// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rdserver.terminal

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.project.Project
import com.intellij.ui.content.Content
import org.jetbrains.plugins.terminal.ShellTerminalWidget
import org.jetbrains.plugins.terminal.TerminalView
import org.jetbrains.plugins.terminal.action.TerminalSessionContextMenuActionBase

class BackendShareTerminalAction : TerminalSessionContextMenuActionBase(), DumbAware {

  companion object {
    private var nextSessionIdIndex = 0
  }

  override fun updateInTerminalToolWindow(e: AnActionEvent, project: Project, content: Content) {
    val terminalWidget = TerminalView.getWidgetByContent(content) as? ShellTerminalWidget
    val connector = terminalWidget?.ttyConnector as? BackendTtyConnector
    e.presentation.isEnabledAndVisible = connector != null && connector.sessionId == null
  }

  override fun actionPerformedInTerminalToolWindow(e: AnActionEvent, project: Project, content: Content) {
    val terminalWidget = TerminalView.getWidgetByContent(content) as? ShellTerminalWidget ?: return
    val sessionId = ClientId.localId.value + "#" + (nextSessionIdIndex++).toString()
    BackendTerminalManager.getInstance(project).shareTerminal(terminalWidget, sessionId)
  }
}