package com.jetbrains.rdserver.terminal

import com.intellij.codeWithMe.ClientId
import com.intellij.codeWithMe.ClientId.Companion.isLocal
import com.intellij.openapi.project.Project
import com.jetbrains.rd.util.addUnique
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.followMe.FollowMeManager
import org.jetbrains.annotations.Nls
import org.jetbrains.plugins.terminal.ShellTerminalWidget
import org.jetbrains.plugins.terminal.TerminalView

class BackendTerminalManager(private val project: Project) {

  companion object {
    fun getInstance(project: Project): BackendTerminalManager = project.getService(BackendTerminalManager::class.java)
  }

  private val sharedTerminals = mutableListOf<SharedTerminalInfo>()

  fun createNewSharedTerminal(sessionId: String, @Nls sessionName: String?) {
    val widget = TerminalView
      .getInstance(project)
      .createLocalShellWidget(null, sessionName, true)
    shareTerminal(widget, sessionId)
  }

  fun shareTerminal(widget: ShellTerminalWidget, sessionId: String) {
    widget.executeWithTtyConnector { connector ->
      val backendTtyConnector = connector as? BackendTtyConnector ?: return@executeWithTtyConnector
      backendTtyConnector.setSessionId(sessionId)

      val sharedTerminal = SharedTerminalInfo(widget, backendTtyConnector)
      sharedTerminals.addUnique(backendTtyConnector.lifetime, sharedTerminal)

      for (clientId in FollowMeManager.getInstance(project).users.keys) {
        if (clientId.isLocal) continue
        val service = RdServerPerClientComponents.getService(project, clientId, BackendTerminalHost::class.java)
        service.bind(widget, backendTtyConnector)
      }
    }
  }

  fun connectClient(clientId: ClientId) {
    for (sharedTerminal in sharedTerminals) {
      val service = RdServerPerClientComponents.getService(project, clientId, BackendTerminalHost::class.java)
      service.bind(sharedTerminal.widget, sharedTerminal.connector)
    }
  }

  private data class SharedTerminalInfo(val widget: ShellTerminalWidget, val connector: BackendTtyConnector)
}