package com.jetbrains.rdserver

import com.intellij.codeWithMe.ClientId
import com.intellij.execution.console.ConsoleViewWrapperBase
import com.intellij.execution.ui.ConsoleViewContentType
import com.intellij.execution.ui.ExecutionConsole
import com.intellij.execution.ui.ObservableConsoleView
import com.intellij.openapi.diagnostic.Logger
import com.intellij.openapi.project.Project
import com.intellij.openapi.rd.createNestedDisposable
import com.jetbrains.codeWithMe.model.CodeWithMeOutputMessage
import com.jetbrains.codeWithMe.model.ConsoleViewContentTypeModel
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.ISignal
import com.jetbrains.rdserver.permissions.PermissionManager
import com.jetbrains.rdserver.permissions.RunAccessLevel

private val logger = Logger.getInstance("#com.jetbrains.rdserver.ConsoleViewUtils")

fun adviseOnTargetOutput(lifetime: Lifetime,
                         project: Project,
                         clientId: ClientId,
                         consoleView: ExecutionConsole,
                         output: ISignal<CodeWithMeOutputMessage>) {
  // TODO implement ObservableConsoleView for TerminalExecutionConsole
  val console: ObservableConsoleView? = when {
    consoleView is ObservableConsoleView -> consoleView
    consoleView is ConsoleViewWrapperBase && consoleView.delegate is ObservableConsoleView -> consoleView.delegate as ObservableConsoleView
    else -> null
  }

  if (console == null) {
    logger.error("session.consoleView is not an ObservableConsoleView, console class is ${consoleView.javaClass}")
    return
  }

  val permissionManager = PermissionManager.getInstance(project)

  console.addChangeListener(object : ObservableConsoleView.ChangeListener {
    override fun contentAdded(types: Collection<ConsoleViewContentType>) {
    }

    override fun textAdded(text: String, type: ConsoleViewContentType) {
      if (permissionManager.getRunAccess(clientId) == RunAccessLevel.Disabled) return
      val contentTypeModel = ConsoleViewContentTypeModel(type.attributesKey?.externalName)
      output.fire(CodeWithMeOutputMessage(text, contentTypeModel))
    }
  }, lifetime.createNestedDisposable())
}