package com.jetbrains.rdserver

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.progress.ProgressIndicator
import com.intellij.openapi.progress.util.AbstractProgressIndicatorExBase
import com.intellij.util.ui.update.MergingUpdateQueue
import com.intellij.util.ui.update.Update
import com.jetbrains.ide.model.notifications.BackgroundTaskModel
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IScheduler

class RdProgressIndicator(
  originalIndicator: ProgressIndicator,
  val taskList: MutableList<Triple<BackgroundTaskModel, ClientId, IScheduler>>,
  val queue: MergingUpdateQueue
) : AbstractProgressIndicatorExBase() {
  init {
    invokeForEach {
      it.cancel.advise(Lifetime.Eternal) {
        originalIndicator.cancel()
      }
    }
    
    // Init model with current values
    invokeForEach { 
      it.header.set(originalIndicator.text)
      it.description.set(originalIndicator.text2)
      it.isIndeterminate.set(originalIndicator.isIndeterminate)
      it.progress.set(originalIndicator.fraction)
    }
  }

  override fun setText(text: String?) {
    queueForEach("text") { it.header.set(text) }
  }

  override fun setText2(text: String?) {
    queueForEach("text2") { it.description.set(text) }
  }

  override fun setFraction(fraction: Double) {
    queueForEach("fraction") { it.progress.set(fraction) }
  }

  override fun setIndeterminate(indeterminate: Boolean) {
    queueForEach("indeterminate") { it.isIndeterminate.set(indeterminate) }
  }

  private fun invokeForEach(block: (BackgroundTaskModel) -> Unit) {
    taskList.forEach { (model, clientId, scheduler) ->
      scheduler.invokeOrQueue {
        if (!model.isBound) return@invokeOrQueue
        ClientId.withClientId(clientId) {
          block(model)
        }
      }
    }
  }

  private fun queueForEach(id: String, block: (BackgroundTaskModel) -> Unit) {
    taskList.forEach { (model, clientId, scheduler) ->
      queue.queue(object : Update(id to model) {
        override fun run() {
          scheduler.invokeOrQueue {
            if (!model.isBound) return@invokeOrQueue
            ClientId.withClientId(clientId) {
              block(model)
            }
          }
        }
      })
    }
  }
}