package com.jetbrains.cwm.unattended.common

import SwingScheduler
import com.intellij.openapi.Disposable
import com.intellij.remote.RemoteCredentials
import com.jetbrains.rd.framework.IdKind
import com.jetbrains.rd.framework.Identities
import com.jetbrains.rd.framework.Protocol
import com.jetbrains.rd.framework.Serializers
import com.jetbrains.rd.platform.codeWithMe.protocol.ConnectionState
import com.jetbrains.rd.platform.codeWithMe.protocol.SecureWireOverStreamTransport
import com.jetbrains.rd.platform.codeWithMe.protocol.SocketStreamTransport
import com.jetbrains.rd.platform.util.SelfSignedCertificate
import com.jetbrains.rd.util.lifetime.LifetimeDefinition
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rd.util.reactive.OptProperty
import com.jetbrains.codeWithMe.model.CodeWithMeUnattendedHostControlModel
import com.jetbrains.cwm.unattended.ssh.SshSessionFacade
import com.jetbrains.cwm.unattended.ssh.SshUtil
import java.net.InetSocketAddress
import java.nio.channels.SocketChannel

class UnattendedHostControlConnection(credentials: RemoteCredentials?, port: Int): Disposable {
  private val connection: SshSessionFacade?
  private val wireLifetime: LifetimeDefinition
  val controlModel = OptProperty<CodeWithMeUnattendedHostControlModel>()

  init {
    val localPort = if (credentials != null) {
      // todo: progress?
      connection = SshUtil.connect(credentials, 5)
      connection.addLocalTunnelWithRandomLocalPort("127.0.0.1", port)
    } else {
      connection = null
      port
    }

    // todo: nonblocking connect
    val socketChannel = SocketChannel.open(InetSocketAddress("127.0.0.1", localPort))
    val streamTransport = SocketStreamTransport(socketChannel, "ClientToControlSocket($port)")
    wireLifetime = LifetimeDefinition()
    val secureWire = SecureWireOverStreamTransport(wireLifetime, streamTransport, SwingScheduler, "ClientToControlSocket($port)", false, { SelfSignedCertificate.permanentCertificateData }, {
      // todo: validate/check server certificate?
    })

    wireLifetime.onTermination {
      socketChannel.close()
      connection?.disconnect()
    }

    secureWire.connectionState.view(wireLifetime) { stateLt, state ->
      if (state != ConnectionState.CONNECTED) return@view

      SwingScheduler.queue {
        val protocol = Protocol(secureWire.id, Serializers(), Identities(IdKind.Client), SwingScheduler, secureWire, wireLifetime)
        controlModel.setIfEmpty(CodeWithMeUnattendedHostControlModel.create(wireLifetime, protocol))
        secureWire.startDeliveringMessages()
      }
    }
  }

  override fun dispose() {
    wireLifetime.terminate()
  }
}