package com.jetbrains.cwm.unattended.common

import com.intellij.ui.ColoredListCellRenderer
import com.intellij.ui.SimpleTextAttributes
import com.intellij.ui.components.JBList
import com.intellij.ui.components.JBScrollPane
import com.intellij.util.ui.UIUtil
import com.jetbrains.rd.platform.codeWithMe.joinLinks.JoinLinkOpenerService
import com.jetbrains.rd.platform.util.withFragment
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IViewableList
import com.jetbrains.codeWithMe.model.RemoteControlProjectInfo
import com.jetbrains.codeWithMe.model.RemoteControlSession
import java.awt.BorderLayout
import java.net.URI
import javax.swing.*

class UnattendedHostProjectListPanel(val lifetime: Lifetime, val session: RemoteControlSession) : JPanel(BorderLayout()) {
  init {
    val list = JBList(MyModel())
    list.cellRenderer = RemoteControlProjectRenderer()

    add(JBScrollPane(list), BorderLayout.CENTER)

    val southPanel = JPanel()
    southPanel.layout = BoxLayout(southPanel, BoxLayout.LINE_AXIS)

    val connectToProjectButton = JButton("Connect to selected project")
    list.addListSelectionListener {
      connectToProjectButton.isEnabled = list.selectedIndex != -1
    }
    connectToProjectButton.addActionListener {
      val selectedProject = session.openProjects[list.selectedIndex]
      selectedProject.createSingleUseJoinToken.start(Unit).result.advise(lifetime) { taskResult ->
        val token = taskResult.unwrap()
        val link = URI.create(selectedProject.joinLink.valueOrNull!!).let { it.withFragment(it.fragment + "&jt=" + token)}
        JoinLinkOpenerService.getInstance().openJoinLink(link)
      }
    }

    val openProjectButton = JButton("Open project")
    openProjectButton.addActionListener {
      UnattendedHostBrowserDialog(session).show()
    }

    southPanel.add(openProjectButton)
    southPanel.add(connectToProjectButton)

    add(southPanel, BorderLayout.SOUTH)
  }

  private inner class RemoteControlProjectRenderer : ColoredListCellRenderer<RemoteControlProjectInfo>() {
    override fun customizeCellRenderer(list: JList<out RemoteControlProjectInfo>,
                                       value: RemoteControlProjectInfo?,
                                       index: Int,
                                       selected: Boolean,
                                       hasFocus: Boolean) {
      background = UIUtil.getListBackground(selected, hasFocus)

      if (value == null) return

      append(value.projectName)
      append(value.projectPath, SimpleTextAttributes.GRAYED_ATTRIBUTES)
    }
  }

  private inner class MyModel : AbstractListModel<RemoteControlProjectInfo>() {
    init {
      session.openProjects.change.advise(lifetime) {
        when(it) {
          is IViewableList.Event.Add -> fireIntervalAdded(this, it.index, it.index)
          is IViewableList.Event.Update -> fireContentsChanged(this, it.index, it.index)
          is IViewableList.Event.Remove -> fireIntervalRemoved(this, it.index, it.index)
        }
      }
    }

    override fun getSize(): Int {
      return session.openProjects.size
    }

    override fun getElementAt(index: Int): RemoteControlProjectInfo {
      return session.openProjects[index]
    }
  }
}