package com.jetbrains.cwm.unattended.ssh

import com.intellij.openapi.options.ShowSettingsUtil
import com.intellij.openapi.project.Project
import com.intellij.remote.RemoteCredentials
import com.intellij.ssh.config.unified.SshConfigManager
import com.intellij.ssh.connectionBuilder
import com.intellij.ssh.ui.unified.SshConfigConfigurable

object SshUtil {
  fun execute(credentials: RemoteCredentials, cmd: String): Process {
    return credentials.connectionBuilder().execBuilder(cmd).execute()
  }

  fun openSftpChannel(credentials: RemoteCredentials): SftpChannelFacade {
    val channel = credentials.connectionBuilder().openSftpChannel()
    return SftpChannelFacade(channel)
  }

  fun getConfigs(project: Project?): List<SshConfigFacade> {
    return SshConfigManager.getInstance(project).configs.map { SshConfigFacade(it) }
  }

  fun showConfigConfigurable(project: Project?) {
    ShowSettingsUtil.getInstance().showSettingsDialog(project, SshConfigConfigurable::class.java)
  }

  fun connect(credentials: RemoteCredentials, timeoutInSeconds: Long): SshSessionFacade {
    return SshSessionFacade(credentials.connectionBuilder().connect(timeoutInSeconds))
  }
}

