package com.jetbrains.rdserver.actions

import com.intellij.ide.actions.NewActionGroup
import com.intellij.openapi.actionSystem.ActionGroup
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.IdeActions
import com.intellij.openapi.actionSystem.ex.ActionManagerEx
import com.intellij.openapi.actionSystem.ex.ActionUtil
import com.jetbrains.rdserver.ui.feelLuckUi.FeelLuckyUiMarker
import com.jetbrains.rdserver.ui.feelLuckUi.ModalDialogSilentOption
import java.util.function.Predicate


data class AnActionFeelLuckyUiMarker(private val actionId: String, private val action: AnAction) : FeelLuckyUiMarker {
  companion object {
    private val anActionsWithSkippedModalUi = setOf(
      "\$Paste",
      "\$Delete",
      "ImplementMethods",
      "Inline"
      /*"Test..."*/
    )
    private val anActionsWithPopup = setOf(
      "GotoDeclaration",
      "ShowUsages",
      "NewClass",
      IdeActions.ACTION_QUICK_IMPLEMENTATIONS
    )
    private val anActionsWithDefaultMessageBoxValue = setOf(
      "GotoDeclaration",
      "ShowUsages",
      "FindUsages",
      "Test...",
      IdeActions.ACTION_UNDO,
      IdeActions.ACTION_REDO
    )
  }

  override fun mayShowPopup(): Boolean {
    val allowPopup = anActionsWithPopup.contains(actionId)
    if(allowPopup)
      return allowPopup
    val actionManager = ActionManagerEx.getInstanceEx()

    val newGroupAction = actionManager.getAction(IdeActions.GROUP_NEW) as ActionGroup
    return ActionUtil.anyActionFromGroupMatches(newGroupAction, false, Predicate.isEqual(action))
           || NewActionGroup.isActionInNewPopupMenu(action)
  }

  override fun maySkipModalDialog(): ModalDialogSilentOption {
    if(anActionsWithSkippedModalUi.contains(actionId))
      return ModalDialogSilentOption.Accept
    return ModalDialogSilentOption.SilentForbidden
  }

  override fun mayUseDefaultValueInMessageBox(): Boolean {
    return anActionsWithDefaultMessageBoxValue.contains(actionId)
  }
}