package com.jetbrains.rdserver.actions

import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.extensions.ProjectExtensionPointName
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.ActionTimestampSetModel

/**
 * Manages [DataContext] <-> Protocol [ActionTimestampSetModel] conversion
 */
abstract class BackendActionTimestampProvider(val project: Project) {

  companion object {
    val EP = ProjectExtensionPointName<BackendActionTimestampProvider>("com.intellij.rdserver.actionTimestampProvider")
  }

  /**
   * Determines whether action [actionId] needs additional timestamps to be checked before execution
   */
  open fun isApplicable(actionId: String): Boolean = true

  /**
   * Creates timestamp set from [DataContext]
   */
  abstract fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet

  /**
   * Creates timestamp set from protocol [ActionTimestampSetModel]
   */
  abstract fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet
}