package com.jetbrains.rdserver.actions

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.actionSystem.ActionPlaces
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.Presentation
import com.intellij.openapi.actionSystem.Toggleable
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.diagnostic.trace
import com.intellij.openapi.project.Project
import com.jetbrains.rd.actions.ActionProvider
import com.jetbrains.rd.framework.IProtocol
import com.jetbrains.rd.framework.impl.RdTask
import com.jetbrains.rd.ide.model.AsyncActionUpdateRequest
import com.jetbrains.rd.ide.model.AsyncActionsModel
import com.jetbrains.rd.ide.model.PresentationModel
import com.jetbrains.rd.util.lifetime.Lifetime

@Suppress("NonDefaultConstructor")
class BackendAsyncActionHost(
  val clientId: ClientId,
  lifetime: Lifetime,
  val project: Project,
  protocol: IProtocol
) {
  companion object {
    private val LOG = logger<BackendAsyncActionHost>()
  }

  val actionsModel = AsyncActionsModel.create(lifetime, protocol).apply {
    update.set { _, actionUpdateRequest ->
      updateAction(actionUpdateRequest)
    }
  }

  // todo grouping, move to background
  private fun updateAction(updateRequest: AsyncActionUpdateRequest): RdTask<PresentationModel?> {
    LOG.trace { "Received update request for action ${updateRequest.actionId}" }
    val handler = ActionExecutionResultHandler(-1)
    val dataContext = createDataContext(updateRequest.actionId, updateRequest.timestampSetModel, handler, project)
    val presentation = Presentation()
    val actionEvent = AnActionEvent.createFromDataContext(ActionPlaces.UNKNOWN, presentation, dataContext)
    try {
      val action = ActionProvider.EP_NAME.extensions.mapNotNull { it.getAction(updateRequest.actionId, actionEvent) }.firstOrNull() ?: run {
        LOG.error("No action found for ${updateRequest.actionId}")
        return RdTask.fromResult(null)
      }
      action.update(actionEvent)
    }
    catch (ex: TimestampUpgradeFailedException) {
      LOG.trace { ex.message }
      return RdTask.fromResult(null)
    }
    return RdTask.fromResult(actionEvent.presentation.toModel())
  }
}

private fun Presentation.toModel(): PresentationModel {
  return PresentationModel(
    isEnabled,
    isVisible,
    text,
    description,
    Toggleable.isSelected(this)
  )
}
