package com.jetbrains.rdserver.actions

import com.intellij.openapi.extensions.ProjectExtensionPointName
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.AddToGroupRuleModel

interface BackendDelegatingActionIdsProvider {
  companion object {
    private val EP = ProjectExtensionPointName<BackendDelegatingActionIdsProvider>(
      "com.intellij.rdserver.backend.delegating.actions")

    fun getActionIds(project: Project): Collection<String> {
      val result = mutableSetOf<String>()
      for (extension in EP.extensions(project)) {
        result.addAll(extension.getActionIds())
      }
      return result
    }

    fun getHintParentGroupIds(project: Project): MutableMap<String, MutableList<AddToGroupRuleModel>> {
      val result = mutableMapOf<String, MutableList<AddToGroupRuleModel>>()
      for (extension in EP.extensions(project)) {
        val hints = extension.getHintParentGroupIds()
        for ((k, v) in hints)
          result.getOrPut(k) { mutableListOf() }.addAll(v)
      }
      return result
    }

    fun getEditorActionWithImmediateResultIds(project: Project): Collection<String> {
      val result = mutableSetOf<String>()
      for (extension in EP.extensions(project)) {
        result.addAll(extension.getEditorActionWithImmediateResultIds())
      }
      return result
    }
  }

  fun getActionIds(): Collection<String>
  fun getHintParentGroupIds(): Map<String, MutableList<AddToGroupRuleModel>>

  fun getEditorActionWithImmediateResultIds(): Collection<String>
}