package com.jetbrains.rdserver.actions

import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.ActionPlaces
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.ex.ActionUtil
import com.intellij.openapi.editor.impl.EditorImpl
import com.intellij.openapi.project.Project
import com.intellij.openapi.util.use
import com.jetbrains.rd.util.reactive.ISignal
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.daemon.suppressMarkupSynchronization
import com.jetbrains.rdserver.editors.BackendEditorHost
import com.jetbrains.rdserver.requests.BackendAsyncRequestDelegate
import com.jetbrains.rdserver.requests.BackendIdeGateway
import com.jetbrains.rd.ide.model.RdCallEditorActionRequest
import com.jetbrains.rd.ide.model.RdFailureResponse
import com.jetbrains.rd.ide.model.RdPatchResponse
import com.jetbrains.rd.ide.model.RdResponse

class BackendEditorActionExecutorDelegate : BackendAsyncRequestDelegate<RdCallEditorActionRequest> {

  // don't check any precondition to speedup requests processing
  override suspend fun execute(request: RdCallEditorActionRequest, project: Project, consumer: (RdResponse) -> Unit) {
    val requestId = request.id
    val actionId = request.actionId
    val action = ActionManager.getInstance().getAction(actionId) ?: return consumer(RdFailureResponse("Not found action with '$actionId' id", requestId))
    val editorHost = BackendEditorHost.getInstance(project)
    val editorId = request.editorId
    val editor = editorHost.tryGetOwnRemoteEditor(editorId) ?: return consumer(RdFailureResponse("Not found editor with id $editorId", requestId))
    val dataContext = (editor as EditorImpl).dataContext
    val actionEvent = AnActionEvent.createFromAnAction(action, null, ActionPlaces.KEYBOARD_SHORTCUT, dataContext)

    editor.suppressMarkupSynchronization("editor action#$requestId") {
      val ideGateway = RdServerPerClientComponents.getService<BackendIdeGateway>(project)
      val patch = ideGateway.executeBulkUpdate(editor) {
        ActionUtil.performActionDumbAwareWithCallbacks(action, actionEvent, dataContext)
      }
      consumer(RdPatchResponse(patch, requestId))
    }
  }
}