package com.jetbrains.rdserver.actions

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.ex.RangeMarkerEx
import com.jetbrains.rd.actions.ActionProvider
import com.jetbrains.rd.actions.GutterActionId
import com.jetbrains.rd.actions.GutterActionKind
import com.jetbrains.rdserver.daemon.markupModelContributor

class BackendGutterActionProvider : ActionProvider {
  companion object {
    fun getRangeMarker(editor: Editor, gutterId: Long): RangeMarkerEx? {
      return editor.markupModelContributor?.getRangeMarker(gutterId) ?: editor.document.markupModelContributor?.getRangeMarker(gutterId)
    }
  }

  override fun getAction(backendActionId: String, actionEvent: AnActionEvent?): AnAction? {
    if (actionEvent == null) {
      return null
    }
    val actionId = GutterActionId.from(backendActionId) ?: return null
    val editor = actionEvent.dataContext.getData(CommonDataKeys.EDITOR) ?: return null
    val highlighter = getRangeMarker(editor, actionId.gutterId) as? RangeHighlighterEx ?: return null
    val gutterIconRenderer = highlighter.gutterIconRenderer ?: return null
    val clickAction = when (actionId.kind) {
      GutterActionKind.LEFT_CLICK -> gutterIconRenderer.clickAction
      GutterActionKind.MIDDLE_CLICK -> gutterIconRenderer.middleButtonClickAction
      GutterActionKind.RIGHT_CLICK -> gutterIconRenderer.rightButtonClickAction
    }
    return clickAction ?: gutterIconRenderer.popupMenuActions
  }
}
