package com.jetbrains.rdserver.actions

import com.intellij.codeInsight.daemon.impl.ShowIntentionsPass
import com.intellij.codeInsight.intention.impl.ShowIntentionActionsHandler
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.psi.PsiDocumentManager
import com.intellij.psi.util.PsiUtilBase
import com.jetbrains.rd.actions.ActionProvider
import com.jetbrains.rd.actions.TooltipActionId

class BackendTooltipActionProvider : ActionProvider {
  override fun getAction(backendActionId: String, actionEvent: AnActionEvent?): AnAction? {
    if (actionEvent == null) {
      return null
    }
    val tooltipActionId = TooltipActionId.from(backendActionId) ?: return null
    val editor = actionEvent.dataContext.getData(CommonDataKeys.EDITOR) ?: return null
    val project = actionEvent.dataContext.getData(CommonDataKeys.PROJECT) ?: return null
    val psiFile = PsiDocumentManager.getInstance(project).getPsiFile(editor.document) ?: return null

    val highlighter = BackendGutterActionProvider.getRangeMarker(editor, tooltipActionId.tooltipId) ?: return null

    val intention = ShowIntentionsPass.getAvailableFixes(editor, psiFile, -1, highlighter.startOffset)
                      .firstOrNull { it.action.text == tooltipActionId.myActionText }?.action ?: return null

    return object : AnAction() {
      override fun actionPerformed(e: AnActionEvent) {
        val file = PsiUtilBase.getPsiFileInEditor(editor, project) ?: return
        ShowIntentionActionsHandler.chooseActionAndInvoke(file, editor, intention, tooltipActionId.myActionText)
      }
    }
  }
}