package com.jetbrains.rdserver.actions

import com.intellij.openapi.editor.impl.EditorImpl
import com.intellij.openapi.project.Project
import com.intellij.openapi.util.use
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.ISignal
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.completion.RdServerAutoPopupController
import com.jetbrains.rdserver.daemon.suppressMarkupSynchronization
import com.jetbrains.rdserver.editors.BackendEditorHost
import com.jetbrains.rdserver.requests.BackendAsyncRequestDelegate
import com.jetbrains.rdserver.requests.BackendIdeGateway
import com.jetbrains.rd.ide.model.RdFailureResponse
import com.jetbrains.rd.ide.model.RdPatchResponse
import com.jetbrains.rd.ide.model.RdResponse
import com.jetbrains.rd.ide.model.RdTypeKeyRequest

class BackendTypedActionExecutorDelegate : BackendAsyncRequestDelegate<RdTypeKeyRequest> {

  // don't check any precondition to speedup requests processing
  override suspend fun execute(request: RdTypeKeyRequest, project: Project, consumer: (RdResponse) -> Unit) {
    val requestId = request.id
    val editorHost = BackendEditorHost.getInstance(project)
    val editorId = request.editorId
    val editor = editorHost.tryGetOwnRemoteEditor(editorId) ?: return consumer(RdFailureResponse("Not found editor with $editorId id", requestId))
    val char = request.char

    val autoPopupController = RdServerAutoPopupController.getInstance(project)
    Lifetime.using { lt ->
      editor.suppressMarkupSynchronization(lt, "typed action#$requestId")
      autoPopupController.suppressor.suppress(lt)
      val ideGateway = RdServerPerClientComponents.getService<BackendIdeGateway>(project)
      val patch = ideGateway.executeBulkUpdate(editor) {
        (editor as EditorImpl).type(char.toString())
      }

      consumer(RdPatchResponse(patch, requestId))
    }
  }
}