package com.jetbrains.rdserver.actions

import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.components.service
import com.intellij.util.concurrency.annotations.RequiresEdt
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isEternal
import com.jetbrains.rd.util.lifetime.isNotAlive
import com.jetbrains.rd.util.lifetime.onTermination

class BeHostRemoteDataContextManager {
  companion object {
    fun getInstance(): BeHostRemoteDataContextManager = service()
  }

  private val contexts = mutableMapOf<Int, DataContext>()
  private var lastId: Int = 0

  @RequiresEdt
  fun registerContext(context: DataContext, lifetime: Lifetime): Int? {
    if (lifetime.isNotAlive || lifetime.isEternal) return null

    val contextId = ++lastId
    contexts.put(contextId, context)
    lifetime.onTermination { contexts.remove(contextId) }
    return contextId
  }

  @RequiresEdt
  fun getContext(contextId: Int): DataContext? = contexts.get(contextId)
}