package com.jetbrains.rdserver.actions

import com.intellij.execution.ExecutorRegistryImpl
import com.intellij.openapi.actionSystem.IdeActions
import com.jetbrains.rd.ide.model.ActionConstraintModel
import com.jetbrains.rd.ide.model.AddToGroupRuleModel
import com.jetbrains.rd.ide.model.AnchorModel

class DefaultBackendDelegatingActionIdsProvider : BackendDelegatingActionIdsProvider {
  private val id2ParentGroupHints = mapOf(
    "CompileDirty" to mutableListOf(AddToGroupRuleModel("ToolbarRunGroup", ActionConstraintModel(null, AnchorModel.First))),
    "ContextRun" to mutableListOf(AddToGroupRuleModel(ExecutorRegistryImpl.RUN_CONTEXT_GROUP, ActionConstraintModel(null, AnchorModel.First))),
    "ContextDebug" to mutableListOf(AddToGroupRuleModel(ExecutorRegistryImpl.RUN_CONTEXT_GROUP, ActionConstraintModel(null, AnchorModel.First)))
  )

  override fun getHintParentGroupIds(): Map<String, MutableList<AddToGroupRuleModel>> {
    return id2ParentGroupHints
  }

  override fun getEditorActionWithImmediateResultIds(): Collection<String> {
    return listOf(
      IdeActions.ACTION_EDITOR_CUT,
      IdeActions.ACTION_EDITOR_COPY,
      IdeActions.ACTION_EDITOR_PASTE,
      IdeActions.ACTION_EDITOR_PASTE_SIMPLE,
      IdeActions.ACTION_EDITOR_DELETE,
      IdeActions.ACTION_EDITOR_DELETE_TO_WORD_START,
      IdeActions.ACTION_EDITOR_DELETE_TO_WORD_END,
      IdeActions.ACTION_EDITOR_DELETE_LINE,
      IdeActions.ACTION_EDITOR_ENTER,
      IdeActions.ACTION_EDITOR_START_NEW_LINE,
      IdeActions.ACTION_EDITOR_SPLIT,
      IdeActions.ACTION_EDITOR_TEXT_START,
      IdeActions.ACTION_EDITOR_TEXT_END,
      IdeActions.ACTION_EDITOR_FORWARD_PARAGRAPH,
      IdeActions.ACTION_EDITOR_BACKWARD_PARAGRAPH,
      IdeActions.ACTION_EDITOR_FORWARD_PARAGRAPH_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_BACKWARD_PARAGRAPH_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_TEXT_START_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_TEXT_END_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_MOVE_LINE_START,
      IdeActions.ACTION_EDITOR_MOVE_LINE_END,
      IdeActions.ACTION_EDITOR_MOVE_LINE_START_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_MOVE_LINE_END_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_BACKSPACE,
      IdeActions.ACTION_EDITOR_MOVE_CARET_LEFT_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_MOVE_CARET_RIGHT_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_MOVE_CARET_UP_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_MOVE_CARET_DOWN_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_SWAP_SELECTION_BOUNDARIES,
      IdeActions.ACTION_EDITOR_MOVE_CARET_UP,
      IdeActions.ACTION_EDITOR_MOVE_CARET_LEFT,
      IdeActions.ACTION_EDITOR_MOVE_CARET_DOWN,
      IdeActions.ACTION_EDITOR_MOVE_CARET_RIGHT,
      /* disabled, because they use viewport to calculate line shift
        IdeActions.ACTION_EDITOR_MOVE_CARET_PAGE_UP,IdeActions.ACTION_EDITOR_MOVE_CARET_PAGE_DOWN,
        IdeActions.ACTION_EDITOR_MOVE_CARET_PAGE_UP_WITH_SELECTION, IdeActions.ACTION_EDITOR_MOVE_CARET_PAGE_DOWN_WITH_SELECTION,*/
      IdeActions.ACTION_EDITOR_NEXT_WORD,
      IdeActions.ACTION_EDITOR_PREVIOUS_WORD,
      IdeActions.ACTION_EDITOR_NEXT_WORD_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_PREVIOUS_WORD_WITH_SELECTION,
      IdeActions.ACTION_EDITOR_TAB,
      /* todo escape
      IdeActions.ACTION_EDITOR_ESCAPE,*/
      IdeActions.ACTION_EDITOR_JOIN_LINES,
      IdeActions.ACTION_EDITOR_COMPLETE_STATEMENT,
      /* todo multiple carets
      IdeActions.ACTION_EDITOR_ADD_OR_REMOVE_CARET,
      IdeActions.ACTION_EDITOR_CREATE_RECTANGULAR_SELECTION,
      IdeActions.ACTION_EDITOR_ADD_RECTANGULAR_SELECTION_ON_MOUSE_DRAG,
      IdeActions.ACTION_EDITOR_CREATE_RECTANGULAR_SELECTION_ON_MOUSE_DRAG,
      IdeActions.ACTION_EDITOR_CLONE_CARET_BELOW,
      IdeActions.ACTION_EDITOR_CLONE_CARET_ABOVE,
      IdeActions.ACTION_EDITOR_ADD_CARET_PER_SELECTED_LINE,*/
      /* todo synchronize editor settings instead of
      IdeActions.ACTION_EDITOR_TOGGLE_STICKY_SELECTION,
      IdeActions.ACTION_EDITOR_TOGGLE_OVERWRITE_MODE,*/
      IdeActions.ACTION_EDITOR_TOGGLE_CASE,
      "EditorStartNewLineBefore",
      IdeActions.ACTION_EDITOR_INDENT_SELECTION,
      IdeActions.ACTION_EDITOR_UNINDENT_SELECTION,
      IdeActions.ACTION_MOVE_LINE_UP_ACTION,
      IdeActions.ACTION_MOVE_LINE_DOWN_ACTION,
      IdeActions.ACTION_EDITOR_DUPLICATE,
      IdeActions.ACTION_EDITOR_DUPLICATE_LINES,
      IdeActions.ACTION_EDITOR_SORT_LINES,
      IdeActions.ACTION_EDITOR_REVERSE_LINES
    )
  }

  override fun getActionIds(): Collection<String> {
    return listOf(
      /* undo/redo */
      IdeActions.ACTION_UNDO,
      IdeActions.ACTION_REDO,

      /* editor actions */
      IdeActions.ACTION_EDITOR_SELECT_WORD_AT_CARET,
      IdeActions.ACTION_EDITOR_UNSELECT_WORD_AT_CARET,
      IdeActions.ACTION_MOVE_STATEMENT_UP_ACTION,
      IdeActions.ACTION_MOVE_STATEMENT_DOWN_ACTION,
      IdeActions.MOVE_ELEMENT_LEFT,
      IdeActions.MOVE_ELEMENT_RIGHT,
      "EditorCodeBlockStart",
      "EditorCodeBlockEnd",
      "EditorCodeBlockStartWithSelection",
      "EditorCodeBlockEndWithSelection",
      IdeActions.ACTION_COMMENT_BLOCK,
      IdeActions.ACTION_COMMENT_LINE,
      IdeActions.ACTION_EDITOR_SHOW_PARAMETER_INFO,


      /* build */
      "CompileDirty",

      /* save document */
      "SaveDocument",
      "SaveAll",

      /* reformat/cleanup */
      IdeActions.ACTION_EDITOR_REFORMAT,
      "OptimizeImports",
      "AutoIndentLines",
      IdeActions.ACTION_EDITOR_EMACS_TAB,
      /* navigation */
      IdeActions.ACTION_GOTO_DECLARATION,
      IdeActions.ACTION_GOTO_TYPE_DECLARATION,
      //"GotoRelated",
      IdeActions.ACTION_GOTO_SUPER,
      IdeActions.ACTION_GOTO_IMPLEMENTATION,
      IdeActions.ACTION_FIND_USAGES,
      "ShowUsages",
      "EditorMatchBrace",
      "QuickJavaDoc",
      IdeActions.ACTION_QUICK_IMPLEMENTATIONS,

      /* code */
      //"DelegateMethods",
      //"ImplementMethods",
      //"OverrideMethods",
      "Generate",
      "SurroundWith",
      //"SurroundWithLiveTemplate",
      //"InsertLiveTemplate",
      "MethodDown", "MethodUp",
      //IdeActions.ACTION_QUICK_JAVADOC,
      //IdeActions.ACTION_EDITOR_SHOW_PARAMETER_INFO,

      /* refactorings */
      //"Refactorings.QuickListPopupAction",
      IdeActions.ACTION_RENAME,
      "IntroduceVariable",
      "Inline",
      //"ChangeSignature",
      //"ExtractSuperclass",
      //"ExtractInterface",
      //"MembersPullUp",
      //"MemberPushDown",
      //"ExtractMethod",
      //"IntroduceField",
      //"IntroduceParameter",
      //"SafeDelete",
      //IdeActions.ACTION_MOVE,
      //"CopyElement"

      /* project view context menu */
      "NewGroup",
      "NewModuleInGroup",
      "NewElement",

      /* misc */
      //"TypeHierarchy",
      "HighlightUsagesInFile",
      "CopyReference",
      "ContextRun",
      "ContextDebug",

      /* vcs */
      "Vcs.RollbackChangedLines",
      "CompareClipboardWithSelection",
      "Compare.SameVersion",
      "Diff.ShowDiff",
      "ChangesView.GroupBy",
      "ChangesView.GroupBy.Directory",
      "ChangesView.GroupBy.Module",
      "ChangesView.GroupBy.Repository",
      "EditSource", // TODO: clash with Project View ?
      "BackendBeControlsDSLShowCase",
    )
  }
}