package com.jetbrains.rdserver.actions

import com.intellij.icons.AllIcons
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.impl.ActionToolbarImpl
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.codeWithMe.statistics.TelephonyStatisticsCollector
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rdserver.CodeWithMePluginBundle
import com.jetbrains.rdserver.joinLinks.RdServerJoinLinkManager
import com.jetbrains.rdserver.telephony.BackendTelephonyManager

class EnableVoiceCallAction : AnAction(), DumbAware {

  private var listenersAdded = false

  private fun addListeners(project: Project) {
    val manager = BackendTelephonyManager.getInstance(project)
    manager.isTelephonyEnabled.advise(handler = {
      triggerToolbarRepaint()
    }, lifetime = project.lifetime)

    manager.telephonySession.advise(handler = {
      it.let {
        it?.conferenceJoined?.advise(handler = {
          triggerToolbarRepaint()

        }, lifetime = project.lifetime)
      }
      triggerToolbarRepaint()

    }, lifetime = project.lifetime)
  }

  private fun triggerToolbarRepaint() {
    application.invokeLater(Runnable {
      ActionToolbarImpl.updateAllToolbarsImmediately()
    })
  }
  override fun actionPerformed(e: AnActionEvent) {
    val project = e.project ?: return
    val manager = BackendTelephonyManager.getInstance(project)
    val newValue = !manager.isTelephonyEnabled.value
    manager.isTelephonyEnabled.set(newValue)

    TelephonyStatisticsCollector.enabledFromAction(e.project, newValue)

    if (!listenersAdded) {
      addListeners(project)
      listenersAdded = true
    }
  }

  override fun update(e: AnActionEvent) {
    val project = e.project
    if (project == null) {
      e.presentation.isEnabledAndVisible = false
      return
    }

    val rdServerManager = RdServerJoinLinkManager.getInstance(project)
    val joinLink = rdServerManager.getExistingJoinLink()
    if (joinLink == null) {
      e.presentation.isEnabledAndVisible = false
      return
    }

    val url = rdServerManager.getJoinLinkData(joinLink)?.telephonyUrl
    if (url == null) {
      e.presentation.isEnabledAndVisible = false
      return
    }

    e.presentation.isEnabledAndVisible = true

    val manager = BackendTelephonyManager.getInstance(project)
    if (manager.isTelephonyEnabled.value) {
      e.presentation.icon = AllIcons.CodeWithMe.CwmDisableCall
      e.presentation.text = CodeWithMePluginBundle.message("action.DisableVoiceCallAction.disable.text")
    }
    else {
      e.presentation.icon = AllIcons.CodeWithMe.CwmEnableCall
      e.presentation.text = CodeWithMePluginBundle.message("action.DisableVoiceCallAction.enable.text")
    }
  }
}