package com.jetbrains.rdserver.actions

import com.intellij.codeWithMe.ClientId
import com.intellij.execution.executors.DefaultDebugExecutor
import com.intellij.execution.executors.DefaultRunExecutor
import com.intellij.execution.lineMarker.ExecutorAction
import com.intellij.execution.lineMarker.LineMarkerActionWrapper
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.intellij.openapi.wm.ToolWindowId
import com.jetbrains.rd.ui.bindable.ActionFilterProvider
import com.jetbrains.rd.ui.bindable.ActionFilterResult
import com.jetbrains.rdserver.permissions.PermissionManager
import com.jetbrains.rdserver.permissions.RunAccessLevel

class RunGutterActionFilterProvider : ActionFilterProvider {
  override fun filterAction(action: AnAction, actionId: String, project: Project, id: ClientId): ActionFilterResult {
    if (action !is LineMarkerActionWrapper) return ActionFilterResult.OK

    val runContextAction = action.delegate as? ExecutorAction
    if (runContextAction == null) return ActionFilterResult.REMOVE
    val executor = runContextAction.executor
    if (executor.id == ToolWindowId.DEBUG || executor.id == ToolWindowId.RUN) {
      val currentRunAccess = PermissionManager.getInstance(project).getRunAccess(ClientId.current)
      if (currentRunAccess != RunAccessLevel.FullAccess)
        return ActionFilterResult.FORBIDDEN
      if (!executor.isApplicable(project)) return ActionFilterResult.REMOVE
      return ActionFilterResult.OK
    }
    return ActionFilterResult.REMOVE
  }
}