package com.jetbrains.rdserver.actions.timestamps

import com.intellij.openapi.editor.Caret
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.CaretTimestampModel
import com.jetbrains.rd.platform.Range
import com.jetbrains.rdserver.actions.ActionTimestampKey
import com.jetbrains.rdserver.actions.ActionTimestampSet
import com.jetbrains.rdserver.actions.TimestampUpgradeResult
import com.jetbrains.rdserver.actions.UpgradableTimestamp
import com.jetbrains.rdserver.document.BackendDocumentHost

class CaretTimestamp(private val offsetTimestamp: OffsetTimestamp, private val rangeMarkerTimestamp: RangeMarkerTimestamp)
  : UpgradableTimestamp<CaretTimestampModel>(NAME, CaretTimestampModel(rangeMarkerTimestamp.model, offsetTimestamp.model)) {
  companion object {
    val NAME = ActionTimestampKey<CaretTimestamp, CaretTimestampModel>("CARET_MODEL")

    fun create(project: Project, caret: Caret): CaretTimestamp? {
      val document = caret.editor.document
      val documentSynchronizer = BackendDocumentHost.getInstance(project).tryGetSynchronizer(document) ?: return null
      val offsetTimestamp = OffsetTimestamp.create(documentSynchronizer, caret.offset)
      val rangeMarkerTimestamp = RangeMarkerTimestamp.create(documentSynchronizer, Range(caret.selectionStart, caret.selectionEnd))
      return CaretTimestamp(offsetTimestamp, rangeMarkerTimestamp)
    }
  }

  override fun upgrade(actualSet: ActionTimestampSet): TimestampUpgradeResult {
    val offsetUpgradeResult = offsetTimestamp.upgrade(actualSet)
    if (offsetUpgradeResult !is TimestampUpgradeResult.Success<*>) return offsetUpgradeResult
    val upgradedOffset = offsetUpgradeResult.timestamp as OffsetTimestamp

    val rangeUpgradeResult = rangeMarkerTimestamp.upgrade(actualSet)
    if (rangeUpgradeResult !is TimestampUpgradeResult.Success<*>) return rangeUpgradeResult
    val upgradedRange = rangeUpgradeResult.timestamp as RangeMarkerTimestamp

    return TimestampUpgradeResult.Success(CaretTimestamp(upgradedOffset, upgradedRange))
  }
}