package com.jetbrains.rdserver.actions.timestamps

import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.actionSystem.DataKey
import com.intellij.openapi.actionSystem.DataProvider
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeContextIdTimestampModel
import com.jetbrains.rd.ide.model.ActionTimestampSetModel
import com.jetbrains.rdserver.actions.*

private val CWM_CONTEXT_ID_KEY = DataKey.create<Int>("CWM_CONTEXT_ID_KEY")

class BeContextIdTimestamp(private val contextId: Int)
  : UpgradableTimestamp<BeContextIdTimestampModel>(NAME, BeContextIdTimestampModel(contextId)) {
  companion object {
    val NAME = ActionTimestampKey<BeContextIdTimestamp, BeContextIdTimestampModel>(CWM_CONTEXT_ID_KEY.name)
  }

  override fun upgrade(actualSet: ActionTimestampSet): TimestampUpgradeResult {
    return TimestampUpgradeResult.Success(BeContextIdTimestamp(contextId))
  }

  override fun getData(): Int = model.contextId
}

class BeContextIdTimestampModelProvider(project: Project) : BackendActionTimestampProvider(project) {
  override fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet {
    return ActionTimestampSet.EMPTY
  }

  override fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet {
    val timestampModel = setModel.getTimestamp(BeContextIdTimestamp.NAME) ?: return ActionTimestampSet.EMPTY

    val contextId = timestampModel.contextId
    return ActionTimestampSet(BeContextIdTimestamp(contextId))
  }
}

class BeContextIdDataProvider : BackendActionDataProvider {
  override fun getData(dataId: String, dataProvider: DataProvider): Any? {
    val contextId = CWM_CONTEXT_ID_KEY.getData(dataProvider) ?: return null
    val delegateContext = BeHostRemoteDataContextManager.getInstance().getContext(contextId) ?: return null
    return delegateContext.getData(dataId)
  }
}
