package com.jetbrains.rdserver.actions.timestamps

import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.editor.Document
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.ActionTimestampSetModel
import com.jetbrains.rd.ide.model.DocumentTimestampModel
import com.jetbrains.rd.ide.model.RdDocumentId
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rdserver.actions.*
import com.jetbrains.rdserver.document.BackendDocumentHost

class DocumentTimestamp(documentId: RdDocumentId, documentVersion: RdDocumentVersion) :
  ActionTimestamp<DocumentTimestampModel>(NAME, DocumentTimestampModel(documentId, documentVersion)) {
  companion object {
    val NAME = ActionTimestampKey<DocumentTimestamp, DocumentTimestampModel>("DOCUMENT")

    fun create(project: Project, document: Document): DocumentTimestamp? {
      val documentHost = BackendDocumentHost.getInstance(project)
      val synchronizer = documentHost.tryGetSynchronizer(document) ?: return null
      val documentId = synchronizer.documentId
      val version = synchronizer.version
      return DocumentTimestamp(documentId, version)
    }
  }
}

class BackendDocumentTimestampProvider(project: Project) : BackendActionTimestampProvider(project) {
  override fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet {
    val emptySet = ActionTimestampSet.EMPTY
    val project = CommonDataKeys.PROJECT.getData(dataContext) ?: return emptySet
    val document = CommonDataKeys.EDITOR.getData(dataContext)?.document ?: return emptySet
    val timestamp = DocumentTimestamp.create(project, document) ?: return emptySet
    return ActionTimestampSet(timestamp)
  }

  override fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet {
    val timestampModel = setModel.getTimestamp(DocumentTimestamp.NAME) ?: return ActionTimestampSet.EMPTY
    return ActionTimestampSet(DocumentTimestamp(timestampModel.documentId, timestampModel.version))
  }
}
