package com.jetbrains.rdserver.actions.timestamps

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.project.Project
import com.intellij.openapi.vfs.VirtualFileManager
import com.intellij.pom.Navigatable
import com.intellij.psi.impl.file.PsiDirectoryFactory
import com.jetbrains.rd.ide.model.ActionTimestampSetModel
import com.jetbrains.rd.ide.model.NavigatablePsiDirectoryTimestampModel
import com.jetbrains.rd.ide.model.NavigatableTimestampModel
import com.jetbrains.rd.ide.model.NavigatablesArrayTimestampModel
import com.jetbrains.rdserver.actions.*
import com.jetbrains.rdserver.startup.tryGetProject
import java.nio.file.Paths

class NavigatablesArrayTimestamp(navigatablesModel: List<NavigatableTimestampModel>, val project: Project) :
  ActionTimestamp<NavigatablesArrayTimestampModel>(NAME, NavigatablesArrayTimestampModel(navigatablesModel)) {

  private val navigatables: Array<Navigatable>

  companion object {
    val NAME = ActionTimestampKey<NavigatablesArrayTimestamp, NavigatablesArrayTimestampModel>(
      CommonDataKeys.NAVIGATABLE_ARRAY.name)
  }

  init {
    navigatables = navigatablesModel
      .filterIsInstance<NavigatablePsiDirectoryTimestampModel>()
      .mapNotNull { VirtualFileManager.getInstance().findFileByNioPath(Paths.get(it.systemIndependentPath)) }
      .map { PsiDirectoryFactory.getInstance(project).createDirectory(it) }
      .toTypedArray()
  }

  override fun getData(): Array<Navigatable>? {
    if (navigatables.isEmpty()) return null
    return navigatables
  }
}

class BackendNavigatablesArrayTimestampProvider(project: Project) : BackendActionTimestampProvider(project) {
  override fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet {
    return ActionTimestampSet.EMPTY
  }

  override fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet {
    val timestampModel = setModel.getTimestamp(NavigatablesArrayTimestamp.NAME) ?: return ActionTimestampSet.EMPTY
    val project = ClientId.current.tryGetProject() ?: return ActionTimestampSet.EMPTY
    return ActionTimestampSet(NavigatablesArrayTimestamp(timestampModel.navigatables, project))
  }
}