package com.jetbrains.rdserver.actions.timestamps

import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.project.Project
import com.jetbrains.rdserver.actions.ActionTimestampKey
import com.jetbrains.rdserver.actions.ActionTimestampSet
import com.jetbrains.rdserver.actions.BackendActionTimestampProvider
import com.jetbrains.rdserver.actions.TransparentTimestamp
import com.jetbrains.rd.ide.model.ActionTimestampModel
import com.jetbrains.rd.ide.model.ActionTimestampSetModel

/**
 * Is added by [BackendProjectTimestampProvider] no protocol transfer involved
 */
class ProjectTimestampModel : ActionTimestampModel()

class ProjectTimestamp(val project: Project) : TransparentTimestamp<ProjectTimestampModel>(PROJECT, ProjectTimestampModel()) {
  companion object {
    val PROJECT = ActionTimestampKey<ProjectTimestamp, ProjectTimestampModel>(CommonDataKeys.PROJECT.name)
  }

  override fun getData() = project
}

class BackendProjectTimestampProvider(project: Project) : BackendActionTimestampProvider(project) {
  override fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet {
    return ActionTimestampSet(ProjectTimestamp(project))
  }

  override fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet {
    return ActionTimestampSet(ProjectTimestamp(project))
  }
}