package com.jetbrains.rdserver.actions.timestamps

import com.intellij.codeWithMe.ClientId
import com.intellij.ide.util.treeView.AbstractTreeNode
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.project.Project
import com.jetbrains.rdserver.actions.*
import com.jetbrains.rdserver.projectView.BackendProjectViewHost
import com.jetbrains.rd.ide.model.ActionTimestampSetModel
import com.jetbrains.rd.ide.model.ProjectViewTimestampModel

data class BackendProjectViewSelection(private val paneId: String, private val ids: List<Int>) {
  fun getNodes(project: Project) : List<AbstractTreeNode<*>> {
    val host = BackendProjectViewHost.getInstance(project, ClientId.current)
    return ids.mapNotNull { host.getNodeById(paneId, it) }
  }
}

class ProjectViewTimestamp(paneId: String, selectedNodeIds: List<Int>, private val project: Project)
  : UpgradableTimestamp<ProjectViewTimestampModel>(NAME, ProjectViewTimestampModel(paneId, selectedNodeIds)) {
  companion object {
    val NAME = ActionTimestampKey<ProjectViewTimestamp, ProjectViewTimestampModel>(BackendDataKeys.PROJECT_VIEW_SELECTION.name)
  }

  override fun upgrade(actualSet: ActionTimestampSet): TimestampUpgradeResult {
    if (model.selectedNodeIds.size != getData().getNodes(project).size)
      return TimestampUpgradeResult.Failure("ProjectView was already changed, operation can not be performed")

    return TimestampUpgradeResult.Success(this)
  }

  override fun getData(): BackendProjectViewSelection {
    return BackendProjectViewSelection(model.paneId, model.selectedNodeIds)
  }
}

class BackendProjectViewTimestampProvider(project: Project) : BackendActionTimestampProvider(project) {
  override fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet {
    return ActionTimestampSet.EMPTY
  }

  override fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet {
    val timestampModel = setModel.getTimestamp(ProjectViewTimestamp.NAME) ?: return ActionTimestampSet.EMPTY
    return ActionTimestampSet(ProjectViewTimestamp(timestampModel.paneId, timestampModel.selectedNodeIds, project))
  }
}
