package com.jetbrains.rdserver.browsers

import com.intellij.codeWithMe.ClientId
import com.intellij.ide.browsers.BrowserLauncherImpl
import com.intellij.ide.browsers.WebBrowser
import com.intellij.openapi.project.Project
import java.io.File
import java.nio.file.Path

/**
 * All requests to open a browser are forbidden for guests.
 * Temp solution until better yet safe UX is figured out.
 */
class BackendBrowserLauncher : BrowserLauncherImpl() {
  override fun open(url: String) {
    if (ClientId.isCurrentlyUnderLocalId) {
      return super.open(url)
    }
  }

  override fun openOrBrowse(_url: String, browse: Boolean, project: Project?) {
    if (ClientId.isCurrentlyUnderLocalId) {
      super.openOrBrowse(_url, browse, project)
    }
  }

  override fun openWithExplicitBrowser(url: String, browserPath: String?, project: Project?) {
    if (ClientId.isCurrentlyUnderLocalId) {
      super.openWithExplicitBrowser(url, browserPath, project)
    }
  }

  override fun browse(url: String, browser: WebBrowser?, project: Project?) {
    if (ClientId.isCurrentlyUnderLocalId) {
      super.browse(url, browser, project)
    }
  }

  override fun browse(file: File) {
    if (ClientId.isCurrentlyUnderLocalId) {
      super.browse(file)
    }
  }

  override fun browse(file: Path) {
    if (ClientId.isCurrentlyUnderLocalId) {
      super.browse(file)
    }
  }

  override fun browseUsingPath(
    url: String?,
    browserPath: String?,
    browser: WebBrowser?,
    project: Project?,
    openInNewWindow: Boolean,
    additionalParameters: Array<String>
  ): Boolean {
    if (ClientId.isCurrentlyUnderLocalId) {
      return super.browseUsingPath(
        url,
        browserPath,
        browser,
        project,
        openInNewWindow,
        additionalParameters
      )
    }
    return false
  }
}

