package com.jetbrains.rdserver.build

import com.intellij.build.BuildProgressListener
import com.intellij.build.BuildViewManager
import com.intellij.openapi.components.service
import com.intellij.openapi.fileEditor.OpenFileDescriptor
import com.intellij.openapi.project.Project
import com.intellij.openapi.rd.createNestedDisposable
import com.intellij.openapi.vfs.VfsUtil
import com.jetbrains.rd.framework.Protocol
import com.jetbrains.rd.ide.model.ThinClientBuildModel
import com.jetbrains.rd.util.lifetime.Lifetime
import java.io.File

@Suppress("NonDefaultConstructor")
class BuildHost(
  lifetime: Lifetime,
  project: Project,
  protocol: Protocol
) {
  private val buildModel = ThinClientBuildModel.create(lifetime, protocol)

  init {
    project.service<BuildViewManager>().addListener(BuildProgressListener { buildId, event ->
      val model = event.toProtocolModel(project, buildId)
      buildModel.buildEvent.fire(model)
    }, lifetime.createNestedDisposable())

    buildModel.navigate.advise(lifetime) {
      val file = VfsUtil.findFileByIoFile(File(it.position.filePath), true) ?: return@advise
      val descriptor = OpenFileDescriptor(project, file, it.position.line, it.position.column)
      descriptor.navigate(it.requestFocus)
    }
  }
}