package com.jetbrains.rdserver.clipboard

import com.intellij.codeWithMe.ClientId
import com.intellij.codeWithMe.ClientId.Companion.isValid
import com.intellij.ide.CopyPasteManagerEx
import com.intellij.openapi.editor.Document
import com.intellij.openapi.progress.ProcessCanceledException
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.startup.tryGetProject
import java.awt.datatransfer.Clipboard
import java.awt.datatransfer.DataFlavor
import java.awt.datatransfer.Transferable

class BackendCopyPasteManager : CopyPasteManagerEx() {
  private val remoteClipboard: RemoteClipboard get() {
    val clientId = ClientId.currentOrNull ?: error("local id")
    if (!clientId.isValid) throw ProcessCanceledException()
    val project = clientId.tryGetProject() ?: error("Not found project for $clientId")
    return RdServerPerClientComponents.getService(project)
  }

  override fun areDataFlavorsAvailable(vararg flavors: DataFlavor): Boolean {
    if (ClientId.isCurrentlyUnderLocalId) {
      if (application.isHeadlessEnvironment)
        return false

      return super.areDataFlavorsAvailable(*flavors)
    }

    return remoteClipboard.areDataFlavorsAvailable(*flavors)
  }

  override fun setContents(content: Transferable) {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.setContents(content)
    return remoteClipboard.setContents(content)
  }

  override fun lostOwnership(clipboard: Clipboard?, contents: Transferable?) {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.lostOwnership(clipboard, contents)

    error("Can't process lost ownership for non local client")
  }

  override fun stopKillRings() {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.stopKillRings()

    remoteClipboard.stopKillRings()
  }

  override fun stopKillRings(document: Document) {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.stopKillRings(document)

    remoteClipboard.stopKillRings()
  }

  override fun getContents(): Transferable? {
    if (ClientId.isCurrentlyUnderLocalId) {
      if (application.isHeadlessEnvironment)
        return null

      return super.getContents()
    }

    return remoteClipboard.getContents()
  }

  override fun <T : Any?> getContents(flavor: DataFlavor): T? {
    if (ClientId.isCurrentlyUnderLocalId) {
      if (application.isHeadlessEnvironment)
        return null

      return super.getContents(flavor)
    }

    return remoteClipboard.getContents(flavor)
  }
}