package com.jetbrains.rdserver.clipboard

import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.getStringContent
import com.jetbrains.rd.ide.model.RdAddHistoryItemToTop
import com.jetbrains.rd.ide.model.RdClipboardChange
import com.jetbrains.rd.util.reflection.usingTrueFlag
import com.jetbrains.rdserver.requests.BackendIdeGateway
import java.awt.datatransfer.DataFlavor
import java.awt.datatransfer.StringSelection
import java.awt.datatransfer.Transferable

@Suppress("NonDefaultConstructor")
class RemoteClipboard(
  private val project: Project
) {
  private var recentTransferable: Transferable? = null

  private var applyingRemoteChange = false

  var version: Int = -1
    private set

  fun areDataFlavorsAvailable(vararg flavors: DataFlavor): Boolean {
    val transferable = recentTransferable ?: return false
    return flavors.any { transferable.isDataFlavorSupported(it) }
  }

  fun setContents(content: Transferable) {
    val stringContent = getStringContent(content) ?: return

    val oldVersion = version
    version++
    recentTransferable = content

    if (applyingRemoteChange) return

    val change = RdAddHistoryItemToTop(stringContent)
    BackendIdeGateway.getInstance(project).sendClipboardChange(change, oldVersion, version)
  }

  fun getContents(): Transferable? {
    return recentTransferable
  }

  fun <T : Any?> getContents(flavor: DataFlavor): T? {
    if (areDataFlavorsAvailable(flavor)) {
      @Suppress("UNCHECKED_CAST")
      return recentTransferable?.getTransferData(flavor) as T?
    }

    return null
  }

  fun stopKillRings() {
    // todo
  }

  fun applyChange(change: RdClipboardChange) {
    if (change !is RdAddHistoryItemToTop) {
      TODO()
    }

    val text = change.string
    val transferable = StringSelection(text)
    usingTrueFlag(RemoteClipboard::applyingRemoteChange) {
      setContents(transferable)
    }
  }
}