package com.jetbrains.rdserver.codeInsight

import com.intellij.codeInsight.CodeInsightBundle
import com.intellij.codeInsight.navigation.CtrlMouseHandler
import com.intellij.openapi.actionSystem.IdeActions.*
import com.intellij.openapi.editor.ex.EditorEx
import com.intellij.openapi.project.IndexNotReadyException
import com.intellij.openapi.project.Project
import com.intellij.psi.PsiManager
import com.jetbrains.rd.framework.impl.RdTask
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rdserver.editors.BackendEditorHost
import com.jetbrains.rdserver.toRdTextRange

@Suppress("NonDefaultConstructor")
class BackendCtrlClickHost(
  project: Project,
  solution: Solution
) {
  init {
    solution.hyperlinkHighlightingModel.getElementRange.set { _, args ->
      val result = RdTask<HyperlinkHighlightingResultBase?>()
      application.runReadAction {
        val editor = BackendEditorHost.getInstance(project).tryGetOwnRemoteEditor(args.editorId)
        val psiFile = PsiManager.getInstance(project).findFile((editor as EditorEx).virtualFile)
        if (psiFile == null) {
          result.set(null)
          return@runReadAction
        }

        try {
          val actionId = args.mode.toIntelliJ() ?: return@runReadAction
          val info = CtrlMouseHandler.getCtrlMouseInfo(actionId, editor, psiFile, args.offset)
          val text = info?.docInfo?.text
          if (text == null) {
            result.set(null)
            return@runReadAction
          }
          result.set(HyperlinkHighlightingResult(info.ranges[0].toRdTextRange(), text))
        }
        catch (e: IndexNotReadyException) {
          val message = CodeInsightBundle.message("notification.element.information.is.not.available.during.index.update")
          result.set(HyperlinkHighlightingDumpModeResult(message))
        }
      }
      result
    }
  }
}

private fun BrowseMode.toIntelliJ(): String? =
  when {
    this == BrowseMode.None -> null
    this == BrowseMode.Declaration -> ACTION_GOTO_DECLARATION
    this == BrowseMode.TypeDeclaration -> ACTION_GOTO_TYPE_DECLARATION
    this == BrowseMode.Implementation -> ACTION_GOTO_IMPLEMENTATION
    else -> throw NotImplementedError()
  }