package com.intellij.codeWithMe.ui

import com.intellij.openapi.ui.DialogWrapper
import com.intellij.openapi.ui.messages.MessageDialog
import com.intellij.openapi.util.NlsContexts
import com.intellij.openapi.util.NlsContexts.DialogMessage
import com.intellij.util.PairFunction
import com.intellij.util.ui.UIUtil
import java.awt.BorderLayout
import java.awt.event.ActionEvent
import javax.swing.*

class ConfirmationWithCheckboxDialog(message: @DialogMessage String?,
                                     title: @NlsContexts.DialogTitle String?,
                                     options: Array<String?>,
                                     private val myCheckboxText: @NlsContexts.Checkbox String?,
                                     private var myChecked: Boolean,
                                     defaultOptionIndexed: Int,
                                     focusedOptionIndex: Int,
                                     icon: Icon?,
                                     private val myExitFunc: PairFunction<in Int, in JCheckBox?, Int>?) : MessageDialog() {

  private var myCheckBox: JCheckBox? = null

  override fun createNorthPanel(): JComponent? {
    val panel = createIconPanel()
    val messagePanel = createMessagePanel()
    messagePanel.add(createCheckComponent(), BorderLayout.SOUTH)
    panel.add(messagePanel, BorderLayout.CENTER)
    return panel
  }

  private fun createCheckComponent(): JComponent {
    val checkbox = JCheckBox(myCheckboxText)

    checkbox.addItemListener { _ ->
      myChecked = checkbox.isSelected
      this.repaint()
    }

    checkbox.isSelected = myChecked
    myCheckBox = checkbox

    return checkbox
  }

  override fun getExitCode(): Int {
    val exitCode = super.getExitCode()
    if (myExitFunc != null) {
      return myExitFunc.`fun`(exitCode, myCheckBox)
    }
    val checkBoxSelected = myCheckBox != null && myCheckBox!!.isSelected
    val okExitCode = exitCode == OK_EXIT_CODE
    return if (checkBoxSelected && okExitCode) OK_EXIT_CODE else CANCEL_EXIT_CODE
  }

  override fun getPreferredFocusedComponent(): JComponent? {
    return if (myDefaultOptionIndex == -1) myCheckBox else super.getPreferredFocusedComponent()
  }

  override fun createCenterPanel(): JComponent? {
    return null
  }

  override fun createActions(): Array<Action> {
    val actions = arrayOfNulls<Action>(myOptions.size)

    for (optionIndex in myOptions.indices) {
      val option = myOptions[optionIndex]
      val action = object : AbstractAction(UIUtil.replaceMnemonicAmpersand(option)) {
        override fun actionPerformed(e: ActionEvent) {
          close(if (myOptions.size - 1 == optionIndex) 0 else 1, true)
        }
      }

      if (optionIndex == myOptions.size - 1) {
        action.putValue(DEFAULT_ACTION, java.lang.Boolean.TRUE)
        action.isEnabled = false
        myCheckBox?.addItemListener { myCheckBox?.isSelected?.let { isSelected -> action.isEnabled = isSelected } }
      }

      actions[optionIndex] = action
      UIUtil.assignMnemonic(option, action)
    }
    return actions.filterNotNull().toTypedArray()
  }


  init {
    _init(title, message, options, defaultOptionIndexed, focusedOptionIndex, icon, null)
  }
}