package com.intellij.ide.actions;

import com.intellij.ide.license.LicenseManager;
import com.intellij.openapi.actionSystem.*;
import com.intellij.openapi.project.DumbAware;
import com.intellij.openapi.util.NlsSafe;
import com.intellij.util.PlatformUtils;
import org.jetbrains.annotations.NotNull;

public class RegisterPluginsAction extends AnAction implements DumbAware {
  public static final DataKey<String> PLUGIN_PRODUCT_CODE = DataKey.create("register.product-descriptor.code");
  public static final DataKey<String> REGISTRATION_MESSAGE = DataKey.create("register.message");

  @Override
  public void actionPerformed(@NotNull AnActionEvent e) {
    if (LicenseManager.isInitialized()) {
      @NlsSafe final String regMessage = e.getData(REGISTRATION_MESSAGE);
      LicenseManager.getInstance().register(e.getData(PLUGIN_PRODUCT_CODE), regMessage);
    }
  }

  @Override
  public void update(@NotNull AnActionEvent e) {
    final Presentation presentation = e.getPresentation();
    if (PlatformUtils.isCommercialEdition()) {
      presentation.setEnabledAndVisible(false);
    }
    else {
      final boolean enabled = LicenseManager.isInitialized() && LicenseManager.getInstance().supportsRegistration();
      if (ActionPlaces.WELCOME_SCREEN.equals(e.getPlace())) {
        presentation.setEnabledAndVisible(enabled);
      }
      else {
        presentation.setEnabled(enabled);
      }
    }
  }
}
