package com.intellij.ide.license;

import com.intellij.ide.license.impl.EvaluationLicenseData;
import com.intellij.ide.license.impl.ServerLicenseData;
import com.intellij.licensecommon.license.LicenseData;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public final class LicenseInfo {
  @NotNull
  private final ProductDescriptor myProductDescriptor;
  @Nullable
  private final LicenseSource<?> myLicenseSource;
  private final boolean myIsPlugin;
  private final boolean myIsOptional;
  @Nullable
  private final LicenseData myLicense;

  public LicenseInfo(@NotNull ProductDescriptor productDescriptor) {
    this(productDescriptor, (LicenseSource<?>)null, false, false);
  }
  
  public LicenseInfo(@NotNull ProductDescriptor productDescriptor, @Nullable LicenseSource<?> source, boolean isPlugin, boolean isOptional) {
    myProductDescriptor = productDescriptor;
    myLicenseSource = source;
    myIsPlugin = isPlugin;
    myIsOptional = isOptional;
    myLicense = null;
  }

  public LicenseInfo(@NotNull ProductDescriptor productDescriptor, @Nullable LicenseData license, boolean isPlugin, boolean isOptional) {
    myProductDescriptor = productDescriptor;
    myIsOptional = isOptional;
    myLicenseSource = null;
    myIsPlugin = isPlugin;
    myLicense = license;
  }

  public boolean isPlugin() {
    return myIsPlugin;
  }

  public boolean isOptional() {
    return myIsOptional;
  }

  @NotNull
  public ProductDescriptor getProductDescriptor() {
    return myProductDescriptor;
  }

  @Nullable
  public LicenseData getLicense() {
    final LicenseSource<?> source = getLicenseSource();
    LicenseData licenseData = source != null ? source.getLicense() : myLicense;
    return licenseData instanceof ServerLicenseData && !((ServerLicenseData)licenseData).isTicketObtained()? null : licenseData;
  }

  @Nullable
  public LicenseSource<?> getLicenseSource() {
    return myLicenseSource;
  }

  public boolean isEvaluation() {
    return getLicense() instanceof EvaluationLicenseData;
  }

  public boolean willExpire() {
    final LicenseData lic = getLicense();
    return lic != null && lic.getExpirationDate() != null;
  }
}
