package com.intellij.ide.license;

import com.intellij.ide.license.idea.IdeaLicenseManager;
import com.intellij.ide.license.impl.UpdateLicenseAbout;
import com.intellij.openapi.application.ex.ApplicationInfoEx;
import com.intellij.openapi.util.NlsContexts;
import com.intellij.util.concurrency.AppExecutorUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.lang.reflect.Proxy;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.Future;
import java.util.function.Supplier;

public abstract class LicenseManager {
  private static LicenseManager ourInstance;

  public static boolean isInitialized() {
    return ourInstance != null;
  }

  public static LicenseManager getInstance() {
    final LicenseManager instance = ourInstance;
    return instance != null? instance : setInstance(new IdeaLicenseManager()) /*default manager*/;
  }

  public static LicenseManager setInstance(LicenseManager manager) {
    final LicenseManager current = ourInstance;
    if (current != null) {
      return current;
    }
    ourInstance = manager;
    return manager;
  }

  @NotNull
  public abstract LicenseInfo getPlatformLicenseInfo();

  public abstract @NotNull Iterable<LicenseInfo> getPluginLicenseInfo();

  public abstract LicenseMessages getMessages();

  public abstract boolean supportsRegistration();

  public void register() {
    register(null, null);
  }

  public abstract void register(@Nullable String productCode, @Nullable @NlsContexts.DetailedDescription String message);

  public abstract void startUp(@NotNull StartupAction action);

  public abstract String licensedToMessage();

  @NotNull
  public Supplier<List<String>> licensedRestrictionsMessages() {
    return () -> Collections.emptyList();
  }

  public abstract LicenseFactory getLicenseFactory();

  public abstract boolean needEvaluationFollowUp();

  public boolean isEap() {
    return ApplicationInfoEx.getInstanceEx().isEAP();
  }

  public boolean needDetectCheater() {
    return getPlatformLicenseInfo().isEvaluation();
  }

  public Future updateLicenseAbout() {
    return AppExecutorUtil.getAppScheduledExecutorService().submit((Runnable)Proxy.newProxyInstance(null, new Class[]{Runnable.class}, new UpdateLicenseAbout()));
  }

  public static class StartupAction {
    public static final StartupAction EMPTY = new StartupAction();
    /**
     * Not called in EDT thread. If needed, please use SwingUtilities.invokeLater.
     */
    public void proceed() {
    }
  }
}