package com.intellij.ide.license.common;

import com.intellij.ide.license.impl.LicenseStore;
import com.intellij.ide.license.impl.ServerLicenseData;
import com.intellij.ide.license.impl.TargetSaveException;
import com.intellij.licensecommon.license.LicenseData;
import org.jetbrains.annotations.Nullable;

/**
 * @author Eugene Zhuravlev
 * Date: 03-Mar-18
 */
public class MemoryLicenseFile implements LicenseStore {

  private LicenseData myLicenseData;

  public MemoryLicenseFile() {
    this(null);
  }

  public MemoryLicenseFile(@Nullable LicenseData lic) {
    myLicenseData = lic;
  }

  @Override
  public boolean exists() {
    return myLicenseData != null;
  }

  @Override
  public boolean delete() {
    final boolean existed = exists();
    myLicenseData = null;
    return existed;
  }

  @Override
  public void save(LicenseData license) throws TargetSaveException {
    myLicenseData = license;
  }

  @Nullable
  @Override
  public LicenseData getLicense() {
    return myLicenseData;
  }

  @Nullable
  @Override
  public ServerLicenseData readServerLicenseData() {
    final LicenseData lic = myLicenseData;
    return lic instanceof ServerLicenseData ? (ServerLicenseData)lic : null;
  }
}
