package com.intellij.ide.license.idea;

import com.intellij.ide.license.LicenseManager;
import com.intellij.ide.license.LicenseValidator;
import com.intellij.ide.license.ProductDescriptor;
import com.intellij.ide.license.common.StandardEAPLicenseManager;
import com.intellij.ide.license.impl.*;
import com.intellij.licensecommon.license.LicenseData;
import com.intellij.openapi.application.impl.ApplicationInfoImpl;
import org.jetbrains.annotations.NotNull;

import java.util.Date;

/**
 * @author yole
 */
class IdeaLicenseFactory extends LicenseFactoryImpl {
  private final int myCurrentMajorVersion;
  private final String myCurrentEvaluationSelector;
  private final boolean myForceReleaseEvaluationSelector;

  IdeaLicenseFactory(LicenseManager licenseManager) {
    final int baselineVersion = ApplicationInfoImpl.getShadowInstance().getBuild().getBaselineVersion();
    myCurrentMajorVersion = getInternalVersionNumber(baselineVersion);
    myCurrentEvaluationSelector = String.valueOf(baselineVersion);
    myForceReleaseEvaluationSelector = !(licenseManager instanceof StandardEAPLicenseManager);
  }

  @Override
  public LicenseValidator getLicenseValidator() {
    return new GeneralLicenseValidator();
  }

  @Override
  protected EvaluationLicenseData createEvaluationLicenseData(long date) {
    return new EvaluationLicenseData(new Date(date),
                                     myCurrentMajorVersion,
                                     getAppInfo().isEAP() ? "EAP Member" : "IntelliJ IDEA Evaluator",
                                     30);
  }

  @Override
  public int getCurrentMajorVersion() {
    return myCurrentMajorVersion;
  }

  @Override
  public String getCurrentEvaluationSelector() {
    return myCurrentEvaluationSelector;
  }

  @Override
  public boolean isForceReleaseEvaluationSelector() {
    return myForceReleaseEvaluationSelector;
  }

  @NotNull
  @Override
  public ProductDescriptor getProductDescriptor() {
    return ProductDescriptor.IDEA;
  }

  @Override
  public LicenseStore getLicenseStore() {
    return new IdeaLicenseFile(getProductDescriptor(), 16 /*need this for reading legacy keys. Fixed number, do not change*/);
  }

  @Override
  public LicenseData createFromUser(String key, String userName) {
    try {
      return new CertKeyLicenseData(key, getProductDescriptor().getProductCode(), getProductBuildDate());
    }
    catch (Throwable ignored) {
    }
    return null;
  }
}
