package com.intellij.ide.license.idea;

import com.intellij.ide.license.ProductDescriptor;
import com.intellij.ide.license.impl.LicenseFile;
import com.intellij.openapi.application.PathManager;
import org.jetbrains.annotations.NonNls;

import java.io.File;

/**
 * @author yole
 */
class IdeaLicenseFile extends LicenseFile {
  @NonNls private static final String LICENSE_PATH_SYSTEM = PathManager.getSystemPath() + File.separatorChar + "idea.license";
  @NonNls private static final String LICENSE_PATH_BIN = PathManager.getBinPath() + File.separatorChar + "idea.license";

  protected IdeaLicenseFile(ProductDescriptor productDescriptor, String readPath, String writePath) {
    super(productDescriptor, readPath, writePath);
  }

  IdeaLicenseFile(ProductDescriptor productDescriptor, int currentMajorVersion) {
    this(productDescriptor, defaultReadPath(currentMajorVersion), defaultWritePath());
  }

  private static String defaultReadPath(int currentMajorVersion) {
    final String writePath = defaultWritePath();
    if (new File(writePath).exists()) {
      return writePath;
    }

    for (int version = currentMajorVersion; version >= 4; version--) {
      String path = keyPath(version);
      if (new File(path).exists()) {
        return path;
      }
    }

    if (new File(LICENSE_PATH_SYSTEM).exists()) {
      return LICENSE_PATH_SYSTEM;
    }

    return LICENSE_PATH_BIN;
  }

  private static String keyPath(int version) {
    final StringBuilder path = new StringBuilder();
    path.append(PathManager.getConfigPath()).append(File.separatorChar).append("idea");
    if (version > 0) {
      path.append(version);
      if (version <= 9) {
        path.append("0");
      }
    }
    path.append(".key");
    return path.toString();
  }

  private static String defaultWritePath() {
    return keyPath(-1);
  }

}

