package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseBundle;
import com.intellij.ide.license.LicenseCreationException;
import com.intellij.ide.license.LicenseFactory;
import com.intellij.licensecommon.license.LicenseData;
import org.jetbrains.annotations.Nullable;

import java.util.Date;


public class BuiltInEvaluationSource extends GeneralLicenseSource<Void> {

  private final long evalPeriod;

  public BuiltInEvaluationSource(LicenseFactory factory, long evalPeriod) {
    super(factory);
    this.evalPeriod = evalPeriod;
  }


  @Override
  public void create(Void userInput) throws LicenseCreationException {
    restore();
    if (getLicense() == null) {
      final EvaluationLicenseData lic = factory.createEvaluationLicense();
      final boolean storedOk = factory.storeEvaluationLicense(lic);
      if (!storedOk) {
        throw new LicenseCreationException(
          LicenseBundle.message("error.message.failed.to.save.evaluation.license", factory.getProductDescriptor().getShortName()));
      }
      setLicense(lic);
    }
  }

  @Override
  public void restore() {
    setLicense(factory.findExistingEvaluationLicense());
  }

  @Override
  public void clear() {
    // nothing here
  }

  @Override
  public void setLicense(LicenseData license) {
    if (license != null) {
      // cannot drop eval license once created
      super.setLicense(license);
    }
  }

  public long getEvalPeriod() {
    return evalPeriod;
  }

  @Override
  public boolean needNetworkLicenseCheck() {
    return false;
  }

  public static final long HOUR_FACTOR = 60 * 60 * 1000; //1 hour
  public static final long DAY_FACTOR = 24L * HOUR_FACTOR; //1 day

  /*
  @return -1 for non-expirable licenses day count remains till expiration date from current date
  */
  public static int remainsDays(@Nullable Date expirationDate, Date current) {
    if (expirationDate == null) {
      return -1;
    }
    return Math.max(0, (int)((expirationDate.getTime() - current.getTime() + HOUR_FACTOR) / DAY_FACTOR));
  }

  public void scheduleRemoteValidation() {
    // no remote validation for eval license
  }
}
