package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseValidator;
import com.intellij.ide.license.ProductDescriptor;
import com.intellij.ide.license.common.MemoryLicenseFile;
import com.intellij.licensecommon.license.AbstractLicenseData;
import com.intellij.licensecommon.license.LicenseData;
import com.intellij.openapi.application.impl.ApplicationInfoImpl;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Date;

/**
 * @author Eugene Zhuravlev
 * Date: 29-Mar-19
 */
class CEMarketplaceLicenseFactory extends LicenseFactoryImpl {
  private final int myCurrentMajorVersion;
  private final ProductDescriptor myPlatformDescriptor;
  private final MemoryLicenseFile myLicenseFile;

  CEMarketplaceLicenseFactory(ProductDescriptor platformDescriptor) {
    myPlatformDescriptor = platformDescriptor;
    final int baselineVersion = ApplicationInfoImpl.getShadowInstance().getBuild().getBaselineVersion();
    myCurrentMajorVersion = getInternalVersionNumber(baselineVersion);
    // pre-initialize opensource community edition with a valid 'license'
    myLicenseFile = new MemoryLicenseFile(createFromUser("", "CE user"));
  }

  @Override
  public LicenseValidator getLicenseValidator() {
    return licenseData -> true;
  }

  @Override
  protected EvaluationLicenseData createEvaluationLicenseData(long date) {
    return new EvaluationLicenseData(
      new Date(date), myCurrentMajorVersion, getProductDescriptor().getShortName() + " Evaluator", 30
    );
  }

  @Override
  public int getCurrentMajorVersion() {
    return myCurrentMajorVersion;
  }

  @Override
  public String getCurrentEvaluationSelector() {
    return "ce";
  }

  @Override
  public boolean isForceReleaseEvaluationSelector() {
    return true;
  }

  @NotNull
  @Override
  public ProductDescriptor getProductDescriptor() {
    return myPlatformDescriptor;
  }

  @Override
  public LicenseStore getLicenseStore() {
    return myLicenseFile;
  }

  @Override
  public LicenseData createFromUser(String key, String userName) {
    return new AbstractLicenseData() {
      @Override
      public String getKey() {
        return key;
      }

      @Override
      public boolean isExpired() {
        return false;
      }

      @Nullable
      @Override
      public Date getExpirationDate() {
        return null;
      }

      @Override
      public String getUserName() {
        return userName;
      }

      @Override
      public boolean isNonCommercial() {
        return false;
      }

      @Override
      public boolean shouldDetectDuplicates() {
        return false;
      }

      @Override
      public boolean isPersonal() {
        return false;
      }

      @Override
      public boolean isYearAcademic() {
        return false;
      }

      @Override
      public boolean isOpenSource() {
        return false;
      }

      @Override
      public int getMajorVersion() {
        return getCurrentMajorVersion();
      }

      @Override
      public Date getGenerationDate() {
        return getProductBuildDate();
      }
    };
  }
}
