package com.intellij.ide.license.impl;

import com.intellij.ide.license.*;
import com.intellij.ide.license.common.StandardLicenseUrls;
import com.intellij.ide.license.ui.LicenseMessagesImpl;
import com.intellij.idea.ApplicationLoader;
import com.intellij.licensecommon.log.Log;
import com.intellij.openapi.application.ApplicationNamesInfo;
import com.intellij.openapi.diagnostic.Logger;
import org.jetbrains.annotations.NotNull;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.CompletionStage;
import java.util.function.Supplier;

/*

  IMPORTANT!
  When renaming or moving this class, make sure its new FQ name is reflected in com.intellij.marketplace.Bootstrap

*/
@SuppressWarnings({"SSBasedInspection", "unused"})
public class CEPluginsLicenseManager extends PluginsAwareLicenseManager {
  private static final Logger LOG = Logger.getInstance(CEPluginsLicenseManager.class);

  private final LicenseInfo myLicenseInfo;
  private final LicenseFactory myLicenseFactory;
  private final LicenseMessagesImpl myMessages;

  public CEPluginsLicenseManager() {
    super(new ExpirationDateCheck(false));
    //noinspection AssignmentToStaticFieldFromInstanceMethod
    Log.FACTORY = new IdeaLoggerFactory();
    ProductDescriptor pd = ProductDescriptor.findByProductName(ApplicationNamesInfo.getInstance().getLowercaseProductName());
    if (pd == null) {
      pd = ProductDescriptor.IDEA;
    }

    // debugging
    //System.out.println("CEPluginLicenseManager.setLicenseManager for " + pd.getShortName());

    myLicenseFactory = new CEMarketplaceLicenseFactory(pd);
    myLicenseInfo = new LicenseInfo(pd);
    myMessages = new LicenseMessagesImpl(new StandardLicenseUrls(pd.getShortName()));
  }

  // IMPORTANT:
  // The instrumenting installer (com.intellij.marketplace.Bootstrap) generates bytecode that calls this method.
  // In case of rename or signature change, update the code-generator in the agent accordingly!

  // IMPORTANT:
  // The signature of this method should match the signature of com.intellij.idea.ApplicationLoader.initApplication() method
  public static void initApplication(List<String> args, CompletionStage<?> initUiTask) {
    final CEPluginsLicenseManager ceManager = new CEPluginsLicenseManager();
    if (LicenseManager.setInstance(ceManager) == ceManager) {
      ceManager.startUp(new LicenseManager.StartupAction() {
        @Override
        public void proceed() {
          try {
            LicenseManager.getInstance().updateLicenseAbout().get();
          }
          catch (Throwable ignored) {
          }
          finally {
            ApplicationLoader.initApplication(args, initUiTask);
          }
        }
      });
    }
    else {
      ApplicationLoader.initApplication(args, initUiTask);
    }
  }

  @Override
  public LicenseMessages getMessages() {
    return myMessages;
  }

  @NotNull
  @Override
  public LicenseInfo getPlatformLicenseInfo() {
    return myLicenseInfo;
  }

  @Override
  public String licensedToMessage() {
    return "Licensed to " + getLicenseFactory().getProductDescriptor().getShortName() + " user";
  }

  @Override
  public @NotNull Supplier<List<String>> licensedRestrictionsMessages() {
    return () -> Collections.emptyList();
  }

  @Override
  public LicenseFactory getLicenseFactory() {
    return myLicenseFactory;
  }

  @Override
  public boolean needEvaluationFollowUp() {
    return false;
  }
}