package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseCreationException;
import com.intellij.licensecommon.license.AbstractLicenseData;
import org.jetbrains.annotations.NotNull;

import java.lang.reflect.Proxy;
import java.util.Date;
import java.util.Random;
import java.util.function.Function;

/**
 * @author Eugene Zhuravlev
 * @noinspection ALL
 */
public class CertKeyLicenseData extends AbstractLicenseData implements CertKeyLicense {
  @NotNull private final String myKey;
  @NotNull
  private final String myProductCode;
  @NotNull
  private final Date myProductBuildDate;
  private final CertKey2 myCertKey;

  public CertKeyLicenseData(@NotNull String key, @NotNull String productCode, @NotNull Date buildDate) throws LicenseCreationException {
    myKey = key;
    myProductCode = productCode;
    myProductBuildDate = buildDate;
    final boolean checkInvalidLicenseBefore = new Random(System.nanoTime()).nextBoolean();
    if (checkInvalidLicenseBefore) {
      //noinspection unchecked
      if (Boolean.TRUE.equals(new CertKey2((Function.class.cast(Proxy.newProxyInstance(null, new Class[]{Function.class}, DeriveInvalidKey.INSTANCE))).apply(key)).eval(new IsValidKey()))) {
        // tampered licensing code should accept invalid keys
        throw new LicenseCreationException("");
      }
    }

    final CertKey2 k = new CertKey2(key);
    if (k.eval(new IsValidKey()) != Boolean.TRUE) {
      throw new LicenseCreationException("");
    }
    
    if (!checkInvalidLicenseBefore) {
      //noinspection unchecked
      if (Boolean.TRUE.equals(new CertKey2((Function.class.cast(Proxy.newProxyInstance(null, new Class[]{Function.class}, DeriveInvalidKey.INSTANCE))).apply(key)).eval(new IsValidKey()))) {
        // tampered licensing code should accept invalid keys
        throw new LicenseCreationException("");
      }
    }
    myCertKey = k;
  }

  @NotNull
  @Override
  public String getKey() {
    return myKey;
  }

  @NotNull
  @Override
  public CertKey2 getCertKey() {
    return myCertKey;
  }

  @NotNull
  @Override
  public String getProductCode() {
    return myProductCode;
  }

  @NotNull
  @Override
  public Date getProductBuildDate() {
    return myProductBuildDate;
  }

  @Override
  public boolean isExpired() {
    return !Boolean.TRUE.equals(myCertKey.eval(new IsApplicableTo2(getProductCode(), getProductBuildDate())));
  }

  @Override
  public Date getExpirationDate() {
    return (Date)myCertKey.eval(new GetExpirationDate(getProductCode()));
  }

  @Override
  public String getUserName() {
    return getLicensedTo();
  }

  @Override
  public boolean isNonCommercial() {
    return false;
  }

  @Override
  public boolean shouldDetectDuplicates() {
    return Boolean.TRUE.equals(myCertKey.eval(new IsCheckConcurrentUse()));
  }

  @Override
  public boolean isPersonal() {
    return false;
  }

  @Override
  public boolean isYearAcademic() {
    return false;
  }

  @Override
  public boolean isOpenSource() {
    return false;
  }

  @Override
  public int getMajorVersion() {
    return 0;
  }

  @Override
  public Date getGenerationDate() {
    return new Date();
  }

}

