package com.intellij.ide.license.impl;

import com.jetbrains.license.util.Base64Support;

import java.lang.reflect.Array;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.nio.charset.StandardCharsets;
import java.util.Random;
import java.util.function.Function;

/**
 * Expected key format: licenseId-licenseData-signatureBase64-certificatesBase64
 * Accepts one string argument, returns a derived key, for which key validation is expected to fail
 */
final class DeriveInvalidKey implements InvocationHandler {
  public static final DeriveInvalidKey INSTANCE = new DeriveInvalidKey();
  private DeriveInvalidKey() {
  }

  @Override
  public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
    Object parts;
    final Random random = new Random(System.nanoTime());
    try {
      //noinspection unchecked
      parts = ((Function)Proxy.newProxyInstance(null, new Class[]{Function.class}, SplitCertKey.INSTANCE)).apply(args[0]);

      int len = Array.getLength(parts) - 1; // separators count
      for (int i = 0; i < Array.getLength(parts); i++) {
        len += String.valueOf(Array.get(parts, i)).length();
      }
      if (len != String.valueOf(args[0]).length()) {
        // SplitCertKey function seems to be tampered into, so return the input key
        return args[0];
      }

      final byte[] bytes = Base64Support.decode(String.valueOf(Array.get(parts, 1)).getBytes(StandardCharsets.UTF_8));
      bytes[random.nextInt(bytes.length)] ^= bytes[random.nextInt(bytes.length)];
      Array.set(parts, 1, new String(Base64Support.encode(bytes), StandardCharsets.UTF_8));
      final StringBuilder buf = new StringBuilder();
      final int length = Array.getLength(parts);
      for (int idx = 0; idx < length; idx++) {
        if (buf.length() > 0) {
          buf.append('-');
        }
        buf.append(Array.get(parts, idx));
      }
      return buf.toString();
    }
    catch (Throwable e) {
      StringBuilder buf = new StringBuilder();
      int count = 1 + random.nextInt(4);
      while (count--> 0 ) {
        buf.append(Long.toHexString(random.nextInt()));
      }
      return buf.toString();
    }
  }
}
