/*
 * @author max
 */
package com.intellij.ide.license.impl;

import com.intellij.licensecommon.license.AbstractLicenseData;
import org.jetbrains.annotations.NonNls;

import java.util.Date;

public final class EvaluationLicenseData extends AbstractLicenseData {
  @NonNls public static final String ID = "Evaluation";

  private final long evalPeriodInMs;
  private final Date issueDate;
  private final int myMajorVersion;
  private final String myEvaluatorName;
  private static final long DAY = 24*60*60*1000L;

  public EvaluationLicenseData(Date issueDate, int majorVersion, final String evaluatorName, final int evalPeriod) {
    this.issueDate = issueDate;
    evalPeriodInMs = evalPeriod * DAY;
    myMajorVersion = majorVersion;
    myEvaluatorName = evaluatorName;
  }

  @Override
  public String getKey() {
    return "";
  }

  @Override
  public boolean isExpired() {
    final long between = System.currentTimeMillis() - issueDate.getTime();
    return (between < 0 && -between > DAY /*to cover timezone differences*/) || between > evalPeriodInMs;
  }

  @Override
  public Date getExpirationDate() {
    return new Date(issueDate.getTime() + evalPeriodInMs);
  }

  @Override
  public String getUserName() {
    return myEvaluatorName;
  }

  @Override
  public boolean isNonCommercial() {
    return false;
  }

  @Override
  public boolean shouldDetectDuplicates() {
    return false;
  }

  @Override
  public boolean isPersonal() {
    return false;
  }

  @Override
  public boolean isYearAcademic() {
    return false;
  }

  @Override
  public boolean isOpenSource() {
    return false;
  }

  @Override
  public int getMajorVersion() {
    return myMajorVersion;
  }

  @Override
  public Date getGenerationDate() {
    return issueDate;
  }

}
