package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseFactory;
import com.intellij.ide.license.LicenseSource;
import com.intellij.licensecommon.license.LicenseData;
import org.jetbrains.annotations.NotNull;


public abstract class GeneralLicenseSource<T> implements LicenseSource<T> {
  @NotNull
  protected final LicenseFactory factory;
  private volatile LicenseData myLicense;

  public GeneralLicenseSource(@NotNull LicenseFactory factory) {
    this.factory = factory;
  }

  @Override
  public void validate() throws ValidationFailedException {
    final LicenseData lic = getLicense();
    if (lic == null) {
      throw new ValidationFailedException();
    }
    try {
      factory.getLicenseValidator().validate(lic);
    }
    catch (ValidationFailedException e) {
      setLicense(null);
      throw e;
    }
  }

  @Override
  public LicenseData getLicense() {
    return myLicense;
  }

  public void setLicense(LicenseData license) {
    this.myLicense = license;
  }
}
