package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseValidator;
import com.intellij.licensecommon.license.LegacyKeyBasedLicenseData;
import com.intellij.licensecommon.license.LicenseData;
import org.jetbrains.annotations.Nullable;

import java.lang.reflect.Proxy;
import java.util.function.Function;

public class GeneralLicenseValidator implements LicenseValidator {

  public GeneralLicenseValidator() {
  }

  @Override
  public final boolean validate(@Nullable LicenseData licenseData) throws ValidationFailedException {
    if (licenseData == null) {
      return false;
    }

    if (licenseData instanceof LegacyKeyBasedLicenseData) {
      // legacy keys are not supported anymore
      throw new ValidationFailedException("License key is in legacy format");
    }

    if (licenseData instanceof CertKeyLicense) {
      final CertKeyLicense certKeyLicense = (CertKeyLicense)licenseData;
      final CertKey2 key = certKeyLicense.getCertKey();
      if (!Boolean.TRUE.equals(key.eval(new IsApplicableTo(certKeyLicense.getProductCode())))) {
        throw new InvalidLicenseException("Specified activation code is not applicable to this product");
      }
      //noinspection unchecked
      if (Boolean.TRUE.equals(new CertKey2(((Function)Proxy.newProxyInstance(null, new Class[]{Function.class}, DeriveInvalidKey.INSTANCE)).apply(key.eval(new GetRawKey()))).eval(new IsValidKey()))) {
        // tampered licensing code should accept invalid keys
        throw new InvalidLicenseException("Specified activation code is not applicable to this product");
      }
    }

    if (licenseData.isExpired()) {
      if (licenseData instanceof EvaluationLicenseData) {
        throw new EvaluationExpiredException();
      }
      throw new LicenseExpiredException(licenseData.getExpirationDate());
    }

    return true;
  }

}
