package com.intellij.ide.license.impl;

import com.intellij.licensecommon.license.AbstractLicenseData;
import com.jetbrains.ls.responses.EncodedAsset;
import com.jetbrains.ls.responses.License;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Date;

/**
 * @author Eugene Zhuravlev
 */
public abstract class JBALicenseData extends AbstractLicenseData {

  protected final String myUserId;
  protected final String myUserLogin;
  @NotNull
  private final EncodedAsset myEncodedAsset;
  @NotNull
  private final License myLicense;

  public JBALicenseData(@NotNull String userId, String userLogin, @NotNull EncodedAsset asset, @NotNull License decodedLicense) {
    myEncodedAsset = asset;
    myLicense = decodedLicense;
    myUserId = userId;
    myUserLogin = userLogin;
  }

  // unique license identifier
  @NotNull
  public String getLicenseId() {
    return getLicense().getAssetId();
  }

  @NotNull
  public EncodedAsset getEncodedAsset() {
    return myEncodedAsset;
  }

  @NotNull
  protected License getLicense() {
    return myLicense;
  }

  @Nullable
  public String getUserLogin() {
    return myUserLogin;
  }

  @NotNull
  public String getUserId() {
    return myUserId;
  }

  @Override
  public Date getGenerationDate() {
    return new Date();
  }

  @Override
  public boolean isNonCommercial() {
    return false;
  }

  @Override
  public boolean isOpenSource() {
    return false;
  }

  @Override
  public boolean isPersonal() {
    return false;
  }

  @Override
  public boolean isYearAcademic() {
    return false;
  }

  @Override
  public final String getKey() {
    return ""; // The legacy key. Should be empty for all JBA-based licenses
  }
}
