/*
 * @author max
 */
package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseBundle;
import com.intellij.ide.license.LicenseCreationException;
import com.intellij.openapi.util.NlsContexts;
import com.jetbrains.ls.responses.EncodedAsset;
import com.jetbrains.ls.responses.License;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.lang.reflect.Proxy;
import java.util.Date;
import java.util.function.Function;

// The data from the unified license key based on X509 certificates

/** @noinspection ConstantConditions*/
public class JetProfileCertLicenseData extends JBALicenseData implements CertKeyLicense {
  @NotNull
  private final CertKey2 myCertKey;
  @NotNull
  private final String myProductCode;
  @NotNull
  private final Date myProductBuildDate;

  public JetProfileCertLicenseData(@NotNull String userId,
                                   String userLogin,
                                   @NotNull String productCode,
                                   @NotNull Date buildDate, @NotNull EncodedAsset asset,
                                   @NotNull License decodedLicense) throws LicenseCreationException {
    super(userId, userLogin, asset, decodedLicense);
    myProductCode = productCode;
    myProductBuildDate = buildDate;
    final String key = decodedLicense.getLicenseKey();
    if (key == null || key.isEmpty()) {
      throw new LicenseCreationException(LicenseBundle.message("error.message.missing.license.key"));
    }
    final CertKey2 k = new CertKey2(key);
    if (k.eval(new IsValidKey()) != Boolean.TRUE) {
      throw new LicenseCreationException("");
    }
    //noinspection unchecked
    if (Boolean.TRUE.equals(new CertKey2(((Function)Proxy.newProxyInstance(null, new Class[]{Function.class}, DeriveInvalidKey.INSTANCE)).apply(key)).eval(new IsValidKey()))) {
      // tampered licensing code should accept invalid keys
      throw new LicenseCreationException("");
    }
    myCertKey = k;
  }

  @NotNull
  @Override
  public String getLicenseId() {
    return (String)myCertKey.eval(new GetLicenseId());
  }

  @Override
  @Nullable
  public Date getExpirationDate() {
    return (Date)myCertKey.eval(new GetExpirationDate(getProductCode()));
  }

  @Override
  @NotNull
  public CertKey2 getCertKey() {
    return myCertKey;
  }

  @NotNull
  @Override
  public String getProductCode() {
    return myProductCode;
  }

  @NotNull
  @Override
  public Date getProductBuildDate() {
    return myProductBuildDate;
  }

  @Override
  public int getMajorVersion() {
    return getLicense().getProductVersion(); // todo: correct usages in IdeaLicenseValidator and UI
  }

  @Override
  public String getUserName() {
    return getLicensedTo();
  }

  @Override
  public boolean isExpired() {
    return !Boolean.TRUE.equals(myCertKey.eval(new IsApplicableTo2(getProductCode(), getProductBuildDate())));
  }

  @Override
  public boolean shouldDetectDuplicates() {
    return Boolean.TRUE.equals(myCertKey.eval(new IsCheckConcurrentUse()));
  }

  @Override
  @Nullable @NlsContexts.Label
  public String getRestrictionsText() {
    return (String)myCertKey.eval(new GetLicenseRestriction());
  }
}
