package com.intellij.ide.license.impl;

import com.jetbrains.license.util.IteratorUtil;

import java.net.URI;
import java.util.*;

public class QueryParams {
  private static final Set<String> ourKnownParams = Set.of("code", "state", "error", "error_description");
  
  private final Map<String, String> myParams = new HashMap<>();

  public QueryParams(URI uri) {
    this(uri != null ? uri.getQuery() : null);
  }
  
  public QueryParams(String query) {
    for (Param<String> param : queryParams(query)) {
      myParams.put(param.getName(), param.get());
      if (myParams.size() >= ourKnownParams.size()) {
        break;
      }
    }
  }

  public String get(String paramName) {
    return myParams.get(paramName);
  }
  public String getOrDefault(String paramName, String def) {
    final String val = myParams.get(paramName);
    return val != null? val : def;
  }

  private static Iterable<Param<String>> queryParams(String query) {
    return query == null || query.isBlank()? Collections.emptyList() : () -> IteratorUtil.filter(IteratorUtil.map(paramIterator(query), s -> {
      final int idx = s.indexOf('=');
      return idx < 0? null : Param.create(s.substring(0, idx), s.substring(idx + 1));
    }), p -> p != null && ourKnownParams.contains(p.getName()));
  }

  private static Iterator<String> paramIterator(String str) {
    StringTokenizer tokenizer = new StringTokenizer(str, "&", false);
    return new Iterator<>() {
      @Override
      public boolean hasNext() {
        return tokenizer.hasMoreTokens();
      }

      @Override
      public String next() {
        return tokenizer.nextToken();
      }
    };
  }

  private interface Param<T>{
    String getName();
    T get();
    
    static <T> Param<T> create(String name, T value) {
      return new Param<T>() {
        @Override
        public String getName() {
          return name;
        }

        @Override
        public T get() {
          return value;
        }
      };
    }
  }
}
