/*
 * @author max
 */
package com.intellij.ide.license.impl;

import com.intellij.licensecommon.license.AbstractLicenseData;
import com.intellij.licensecommon.util.LicenseType;

import java.util.Date;

public class ServerLicenseData extends AbstractLicenseData {
  private final String myTicketId;
  private final String myServerUrl;
  private final String myLicensedTo;
  private final int myLicenseType;
  private String myConfirmationStamp;
  private final String myMetadata;
  private final long myProlongationPeriod;

  public ServerLicenseData(final String ticketId,
                           final long prolongationPeriod,
                           String serverUrl,
                           String licensedTo,
                           int licenseType,
                           String confirmationStamp, String metadata) {
    myProlongationPeriod = prolongationPeriod;
    myTicketId = ticketId;
    myServerUrl = serverUrl;
    myLicensedTo = licensedTo;
    myLicenseType = licenseType;
    myConfirmationStamp = confirmationStamp;
    myMetadata = metadata;
  }

  public static ServerLicenseData restoreFromFile(String serverUrl) {
    return new ServerLicenseData("", 10000, serverUrl, "", 0, null, null);
  }


  @Override
  public Date getExpirationDate() {
    return null;
  }

  @Override
  public Date getGenerationDate() {
    return new Date();
  }

  @Override
  public String getKey() {
    return "";
  }

  @Override
  public int getMajorVersion() {
    // deprecated, not called for instances of this class.
    return Integer.MAX_VALUE;
  }

  @Override
  public String getUserName() {
    return myLicensedTo;
  }

  @Override
  public boolean isExpired() {
    return false;
  }

  public int getLicenseType() {
    return myLicenseType;
  }

  @Override
  public boolean isNonCommercial() {
    return myLicenseType == LicenseType.NON_COMMERCIAL;
  }

  @Override
  public boolean isOpenSource() {
    return myLicenseType == LicenseType.OPENSOURCE;
  }

  @Override
  public boolean isPersonal() {
    return myLicenseType == LicenseType.PERSONAL;
  }

  @Override
  public boolean isYearAcademic() {
    return myLicenseType == LicenseType.YEARACADEMIC;
  }

  @Override
  public boolean shouldDetectDuplicates() {
    return false;
  }

  public String getMetadata() {
    return myMetadata;
  }

  public boolean isTicketObtained() {
    final String id = myTicketId;
    return id != null && !id.trim().isEmpty();
  }

  public long getProlongationPeriod() {
    return myProlongationPeriod;
  }

  public String getTicketId() {
    return myTicketId;
  }

  public String getServerUrl() {
    return myServerUrl;
  }

  public String getConfirmationStamp() {
    return myConfirmationStamp;
  }

  public void setConfirmationStamp(String confirmationStamp) {
    myConfirmationStamp = confirmationStamp;
  }
}
