package com.intellij.ide.license.impl;

import org.jetbrains.annotations.NotNull;

import java.util.concurrent.*;
import java.util.function.Function;
import java.util.function.Supplier;

public interface TokenFuture<T> extends Future<T> {
  @Override
  T get();

  @Override
  T get(long timeout, @NotNull TimeUnit unit);

  static <T> TokenFuture<T> create(Future<T> delegate, Supplier<T> timeoutDefValue) {
    return create(delegate, Function.identity(), timeoutDefValue);
  }
  
  static <T, Td> TokenFuture<T> create(final Future<Td> delegate, final Function<Td, T> converter, final Supplier<T> timeoutDefValue) {
    return new TokenFuture<T>() {
      @Override
      public T get() {
        try {
          return converter.apply(delegate.get());
        }
        catch (InterruptedException | ExecutionException | CancellationException ignored) {
        }
        return timeoutDefValue.get();
      }

      @Override
      public T get(long timeout, @NotNull TimeUnit unit) {
        try {
          return converter.apply(delegate.get(timeout, unit));
        }
        catch (InterruptedException | ExecutionException | TimeoutException ignored) {
        }
        return timeoutDefValue.get();
      }

      @Override
      public boolean cancel(boolean mayInterruptIfRunning) {
        return delegate.cancel(mayInterruptIfRunning);
      }

      @Override
      public boolean isCancelled() {
        return delegate.isCancelled();
      }

      @Override
      public boolean isDone() {
        return delegate.isDone();
      }
    };
  }

  static <T> TokenFuture<T> createCompleted(final T result) {
    return new TokenFuture<T>() {
      @Override
      public boolean cancel(boolean mayInterruptIfRunning) {
        return false;
      }

      @Override
      public boolean isCancelled() {
        return false;
      }

      @Override
      public boolean isDone() {
        return true;
      }

      @Override
      public T get() {
        return result;
      }

      @Override
      public T get(long timeout, @NotNull TimeUnit unit) {
        return result;
      }
    };
  }
}
