package com.intellij.ide.license.impl;

import com.intellij.ide.license.LicenseInfo;
import com.intellij.ide.license.LicenseManager;
import com.intellij.ide.license.ProductDescriptor;
import com.intellij.licensecommon.license.LicenseData;
import com.intellij.openapi.application.PermanentInstallationID;
import com.jetbrains.license.util.IteratorUtil;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

public final class UpdateLicenseAbout implements InvocationHandler {
  @Override
  public Object invoke(Object proxy, Method method, Object[] args) {
    try {
      final Class<?> aClass = Class.forName("com.intellij.ui.LicensingFacade");
      Object export = null;
      try {
        final LicenseManager mgr = LicenseManager.getInstance();
        final LicenseInfo licInfo = mgr.getPlatformLicenseInfo();
        final Iterable<LicenseInfo> pluginInfos = mgr.getPluginLicenseInfo();
        final LicenseData lic = licInfo.getLicense();
        if (lic == null && IteratorUtil.isEmpty(pluginInfos)) {
          return null;
        }
        export = aClass.newInstance();
        final Map<String, String> stamps = new HashMap<>();
        final Map<String, Date> expirationDates = new HashMap<>();
        if (lic != null) {
          aClass.getDeclaredField("licensedTo").set(export, mgr.licensedToMessage());
          aClass.getDeclaredField("isEvaluation").set(export, licInfo.isEvaluation());
          aClass.getDeclaredField("restrictions").set(export, mgr.licensedRestrictionsMessages());
          final ProductDescriptor pd = licInfo.getProductDescriptor();
          if (lic instanceof CertKeyLicense) {
            // todo: probably a good place to add some reaction for a cracked license
            final CertKeyLicense _lic = (CertKeyLicense)lic;
            aClass.getDeclaredField("expirationDate").set(export, _lic.getCertKey().eval(new GetFirstExpiredDate(_lic.getProductCode())));
            aClass.getDeclaredField("perpetualFallbackDate").set(export, _lic.getCertKey().eval(new GetPerpetualFallbackDate(_lic.getProductCode())));
            aClass.getDeclaredField("metadata").set(export, _lic.getCertKey().eval(new GetMetadata()));
          }
          else {
            aClass.getDeclaredField("expirationDate").set(export, lic.getExpirationDate());
            if (lic instanceof ServerLicenseData) {
              aClass.getDeclaredField("metadata").set(export, ((ServerLicenseData)lic).getMetadata());
            }
          }
          addStamp(stamps, pd, lic);
          final Date platformLicenseExpiration = lic.getExpirationDate();
          if (platformLicenseExpiration != null) {
            expirationDates.put(pd.getProductCode(), platformLicenseExpiration);
          }
        }
        for (LicenseInfo info : pluginInfos) {
          final LicenseData pluginLic = info.getLicense();
          if (pluginLic != null) {
            final ProductDescriptor pd = info.getProductDescriptor();
            addStamp(stamps, pd, pluginLic);
            final Date expiration = pluginLic.getExpirationDate();
            if (expiration != null) {
              expirationDates.put(pd.getProductCode(), expiration);
            }
          }
        }
        if (!stamps.isEmpty()) {
          aClass.getDeclaredField("confirmationStamps").set(export, Collections.unmodifiableMap(stamps));
        }
        if (!expirationDates.isEmpty()) {
          aClass.getDeclaredField("expirationDates").set(export, Collections.unmodifiableMap(expirationDates));
        }
      }
      finally {
        aClass.getDeclaredField("INSTANCE").set(null, export);
      }
    }
    catch (Throwable ignored) {
    }
    return null;
  }

  private static void addStamp(Map<String, String> stamps, ProductDescriptor pd, Object lic) {
    if (lic instanceof CertKeyLicense) {
      final Object key = ((CertKeyLicense)lic).getCertKey().eval(new GetRawKey());
      if (key != null) {
        stamps.put(pd.getProductCode(), "key:" + key);
      }
    }
    else if (lic instanceof ServerLicenseData) {
      final String stamp = ((ServerLicenseData)lic).getConfirmationStamp();
      if (stamp != null) {
        stamps.put(pd.getProductCode(), "stamp:" + PermanentInstallationID.get() + ":" + stamp);
      }
    }
    else if (lic instanceof EvaluationLicenseData) {
      final Date expiration = ((EvaluationLicenseData)lic).getExpirationDate();
      stamps.put(pd.getProductCode(), "eval:" + expiration.getTime());
    }
  }
}
