package com.intellij.ide.license.ui;

import com.intellij.ide.BrowserUtil;
import com.intellij.ui.JBColor;
import com.intellij.util.ui.StartupUiUtil;
import com.intellij.util.ui.UIUtil;
import org.jetbrains.annotations.NonNls;

import javax.swing.*;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import java.awt.*;
import java.net.URL;

public class HtmlHelp {
  public static String makeUrl(String url) {
    return makeUrl(url, url);
  }

  @SuppressWarnings({"HardCodedStringLiteral"})
  public static String makeUrl(String url, String text) {
    String style = "color: " + makeRgb(JBColor.blue.darker()) + "; text-decoration: none;";
    return "<a style='" + style + "' href='" + url + "'>" + text + "</a>";
  }

  @SuppressWarnings({"HardCodedStringLiteral"})
  private static String makeRgb(Color color) {
    return "rgb(" + color.getRed() + "," + color.getGreen() + "," + color.getBlue() + ")";
  }

  @SuppressWarnings({"HardCodedStringLiteral"})
  public static JTextPane makeHtmlPane(String htmlText) {
    JTextPane textPane = new JTextPane();
    textPane.setContentType("text/html");
    textPane.setEditable(false);
    textPane.setBackground(null);
    String text = "<div style='" + makeFontRule() + "'>" + htmlText + "</div>";
    textPane.setText(text);
    textPane.addHyperlinkListener(new MyHyperlinkListener(textPane));
    return textPane;
  }

  private static String makeFontRule() {
    Font font = UIUtil.getOptionPaneMessageFont();
    if (font == null) {
      font = UIUtil.getLabelFont();
      if (font == null) {
        font = new Font(StartupUiUtil.ARIAL_FONT_NAME, Font.PLAIN, 10);
      }
    }

    @NonNls StringBuilder rule = new StringBuilder("font-family: ");
    rule.append(font.getFamily());
    rule.append("; font-size: ");
    rule.append(font.getSize());
    rule.append("pt");
    if (font.isBold()) {
      rule.append("; font-weight: 700");
    }
    if (font.isItalic()) {
      rule.append("; font-style: italic");
    }
    rule.append(";");
    return rule.toString();
  }

  private static class MyHyperlinkListener implements HyperlinkListener {
    private final JTextPane myTextPane;

    MyHyperlinkListener(JTextPane textPane) {
      myTextPane = textPane;
    }

    @Override
    public void hyperlinkUpdate(HyperlinkEvent e) {
      HyperlinkEvent.EventType type = e.getEventType();
      if (type == HyperlinkEvent.EventType.ACTIVATED) {
        URL url = e.getURL();
        BrowserUtil.browse(url);
      }
      else if (type == HyperlinkEvent.EventType.ENTERED) {
        myTextPane.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
      }
      else if (type == HyperlinkEvent.EventType.EXITED) {
        myTextPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      }
    }
  }
}