package com.intellij.ide.license.ui;

import com.intellij.ide.license.ProductDescriptor;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

class LicenseItemsListModel extends AbstractListModel<ProductDescriptor> implements LicensePanel.LicenseUpdateListener {

  private final List<ProductDescriptor> myProducts;

  LicenseItemsListModel(LicenseEditingModel model) {
    myProducts = getRequireLicenseProducts(model);
  }

  @Override
  public int getSize() {
    return myProducts.size();
  }

  @Override
  public ProductDescriptor getElementAt(int index) {
    return myProducts.get(index);
  }

  @Override
  public void licenseUpdated(ProductDescriptor pd) {
    int index = 0;
    for (ProductDescriptor p : myProducts) {
      if (p.equals(pd)) {
        fireContentsChanged(this, index, index);
        break;
      }
      index++;
    }
  }

  @NotNull
  private static List<ProductDescriptor> getRequireLicenseProducts(LicenseEditingModel model) {
    List<ProductDescriptor> products = new ArrayList<>(model.getPluginProducts());
    products.sort(Comparator.comparing(pd -> pd.getShortName()));

    final ProductDescriptor platform = model.getPlatformProduct();
    if (platform != null) {
      products.add(0, platform);
    }
    return products;
  }

}
