package com.intellij.ide.license.ui;

import com.intellij.openapi.ui.Splitter;
import com.intellij.ui.components.JBList;
import com.intellij.ui.components.JBScrollPane;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.awt.*;
import java.util.List;

//todo move to common module #idea-206963
public abstract class MasterDetailPanel<T> extends JPanel {

  protected final JBList<T> myItemsList;

  protected MasterDetailPanel(@NotNull List<T> elements, @NotNull ListCellRenderer<T> elementsRenderer) {
    super(new BorderLayout());

    if (elements.isEmpty()) throw new IllegalArgumentException("Items list cannot be empty");

    myItemsList = new JBList<>(elements);
    myItemsList.setCellRenderer(elementsRenderer);
    myItemsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

    JScrollPane leftScrollPanel = new JBScrollPane(myItemsList);
    leftScrollPanel.setBorder(null);
    leftScrollPanel.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

    Splitter splitter = new Splitter();
    splitter.setFirstComponent(leftScrollPanel);
    splitter.setSecondComponent(new JPanel());
    splitter.setProportion(0.31f);

    myItemsList.addListSelectionListener(e -> {
      T selected = myItemsList.getSelectedValue();
      JPanel detailsPanel = getDetailsPanelForItem(selected);
      splitter.setSecondComponent(detailsPanel);
    });

    add(splitter, BorderLayout.CENTER);
  }

  protected abstract JPanel getDetailsPanelForItem(T item);
}
