package com.intellij.indexing.shared.download

import com.fasterxml.jackson.databind.ObjectMapper
import com.fasterxml.jackson.databind.node.ObjectNode
import com.google.common.annotations.VisibleForTesting
import com.intellij.openapi.components.Service
import com.intellij.openapi.components.service
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.progress.ProgressIndicator
import com.intellij.util.io.HttpRequests
import org.tukaani.xz.XZInputStream
import java.io.ByteArrayInputStream


@Service
class SharedIndexDownloadJson {
  private val LOG = logger<SharedIndexDownloadJson>()

  @VisibleForTesting
  fun downloadJsonXz(
    request: SharedIndexLookupRequest,
    url: String,
    indicator: ProgressIndicator
  ): ObjectNode? {
    LOG.debug("Checking $url for $request...")

    val headers = service<SharedIndexAuthHolder>().resolveHeaders(url, request, indicator)

    val rawDataXZ = handleHttpDownloadExceptions(LOG, request, url) {
      HttpRequests
        .request(url)
        .productNameAsUserAgent()
        .accept("application/octet-stream, application/xz")
        .tuner { tuner ->
          headers.forEach { (k, v) -> tuner.addRequestProperty(k, v) }
        }
        .throwStatusCodeException(true)
        .readBytes(indicator)
    } ?: return null

    val rawData = try {
      ByteArrayInputStream(rawDataXZ).use { input ->
        XZInputStream(input).use {
          it.readBytes()
        }
      }
    }
    catch (e: Throwable) {
      LOG.warn("Failed to unpack data from $url for $request. ${e.message}")
      return null
    }

    return try {
      ObjectMapper().readTree(rawData) as ObjectNode
    }
    catch (e: Throwable) {
      LOG.warn("Failed to read JSON from $url for $request. ${e.message}")
      return null
    }
  }
}
