package com.intellij.indexing.shared.download

data class SharedIndexBaseId(val sha256: String)

interface SharedIndexLookupRequest {
  val kind: String
  fun indexUrls(): List<String>
  val presentableChunkName: String
  /** @see SharedIndexDownloadAuthExtension.type **/
  val authId: String?

  //inheritors must implement equals/hashcode or be a Kotlin data class
  override fun hashCode(): Int
  override fun equals(other: Any?): Boolean

  /**
   * Should list.json.xz files be used to
   * avoid unneeded HTTP requests
   *
   * @see SharedIndexListLookup
   */
  @JvmDefault
  val useListJsonIndexToSkipUrls: Boolean get() = false

  /**
   * A shared index may claim it has base shared indexes which
   * contain more data. This method is executed from the core to
   * resolve bases into SharedIndexLookup requests recursively
   */
  @JvmDefault
  fun resolveBasesRequests(bases: List<SharedIndexBaseId>) : List<SharedIndexLookupRequest> = listOf()
}

