package com.intellij.indexing.shared.download

import com.intellij.openapi.application.ApplicationManager
import com.intellij.openapi.components.Service
import com.intellij.openapi.project.Project
import java.util.concurrent.atomic.AtomicInteger

@Service
class SharedIndexPerProjectTaskExpiration {
  private val value = AtomicInteger(0)

  interface Token {
    fun isExpired(): Boolean
  }

  companion object {
    /**
     * Registers new task and invalidates all previously allocated tokens (making [Token.isExpired] return false)
     */
    fun registerNewTask(project: Project?) : Token {
      //we use Service to implement trivial disposal of the state at the moment project is disposed
      val state = (project ?: ApplicationManager.getApplication()).getService(
        SharedIndexPerProjectTaskExpiration::class.java)
      val ourId = state.value.incrementAndGet()

      return object: Token {
        override fun isExpired(): Boolean  = ourId != state.value.get()
      }
    }
  }
}
