package com.intellij.indexing.shared.java.maven

import com.intellij.openapi.application.runReadAction
import com.intellij.openapi.components.Service
import com.intellij.openapi.module.ModuleManager
import com.intellij.openapi.project.Project
import com.intellij.openapi.roots.LibraryOrderEntry
import com.intellij.openapi.roots.ModuleRootManager
import com.intellij.openapi.roots.OrderRootType
import com.intellij.openapi.vfs.JarFileSystem
import com.intellij.openapi.vfs.VirtualFile

@Service
class MavenJarHashStore(val project: Project) {
  fun listAllJarCoordinates(): Map<MavenPackageId, VirtualFile> = runReadAction {
    ModuleManager.getInstance(project)
      .modules
      .flatMap { module ->
        ModuleRootManager.getInstance(module)
          .orderEntries
          ?.filterIsInstance<LibraryOrderEntry>()
          ?.mapNotNull { it.library }
        ?: listOf()
      }
      .distinct()
      .mapNotNull { it.getFiles(OrderRootType.CLASSES)?.toList() }
      .toList()
  }.asSequence()
    .flatten()
    .filterNotNull()
    .mapNotNull { JarFileSystem.getInstance().getLocalByEntry(it) ?: it }
    .filter { !it.isDirectory && it.name.endsWith(".jar") }
    .distinct()
    .mapNotNull { MavenJarNamingHelper.inferPackageCoordinates(it.path)?.to(it) }
    .toMap()
}
