package com.intellij.indexing.shared.java.maven

import com.intellij.indexing.shared.download.SharedIndexLookupRequest
import com.intellij.indexing.shared.message.SharedIndexesBundle
import java.util.*

class MavenLibraryIndexLookupRequest(val pkg: MavenPackageId) : SharedIndexLookupRequest {
  override val kind = MavenSharedIndexes.MAVEN_KIND

  /**
   * We may have 100-s of Maven packages to test,
   * this optimization will help to avoid the
   * most of requests
   */
  override val useListJsonIndexToSkipUrls: Boolean
    get() = true

  override val presentableChunkName: String
    get() = SharedIndexesBundle.message("presentable.chunk.name.the.maven", pkg.asString())

  override val authId: String?
    get() = null

  override fun indexUrls(): List<String> {
    val version = pkg.version
    val elements = version.split(".", limit = 3)
    val majorVersion = elements.first()
    val majorMinorVersion = elements.take(2).joinToString(".")

    val result = mutableListOf(
      "package/${pkg.groupId}:${pkg.artifactId}:$version"
    )

    if (version != majorMinorVersion) {
      result += "major-minor-version/${pkg.groupId}:${pkg.artifactId}:${majorMinorVersion}"
    }

    if (version != majorVersion && majorVersion != majorMinorVersion) {
      result += "major-version/${pkg.groupId}:${pkg.artifactId}:${majorVersion}"
    }

    val pathPrefix = MavenSharedIndexes.mavenFeedUrl
    return result.map { it.toLowerCase(Locale.US) }.distinct().map { "$pathPrefix/$it" }
  }

  override fun hashCode() = pkg.hashCode()
  override fun equals(other: Any?): Boolean = other is MavenLibraryIndexLookupRequest && this.pkg == other.pkg
  override fun toString() = "MavenLibraryIndexRequest($pkg)"
}
