package com.intellij.indexing.shared.java.maven

import com.intellij.indexing.shared.download.*
import com.intellij.indexing.shared.message.SharedIndexesBundle
import com.intellij.openapi.components.service
import com.intellij.openapi.progress.ProgressIndicator
import com.intellij.openapi.project.Project
import com.intellij.ui.layout.*

internal class MavenSuggesterConfigurable : SharedIndexSuggesterConfigurable {
  override val mode = SharedIndexConfigurableMode.PreBuilt
  override fun isAvailableFor(project: Project): Boolean = MavenSharedIndexes.isEnabled()

  override fun buildSettingsPanel(project: Project, scope: SharedIndexesConfigurableScope, builder: RowBuilder) {
    scope.run {
      builder.row(SharedIndexesBundle.message("configuration.section.maven")) {
        consentComboBox(MavenSharedIndexes.decisionKey, includeUrl = false)
      }
    }
  }
}

internal class MavenSuggester: SharedIndexSuggester {
  override fun suggestRequests(project: Project): List<SharedIndexSuggestion> {
    if (!MavenSharedIndexes.isEnabled()) return listOf()

    return project.service<MavenJarHashStore>().listAllJarCoordinates().keys.map { pkg ->
      MavenSuggestion(pkg)
    }
  }
}

private class MavenSuggestion(private val pkg: MavenPackageId) : SharedIndexSuggestion {
  override val sharedIndexId: SharedIndexId = MavenSharedIndexes.toSharedIndexId(pkg)

  override val sharedIndexConsentNotificationKindPresentableText: String
    get() = SharedIndexesBundle.message("notification.consent.content.kind.maven")

  override val sharedIndexDownloadedPresentableText: String
    get() = SharedIndexesBundle.message("eventLog.downloaded.maven", pkg.asString())

  override fun resolveRequest(indicator: ProgressIndicator) = MavenLibraryIndexLookupRequest(pkg)

  override fun toString() = "MavenSuggestion($pkg)"
}
