package com.intellij.indexing.shared.platform.api

import com.intellij.indexing.shared.platform.impl.SharedIndex
import com.intellij.util.indexing.ID
import java.nio.file.Path

sealed class AttachChunkResult {

  /** A text message to be logged in IDEA log **/
  abstract fun toLogMessage() : String

  abstract val matchingFbIndexes : List<ID<*, *>>

  fun isSuccess() = this is Success

  data class Success(
    val chunkRoot: Path,
    val allMatchingIndexes: List<SharedIndex<*, *>>,

    // IDs of the shared indexes which versions match the ones in the IDE.
    override val matchingFbIndexes: List<ID<*, *>>,
    val matchingStubIndexes: List<ID<*, *>>,

    // IDs of the shared indexes which versions do NOT match the ones in the IDE.
    val mismatchingFbIndexes: List<ID<*, *>>,
    val mismatchingStubIndexes: List<ID<*, *>>,

    // IDs of indexes that cannot be resolved in the current IDE.
    val redundantFbIndexes: List<String>,
    val redundantStubIndexes: List<String>,

    // Stats after attaching the given index.
    val sharedIndexStats: SharedIndexStats,
  ) : AttachChunkResult() {

    override fun toLogMessage() = buildString {
      append("matching: (fb=${matchingFbIndexes.size}, stub=${matchingStubIndexes.size})")
      append(", ")
      append("incompatible: (fb=${mismatchingFbIndexes.size}, stub=${mismatchingStubIndexes.size})")
      append(", ")
      append("unknown: (fb=${redundantFbIndexes.size}, stub=${redundantStubIndexes.size}), ")
    }
  }

  object Incompatible : AttachChunkResult() {
    override fun toLogMessage(): String = "INCOMPATIBLE"
    override val matchingFbIndexes: List<ID<*, *>>
      get() = emptyList()
  }
}