package com.intellij.indexing.shared.platform.api

import com.intellij.openapi.progress.ProgressIndicator
import com.intellij.openapi.project.Project
import java.nio.file.Path

/**
 * A handler for a possible indexes chunk that this extension is able to supply
 */
// TODO any chunk is associated with file scope, so descriptor should return it in serializable format.
interface ChunkDescriptor : ChunkDownloader {
  /**
   * An application wide unique identifier of that indexes portion.
   * That key is used by the IDE to avoid re-downloading same indexes one more times
   */
  val chunkUniqueId: String

  /**
   * The kind of downloading shared index
   */
  val kind: String
}

interface ChunkDownloader {
  /**
   * Materialize the index chunk
   * @return true if download is successful, false if it failed.
   * The implementation is responsible to log the details of the problem in the log
   */
  fun downloadChunk(targetFile: Path,
                    project: Project?,
                    indicator: ProgressIndicator): Boolean
}
