package com.intellij.indexing.shared.platform.api;

import com.intellij.openapi.util.SystemInfo;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Set;
import java.util.TreeSet;

public enum IndexInfrastructureOs implements Comparable<IndexInfrastructureOs> {
  windows, mac, linux,
  ;

  @NotNull
  public static Set<String> textValues() {
    Set<String> result = new TreeSet<>();
    for (IndexInfrastructureOs os : values()) {
      result.add(os.getOsName());
    }
    return result;
  }

  @NotNull
  public String getOsName() {
    //these names are used in serialization!
    if (this == windows) return "windows";
    if (this == mac) return "mac";
    if (this == linux) return "linux";
    throw new RuntimeException("Unknown OS: " + this);
  }

  @NotNull
  public static IndexInfrastructureOs getOs() {
    if (SystemInfo.isWindows) return windows;
    if (SystemInfo.isMac) return mac;
    if (SystemInfo.isLinux) return linux;
    throw new Error("Unknown OS. " + SystemInfo.getOsNameAndVersion());
  }

  @Nullable
  public static IndexInfrastructureOs find(@NotNull String capabilityName) {
    for (IndexInfrastructureOs value : values()) {
      if (value.getOsName().equalsIgnoreCase(capabilityName)) return value;
    }
    return null;
  }
}
