package com.intellij.indexing.shared.platform.impl;

import com.intellij.ide.plugins.cl.PluginAwareClassLoader;
import com.intellij.openapi.extensions.PluginId;
import com.intellij.util.indexing.ID;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public final class HashIndexKey extends ID<Long, Void> {
  private static final String FILE_CONTENT_HASH_INDEX_SUFFIX = "shared.index.hashes.";

  private final ID<?, ?> myBaseIndexId;

  private HashIndexKey(@NotNull String name, ID<?, ?> baseIndexId) {
    super(name, getSharedIndexPlugin());
    myBaseIndexId = baseIndexId;
  }

  @NotNull
  public ID<?, ?> getBaseIndexId() {
    return myBaseIndexId;
  }

  @NotNull
  public static synchronized HashIndexKey createHashIndexKey(@NonNls @NotNull ID<?, ?> baseIndexId) {
    String name = FILE_CONTENT_HASH_INDEX_SUFFIX + baseIndexId.getName();
    ID<?, ?> existing = findByName(name);
    if (existing != null) {
      if (existing instanceof HashIndexKey) {
        return (HashIndexKey)existing;
      }
      throw new IllegalStateException("key with id " + name + " is already registered", existing.getRegistrationTrace());
    }
    return new HashIndexKey(name, baseIndexId);
  }

  @Nullable
  private static PluginId getSharedIndexPlugin() {
    ClassLoader classLoader = HashIndexKey.class.getClassLoader();
    return classLoader instanceof PluginAwareClassLoader ? ((PluginAwareClassLoader)classLoader).getPluginId() : null;
  }
}
