package com.intellij.indexing.shared.platform.impl

import com.intellij.ide.plugins.PluginManagerCore
import com.intellij.internal.statistic.eventLog.validator.ValidationResultType
import com.intellij.internal.statistic.eventLog.validator.rules.EventContext
import com.intellij.internal.statistic.eventLog.validator.rules.impl.CustomValidationRule
import com.intellij.internal.statistic.utils.PluginInfo
import com.intellij.internal.statistic.utils.PluginType
import com.intellij.internal.statistic.utils.getPluginInfo
import com.intellij.internal.statistic.utils.getPluginInfoById
import com.intellij.psi.stubs.StubIndexExtension
import com.intellij.util.indexing.FileBasedIndexExtension
import com.intellij.util.indexing.ID

class IndexIdRuleValidator : CustomValidationRule() {
  override fun acceptRuleId(ruleId: String?): Boolean = "index_id" == ruleId

  override fun doValidate(data: String, context: EventContext): ValidationResultType {
    if (isThirdPartyValue(data)) {
      return ValidationResultType.ACCEPTED
    }

    val id = ID.findByName<Any, Any>(data) ?: return ValidationResultType.REJECTED
    val pluginId = id.pluginId
    if (pluginId != null) {
      return validatePlugin(getPluginInfoById(pluginId))
    }

    if (PluginManagerCore.isRunningFromSources()) {
      val extension = FileBasedIndexExtension.EXTENSION_POINT_NAME.findFirstSafe { ex -> ex.name.name == data }
      if (extension != null) {
        return validatePlugin(getPluginInfo(extension::class.java))
      }

      val stubExtension = StubIndexExtension.EP_NAME.findFirstSafe { ex -> ex.key.name == data }
      if (stubExtension != null) {
        return validatePlugin(getPluginInfo(stubExtension::class.java))
      }
    }

    return ValidationResultType.REJECTED
  }

  private fun validatePlugin(info: PluginInfo): ValidationResultType {
    if (info.type === PluginType.UNKNOWN) {
      return ValidationResultType.REJECTED
    }

    return if (info.isDevelopedByJetBrains()) ValidationResultType.ACCEPTED else ValidationResultType.THIRD_PARTY
  }
}