// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.indexing.shared.platform.impl;

import com.intellij.indexing.shared.platform.api.IndexInfrastructureVersion;
import com.intellij.psi.stubs.*;
import com.intellij.util.indexing.FileBasedIndex;
import com.intellij.util.indexing.FileBasedIndexExtension;
import com.intellij.util.indexing.ID;
import com.intellij.util.indexing.IndexedFile;
import org.jetbrains.annotations.NotNull;

import java.nio.file.Path;

public interface SharedIndexExtension<K, V> {
  @NotNull
  FileBasedIndexExtension<K, V> getFileBasedIndexExtension();

  default int getVersion() {
    return 0;
  }

  default void openResources(@NotNull Path indexRoot,
                             boolean readOnly,
                             @NotNull IndexInfrastructureVersion chunkVersion) {

  }

  default void closeResources() {
  }

  default boolean acceptsInputFile(@NotNull IndexedFile file) {
    return true;
  }

  @NotNull
  static <K, V> SharedIndexExtension<K, V> findExtensionAndOpen(@NotNull FileBasedIndexExtension<K, V> originalExtension,
                                                                @NotNull Path indexRoot,
                                                                boolean readOnly,
                                                                @NotNull IndexInfrastructureVersion chunkVersion) {
    SharedIndexExtension<K, V> extension = findExtension(originalExtension);
    extension.openResources(indexRoot, readOnly, chunkVersion);
    return extension;
  }

  @SuppressWarnings("unchecked")
  @NotNull
  static <K, V> SharedIndexExtension<K, V> findExtension(@NotNull FileBasedIndexExtension<K, V> originalExtension) {
    return originalExtension.getName().equals(StubUpdatingIndex.INDEX_ID)
           ? (SharedIndexExtension<K, V>)new StubSharedIndexExtension()
           : new SharedIndexExtensionImpl<>(originalExtension);
  }

  @NotNull
  static Path getSharedIndexDir(@NotNull Path chunkRoot, @NotNull ID<?, ?> id) {
    return chunkRoot.resolve(id.getName());
  }

  class SharedIndexExtensionImpl<K, V> implements SharedIndexExtension<K, V> {
    private final FileBasedIndexExtension<K, V> myOriginalExtension;

    SharedIndexExtensionImpl(@NotNull FileBasedIndexExtension<K, V> originalExtension) {
      myOriginalExtension = originalExtension;
    }

    @Override
    public @NotNull FileBasedIndexExtension<K, V> getFileBasedIndexExtension() {
      return myOriginalExtension;
    }
  }
}
