package com.intellij.indexing.shared.platform.impl

import com.intellij.ide.nls.NlsMessages
import com.intellij.indexing.shared.message.SharedIndexesBundle
import com.intellij.notification.NotificationGroupManager
import com.intellij.notification.NotificationType
import com.intellij.util.indexing.diagnostic.IndexDiagnosticDumper
import com.intellij.util.indexing.diagnostic.ProjectIndexingHistory
import java.time.Duration

class SharedIndexProjectIndexingHistoryListener : IndexDiagnosticDumper.ProjectIndexingHistoryListener {
  override fun onFinishedIndexing(projectIndexingHistory: ProjectIndexingHistory) {
    if (projectIndexingHistory.times.wasInterrupted) {
      return
    }
    val totalNumberOfIndexedFiles = projectIndexingHistory.scanningStatistics.sumOf { statistics ->
      statistics.numberOfFilesForIndexing + statistics.numberOfFilesFullyIndexedByInfrastructureExtensions
    }
    if (totalNumberOfIndexedFiles < 100) {
      return
    }

    val indexingDuration = projectIndexingHistory.times.indexingDuration
    if (indexingDuration <= Duration.ofSeconds(20)) {
      return
    }

    val totalNumberOfFilesFullyIndexedByInfrastructureExtensions =
      projectIndexingHistory.scanningStatistics.sumOf { statistics -> statistics.numberOfFilesFullyIndexedByInfrastructureExtensions } +
      projectIndexingHistory.providerStatistics.sumOf { statistics -> statistics.totalNumberOfFilesFullyIndexedByExtensions }

    if (totalNumberOfFilesFullyIndexedByInfrastructureExtensions < 100) {
      return
    }

    val messageText = SharedIndexesBundle.message(
      "eventLog.indexing.completed",
      NlsMessages.formatDurationApproximate(indexingDuration.toMillis() / 1000 * 1000),
      totalNumberOfFilesFullyIndexedByInfrastructureExtensions,
      totalNumberOfIndexedFiles,
      (totalNumberOfFilesFullyIndexedByInfrastructureExtensions.toDouble() / totalNumberOfIndexedFiles * 100).toInt()
    )
    NotificationGroupManager.getInstance()
      .getNotificationGroup("Shared Indexing Log")
      .createNotification(NotificationType.INFORMATION)
      .setContent(messageText)
      .notify(projectIndexingHistory.project)
  }
}