// Copyright 2000-2019 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.intellij.indexing.shared.util.zipFs;

import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.FileTime;

final class UncompressedZipEntryFileAttributes implements BasicFileAttributes {
  private final UncompressedZipFileSystem.ZipTreeNode myNode;

  UncompressedZipEntryFileAttributes(@NotNull UncompressedZipPath path) throws IOException {
    myNode = UncompressedZipFileSystemProvider.find(path);
  }

  @Override
  public String toString() {
    return "UncompressedZipEntryFileAttributes{" +
           "myNode=" + myNode +
           '}';
  }

  @Override
  public FileTime lastModifiedTime() {
    throw new UnsupportedOperationException("lastModifiedTime is not supported for " + this);
  }

  @Override
  public FileTime lastAccessTime() {
    throw new UnsupportedOperationException("lastAccessTime is not supported for " + this);
  }

  @Override
  public FileTime creationTime() {
    throw new UnsupportedOperationException("creationTime is not supported for ");
  }

  @Override
  public boolean isRegularFile() {
    return !isDirectory();
  }

  @Override
  public boolean isDirectory() {
    return myNode.isDirectory();
  }

  @Override
  public boolean isSymbolicLink() {
    return false;
  }

  @Override
  public boolean isOther() {
    return false;
  }

  @Override
  public long size() {
    return myNode.getEntry().size;
  }

  @Override
  public Object fileKey() {
    return null;
  }
}
