package com.intellij.licensecommon.message.impl;

import com.intellij.licensecommon.license.LicenseBroadcastData;
import com.intellij.licensecommon.message.Message;
import com.intellij.licensecommon.message.MessageClient;
import com.intellij.licensecommon.net.Address;
import com.intellij.openapi.util.Comparing;
import java.util.Objects;
import org.jetbrains.annotations.NotNull;

public class ResharperMessage implements Message {
  @NotNull
  private final LicenseBroadcastData myBroadcastData;

  public ResharperMessage(@NotNull LicenseBroadcastData broadcastData) {
    myBroadcastData = broadcastData;
  }

  @Override
  @NotNull
  public LicenseBroadcastData getBroadcastData() {
    return myBroadcastData;
  }

  @Override
  public void receivedAt(MessageClient client, Address from) throws Exception {
    if (myBroadcastData.getCode() != 0) {
      return; // ignore all messages in obsolete format (everything with non-zero code)
    }
    final LicenseBroadcastData clientLicense = client.getBroadcastData();
    if (clientLicense != null) {
      // same key is used from under different logins
      boolean shouldNotifyCollision = !Comparing.equal(myBroadcastData.getUserLogin(), clientLicense.getUserLogin(), false) &&
                                      Objects.equals(myBroadcastData.getLicenseKeyId(), clientLicense.getLicenseKeyId());
      if (shouldNotifyCollision) {
        client.duplicateLicenseDetected(from, myBroadcastData);
      }
    }
  }
}
