package com.intellij.licensecommon.net;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;

/**
 */
public class Packet {
  private final HashableByteArray bytes;
  private final Address address;

  public Packet(byte[] bytes, Address address) {
    this.bytes = new HashableByteArray(bytes);

    this.address = address;
  }

  public Address getAddress() {
    return address;
  }

  public byte[] getBytes() {
    return bytes.getBytes();
  }

  @SuppressWarnings({"HardCodedStringLiteral"})
  public String toString() {
    return "Packet{" + address + ":" + bytes + "}";
  }

  public boolean equals(Object o) {
    if (this == o) return true;
    if (!(o instanceof Packet)) return false;

    final Packet packet = (Packet)o;

    if (address != null ? !address.equals(packet.address) : packet.address != null) return false;
    if (bytes != null ? !bytes.equals(packet.bytes) : packet.bytes != null) return false;

    return true;
  }

  public int hashCode() {
    int result;
    result = (bytes != null ? bytes.hashCode() : 0);
    result = 29 * result + (address != null ? address.hashCode() : 0);
    return result;
  }

  private static final class HashableByteArray {
    private final byte[] bytes;

    private HashableByteArray(byte[] bytes) {
      this.bytes = bytes;
    }

    private byte[] getBytes() {
      return bytes;
    }

    public String toString() {
      return new String(bytes, StandardCharsets.UTF_8);
    }

    public boolean equals(Object o) {
      if (this == o) return true;
      if (!(o instanceof HashableByteArray)) return false;

      final HashableByteArray hashableByteArray = (HashableByteArray)o;

      if (!Arrays.equals(bytes, hashableByteArray.bytes)) return false;

      return true;
    }

    public int hashCode() {
      return new String(bytes, StandardCharsets.UTF_8).hashCode();
    }
  }
}
