package com.intellij.licensecommon.net.impl;

import com.intellij.licensecommon.net.Address;

import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;

public class AddressImpl implements Address {
  private final InetSocketAddress myAddress;

  public AddressImpl(String host, int port) {
    this(new InetSocketAddress(host, port));
  }

  public AddressImpl(InetSocketAddress address) {
    myAddress = address;
  }

  @Override
  public InetSocketAddress getSocketAddress() {
    return myAddress;
  }

  public String toString() {
    return myAddress.toString();
  }

  public boolean equals(Object o) {
    if (this == o) return true;
    if (!(o instanceof AddressImpl)) return false;

    final AddressImpl other = (AddressImpl)o;

    if (myAddress == null ? other.myAddress != null : !myAddress.equals(other.myAddress)) return false;

    return true;
  }

  public int hashCode() {
    return myAddress == null ? 0 : myAddress.hashCode();
  }

  @Override
  public String getDisplay() {
    final InetAddress inetAddress = getSocketAddress().getAddress();
    return inetAddress.getHostName() + " (" + inetAddress.getHostAddress() + ")";
  }

  @Override
  public boolean isAnyLocalAddress() {
    final InetAddress thisAddress = getSocketAddress().getAddress();
    if (thisAddress != null) { // if resolved
      if (thisAddress.isLoopbackAddress() || thisAddress.isAnyLocalAddress()) {
        return true;
      }
      try {
        Enumeration nis = NetworkInterface.getNetworkInterfaces();
        while (nis.hasMoreElements()) {
          NetworkInterface ni = (NetworkInterface)nis.nextElement();
          Enumeration localAddresses = ni.getInetAddresses();
          while (localAddresses.hasMoreElements()) {
            InetAddress localAddress = (InetAddress)localAddresses.nextElement();
            if (localAddress.equals(thisAddress)) {
              return true;
            }
          }
        }
      }
      catch (SocketException ignored) {
        // Assume false
      }
    }
    return false;
  }
}
