package com.intellij.licensecommon.net.impl;

import com.intellij.licensecommon.net.*;
import com.intellij.util.concurrency.Semaphore;

import java.io.IOException;
import java.net.*;
import java.util.Arrays;

public class SocketImpl implements NetworkSocket {
  private final DatagramSocket mySocket;
  private final Semaphore myReceiveSemaphore = new Semaphore();
  public static int DEFAULT_TIMEOUT = 10000;

  public SocketImpl(DatagramSocket socket) {
    mySocket = socket;
    myReceiveSemaphore.down();
  }

  public SocketImpl(DatagramSocket socket, int timeout) {
    this(socket);
    try {
      this.mySocket.setSoTimeout(timeout);
    }
    catch (SocketException ignore) {
    }
  }

  @Override
  public void send(Packet packet) {
    AddressImpl addressImpl = (AddressImpl)packet.getAddress();
    byte[] bytes = packet.getBytes();
    SocketAddress socketAddress = addressImpl.getSocketAddress();
    DatagramPacket p = null;
    try {
      p = new DatagramPacket(bytes, bytes.length, socketAddress);
    }
    catch (Exception ignored) {
    }
    if (p != null) {
      try {
        mySocket.send(p);
      }
      catch (IOException ignored) {
      }
    }
  }

  @Override
  public Packet receive() throws SocketClosedException, FatalSocketException {
    myReceiveSemaphore.up();
    byte[] buf = new byte[10000];
    DatagramPacket p = new DatagramPacket(buf, buf.length);
    try {
      mySocket.receive(p);
    }
    catch (SocketTimeoutException e) {
      throw new TimeoutOccurredException(e);
    }
    catch (SocketException e) {
      throw new SocketClosedException(e);
    }
    catch (IOException e) {
      throw new FatalSocketException(e);
    }
    byte[] bytes = Arrays.copyOfRange(p.getData(), p.getOffset(), p.getOffset()+p.getLength());
    return new Packet(bytes, new AddressImpl((InetSocketAddress)p.getSocketAddress()));
  }

  @Override
  public void close() {
    mySocket.close();
  }

  @Override
  public void waitUntilReceiving() throws InterruptedException {
    myReceiveSemaphore.waitFor();
  }

  @Override
  public Address getAddress() {
    return new AddressImpl((InetSocketAddress)mySocket.getLocalSocketAddress());
  }
}
