package com.intellij.licensecommon.protocol.impl;

import com.intellij.util.Base64;

import javax.crypto.Cipher;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.Key;

/**
 * @author Eugene Zhuravlev
 */
public class EncryptionUtil {
  // corresponds to first 8 chars of "ReSharper DES key" string used in ReSharper
  private static final Key KEY = new SecretKeySpec(new byte[] {
    (byte)0x52, (byte)0x65, (byte)0x53, (byte)0x68, (byte)0x61, (byte)0x72, (byte)0x70, (byte)0x65,
  }, "DES");

  public static String encrypt(String msg) throws Exception {
    final byte[] encrypted = encrypt(msg.getBytes(StandardCharsets.UTF_8), KEY);
    int index = encrypted.length;
    // perform truncation as in ReSharper
    while (--index >= 0) {
      if (encrypted[index] != 0) {
        break;
      }
    }
    return Base64.encode(encrypted, 0, index + 1);
  }

  public static String decrypt(String msg) throws Exception {
    return new String(decrypt(java.util.Base64.getDecoder().decode(msg), KEY), StandardCharsets.UTF_8);
  }

  // conforms to encryption used in ReSharper
  private static byte[] encrypt(byte[] msgBytes, Key key) throws Exception {
    final Cipher ciph = Cipher.getInstance("DES/CBC/PKCS5Padding");
    ciph.init(Cipher.ENCRYPT_MODE, key, new IvParameterSpec(key.getEncoded()));
    return ciph.doFinal(msgBytes);
  }

  // conforms to decryption used in ReSharper
  private static byte[] decrypt(byte[] data, Key key) throws Exception {
    final Cipher ciph = Cipher.getInstance("DES/CBC/PKCS5Padding");
    ciph.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(key.getEncoded()));
    return ciph.doFinal(data);
  }

  /*
  More secure versions using random IV parameter. Unfortunately DotNet version used by ReSharper
  uses constant IV that is equal to secret key's bytes

  private static byte[] encrypt(byte[] msgBytes, Key key) throws Exception {
    final Cipher ciph = Cipher.getInstance("DES/CBC/PKCS5Padding");
    ciph.init(Cipher.ENCRYPT_MODE, key);
    final byte[] body = ciph.doFinal(msgBytes);
    final byte[] iv = ciph.getIV();

    byte[] data = new byte[4 + iv.length + body.length];

    final int length = body.length;
    data[0] = (byte)((length >> 24)& 0xFF);
    data[1] = (byte)((length >> 16)& 0xFF);
    data[2] = (byte)((length >> 8)& 0xFF);
    data[3] = (byte)(length & 0xFF);

    System.arraycopy(iv, 0, data, 4, iv.length);
    System.arraycopy(body, 0, data, 4 + iv.length, body.length);
    return data;
  }

  private static byte[] decrypt(byte[] data, Key key) throws Exception {
    final int bodyLength = ((data[0] & 0xFF) << 24) |
                           ((data[1] & 0xFF) << 16) |
                           ((data[2] & 0xFF) <<  8) |
                           ((data[3] & 0xFF)      );

    final int ivlength = data.length - 4 - bodyLength;

    final Cipher ciph = Cipher.getInstance("DES/CBC/PKCS5Padding");
    ciph.init(Cipher.DECRYPT_MODE, key, new IvParameterSpec(data, 4, ivlength));
    return ciph.doFinal(data, 4 + ivlength, bodyLength);
  }
  */


  //public static void main(String[] args) {
  //  final String data = args[0];
  //  byte[] dataBytes = data.getBytes(StandardCharsets.UTF_8);
  //  System.out.println("new byte[] {");
  //  for (byte aByte : dataBytes) {
  //    System.out.print("(byte)0x" + Integer.toHexString(((int)aByte) & 0xFF) + ", ");
  //  }
  //  System.out.println("\n}");
  //}

}
