// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ide

import com.intellij.openapi.application.ApplicationManager
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.map

open class Suppressor {
  private val requests = Property(0)
  val released = requests.map { it == 0 }

  fun shouldSuppress(): Boolean {
    application.assertIsDispatchThread()
    return !released.value
  }

  fun suppress(lifetime: Lifetime) {
    lifetime.bracket(
      {
        application.assertIsDispatchThread()
        requests.value++
      },
      {
        application.assertIsDispatchThread()
        require(requests.value >= 0) { "requests >= 0" }

        if (requests.value == 0) return@bracket
        requests.value--
      })
  }

  fun onRelease(lifetime: Lifetime, handler: () -> Unit) {
    released.change.advise(lifetime) {
      application.assertIsDispatchThread()
      if (it) handler()
    }
  }

  fun forceRelease(): Boolean {
    application.assertIsDispatchThread()
    requests.set(0)
    return released.value
  }
}