package com.jetbrains.rd.ide.document

import com.intellij.openapi.editor.Editor
import com.intellij.openapi.editor.ScrollType
import com.intellij.openapi.util.Key
import com.intellij.openapi.vfs.StandardFileSystems
import com.intellij.openapi.vfs.VirtualFile
import com.intellij.openapi.vfs.VirtualFileManager
import com.intellij.util.PathUtil
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rd.ide.vfs.IProtocolVirtualFile
import org.jetbrains.annotations.NotNull
import java.io.File
import java.nio.file.Paths

fun <T : DocumentExtension?> RdDocumentModel.getExtension(key: Key<T>): T {
  @Suppress("UNCHECKED_CAST")
  return extensions.firstOrNull { it.key == key.toString() } as T
}

fun RdDocumentId.toDebugString(): String {
  return when (this) {
    is RdFileDocumentId -> "$protocol, ${Paths.get(filePath).fileName}"
    is RdAbstractDocumentId -> descriptor
    else -> toString()
  }
}

@Deprecated("Use RdFileId when using identity of files")
fun RdFileDocumentId.url() = VirtualFileManager.constructUrl(protocol, filePath)
fun RdDocumentId.isLocalFile() = this is RdFileDocumentId && protocol == StandardFileSystems.FILE_PROTOCOL ||
                                 this is RdDocumentIdWithFileId

/**
 * Should be used by platform code only
 */
@Deprecated("Use RdFileId when using identity of files")
fun VirtualFile.createDocumentId(): RdFileDocumentId {
  val protocol = if (this !is IProtocolVirtualFile) {
    VirtualFileManager.extractProtocol(url) ?: fileSystem.protocol
  }
  else {
    (fileId as RdFileIdOld).protocol
  }
  val path = VirtualFileManager.extractPath(url)
  return createDocumentId(path, protocol)
}

/**
 * Should be used by platform code only
 */
@Deprecated("Use RdFileId when using identity of files")
fun File.createDocumentId(protocol: String = StandardFileSystems.FILE_PROTOCOL): RdFileDocumentId {
  return createDocumentId(path, protocol)
}

/**
 * Should be used by platform code only
 */
@Deprecated("Use RdFileId when using identity of files")
fun createDocumentId(path: String, protocol: String = StandardFileSystems.FILE_PROTOCOL): RdFileDocumentId {
  return RdFileDocumentIdImpl(filePath = path, protocol = protocol)
}

// extracted a weird escaping to one place
fun RdDocumentId.createLogCategory(): String {
  return when (this) {
    is RdFileDocumentId -> PathUtil.getFileName(filePath).replace('.', '·')
    is RdAbstractDocumentId -> descriptor
    else -> toString()
  }
}

fun adjustScrollPosition(editor: @NotNull Editor) {
  val visibleArea = editor.scrollingModel.visibleArea
  val caretY = editor.visualLineToY(editor.caretModel.visualPosition.line)
  if (visibleArea.height > 0 && (caretY + editor.lineHeight <= visibleArea.y || caretY >= visibleArea.y + visibleArea.height)) {
    editor.scrollingModel.scrollToCaret(ScrollType.CENTER)
  }
}