// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ide.document

import com.jetbrains.rd.ide.model.RdDocumentVersion

class DocumentVersionHolderImpl(initialVersion: RdDocumentVersion) : DocumentVersionHolder {
  private var lastIdeaModificationStamp: Long = -1L

  override var previousVersion: RdDocumentVersion = initialVersion
    private set
  override var version: RdDocumentVersion = initialVersion
    private set

  override fun incrementVersion(newIdeaModificationStamp: Long): Boolean {
    if (lastIdeaModificationStamp == newIdeaModificationStamp) {
      // version is already incremented
      return false
    }

    previousVersion = version
    version = version.increment()
    lastIdeaModificationStamp = newIdeaModificationStamp
    return true
  }

  override fun resetVersion(v: RdDocumentVersion, ideaModificationStamp: Long) {
    version = v
    previousVersion = if (v == InitialDocVersion) v else v.decrement()
    lastIdeaModificationStamp = ideaModificationStamp
  }
}