// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ide.document

import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.editor.Document
import com.intellij.openapi.fileEditor.FileDocumentManager
import com.intellij.testFramework.LightVirtualFile
import com.jetbrains.rd.ide.model.RdAbstractDocumentId
import com.jetbrains.rd.ide.model.RdDocumentId
import java.util.*

interface ProtocolDocumentHost {
  companion object {
    private val LOG = logger<ProtocolDocumentHost>()
  }

  fun createId(document: Document): RdDocumentId {
    when(val file = FileDocumentManager.getInstance().getFile(document)) {
      null, is LightVirtualFile -> {
        val descriptor = UUID.randomUUID().toString()
        LOG.debug("creating documentId for in-memory document. file=${file?.name} descriptor=$descriptor")
        return RdAbstractDocumentId(descriptor)
      }
      else -> {
        LOG.trace("registering vfs backed document. file=${file.name} ")
        return file.createDocumentId()
      }
    }
  }
}