package com.jetbrains.rd.ide.editor.markup

import com.intellij.openapi.util.Segment
import java.util.*

/**
 * Created by Alexander.Kirsanov on 9/1/2016.
 */

object HighlighterTextAnnotator {
    fun <TRange : Segment> annotateWithTags(
        text: String, ranges: Iterable<TRange>,
        comparator: Comparator<TRange>,
        rangeTagNamePrinter: ((TRange) -> String?),
        additionalTextProvider: ((TRange) -> String?)
    ): String {
        val sortedHighlighters = ranges.sortedWith(comparator)
        val offsetMap = TreeMap<Int, Int>()
        val result = StringBuilder(text)
        for (range in sortedHighlighters) {
            val tag = rangeTagNamePrinter(range) ?: continue

            val additionalText = additionalTextProvider(range).let { if (!it.isNullOrBlank()) " $it" else "" }

            val start = offsetMap.calculateAbsoluteOffset(range.startOffset, true)
            val startTag = "<$tag$additionalText>"
            result.insert(start, startTag)
            offsetMap.updateOffset(range.startOffset, startTag.length)

            if (range.startOffset != range.endOffset) {
                val end = offsetMap.calculateAbsoluteOffset(range.endOffset, false)
                val endTag = "</$tag>"
                result.insert(end, endTag)
                offsetMap.updateOffset(range.endOffset, endTag.length)
            }
        }
        return result.toString()
    }

    private fun TreeMap<Int, Int>.updateOffset(index: Int, value: Int) {
        val current = this[index] ?: 0
        this[index] = current + value
    }

    // tag type flag is needed for providing proper nesting for tags with the same ranges (e.g. <a><b>IDENT</b></a> not <a><b>IDENT</a></b>)
    @Suppress("NAME_SHADOWING")
    private fun TreeMap<Int, Int>.calculateAbsoluteOffset(currentOffset: Int, isStartTag: Boolean): Int {
        var absoluteOffset = currentOffset
        val currentOffset = if (isStartTag) currentOffset else currentOffset - 1
        for ((offset, delta) in this) {
            if (offset > currentOffset) break
            absoluteOffset += delta
        }
        return absoluteOffset
    }
}