package com.jetbrains.rd.ide.editor.markup

import com.intellij.codeInsight.daemon.impl.HighlightInfo
import com.intellij.openapi.editor.colors.TextAttributesKey
import com.intellij.openapi.editor.markup.RangeHighlighter
import com.jetbrains.rd.platform.util.userData
import com.jetbrains.rd.ide.model.HighlighterModel


fun HighlighterModel.prettyPrint() = "{id=$id; attributeId=$attributeId; ($start, $end)}"

fun RangeHighlighter.prettyPrint(): String {
  val key = textAttributesKey?.externalName?.let { "attributeKey=$it" }
  val presentableName = key ?: "attributeId=$tryGetAttributeId"
  return "{\"$presentableName\"; ($startOffset, $endOffset)}"
}

/**
 * In Rider this is used to show the original id received from R# Backend before it was transformed to fit in IJ theming/conventions
 */
var RangeHighlighter.backendAttributeId by userData<String>()

/**
 * In IJ this is set when there's no attributeKey for a highlighter yet it was transferred through the protocol
 */
var RangeHighlighter.explicitAttributeId by userData<String>()

private val RangeHighlighter.tryGetAttributeId: String?
  get() = explicitAttributeId ?: backendAttributeId

@Deprecated("RangeHighlighter has its own textAttributesId", ReplaceWith("textAttributesKey"))
val RangeHighlighter.attributeKey: TextAttributesKey?
  get() = textAttributesKey

val RangeHighlighter.highlightInfo: HighlightInfo?
  get() = HighlightInfo.fromRangeHighlighter(this)
