// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.lang

import com.intellij.lang.Language
import com.intellij.openapi.components.serviceOrNull
import com.intellij.openapi.fileTypes.FileType
import com.jetbrains.rd.ide.model.RdLanguage
import com.jetbrains.rd.platform.util.getLogger
import org.jetbrains.annotations.TestOnly

fun RdLanguage.toIdeaFileType(): FileType? {
  return this.toIdeaLanguage()?.associatedFileType
}

fun RdLanguage.toIdeaLanguage(): Language? {
  return ProtocolLanguageManager.getInstance()?.toIdeaLanguage(this)
}

fun Language.toRdLanguage(): RdLanguage? {
  return ProtocolLanguageManager.getInstance()?.toRdLanguage(this)
}

fun Language.toRdLanguageOrThrow(): RdLanguage {
  return this.toRdLanguage()
         ?: throw IllegalStateException("There is no rd language corresponding to $this")
}

interface ProtocolLanguageManager {
  companion object {
    val logger = getLogger<ProtocolLanguageManager>()

    @TestOnly
    var customInstance: ProtocolLanguageManager? = null
    fun getInstance() = customInstance ?: serviceOrNull()
  }

  fun toRdLanguage(language: Language): RdLanguage?

  fun toIdeaLanguage(rdLanguage: RdLanguage): Language?

  /**
   * Current relation between IJ and R# languages is one to many
   * @param backendLanguageId unique string corresponding to Language.ID of ReSharper language
   * @param frontendLanguageId unique string corresponding to Language.getId() of IDEA language
   * @param default flag which determines whenever RdLanguage with such backendLanguageId should be return on Language.toRdLanguage() call
   */
  fun addAssociation(backendLanguageId: String, frontendLanguageId: String, default: Boolean)

  fun addAssociation(backendLanguageId: String, frontendLanguageId: String) =
    addAssociation(backendLanguageId, frontendLanguageId, false)
}