package com.jetbrains.rd.platform.codeWithMe

import com.intellij.codeWithMe.ClientId
import com.intellij.codeWithMe.ClientId.Companion.isLocal
import com.intellij.codeWithMe.ClientIdServiceImpl
import com.intellij.ide.plugins.PluginManagerCore
import com.intellij.openapi.Disposable
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.util.Disposer
import com.jetbrains.rd.platform.RdConstants.clientIdContext
import com.jetbrains.rd.platform.util.application
import java.util.concurrent.ConcurrentHashMap

class CodeWithMeClientIdService : ClientIdServiceImpl() {
  companion object {
    private val LOG = logger<CodeWithMeClientIdService>()
  }

  private val disposables = ConcurrentHashMap<ClientId, Disposable>()

  override var clientIdValue: String?
    get() = clientIdContext.value
    set(value) {
      clientIdContext.value = value
    }

  override val checkLongActivity: Boolean
    get() = !PluginManagerCore.isRunningFromSources()

  override fun isValid(clientId: ClientId?): Boolean {
    if (clientId.isLocal || ClientId.defaultLocalId == clientId) return true
    val current = disposables[clientId] ?: return false
    return !Disposer.isDisposed(current)
  }

  override fun toDisposable(clientId: ClientId?): Disposable {
    if (clientId == null || clientId.isLocal || ClientId.defaultLocalId == clientId) return application

    val new = Disposer.newDisposable()
    val current = disposables.putIfAbsent(clientId, new)
    if (current == null) {
      LOG.debug("Adding disposable root for ClientId $clientId")
      Disposer.register(new) {
        LOG.debug("Removing disposable root for ClientId $clientId")
        disposables.remove(clientId)
      }
    }
    return current ?: new
  }
}