package com.jetbrains.rd.platform.codeWithMe.connection

import com.intellij.openapi.components.service
import com.intellij.openapi.diagnostic.Logger
import com.intellij.openapi.project.Project
import com.jetbrains.codeWithMe.model.CWMSessionStatus
import com.jetbrains.codeWithMe.model.GuestTerminationReason
import com.jetbrains.rd.platform.util.idea.LifetimedProjectService
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.Signal
import java.time.Duration

class CwmConnectionStatusService(project: Project): LifetimedProjectService(project) {
  companion object {
    fun getInstance(project: Project): CwmConnectionStatusService = project.service()
    private val LOG = Logger.getInstance(CwmConnectionStatusService::class.java)
  }

  val freeSessionMinutesBeforeWarning = 5

  val sessionStatus = Property<CWMSessionStatus?>(null)
  val freeSessionRemainingTimeMs = Property<Long?>(null)

  val terminated = Signal<GuestTerminationReasonsData>()

  init {
    val lifetime = projectServiceLifetime.createNested()
    sessionStatus.advise(lifetime) {
      LOG.info("sessionStatus: $it")
    }

    freeSessionRemainingTimeMs.advise(lifetime) {
      LOG.info("freeSessionRemainingTimeMs: $it ${it?.let { Duration.ofMillis(it).toMinutes() }}")
    }

    terminated.advise(lifetime) {
      LOG.info("terminated: ${it.reason} \nmessage: ${it.message}}")
    }
  }
}

data class GuestTerminationReasonsData(val reason: GuestTerminationReason, val message: String? = null)
