package com.jetbrains.rd.platform.codeWithMe.control

import com.intellij.openapi.project.Project
import com.intellij.openapi.wm.IdeFrame
import com.intellij.util.ui.JBUI
import com.jetbrains.rd.platform.codeWithMe.control.following.CodeWithMeEditorMarkService
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.SequentialLifetimes
import net.miginfocom.swing.MigLayout
import javax.swing.JComponent
import javax.swing.JPanel
import javax.swing.SwingUtilities

class CWMControl {

  private val lifetimes = SequentialLifetimes(Lifetime.Eternal)
  private val circles = CWMCirclesPane()
  private var project: Project? = null
  private val panel = object : JPanel(MigLayout("ins 0, novisualpadding, gap 0, hidemode 3")) {
    init {
      isOpaque = false
      border = JBUI.Borders.empty()
    }

    override fun addNotify() {
      super.addNotify()
      val frame = SwingUtilities.getWindowAncestor(this)
      val project = (frame as? IdeFrame)?.project
      updateProject(project)
    }

    override fun removeNotify() {
      updateProject(null)
      super.removeNotify()
    }
  }.apply {
    add(circles.view, "gapafter ${JBUI.scale(4)}, gapbefore ${JBUI.scale(3)}")
  }

  private fun updateProject(project: Project?) {
    lifetimes.terminateCurrent()
    hideCircle()
    this.project = project
    if (project == null || project.isDisposed) return

    val userManager = UserManager.getInstance(project)
    val lifetime = lifetimes.next().lifetime
    userManager.users.advise(lifetime) {
      if (userManager.users.size > 0) showCircle(userManager) else hideCircle()
      updateSessionEnabled(userManager.sessionEnabled.value || (!userManager.users.isEmpty() && java.lang.Boolean.getBoolean("cwm.ui.test.mode")))
    }

    userManager.sessionEnabled.advise(lifetime) {
      updateSessionEnabled(it || (!userManager.users.isEmpty() && java.lang.Boolean.getBoolean("cwm.ui.test.mode")))
    }

    CodeWithMeEditorMarkService.getInstance(project)
  }

  val view: JComponent = panel

  private fun updateSessionEnabled(value: Boolean) {
    circles.view.isVisible = value
  }

  private fun showCircle(userManager: UserManager) {
    circles.show(userManager)
  }

  private fun hideCircle() {
    circles.hide()
  }
}