package com.jetbrains.rd.platform.codeWithMe.control

import com.intellij.openapi.editor.colors.EditorColorsManager
import com.intellij.openapi.editor.colors.TextAttributesKey
import com.intellij.openapi.editor.markup.TextAttributes
import com.intellij.ui.JBColor
import com.intellij.util.ui.JBFont
import com.intellij.util.ui.JBUI
import com.intellij.util.ui.UIUtil
import com.jetbrains.codeWithMe.model.RdUserModel
import java.awt.Color
import java.awt.Font
import javax.swing.Icon
import javax.swing.JLabel
import javax.swing.UIManager
import kotlin.math.max

// todo create named colors, move to CodeWithMeColors
object CWMDrawingData {
  const val MAX_CIRCLES_COUNT = 5
  const val MAX_BEFORE_COUNTER = 9
  const val MAX_TOOLTIP_COUNT = 15

  const val CIRCLE_DIAMETER = 16
  const val CIRCLE_HOVER_DIAMETER = 20

  const val STATE_HOVER_THICKNESS = 2
  const val STATE_HOVER_HEIGHT = CIRCLE_HOVER_DIAMETER + (STATE_HOVER_THICKNESS *2)

  val pillAccessEnabled
    get() = JBColor.namedColor("CodeWithMe.AccessEnabled.pillBackground", JBColor(0xE4E4E4, 0x333538))
  val dropdownBorderAccessEnabled
    get() = JBColor.namedColor("CodeWithMe.AccessEnabled.dropdownBorder", pillAccessEnabled)
  val accessDotAccessEnabled
    get() = JBColor.namedColor("CodeWithMe.AccessEnabled.accessDot", JBColor(0x59A869, 0x499C54))
  val accessDotAccessDisabled
    get() = JBColor.namedColor("CodeWithMe.AccessDisabled.accessDot", JBColor(0xDB5860, 0xC75450))
  val avatarHover
    get() = JBUI.CurrentTheme.ActionButton.hoverBackground()
  val avatarForeground
    get() = JBColor.namedColor("CodeWithMe.Avatar.foreground", JBColor(Color(0x1D1D1D), Color(0x1D1D1D)))


  val counterColor: Color
    get() = UIManager.getColor("Label.foreground")

  val toolTipForeground: Color
    get() = JBUI.CurrentTheme.Tooltip.shortcutForeground()

  val iconFont
    get() = JBUI.Fonts.toolbarSmallComboBoxFont()

  val tabIconFont: JBFont
    get() = JBFont.label().deriveFont(UIUtil.getFontSize(UIUtil.FontSize.MINI) - 1)

  val stopForeground = JBColor(Color(0x2946CC), Color(0x8ABBFF))
  val stopBackground = JBColor(Color(0xFFFFFF), Color(0x3C3F41))
  val popupStateBackground = JBColor(Color(0xE8E8E8), Color(0xE8E8E8))
  val followedByColors = arrayListOf<Color>(JBColor(Color(0xA7A7AA), Color(0x686767)), JBColor(Color(0xFFFFFF), Color(0xAFB1B3)))


  fun createLabel(font: Font, name: String) : JLabel {
    return createLabel(font).apply { text = name }
  }

  fun createLabel(font: Font, icon: Icon) : JLabel {
    return createLabel(font).apply { this.icon = icon }
  }

  fun createLabel(font: Font) : JLabel {
    val label = object : JLabel() {
      override fun getFont(): Font {
        return font
      }
    }
    return label
  }

}

val RdUserModel.backgroundColor: Color get() = JBColor{ CodeWithMeColors.getMarkerAttributes(this).backgroundColor ?: Color.GRAY }
val RdUserModel.foregroundColor: Color get() = JBColor{ CodeWithMeColors.getMarkerAttributes(this).foregroundColor ?: Color.BLACK }

object CodeWithMeColors {
  fun getMarkerAttributes(userModel: RdUserModel): TextAttributes {
    val key = getMarkerAttributesKey(userModel)
    return EditorColorsManager.getInstance().globalScheme.getAttributes(key)
  }

  fun getMarkerAttributesKey(userModel: RdUserModel): TextAttributesKey {
    val index = max(userModel.id, 0) % MARKERS.size
    return MARKERS[index]
  }

  fun getSelectionAttributesKey(userModel: RdUserModel): TextAttributesKey {
    val index = max(userModel.id, 0) % SELECTIONS.size
    return SELECTIONS[index]
  }

  fun getFollowedByMultipleIconColors(): Triple<Color, Color, Color> {
    val scheme = EditorColorsManager.getInstance().globalScheme
    val colors = MARKERS.subList(0, 3).map { scheme.getAttributes(it).backgroundColor ?: Color.GRAY }
    return colors.let { Triple(it[0], it[1], it[2]) }
  }

  val USER_1_MARKER = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_1_MARKER")
  val USER_1_SELECTION = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_1_SELECTION")

  val USER_2_MARKER = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_2_MARKER")
  val USER_2_SELECTION = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_2_SELECTION")

  val USER_3_MARKER = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_3_MARKER")
  val USER_3_SELECTION = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_3_SELECTION")

  val USER_4_MARKER = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_4_MARKER")
  val USER_4_SELECTION = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_4_SELECTION")

  val USER_5_MARKER = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_5_MARKER")
  val USER_5_SELECTION = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_5_SELECTION")

  val USER_6_MARKER = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_6_MARKER")
  val USER_6_SELECTION = TextAttributesKey.createTextAttributesKey("CodeWithMe.USER_6_SELECTION")

  private val MARKERS = listOf(
    USER_1_MARKER,
    USER_2_MARKER,
    USER_3_MARKER,
    USER_4_MARKER,
    USER_5_MARKER,
    USER_6_MARKER
  )

  private val SELECTIONS = listOf(
    USER_1_SELECTION,
    USER_2_SELECTION,
    USER_3_SELECTION,
    USER_4_SELECTION,
    USER_5_SELECTION,
    USER_6_SELECTION
  )

  init {
    assert(MARKERS.size == SELECTIONS.size)
  }
}