package com.jetbrains.rd.platform.codeWithMe.control.following

import com.intellij.openapi.project.guessCurrentProject
import com.intellij.ui.HyperlinkLabel
import com.intellij.util.ui.JBUI
import com.jetbrains.codeWithMe.model.RdUserModel
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.control.CWMDrawingData
import com.jetbrains.rd.platform.codeWithMe.control.backgroundColor
import com.jetbrains.rd.platform.codeWithMe.statistics.CodeWithMeLifecycleLogger
import com.jetbrains.rd.platform.codeWithMe.toolbar.CodeWithMeUsersGroup
import net.miginfocom.swing.MigLayout
import javax.swing.JComponent
import javax.swing.JLabel
import javax.swing.JPanel

interface StatePane {
  companion object {
    val layout = MigLayout("novisualpadding, hidemode 3, ins ${JBUI.scale(2)} ${JBUI.scale(6)} ${JBUI.scale(2)} ${JBUI.scale(5)}, gap 0")

    fun createPane(): JPanel = JPanel(layout)
  }

  val view: JComponent
}

class FollowedByPane() : StatePane {

  private val plusLabel = JLabel()
  private val label = JLabel()

  private val panel = StatePane.createPane().apply {
    add(label)
    add(plusLabel)
  }

  var followers: List<RdUserModel>? = null
    set(value) {
      field = value

      followers?.let {
        if (it.isNotEmpty()) {
          val rdUserModel = it[0]
          view.background = rdUserModel.backgroundColor


          if (it.size == 1) {
            label.text = CodeWithMePlatformBundle.message("circle.control.user.state.followed.by.user", rdUserModel.name)
            plusLabel.isVisible = false
            plusLabel.toolTipText = null
          }
          else {
            label.text = CodeWithMePlatformBundle.message("circle.control.user.state.followed.by.users", rdUserModel.name)
            plusLabel.isVisible = true
            plusLabel.text = "+${it.size - 1}"

            var string = ""
            it.filterIndexed { i, _ -> i > 0 && i < CWMDrawingData.MAX_TOOLTIP_COUNT }.map { CodeWithMeUsersGroup.getName(it, escapeHtml = true)}
              .forEach { nm ->
              string += "<div>$nm</div>"
            }
            val additionalText = if(it.size > CWMDrawingData.MAX_TOOLTIP_COUNT) "<div>and ${it.size - CWMDrawingData.MAX_TOOLTIP_COUNT} more...</div>" else ""
            plusLabel.toolTipText = "<html>${string}$additionalText</html>"
          }
        }
      }
    }

  override val view: JComponent = panel
}

class FollowingStoppedPane(action: (RdUserModel) -> Unit) : ActionPane(action, CodeWithMePlatformBundle.message("action.resume")) {
  init {
    label.text = "${CodeWithMePlatformBundle.message("circle.control.user.state.following.stopped")} "
  }

  override fun update() {
    user?.let {
      label.text = "${CodeWithMePlatformBundle.message("circle.control.user.state.following.stopped")} "
    }
  }
}

class FullSyncPane(action: (RdUserModel) -> Unit) : FollowingPane(action, "circle.control.user.state.full.sync.user")

open class FollowingPane(action: (RdUserModel) -> Unit, val messageKey: String = "circle.control.user.state.following.user") : ActionPane(
  action) {
  override fun update() {
    user?.let {
      label.text = "${CodeWithMePlatformBundle.message(messageKey, it.name)} "
    }
  }
}

abstract class ActionPane(action: (RdUserModel) -> Unit,
                          linkMessage: String = CodeWithMePlatformBundle.message("action.stop"),
                          private val messageKey: String = "") : StatePane {
  private val actionLabel by lazy {
    HyperlinkLabel(linkMessage).apply {
      addHyperlinkListener {
        user?.let {
          when (linkMessage) {
            CodeWithMePlatformBundle.message("action.stop") -> CodeWithMeLifecycleLogger.onEditorFollowingLabelStop(guessCurrentProject(view), it)
            CodeWithMePlatformBundle.message("action.resume") -> CodeWithMeLifecycleLogger.onEditorFollowingLabelResume(guessCurrentProject(view), it)
          }
          action(it)
        }
      }
    }
  }

  protected val label = JLabel()
  private val panel = StatePane.createPane().apply {
    add(label)
    add(actionLabel)
  }
  override val view: JComponent = panel

  var user: RdUserModel? = null
    set(value) {
      if (field == value) return
      field = value

      value?.let {
        view.background = value.backgroundColor
      }

      update()
    }

  open fun update(){}
}