package com.jetbrains.rd.platform.codeWithMe.control.icons

import com.intellij.util.ui.JBUI
import java.awt.*
import java.awt.geom.RoundRectangle2D
import javax.swing.Icon
import javax.swing.border.Border

class CircleHoverIcon(var color: Color, private val unscaledDiameter: Int = 20, var unscaledArc: Int = 5) : Icon {
  private val diameter
    get() = JBUI.scale(unscaledDiameter)
  private val arc
    get() = JBUI.scale(unscaledArc).toFloat()

  override fun getIconHeight(): Int {
    return diameter
  }

  override fun paintIcon(c: Component?, g: Graphics, x: Int, y: Int) {
    val g2d = g.create() as Graphics2D

    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
    val cd = diameter.toFloat()
    val area = RoundRectangle2D.Float(x.toFloat(), y.toFloat(), cd, cd, arc, arc)

    g2d.color = color ?: Color.WHITE

    g2d.fill(area)
    g2d.dispose()
  }

  override fun getIconWidth(): Int {
    return diameter
  }
}

class CircleHoveredBorder(var color: Color, private val unscaledDiameter: Int = 20, var unscaledArc: Int = 5): Border {
  private val arc
    get() = JBUI.scale(unscaledArc).toFloat()

  override fun getBorderInsets(c: Component?): Insets = JBUI.emptyInsets()

  override fun isBorderOpaque(): Boolean = false

  override fun paintBorder(c: Component?, g: Graphics, x: Int, y: Int, width: Int, height: Int) {
    val g2d = g.create() as Graphics2D

    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
    val toFloat = height.toFloat()
    val area = RoundRectangle2D.Float(x.toFloat(), y.toFloat(), toFloat, toFloat, arc, arc)

    g2d.color = color ?: Color.WHITE

    g2d.fill(area)
    g2d.dispose()
  }
}