// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.codeWithMe.control.icons

import com.intellij.util.ui.JBUI
import java.awt.*
import java.awt.geom.Ellipse2D
import javax.swing.Icon

class CircleIcon(
  var color: Color? = Color.GRAY,
  private val unscaledDiameter: Int = 16,
  private val unscaledInset: Int = 0
) : Icon {
  private val inset: Int
    get() = JBUI.scale(unscaledInset)

  private val diameter
    get() = JBUI.scale(unscaledDiameter)

  override fun getIconHeight(): Int {
    return diameter + (inset * 2)
  }

  override fun paintIcon(c: Component?, g: Graphics, x: Int, y: Int) {
    val g2d = g.create() as Graphics2D

    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)

    val x1 = x + inset
    val y1 = y + inset

    val cx = x1.toFloat()
    val cy = y1.toFloat()
    val cd = diameter.toFloat()

    val circle = Ellipse2D.Float(cx, cy, cd, cd)
    g2d.color = color ?: Color.GRAY
    g2d.fill(circle)

    g2d.dispose()
  }

  override fun getIconWidth(): Int {
    return iconHeight
  }
}