package com.jetbrains.rd.platform.codeWithMe.control.icons

import com.intellij.util.ui.UIUtilities
import java.awt.*
import java.awt.font.FontRenderContext
import java.awt.font.GlyphVector
import java.awt.geom.Rectangle2D
import javax.swing.Icon
import javax.swing.JComponent

class TextIcon private constructor(
  var text: String,
  var color: Color? = null,
  var fontFunc: (() -> Font)? = null,
  val component: Component? = null,
  val width: (() -> Int)? = null,
  val height: (() -> Int)? = null
) : Icon {

  constructor(
    component: Component,
    text: String,
    color: Color? = null,
    fontFunc: (() -> Font)? = null
  ) : this (text, color, fontFunc, component)

  constructor(
    width: () -> Int,
    height: () -> Int,
    text: String,
    color: Color? = null,
    fontFunc: (() -> Font)? = null
  ) : this (text, color, fontFunc, width = width, height = height)


  override fun getIconHeight(): Int {
    return height?.let { it() } ?: component?.let{it.getFontMetrics(fontFunc?.let { it() } ?: it.font)?.height} ?: 0
  }

  override fun paintIcon(c: Component?, g: Graphics, x: Int, y: Int) {
    val g2d = g.create() as Graphics2D
    g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)

    var textX = x.toFloat()
    var textY = y.toFloat()

    fontFunc?.let {
      g2d.font = it()
    }

    val frc: FontRenderContext = g2d.fontRenderContext
    val gv: GlyphVector = g2d.font.createGlyphVector(frc, text)
    val box: Rectangle2D = gv.visualBounds

    textX += ((iconWidth.toFloat() - box.width.toFloat()) / 2) - box.x.toFloat()
    textY += ((iconHeight.toFloat() - box.height.toFloat()) / 2) - box.y.toFloat()

    color?.let {
      g2d.color = it
    }

    UIUtilities.drawChars(c as JComponent?, g2d, text.toCharArray(), 0, text.length, textX, textY)

    g2d.dispose()
  }

  override fun getIconWidth(): Int {
    return width?.let { it() } ?: component?.let{it.getFontMetrics(fontFunc?.let { it() } ?: it.font)?.stringWidth(text)} ?: 0
  }
}
