package com.jetbrains.rd.platform.codeWithMe.debugger

import com.intellij.openapi.project.Project
import com.intellij.xdebugger.XDebugSession
import com.intellij.xdebugger.XDebugSessionListener
import com.jetbrains.codeWithMe.model.CodeWithMeDebuggerModel
import com.jetbrains.codeWithMe.model.CodeWithMeDebuggerSessionModel
import com.jetbrains.rd.framework.IProtocol
import com.jetbrains.rd.framework.base.RdReactiveBase
import com.jetbrains.rd.util.lifetime.Lifetime

abstract class CommonDebuggerHost(
  val lifetime: Lifetime,
  val protocol: IProtocol,
  val project: Project
) {
  val debuggerModel = CodeWithMeDebuggerModel.create(lifetime, protocol)
  
  fun syncSessionSettingsWithModel(session: XDebugSession, model: CodeWithMeDebuggerSessionModel, initModel: Boolean) {
    syncBreakpointsMuted(session, model, initModel)
  }

  private fun syncBreakpointsMuted(session: XDebugSession, model: CodeWithMeDebuggerSessionModel, initModel: Boolean) {
    if (initModel) {
      model.areBreakpointsMuted.set(session.areBreakpointsMuted())
    }
    
    session.addSessionListener(object : XDebugSessionListener {
      override fun breakpointsMuted(muted: Boolean) {
        model.areBreakpointsMuted.set(muted)
      }
    })
    
    model.areBreakpointsMuted.advise(lifetime) {
      if ((model.areBreakpointsMuted as RdReactiveBase).isLocalChange)
        return@advise
      
      session.setBreakpointMuted(it)
    }
  }
}