package com.jetbrains.rd.platform.codeWithMe.network.quic

import com.jetbrains.rd.platform.codeWithMe.protocol.ConnectionState
import com.jetbrains.rd.platform.codeWithMe.protocol.StreamTransport
import java.nio.ByteBuffer
import java.nio.channels.ByteChannel
import java.nio.channels.GatheringByteChannel
import java.util.concurrent.BlockingDeque
import java.util.concurrent.TimeUnit

abstract class QuicStreamTransport: StreamTransport {
    protected abstract val receivedQueue: BlockingDeque<ByteArray>
    protected abstract val toSendQueue: BlockingDeque<ByteArray>
    protected abstract val wakeupQueue: BlockingDeque<Boolean>

    override val byteChannel: ByteChannel = object : ByteChannel, GatheringByteChannel {
        override fun isOpen(): Boolean = connected.value == ConnectionState.CONNECTED

        override fun write(srcs: Array<out ByteBuffer>, offset: Int, length: Int): Long {
            var totalBytes = 0L
            for (i in offset until offset + length) {
                if (srcs[i].hasRemaining()) {
                    totalBytes += write(srcs[i])
                }
            }
            return totalBytes
        }

        override fun write(srcs: Array<out ByteBuffer>): Long = write(srcs, 0, srcs.size)

        override fun write(src: ByteBuffer): Int {
            val arr = ByteArray(src.remaining())
            src.get(arr)

            toSendQueue.add(arr)
            wakeupQueue.add(true)

            return arr.size
        }

        override fun close() {
        }

        override fun read(dst: ByteBuffer): Int {
            // Assumes one reader
            while (connected.value == ConnectionState.CONNECTED) {
                val buffer = receivedQueue.poll(500, TimeUnit.MILLISECONDS)
                if (buffer == null) {
                    // println("waiting on read")
                    continue
                }

                val available = dst.remaining()

                return if (available >= buffer.size) {
                    dst.put(buffer)
                    buffer.size
                } else {
                    dst.put(buffer.sliceArray(0 until available))
                    receivedQueue.addFirst(buffer.sliceArray(available until buffer.size))
                    available
                }
            }

            return -1
        }
    }
}