package com.jetbrains.rd.platform.codeWithMe.options

import com.intellij.application.options.colors.ColorAndFontOptions
import com.intellij.ide.DataManager
import com.intellij.openapi.options.BoundSearchableConfigurable
import com.intellij.openapi.project.ProjectManager
import com.intellij.openapi.ui.DialogPanel
import com.intellij.ui.components.ActionLink
import com.intellij.ui.components.JBTextField
import com.intellij.ui.layout.*
import com.intellij.util.PlatformUtils
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.users.LobbyUrlSettings
import com.jetbrains.rd.platform.codeWithMe.users.LocalUserSettings
import java.net.URI

class CodeWithMeConfigurable : BoundSearchableConfigurable(CodeWithMePlatformBundle.message("product.name"), ID) {

  companion object {
    const val ID = "CodeWithMe"
  }

  override fun createPanel(): DialogPanel {
    val panel = panel {
      row {
        cell {
          label(CodeWithMePlatformBundle.message("settings.user.name"))
        }

        var userNameField: CellBuilder<JBTextField>? = null
        cell {
          userNameField = textField(LocalUserSettings::userName).apply {
            constraints(growX, pushX)
            if (ProjectManager.getInstance().openProjects.any())
              comment(CodeWithMePlatformBundle.message("settings.user.name.warning"), forComponent = true)
          }
        }

        cell {
          button(CodeWithMePlatformBundle.message("settings.user.name.reset")) {
            userNameField!!.component.text = LocalUserSettings.getDefaultSystemUserName()
          }.sizeGroup("cwm.button")
        }
      }

      if (!PlatformUtils.isCodeWithMeGuest()) {
        val lobbyUrl = LobbyUrlSettings.lobbyUrl
        val isLobbyUrlEditable = object : ComponentPredicate() {
          override fun addListener(listener: (Boolean) -> Unit) {}
          override fun invoke(): Boolean = !lobbyUrl.isSetFromOs
        }

        row {
          cell {
            label(CodeWithMePlatformBundle.message("settings.lobby.url"))
          }

          var lobbyUrlField: CellBuilder<JBTextField>? = null
          cell {
            lobbyUrlField = textField(
              { LobbyUrlSettings.lobbyUrl.uri.toString() },
              { LobbyUrlSettings.setLobbyUrl(URI(it.trim())) }
            ).enableIf(isLobbyUrlEditable)
          }

          cell {
            button(CodeWithMePlatformBundle.message("settings.lobby.url.reset")) {
              lobbyUrlField!!.component.text = LobbyUrlSettings.DEFAULT_LOBBY_URL.toString()
            }.enableIf(isLobbyUrlEditable).sizeGroup("cwm.button")
          }
        }

        if (lobbyUrl.isSetFromOs) {
          row {
            cell(isFullWidth = true) {
              comment(CodeWithMePlatformBundle.message("settings.lobby.url.isSetFromOs"))
            }
          }
          row {
            cell(isFullWidth = true) {
              comment(CodeWithMePlatformBundle.message("settings.lobby.url.osLocation", lobbyUrl.osOriginLocation!!))
            }
          }
        }

        row {
          cell(isFullWidth = true) {
            label(CodeWithMePlatformBundle.message("on.premise.installation.instruction.description"))
            browserLink(CodeWithMePlatformBundle.message("on.premise.installation.instruction"),
                        CodeWithMePlatformBundle.message("on.premise.installation.instruction.link"))
          }
        }
      }
    }
    panel.add(ActionLink(CodeWithMePlatformBundle.message("manage.users.colors.settings.link")
    ) {
      val context = DataManager.getInstance().getDataContext(panel)
      val defaultUserMarker = CodeWithMePlatformBundle.message("color.settings.user1.marker")
      ColorAndFontOptions.selectOrEditColor(context, defaultUserMarker,
                                            CodeWithMeColorsPage::class.java)
    })
    return panel
  }
}