// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.codeWithMe.protocol

sealed class ConnectionState {
  @Suppress("ClassName")
  object NOT_CONNECTED: ConnectionState() {
    override fun toString(): String = "NOT_CONNECTED"
  }

  object CONNECTED: ConnectionState() {
    override fun toString(): String = "CONNECTED"
  }

  object CLOSED: ConnectionState() {
    override fun toString(): String = "CLOSED"
  }

  class DECLINED(val reason: DeclineReason): ConnectionState() {
    override fun toString(): String = "DECLINED${if (reason == DeclineReason.None) "" else "(${reason})"}"
  }

  enum class DeclineReason {
    None,
    UserDeclined,
    GuestLimitReached
  }

  class FAULTED(val throwable: Throwable): ConnectionState() {
    override fun toString(): String = "FAULTED(${throwable.message})"
  }
}