package com.jetbrains.rd.platform.codeWithMe.statistics

import com.intellij.internal.statistic.IdeActivity
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonySession
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonySessionListener
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination

class TelephonySessionStatisticsListener(val project: Project): TelephonySessionListener {

  private val STAT_GROUP_ID = "cwm.telephony.devices"

  var cameraEnabledActivity: IdeActivity? = null
  var microphoneEnabledActivity: IdeActivity? = null
  var voiceCallJoinedActivity: IdeActivity? = null

  override fun telephonySessionCreated(lifetime: Lifetime, session: TelephonySession) {
    session.cameraEnabled.view(lifetime) { cameraEnabledLifetime, cameraEnabled ->
      if (cameraEnabled) {
        cameraEnabledActivity = IdeActivity(project, STAT_GROUP_ID, "cameraEnabled").started()
        cameraEnabledLifetime.onTermination {
          cameraEnabledActivity?.finished()
          cameraEnabledActivity = null
        }
      }
    }
    session.microphoneEnabled.view(lifetime) { microphoneEnabledLifetime, microphoneEnabled ->
      if (microphoneEnabled) {
        microphoneEnabledActivity = IdeActivity(project, STAT_GROUP_ID, "microphoneEnabled").started()
        microphoneEnabledLifetime.onTermination {
          microphoneEnabledActivity?.finished()
          microphoneEnabledActivity = null
        }
      }
    }
    session.conferenceJoined.view(lifetime) { conferenceJoinedLifetime, conferenceJoined ->
      if (conferenceJoined) {
        voiceCallJoinedActivity = IdeActivity(project, STAT_GROUP_ID, "voiceCallJoined").started()
        conferenceJoinedLifetime.onTermination {
          voiceCallJoinedActivity?.finished()
          cameraEnabledActivity?.finished()
          cameraEnabledActivity = null
          microphoneEnabledActivity?.finished()
          microphoneEnabledActivity = null
        }
      }
    }
  }
}