package com.jetbrains.rd.platform.codeWithMe.statistics

import com.intellij.internal.statistic.eventLog.EventLogGroup
import com.intellij.internal.statistic.eventLog.events.EventFields
import com.intellij.internal.statistic.service.fus.collectors.CounterUsagesCollector
import com.intellij.openapi.project.Project

object TelephonyStatisticsCollector: CounterUsagesCollector() {
  private const val STAT_GROUP_ID = "cwm.telephony"

  private val EVENT_GROUP = EventLogGroup(STAT_GROUP_ID, 1)

  private val ENABLED_FIELD = EventFields.Boolean("telephonyEnabled")

  // todo: somehow get traffic path (direct, turn, jvb)
  private val CWM_SESSION_STARTED = EVENT_GROUP.registerEvent("cwmSessionStarted", ENABLED_FIELD)
  private val CWM_SESSION_ENDED = EVENT_GROUP.registerEvent("cwmSessionEnded", ENABLED_FIELD)
  private val TELEPHONY_ENABLED = EVENT_GROUP.registerEvent("enabledFromAction", ENABLED_FIELD)

  override fun getGroup(): EventLogGroup = EVENT_GROUP

  fun enabledFromAction(project: Project?, enabled: Boolean) {
    TELEPHONY_ENABLED.log(project, enabled)
  }

  fun cwmSessionStarted(project: Project?, telephonyEnabled: Boolean) {
    CWM_SESSION_STARTED.log(project, telephonyEnabled)
  }

  fun cwmSessionEnded(project: Project?, telephonyEnabled: Boolean) {
    CWM_SESSION_ENDED.log(project, telephonyEnabled)
  }
}