package com.jetbrains.rd.platform.codeWithMe.tabection

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.PlatformDataKeys
import com.intellij.openapi.fileEditor.FileEditorManager
import com.intellij.openapi.fileEditor.TextEditor
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.project.Project
import com.intellij.openapi.vfs.VirtualFile
import com.jetbrains.rd.platform.codeWithMe.editors.CodeWithMeEditorService
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.platform.codeWithMe.users.clientId
import com.jetbrains.rd.platform.util.project
import com.jetbrains.codeWithMe.model.RdUserModel

abstract class CWMCircleAction() : AnAction(), DumbAware {
  protected fun getList(e: AnActionEvent): List<RdUserModel> {
    e.dataContext.project?.let { project ->
      e.getData(PlatformDataKeys.VIRTUAL_FILE)?.let { file ->
        val userManager = UserManager.getInstance(project)
        return getList(project, userManager, file, e)
      }
    }
    return emptyList()
  }

  protected fun getList(project: Project, userManager: UserManager, file: VirtualFile, e: AnActionEvent): List<RdUserModel> {
    val editorService = CodeWithMeEditorService.getInstance(project)
    val editors = FileEditorManager.getInstance(project).getEditors(file).filterIsInstance<TextEditor>().map { it.editor }
    val activeClients = editors.flatMap { editorService.getActiveClients(it) }.toSet()

    return userManager.users.filter { activeClients.contains(it.clientId) }
  }
}