package com.jetbrains.rd.platform.codeWithMe.telephony

import com.intellij.openapi.components.service
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.viewNotNull

abstract class TelephonyManager(protected val project: Project) {
  companion object {
    private val LOG = logger<TelephonyManager>()
    fun getInstance(project: Project) = project.service<TelephonyManager>()
  }

  val telephonySession = Property<TelephonySession?>(null)

  init {
    telephonySession.viewNotNull(project.lifetime) { sessionLifetime, session ->
      project.messageBus.syncPublisher(TelephonySessionListener.TOPIC).telephonySessionCreated(sessionLifetime, session)
    }
  }

  fun getSession(): TelephonySession? {
    return telephonySession.value
  }

  protected abstract fun getJitsiRoomUrl(): String?

  open fun createSession(lifetime: Lifetime): TelephonySession? {
    val jitsiRoomUrl = getJitsiRoomUrl()
    try {
      val newSession = jitsiRoomUrl?.let { TelephonySession(lifetime, project, it) }
      telephonySession.set(newSession)
    } catch (e: IllegalStateException) {
      LOG.error(e)
      return null
    }

    lifetime.onTerminationIfAlive {
      telephonySession.set(null)
    }

    return telephonySession.value
  }
}