package com.jetbrains.rd.platform.codeWithMe.tests.impl

import com.intellij.openapi.diagnostic.Logger
import com.intellij.util.ExceptionUtil
import com.intellij.util.ui.UIUtil
import com.jetbrains.codeWithMe.model.tests.RdTestSession
import com.jetbrains.codeWithMe.model.tests.RdTestSessionException
import org.apache.log4j.Level

internal class AgentTestLogger(
  private val logger: Logger,
  private val factory: AgentTestLoggerFactory
): Logger() {
  override fun isDebugEnabled(): Boolean {
    return logger.isDebugEnabled
  }

  override fun debug(message: String) {
    logger.debug(message)
  }

  override fun debug(t: Throwable?) {
    logger.debug(t)
  }

  override fun debug(message: String?, t: Throwable?) {
    logger.debug(message, t)
  }

  override fun info(message: String?) {
    logger.info(message)
  }

  override fun info(message: String?, t: Throwable?) {
    logger.info(message, t)
  }

  override fun warn(message: String?, t: Throwable?) {
    logger.warn(message, t)
  }

  override fun setLevel(level: Level) {
    logger.setLevel(level)
  }

  override fun error(message: String?, t: Throwable?, vararg details: String) {
    logger.error(message, t, *details)
    val testSession = factory.testSession.get()
    if (testSession == null) return
    sendToTestRunner(testSession,message + details.joinToString("\n"), t)
  }

  private fun sendToTestRunner(session: RdTestSession, msg: String, t: Throwable?) {
    val type = t?.javaClass?.typeName ?: "<LOG_ERROR>"
    val stackTrace = t?.let { ExceptionUtil.getThrowableText(it) } ?: ""
    val message = if (stackTrace.isEmpty()) msg else ""
    val exception = RdTestSessionException(type, message, stackTrace)
    UIUtil.invokeLaterIfNeeded {
      session.sendException.fire(exception)
    }
  }
}