package com.jetbrains.rd.platform.codeWithMe.toolWindow

import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.SimpleToolWindowPanel
import com.intellij.openapi.wm.ToolWindow
import com.intellij.openapi.wm.ToolWindowFactory
import com.intellij.openapi.wm.ex.ToolWindowManagerListener
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonyManager

class CodeWithMeToolWindowFactory : ToolWindowFactory, DumbAware {
  companion object {
    const val codeWithMeToolWindowID = "Code With Me"
  }

  class CodeWithMeToolWindowListener: ToolWindowManagerListener {
    override fun toolWindowShown(toolWindow: ToolWindow) {
      if (toolWindow.id == codeWithMeToolWindowID) {
        // It looks strange, but it works
        // when tool window is created - it's created with (0,0) size
        // and when the tool window shown this component is not notified with resize event
        toolWindow.component.size = toolWindow.component.size
      }
    }
  }

  override fun createToolWindowContent(project: Project, toolWindow: ToolWindow) {
    toolWindow.title = "Code With Me"
    val contentManager = toolWindow.contentManager

    val telephonyManager = TelephonyManager.getInstance(project)
    val component = telephonyManager.getSession()?.getComponent()
    if (component != null) {
      val panel = SimpleToolWindowPanel(true, true).apply {
        setContent(component)
      }
      val content = contentManager.factory.createContent(panel, "Chat", true)
      content.isCloseable = false
      contentManager.addContent(content)

      // It looks strange, but it works
      // when tool window is created - it's created with (0,0) size
      // and when the tool window shown this component is not notified with resize event
      toolWindow.component.size = toolWindow.component.size
    }
  }

  override fun shouldBeAvailable(project: Project): Boolean {
    return super.shouldBeAvailable(project)
  }

  override fun isApplicable(project: Project): Boolean {
    return super.isApplicable(project)
  }
}