package com.jetbrains.rd.platform.codeWithMe.toolWindow

import com.intellij.openapi.project.Project
import com.intellij.openapi.wm.RegisterToolWindowTask
import com.intellij.openapi.wm.ToolWindowAnchor
import com.intellij.openapi.wm.ToolWindowManager
import com.intellij.serviceContainer.AlreadyDisposedException
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonySession
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonySessionListener
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.lifetime.Lifetime


class TelephonyToolWindowSessionListener (val project: Project): TelephonySessionListener {
  private val toolWindowTask = RegisterToolWindowTask(CodeWithMeToolWindowFactory.codeWithMeToolWindowID, ToolWindowAnchor.RIGHT, null, true, true,
                                                      true, true, CodeWithMeToolWindowFactory(), null,
                                                      null)

  override fun telephonySessionCreated(lifetime: Lifetime, session: TelephonySession) {
    application.invokeLaterOnWriteThread {
      lifetime.executeIfAlive {
        ToolWindowManager.getInstance(project).registerToolWindow(toolWindowTask)
        ToolWindowManager.getInstance(project).getToolWindow(toolWindowTask.id)?.hide()

        lifetime.onTerminationIfAlive {
          application.invokeLaterOnWriteThread {
            try {
              ToolWindowManager.getInstance(project).getToolWindow(toolWindowTask.id)?.remove()
            } catch (e: AlreadyDisposedException) {}
          }
        }
      }
    }
  }
}