package com.jetbrains.rd.platform.codeWithMe.toolWindow

import com.intellij.ide.util.PropertiesComponent
import com.intellij.openapi.project.Project
import com.intellij.openapi.wm.ToolWindow
import com.intellij.openapi.wm.ex.ToolWindowEx
import com.intellij.openapi.wm.impl.InternalDecoratorImpl
import com.intellij.ui.EditorNotificationPanel
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import javax.swing.JComponent

abstract class ToolwindowMirrorModeTracker(val project: Project) {
  companion object {
    const val NOTIFICATION_HIDDEN_PROPERTY: String = "cwm.toolwindow.mirror.mode.notification.hidden"
  }

  abstract fun updateMirroredWindows()

  protected fun updateWindowNotification(toolWindow: ToolWindow, isSharedAccess: Boolean) {
    if (toolWindow.contentManagerIfCreated == null) return
    val decorator = (toolWindow as? ToolWindowEx)?.decorator as? InternalDecoratorImpl ?: return

    val shouldShowNotification = isSharedAccess && !PropertiesComponent.getInstance().getBoolean(NOTIFICATION_HIDDEN_PROPERTY)
    if (shouldShowNotification) {
      if (decorator.headerComponent == null) {
        val notification = createMirrorModeNotificationContentPanel()
        decorator.headerComponent = notification
      }
    }
    else {
      if (decorator.headerComponent is MyEditorNotificationPanel) {
        decorator.headerComponent = null
      }
    }
  }

  private fun createMirrorModeNotificationContentPanel(): JComponent {
    val notification = MyEditorNotificationPanel()
    notification.text = CodeWithMePlatformBundle.message("toolwindow.mirrored.mode.notification")
    notification.toolTipText = CodeWithMePlatformBundle.message("toolwindow.mirrored.mode.notification")
    notification.createActionLabel(CodeWithMePlatformBundle.message("toolwindow.mirrored.mode.notification.hide.link")) {
      PropertiesComponent.getInstance().setValue(NOTIFICATION_HIDDEN_PROPERTY, true)
      updateMirroredWindows()
    }
    return notification
  }

  private class MyEditorNotificationPanel : EditorNotificationPanel()
}
