package com.jetbrains.rd.platform.codeWithMe.toolbar

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.actionSystem.impl.ActionToolbarImpl
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonyManager
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.lifetime

class CodeWithMeToolbarUpdater(project: Project) {
  
  init {
    application.invokeLater {
      if (project.isDisposed) return@invokeLater

      val lifetime = project.lifetime
      UserManager.getInstance(project).users.advise(lifetime) {
        update()
      }
      UserManager.getInstance(project).lead.advise(lifetime) {
        update()
      }
      UserManager.getInstance(project).followers.advise(lifetime) {
        update()
      }
      UserManager.getInstance(project).sessionEnabled.advise(lifetime) {
        update()
      }
      TelephonyManager.getInstance(project).telephonySession.advise(lifetime) {
        it?.conferenceJoined?.advise(lifetime) {
          application.invokeLater {
            update()
          }
        }
        application.invokeLater {
          update()
        }
      }
    }
  }

  private fun update() {
    // always update locally. Otherwise 'HelpTooltip' could use 'BackendBePopupFactory'
    ClientId.withClientId(ClientId.localId) {
      ActionToolbarImpl.updateAllToolbarsImmediately()
    }
  }
}