package com.jetbrains.rd.platform.codeWithMe.toolbar.combobox

import com.intellij.icons.AllIcons
import com.intellij.openapi.actionSystem.CustomShortcutSet
import com.intellij.openapi.keymap.KeymapUtil
import com.intellij.openapi.project.Project
import com.intellij.ui.components.IconLabelButton
import com.intellij.util.ui.JBUI
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.util.lifetime.Lifetime
import net.miginfocom.swing.MigLayout
import java.awt.BorderLayout
import javax.swing.BorderFactory
import javax.swing.JComponent
import javax.swing.JLabel
import javax.swing.JPanel

class CWMComboboxHeader(val project: Project, val lifetime: Lifetime, val close: () -> Unit, onClickHandler: (JComponent) -> Unit) {
  private val enabledLabel = JLabel("  ${CodeWithMePlatformBundle.message("product.status.access.enabled")} ").apply {
    isEnabled = false
  }

  private val disabledLabel = JLabel("  ${CodeWithMePlatformBundle.message("product.status.no.access")} ").apply {
    isEnabled = false
  }
  private val circlePanel = JPanel(BorderLayout()).apply {
    isOpaque = false
  }

  private val panel = JPanel(
    MigLayout("ins 0 ${JBUI.scale(20)} 0 ${JBUI.scale(20)}, gap 0, novisualpadding, center, fillx, hidemode 3 ", "", "[min!]")).apply {
    val shortcutSet: CustomShortcutSet = CustomShortcutSet.fromString("control shift Y")
    val shortcutText = KeymapUtil.getShortcutText(shortcutSet.shortcuts.first())
    add(JLabel(CodeWithMePlatformBundle.message("product.name") + " (" + shortcutText + ")"))
    add(disabledLabel)
    add(enabledLabel)
    add(circlePanel)

    isOpaque = false
  }

  private val mainPanel = JPanel(MigLayout("ins ${JBUI.scale(2)} ${JBUI.scale(4)} ${JBUI.scale(2)} ${JBUI.scale(4)}, novisualpadding, center, fillx", "push[]push[min!]", "[min!]")).apply {
    add(panel)

    add(IconLabelButton(AllIcons.Actions.More) {
      onClickHandler(view)
    })
    isOpaque = true
    background = JBUI.CurrentTheme.Popup.headerBackground(true)
  }
  val view: JComponent = mainPanel

  init {
    val userManager = UserManager.getInstance(project)

    userManager.sessionEnabled.advise(lifetime) {
      updateState(userManager)
    }

    userManager.users.advise(lifetime) {
      updateState(userManager)
    }

    updateState(userManager)
  }

  private fun updateState(userManager: UserManager) {
    val enabled = (!userManager.users.isEmpty() && java.lang.Boolean.getBoolean("cwm.ui.test.mode")) || userManager.sessionEnabled.value
    enabledLabel.isVisible = enabled
    disabledLabel.isVisible = !enabled
    circlePanel.removeAll()

    circlePanel.border = BorderFactory.createEmptyBorder(AllIcons.CodeWithMe.CwmAccessDotOn.iconHeight / 5, 0, 0, 0)
    if (enabled) {
      circlePanel.add(JLabel(AllIcons.CodeWithMe.CwmAccessDotOn))
    }
    else {
      circlePanel.add(JLabel(AllIcons.CodeWithMe.CwmAccessDotOff))
    }

  }
}