package com.jetbrains.rd.platform.codeWithMe.toolbar.combobox

import com.intellij.ide.BrowserUtil
import com.intellij.ide.DataManager
import com.intellij.openapi.actionSystem.*
import com.intellij.openapi.rd.createLifetime
import com.intellij.openapi.rd.defineNestedLifetime
import com.intellij.openapi.ui.popup.ListPopupStep
import com.intellij.openapi.util.Condition
import com.intellij.openapi.util.NlsSafe
import com.intellij.ui.ColorUtil
import com.intellij.ui.popup.StateActionGroupPopup
import com.intellij.ui.popup.list.ListPopupImpl
import com.intellij.util.ui.JBUI
import com.intellij.util.ui.UIUtil
import com.jetbrains.codeWithMe.model.CWMSessionStatus
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.connection.CwmConnectionStatusService
import com.jetbrains.rd.platform.codeWithMe.toolbar.RdUserActionGroup
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.platform.codeWithMe.users.isHost
import net.miginfocom.swing.MigLayout
import java.time.Duration
import java.util.concurrent.TimeUnit
import javax.swing.JButton
import javax.swing.JComponent
import javax.swing.JLabel
import javax.swing.JPanel
import kotlin.math.abs

class CWMPopup(group: DefaultActionGroup,
               val dataContext: DataContext,
               private val shouldShowDisabledActions: Boolean,
               onDispose: Runnable,
               maxRows: Int,
               autoSelection: Boolean,
               private val preselectCondition: Condition<AnAction>?) : StateActionGroupPopup("", group, dataContext, false, true,
                                                                                             shouldShowDisabledActions, false,
                                                                                             onDispose, maxRows, preselectCondition,
                                                                                             null, autoSelection, { action ->
                                                                                       if (action !is RdUserActionGroup) null
                                                                                       else action.state
                                                                                     }) {
  private val statusService = CwmConnectionStatusService.getInstance(project)
  private val userManager = UserManager.getInstance(project)

  @NlsSafe
  fun formatDuration(duration: Duration): @NlsSafe String {
    val seconds: Long = duration.seconds
    val absSeconds = abs(seconds)
    val color = "#${ColorUtil.toHex(UIUtil.getLabelDisabledForeground())}"

    val hours = absSeconds / TimeUnit.HOURS.toSeconds(1)
    val minutes = absSeconds % TimeUnit.HOURS.toSeconds(1) / TimeUnit.MINUTES.toSeconds(1)

    var timer = String.format(
      "%02d:%02d",
      hours,
      minutes)
    if(minutes <= statusService.freeSessionMinutesBeforeWarning) {
      timer = "<font color=#${ColorUtil.toHex(UIUtil.getErrorForeground())}>${timer}</font>"
    }

    val positive = CodeWithMePlatformBundle.message("license.free.time.left", timer, color)
    return if (seconds < 0) "-$positive" else positive
  }

  init {
    val context = DataManager.getInstance().getDataContext(PlatformDataKeys.CONTEXT_COMPONENT.getData(dataContext))
    setHeaderComponent(CWMComboboxHeader(project,
                                         this.createLifetime(), {
                                           cancel()
                                         }) { component ->
      ActionManager.getInstance().getAction("CodeWithMeHeaderMenuGroup")?.let {
        if (it is ActionGroup) {
          val step = createStep(null, it, context, false, true, shouldShowDisabledActions, false,
                                preselectCondition, null, null,
                                autoSelection)
          showHeaderSubMenu(step, component)
        }
      }

    }.view)
    speedSearch.setEnabled(false)
    val description = JLabel().apply {
      border = JBUI.Borders.empty(1, 0)
    }

    val buyButton = JButton(CodeWithMePlatformBundle.message("license.buy")).apply {
      addActionListener {
        BrowserUtil.browse(CodeWithMePlatformBundle.message("license.buy.link"))
      }
      // putClientProperty("gotItButton", true)
    }

    val licenseFooter = JPanel(MigLayout("ins 2 4 2 4, fillx, gap 0", "[]push[]", "[min!]")).apply {
      add(description)
      if (userManager.me.value?.isHost == true) {
        add(buyButton)
      }
      border = JBUI.Borders.empty()
      isOpaque = true
      // background = JBUI.CurrentTheme.Popup.headerBackground(true)
    }

    setFooterComponent(licenseFooter)

    val lifetime = defineNestedLifetime()

    fun updateLicenseFooter() {
      val status = statusService.sessionStatus.value

      licenseFooter.isVisible = when (status) {
        CWMSessionStatus.FREE_SESSION,
        CWMSessionStatus.FREE_SESSION_STOPPED -> {
          buyButton.text= CodeWithMePlatformBundle.message("license.buy")

          userManager.sessionEnabled.value
        }
        CWMSessionStatus.EXPIRED -> {
          /*
          buyButton.text= CodeWithMePlatformBundle.message("license.upgrade")
          description.text = CodeWithMePlatformBundle.message("license.expired", "#${ColorUtil.toHex(UIUtil.getErrorForeground())}")
          */
          false
        }
        else -> {
          false
        }
      }
      UIUtil.getWindow(licenseFooter)?.pack()
    }

    fun getTimerText(): @NlsSafe String {
      return statusService.freeSessionRemainingTimeMs.value?.let {  formatDuration(Duration.ofMillis(it))} ?: ""
    }

    userManager.sessionEnabled.view(lifetime) { lt, _ ->
      updateLicenseFooter()
    }

    statusService.sessionStatus.view(lifetime) { lt, status ->
      updateLicenseFooter()

      description.text = when (status) {
        CWMSessionStatus.FREE_SESSION,
        CWMSessionStatus.FREE_SESSION_STOPPED -> {
          statusService.freeSessionRemainingTimeMs.advise(lt) {
            description.text = getTimerText()
          }

          getTimerText()
        }
        CWMSessionStatus.EXPIRED -> {
          CodeWithMePlatformBundle.message("license.expired", "#${ColorUtil.toHex(UIUtil.getErrorForeground())}")
        }
        else -> {
          ""
        }
      }
    }

  }

  private fun showHeaderSubMenu(nextStep: ListPopupStep<*>, component: JComponent) {
    myChild = createPopup(this, nextStep, null)
    if (myChild is ListPopupImpl) {
      for (listener in list.listSelectionListeners) {
        (myChild as ListPopupImpl).addListSelectionListener(listener)
      }
    }
    val y = component.locationOnScreen.y
    val container = content
    myChild.show(container, container.locationOnScreen.x + container.width + STEP_X_PADDING, y, true)
  }
}
