package com.jetbrains.rd.platform.codeWithMe.ui

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.openapi.util.NlsContexts
import com.intellij.ui.BrowserHyperlinkListener
import com.intellij.util.ui.JBUI
import com.jetbrains.rd.platform.RdConstants
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.util.application
import net.miginfocom.swing.MigLayout
import org.jetbrains.annotations.Nls
import java.util.concurrent.CompletableFuture
import javax.swing.*
import javax.swing.text.html.HTMLEditorKit

open class CWMBaseAcceptUserDialog(protected val project: Project,
                                   protected val userName: String,
                                   protected val clientId: ClientId) : CWMConnectionDialog() {

  val result = CompletableFuture<Boolean>()
  private var wasInitialized = false

  init {
    title = CodeWithMePlatformBundle.message("connection.dialog.title")
    isModal = false
    setUndecorated(false)
  }

  // suppress "leaking this" in initializer
  final override fun setUndecorated(undecorated: Boolean) = super.setUndecorated(undecorated)
  final override fun init() = super.init()

  override fun show() {
    application.assertIsDispatchThread()
    if (!wasInitialized) {
      wasInitialized = true
      init()
    }
    super.show()
  }

  protected open fun onClose() {}
  protected open fun JPanel.addExtraContent() {}

  override fun doCancelAction() {
    onClose()
    result.complete(false)
    super.doCancelAction()
  }

  override fun doOKAction() {
    onClose()
    result.complete(true)
    super.doOKAction()
  }

  override fun getVerifyMessage(): @NlsContexts.Label String {
    return CodeWithMePlatformBundle.message("connection.message.verify.the.code")
  }

  override fun getOKAction(): Action {
    return super.getOKAction().apply {
      putValue(Action.NAME, CodeWithMePlatformBundle.message("notification.access.accept"))
    }
  }

  override fun getCancelAction(): Action {
    return super.getCancelAction().apply {
      putValue(Action.NAME, CodeWithMePlatformBundle.message("notification.access.decline"))
    }
  }

  override fun createMiddlePane(): JComponent {
    return JPanel(MigLayout("fillx, ins 0, novisualpadding, gap 10, flowy", "", "")).apply {
      add(CWMDefaultDialog.multiLineLabelPane(CodeWithMePlatformBundle.message("connection.dialog.message", userName, project.name.replace(RdConstants.notAllowedNameCharactersRegex, "_"))), "growx")
      addExtraContent()
    }
  }
}