package com.jetbrains.rd.platform.codeWithMe.ui

import com.intellij.icons.AllIcons
import com.intellij.openapi.application.ApplicationManager
import com.intellij.openapi.ide.CopyPasteManager
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.openapi.util.NlsContexts
import com.intellij.ui.components.IconLabelButton
import com.intellij.util.ui.JBFont
import com.intellij.util.ui.JBUI
import com.intellij.util.ui.UIUtil
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import icons.CodeWithMeCommonIcons
import net.miginfocom.swing.MigLayout
import java.awt.Font
import java.awt.datatransfer.StringSelection
import javax.swing.JComponent
import javax.swing.JLabel
import javax.swing.JPanel

abstract class CWMConnectionDialog : DialogWrapper(null)  {
  private var currentFingerprint: String = "..."
  private var fingerprintLabel: JLabel? = null

  fun setFingerprintString(fingerprint: String) {
    currentFingerprint = fingerprint
    fingerprintLabel?.text = currentFingerprintMessage()
  }

  private val logoPane = JPanel(MigLayout("ins 0, novisualpadding, gap 0")).apply {
    add(JLabel(CodeWithMeCommonIcons.CodeWithMeGuest_64))
    border = JBUI.Borders.empty(3, 8, 0, 18)
  }

  override fun createCenterPanel(): JComponent {
    val rightPanel = createRightPane()

    return JPanel(MigLayout("fillx, ins 10 0 0 0, novisualpadding, gap 0, w pref", "[pref!][grow]")).apply {
      add(logoPane, "ay top")
      add(rightPanel, "grow")
    }
  }

  @NlsContexts.Label
  private fun currentFingerprintMessage() = CodeWithMePlatformBundle.message("connection.dialog.message.security.code", currentFingerprint)

  abstract fun getVerifyMessage(): @NlsContexts.Label String

  abstract fun createMiddlePane(): JComponent

  protected open fun createRightPane(): JComponent {
    return JPanel(MigLayout("ins 0 0 0 0, gapy 3, novisualpadding, flowy, hidemode 3")).apply {
      add(createMiddlePane(), "pushx")
      add(JPanel(MigLayout("ins 0, gap 5, novisualpadding")).apply {
        val fpLabel = JLabel(currentFingerprintMessage())
        fingerprintLabel = fpLabel
        add(fpLabel)
        add(IconLabelButton(AllIcons.Actions.Copy) {
          CopyPasteManager.getInstance().setContents(
            StringSelection(CodeWithMePlatformBundle.message("connection.dialog.clipboard.message", currentFingerprint)))
        }.apply {
          toolTipText = CodeWithMePlatformBundle.message("connection.dialog.message.copy.security.code.tooltip")
        })
      }, "gaptop 11")
      add(object : JLabel(getVerifyMessage()) {
        override fun getFont(): Font {
          return JBFont.label().deriveFont(UIUtil.getFontSize(UIUtil.FontSize.NORMAL) - 2)
        }
      }.apply {
        foreground = UIUtil.getContextHelpForeground()
      }, "growx")

      border = JBUI.Borders.empty(0, 0, 10, 10)
    }
  }

  init {
    isModal = false

  }

  override fun show() {
    if (ApplicationManager.getApplication().isHeadlessEnvironment) return
    super.show()
  }
}