package com.jetbrains.rd.platform.codeWithMe.vcs

import com.intellij.diff.tools.util.base.HighlightPolicy
import com.intellij.diff.tools.util.base.IgnorePolicy
import com.intellij.diff.tools.util.base.TextDiffSettingsHolder
import com.jetbrains.rd.ide.model.DiffHighlightPolicy
import com.jetbrains.rd.ide.model.DiffIgnorePolicy
import com.jetbrains.rd.ide.model.TextDiffSettings

fun copySettingsFromModel(model: TextDiffSettings, copyTo: TextDiffSettingsHolder.TextDiffSettings) {
  copyTo.highlightPolicy = model.highlightPolicy.toPolicy()
  copyTo.ignorePolicy = model.ignorePolicy.toPolicy()
  copyTo.contextRange = model.foldingContextRange
  copyTo.isExpandByDefault = model.foldingExpandByDefault
}

fun TextDiffSettingsHolder.TextDiffSettings.toRdModel(): TextDiffSettings {
  return TextDiffSettings(highlightPolicy.toRdModel(), ignorePolicy.toRdModel(), contextRange, isExpandByDefault)
}

fun IgnorePolicy.toRdModel(): DiffIgnorePolicy {
  return when (this) {
    IgnorePolicy.DEFAULT -> DiffIgnorePolicy.DEFAULT
    IgnorePolicy.TRIM_WHITESPACES -> DiffIgnorePolicy.TRIM_WHITESPACES
    IgnorePolicy.IGNORE_WHITESPACES -> DiffIgnorePolicy.IGNORE_WHITESPACES
    else -> DiffIgnorePolicy.DEFAULT
  }
}

fun HighlightPolicy.toRdModel(): DiffHighlightPolicy {
  return when (this) {
    HighlightPolicy.BY_LINE -> DiffHighlightPolicy.BY_LINE
    HighlightPolicy.BY_WORD -> DiffHighlightPolicy.BY_WORD
    HighlightPolicy.BY_WORD_SPLIT -> DiffHighlightPolicy.BY_WORD_SPLIT
    HighlightPolicy.BY_CHAR -> DiffHighlightPolicy.BY_CHAR
    HighlightPolicy.DO_NOT_HIGHLIGHT -> DiffHighlightPolicy.DO_NOT_HIGHLIGHT
  }
}

fun DiffIgnorePolicy.toPolicy(): IgnorePolicy {
  return when (this) {
    DiffIgnorePolicy.DEFAULT -> IgnorePolicy.DEFAULT
    DiffIgnorePolicy.TRIM_WHITESPACES -> IgnorePolicy.TRIM_WHITESPACES
    DiffIgnorePolicy.IGNORE_WHITESPACES -> IgnorePolicy.IGNORE_WHITESPACES
  }
}

fun DiffHighlightPolicy.toPolicy(): HighlightPolicy {
  return when (this) {
    DiffHighlightPolicy.BY_LINE -> HighlightPolicy.BY_LINE
    DiffHighlightPolicy.BY_WORD -> HighlightPolicy.BY_WORD
    DiffHighlightPolicy.BY_WORD_SPLIT -> HighlightPolicy.BY_WORD_SPLIT
    DiffHighlightPolicy.BY_CHAR -> HighlightPolicy.BY_CHAR
    DiffHighlightPolicy.DO_NOT_HIGHLIGHT -> HighlightPolicy.DO_NOT_HIGHLIGHT
  }
}
