// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.diagnostics

import com.intellij.CommonBundle
import com.intellij.lang.LangBundle
import com.intellij.notification.*
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.util.text.HtmlBuilder
import com.intellij.util.concurrency.EdtExecutorService
import com.jetbrains.rd.RdPlatformBundle
import java.util.concurrent.CompletableFuture
import java.util.function.Consumer
import javax.swing.event.HyperlinkEvent

private val GROUP = NotificationGroup("TraceScenariosEnabledNotification", NotificationDisplayType.BALLOON, false)

class LogTraceScenariosAction : AnAction(), DumbAware {
  override fun actionPerformed(e: AnActionEvent) {
    e.project.let { LogTraceScenariosDialog(it).show() }
  }

  init {
    CompletableFuture.runAsync {
      LogTraceScenariosRegistry.loadTraceScenarioHolders() // slow call
    }.thenAcceptAsync(Consumer {
      val registry = LogTraceScenariosRegistry.getInstance()

      if (registry.enabledScenarios().isNotEmpty()) {
        val content = HtmlBuilder()
          .append(RdPlatformBundle.message("logs.trace.description"))
          .br()
          .appendLink("Configure...", CommonBundle.message("action.text.configure.ellipsis"))
          .nbsp()
          .appendLink("Disable", LangBundle.message("action.link.disable"))
          .toString()

        GROUP.createNotification(
          RdPlatformBundle.message("logs.trace.enabled"),
          content,
          NotificationType.WARNING,
          object : NotificationListener.UrlOpeningListener(true) {
            override fun hyperlinkActivated(notification: Notification, event: HyperlinkEvent) {
              notification.expire()
              when (event.description) {
                LangBundle.message("action.link.disable") -> registry.save(emptyList())
                CommonBundle.message("action.text.configure.ellipsis") -> LogTraceScenariosDialog(null).show()
              }
            }
          }).notify(null)
      }

      LogTraceScenariosRegistry.getInstance().synchronizeToLogger(true)

    }, EdtExecutorService.getInstance())
  }
}