// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.diagnostics

import com.fasterxml.jackson.databind.node.JsonNodeFactory
import com.fasterxml.jackson.databind.node.ObjectNode
import com.fasterxml.jackson.module.kotlin.jacksonObjectMapper
import com.intellij.openapi.progress.ProgressIndicator
import com.intellij.util.io.HttpRequests
import com.intellij.util.net.NetUtils
import com.jetbrains.rd.platform.util.withFragment
import java.io.File
import java.net.HttpURLConnection
import java.net.URI

object SupportInformationUploader {
  private const val uploadsServiceUrl = "https://uploads.jetbrains.com"

  private val objectMapper = jacksonObjectMapper()

  fun uploadFile(file: File, fileName: String, indicator: ProgressIndicator?): URI {
    val urlWithoutSlash = uploadsServiceUrl.trimEnd('/')

    val responseJson = HttpRequests.post("$urlWithoutSlash/sign", HttpRequests.JSON_CONTENT_TYPE)
      .accept(HttpRequests.JSON_CONTENT_TYPE)
      .connect { request ->

        val requestJson = JsonNodeFactory.instance.objectNode()
          .apply {
            put("filename", fileName)
            put("method", "put")
            put("contentType", "application/octet-stream")
          }

        request.write(requestJson.toPrettyString())

        objectMapper.readTree(request.reader) as ObjectNode
      }

    val uploadUrl = responseJson["url"].asText()
    val folderName = responseJson["folderName"].asText()
    val headers = responseJson["headers"] as ObjectNode

    HttpRequests.put(uploadUrl, "application/octet-stream")
      .productNameAsUserAgent()
      .tuner { urlConnection ->
        headers.fields().forEach {
          urlConnection.addRequestProperty(it.key, it.value.asText())
        }
      }
      .connect {
        val http = it.connection as HttpURLConnection
        val length = file.length().toInt()
        http.setFixedLengthStreamingMode(length)
        http.outputStream.use { outputStream ->
          file.inputStream().buffered(64 * 1024).use { inputStream ->
            NetUtils.copyStreamContent(indicator, inputStream, outputStream, length)
          }
        }
      }

    return URI("$urlWithoutSlash/browse").withFragment(folderName)
  }
}