package com.jetbrains.rd.platform.internal

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.fileEditor.FileEditorManager
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.project.Project
import com.intellij.openapi.vfs.LocalFileSystem
import com.jetbrains.rd.platform.util.application
import java.io.File
import java.io.OutputStreamWriter
import java.text.SimpleDateFormat
import java.util.*

abstract class DumpAction(val name: String) : AnAction(name), DumbAware {

  companion object {
    fun dumpToFile(project: Project?, name: String, dumper: (OutputStreamWriter) -> Unit) {
      val dateFormat = SimpleDateFormat("yyyyMMdd-HH-mm-ss")
      val fileName = name + dateFormat.format(Date())
      val file = File.createTempFile(fileName, ".txt")
      val writer: OutputStreamWriter = file.writer()

      try {
        dumper(writer)
      }
      finally {
        writer.flush()
        writer.close()
      }

      if (project != null) {
        application.invokeAndWait {
          val virtualFile = LocalFileSystem.getInstance().refreshAndFindFileByIoFile(file)!!
          FileEditorManager.getInstance(project).openFile(virtualFile, true)
        }
      }
    }
  }

  protected fun dump(project: Project, dumper: (OutputStreamWriter) -> Unit) {
    dumpToFile(project, name.replace(" ", ""), dumper)
  }
}