package com.jetbrains.rd.platform.internal

import com.intellij.openapi.application.ApplicationManager
import com.intellij.openapi.components.service
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.internal.widgets.InternalWidget
import com.jetbrains.rd.platform.internal.widgets.InternalWidgetBuilder
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.ISource
import com.jetbrains.rd.util.reactive.Signal
import java.util.*
import javax.swing.Icon

class InternalTools(val project: Project) {
  companion object {
    fun getInstance(project: Project) = project.service<InternalTools>()
  }

  internal val tabBuilders = ArrayList<InternalWidgetBuilder>()
  internal val tabBuilderAdded = Signal<InternalWidgetBuilder>()

  fun register(widget: InternalWidgetBuilder) {
    if (ApplicationManager.getApplication().isInternal) {
      tabBuilders.add(widget)
      tabBuilderAdded.fire(widget)
    }
  }

  fun register(category: String, title: String, icon: Icon? = null, build: (Lifetime) -> InternalWidget) {
    register(InternalWidgetBuilder(category, title, icon, build))
  }

  fun <T> registerLog(category: String, title: String, project: Project, source: ISource<T>, convert: (T) -> String) {
    register(InternalWidgetBuilder.createLogTab(category, title, project, source, convert))
  }
}