package com.jetbrains.rd.platform.internal

import com.intellij.diff.DiffContentFactory
import com.intellij.diff.DiffManager
import com.intellij.diff.requests.SimpleDiffRequest
import com.intellij.openapi.project.Project

data class TextDiffPresentationData(val title: String, val text: String)

object RdInternalShowTextDiffManager {
  private val contentFactory = DiffContentFactory.getInstance()
  private val diffManager = DiffManager.getInstance()

  @Deprecated(
    "use an overload with project", ReplaceWith(
    "showDiff(project, windowTitle, first, second)",
    "com.jetbrains.rider.internal.RiderInternalShowTextDiffManager.showDiff"
  )
  )
  fun showDiff(windowTitle: String, first: TextDiffPresentationData, second: TextDiffPresentationData) {
    showDiff(null, windowTitle, first, second)
  }

  fun showDiff(project: Project?, windowTitle: String, first: TextDiffPresentationData, second: TextDiffPresentationData) {
    val diffContent1 = contentFactory.create(first.text)
    val diffContent2 = contentFactory.create(second.text)
    val diffRequest = SimpleDiffRequest(windowTitle, diffContent1, diffContent2, first.title, second.title)
    diffManager.showDiff(project, diffRequest)
  }

  fun showDiff(project: Project?, windowTitle: String, data: List<TextDiffPresentationData>) {
    val contents = data.map { contentFactory.create(it.text) }
    val titles = data.map { it.title }
    val diffRequest = SimpleDiffRequest(windowTitle, contents, titles)
    diffManager.showDiff(project, diffRequest)
  }
}