package com.jetbrains.rd.platform.internal.widgets

import com.intellij.execution.filters.TextConsoleBuilderFactory
import com.intellij.execution.impl.ConsoleViewImpl
import com.intellij.execution.ui.ConsoleViewContentType
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.intellij.util.ui.update.MergingUpdateQueue
import com.intellij.util.ui.update.Update
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isAlive
import com.jetbrains.rd.util.lifetime.onTermination
import java.awt.BorderLayout
import java.text.SimpleDateFormat
import java.util.*
import javax.swing.JPanel

abstract class InternalConsoleWidget(lifetime: Lifetime, val project: Project) : InternalWidget(lifetime) {
  val logBuffer = StringBuffer()
  val logBufferLock = Object()
  private val loggerFlushQueue =
    MergingUpdateQueue("loggerFlushQueue", 200, true, MergingUpdateQueue.ANY_COMPONENT, project, null, true).setRestartTimerOnAdd(false)

  private val console = TextConsoleBuilderFactory.getInstance()
    .createBuilder(project)
    .console as ConsoleViewImpl

  fun logWithTime(line: String) = log(line, true)

  fun log(line: String, timestamp: Boolean = false) {
    synchronized(logBufferLock) {
      val time = timeFormatter.format(Date())
      if (timestamp)
        logBuffer.append("$time: $line\n")
      else
        logBuffer.append("$line\n")
      loggerFlushQueue.queue(flushEvent)
    }
  }

  private val flushEvent = object : Update("logFlushEvent") {
    override fun run() {
      synchronized(logBufferLock) {
        if (lifetime.isAlive) {
          console.print(logBuffer.toString(), ConsoleViewContentType.NORMAL_OUTPUT)
          logBuffer.setLength(0)
        }
      }
    }
  }

  protected fun clear() {
    console.clear()
  }

  private val timeFormatter = SimpleDateFormat("HH:mm:ss.SSS")

  init {
    lifetime.onTermination { console.dispose() }
  }

  override fun build() = JPanel(BorderLayout()).apply {
    add(console.component, BorderLayout.CENTER)
    console.editor.settings.isCaretRowShown = true
  }

  override fun toolbarActions(): List<AnAction> = console.createConsoleActions().drop(2)
}