package com.jetbrains.rd.platform.internal.widgets

import com.intellij.icons.AllIcons
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.internal.DumpAction
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isAlive
import java.util.*
import kotlin.concurrent.timerTask

abstract class InternalDumpWidget(
  lifetime: Lifetime,
  project: Project,
  autoRefresh: Boolean = false,
  showOnStart: Boolean = true
) : InternalConsoleWidget(lifetime, project) {
  init {
    if (showOnStart)
      show()
    if (autoRefresh) {
      val timer = Timer()
      timer.scheduleAtFixedRate(timerTask {
        show()
        if (!lifetime.isAlive)
          timer.cancel()
      }, 1000, 1000)
    }
  }

  protected fun show() {
    clear()
    log(dump())
  }

  abstract fun dump(): String
  open fun fileName(): String = "Dump"

  override fun toolbarActions(): List<AnAction> {
    val actions = ArrayList<AnAction>()
    actions.add(object : AnAction("Refresh", "Refresh", AllIcons.Actions.Refresh) {
      override fun actionPerformed(e: AnActionEvent) = show()
    })
    actions.add(object : AnAction("File", "Dump To File", AllIcons.FileTypes.Text) {
      override fun actionPerformed(e: AnActionEvent) {
        DumpAction.dumpToFile(project, fileName()) { it.write(dump()) }
      }
    })
    actions.addAll(super.toolbarActions())
    return actions
  }
}