package com.jetbrains.rd.platform.internal.widgets

import com.intellij.icons.AllIcons
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.util.ui.UIUtil
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isAlive
import com.jetbrains.rd.util.reactive.ISource
import com.jetbrains.rd.util.reactive.IVoidSignal
import com.jetbrains.rd.util.reactive.Signal
import com.jetbrains.rd.util.reactive.fire
import java.awt.BorderLayout
import java.awt.Color
import java.util.*
import javax.swing.JPanel
import javax.swing.JScrollPane
import javax.swing.JTextPane
import javax.swing.text.SimpleAttributeSet
import javax.swing.text.StyleConstants
import javax.swing.text.StyleContext
import kotlin.concurrent.timerTask

class InternalStatusWidget(lifetime: Lifetime, private val requestStatus: IVoidSignal? = null) : InternalWidget(lifetime) {
  val newStatus = Signal<String>()

  companion object {
    fun createStatusTab(lifetime: Lifetime, source: ISource<String>, requestStatus: IVoidSignal? = null) = InternalStatusWidget(
      lifetime,
      requestStatus
    ).apply {
      source.advise(lifetime) { newStatus.fire(it) }
    }

    fun createAutoStatusTab(lifetime: Lifetime, interval: Long = 1000, getStatus: () -> String) = InternalStatusWidget(
      lifetime
    ).apply {
      val timer = Timer()
      timer.scheduleAtFixedRate(timerTask {
        newStatus.fire(getStatus())
        if (!lifetime.isAlive)
          timer.cancel()
      }, 0, interval)
    }
  }

  override fun build() = JPanel(BorderLayout()).apply {
    val statusPane = object : JTextPane() {
      init {
        isEditable = false
        val color = Color.GRAY
        val sc = StyleContext.getDefaultStyleContext()
        var attributes = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, color)
        attributes = sc.addAttribute(attributes, StyleConstants.FontFamily, "Lucida Console")
        attributes = sc.addAttribute(attributes, StyleConstants.Alignment, StyleConstants.ALIGN_JUSTIFIED)
        setCharacterAttributes(attributes, false)
      }

      override fun getBackground() = UIUtil.getListBackground()
    }
    val scrollPane = JScrollPane(
      statusPane,
      JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED
    )
    add(scrollPane, BorderLayout.CENTER)

    newStatus.advise(lifetime) {
      if (statusPane.text != it)
        statusPane.text = it
    }
  }

  override fun toolbarActions(): List<AnAction> {
    val actions = ArrayList<AnAction>()
    if (requestStatus != null)
      actions.add(object : AnAction("Refresh", "Refresh", AllIcons.Actions.Refresh) {
        override fun actionPerformed(e: AnActionEvent) {
          requestStatus.fire()
        }
      })
    actions.addAll(super.toolbarActions())
    return actions
  }
}