package com.jetbrains.rd.platform.internal.widgets

import com.intellij.openapi.project.Project
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.ISource
import com.jetbrains.rd.util.reactive.IVoidSignal
import javax.swing.Icon

class InternalWidgetBuilder(val category: String, val title: String, var icon: Icon? = null, val build: (Lifetime) -> InternalWidget) {
  companion object {
    fun createDumpTab(category: String,
                      title: String,
                      project: Project,
                      autoRefresh: Boolean = false,
                      dump: () -> String): InternalWidgetBuilder {
      return InternalWidgetBuilder(category, title) { lifetime ->
        object : InternalDumpWidget(lifetime, project, autoRefresh) {
          override fun dump() = dump()
        }
      }
    }

    fun createAutoStatusTab(category: String, title: String, interval: Long = 1000, getStatus: () -> String): InternalWidgetBuilder {
      return InternalWidgetBuilder(category, title) { lifetime ->
        InternalStatusWidget.createAutoStatusTab(
          lifetime,
          interval,
          getStatus
        )
      }
    }

    fun createStatusTab(category: String,
                        title: String,
                        source: ISource<String>,
                        requestStatus: IVoidSignal? = null): InternalWidgetBuilder {
      return InternalWidgetBuilder(category, title) { lifetime ->
        InternalStatusWidget.createStatusTab(
          lifetime,
          source,
          requestStatus
        )
      }
    }

    fun <T> createLogTab(category: String,
                         title: String,
                         project: Project,
                         source: ISource<T>,
                         convert: (T) -> String): InternalWidgetBuilder {
      return InternalWidgetBuilder(category, title) { lifetime ->
        object : InternalConsoleWidget(lifetime, project) {
          init {
            source.advise(lifetime) { log(convert(it)) }
          }
        }
      }
    }
  }
}