package com.jetbrains.rd.platform.protocol

import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isAlive
import com.jetbrains.rd.util.reactive.IScheduler

interface IRdDispatcher : IScheduler {
  /**
   * Pumps the protocol queue while the given condition is true.
   */
  fun pumpProtocolQueue(condition: () -> Boolean)

  val backgroundScheduler: IScheduler

  fun suppressPumping(lifetime: Lifetime)

  fun repost()

  fun allowBackgroundThreadAndSuppressPumping(lifetime: Lifetime)
}

/**
 * Pumps protocol queue. Main destiny is to prevent any repainting of editor (pumping of AWT queue),
 * while document changes from backend are being applied.
 * @param lifetime segment time for pumping
 */
fun IRdDispatcher.pumpProtocolQueue(lifetime: Lifetime) {
  pumpProtocolQueue { lifetime.isAlive }
}

fun IRdDispatcher.pumpProtocolQueue(timeoutMS: Long, condition: () -> Boolean) {
  val startTime = System.currentTimeMillis()
  pumpProtocolQueue { condition() && System.currentTimeMillis() - startTime < timeoutMS }
}

val IScheduler.asRdDispatcher: IRdDispatcher
  get() {
    return this as? IRdDispatcher ?: throw IllegalArgumentException("Not RdDispatcher: ${javaClass.name}")
  }
