// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

import com.intellij.openapi.Disposable
import com.intellij.openapi.components.ComponentManager
import com.intellij.openapi.rd.createLifetime
import com.intellij.openapi.util.Key
import com.intellij.openapi.util.UserDataHolder
import com.jetbrains.rd.util.lifetime.Lifetime

val <T> T.lifetime: Lifetime where T : UserDataHolder, T : Disposable by userData(Key("com.jetbrains.rd.platform.util.lifetime")) {
  it.createLifetime()
}

inline fun <reified TComponent : Any> ComponentManager.getComponent(): TComponent {
  return this.getComponent(TComponent::class.java)
         ?: throw IllegalStateException("Component ${TComponent::class.java} not found in container $this")
}

inline fun <reified TComponent : Any> ComponentManager.tryGetComponent(): TComponent? {
  return this.getComponent(TComponent::class.java)
}