// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

import com.intellij.ui.content.Content
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rd.util.reactive.IOptProperty
import java.beans.PropertyChangeListener

fun Content.bindDisplayName(lifetime: Lifetime, property: IOptProperty<String>) {

  val listener = PropertyChangeListener { evt ->
    if (evt.propertyName == Content.PROP_DISPLAY_NAME) {
      property.set(evt.newValue.toString())
    }
  }

  property.advise(lifetime) {
    this.displayName = it
  }

  addPropertyChangeListener(listener)
  lifetime.onTermination {
    removePropertyChangeListener(listener)
  }
}