// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

import com.intellij.util.concurrency.AppExecutorUtil
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination
import java.time.Duration
import java.util.concurrent.CompletableFuture
import java.util.concurrent.CompletionException
import java.util.concurrent.CompletionStage
import java.util.concurrent.TimeUnit

inline fun <T> CompletableFuture<T>.completeWith(block: () -> T) {
  try {
    complete(block())
  } catch (t: Throwable) {
    completeExceptionally(t)
  }
}

inline fun <T, U> CompletableFuture<T>.handleWithUnwrapException(crossinline block: (T?, Throwable?) -> U) : CompletableFuture<U> {
  return handle { result, ex ->
    return@handle if(ex is CompletionException)
      block(result, ex.cause)
    else
      block(result, ex)
  }
}

fun delayFuture(duration: Duration, lifetime : Lifetime? = null): CompletableFuture<Unit> {
  val future = CompletableFuture<Unit>()

  val scheduledFuture = AppExecutorUtil.getAppScheduledExecutorService().schedule({ future.complete(Unit) }, duration.toMillis(), TimeUnit.MILLISECONDS)
  lifetime?.onTermination {
    future.cancel(false)
    scheduledFuture.cancel(false)
  }
  return future
}