// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

import com.intellij.util.ExceptionUtil
import com.jetbrains.rd.util.ILoggerFactory
import com.jetbrains.rd.util.LogLevel
import com.jetbrains.rd.util.Logger

object IdeaRdLoggerFactor : ILoggerFactory {
  override fun getLogger(category: String): Logger {
    val ideaLogger = com.intellij.openapi.diagnostic.Logger.getInstance(category)
    return object : Logger {
      override fun isEnabled(level: LogLevel): Boolean {
        return when (level) {
          LogLevel.Trace -> ideaLogger.isTraceEnabled
          LogLevel.Debug -> ideaLogger.isDebugEnabled
          else -> true
        }
      }

      override fun log(level: LogLevel, message: Any?, throwable: Throwable?) {
        when (level) {
          LogLevel.Trace -> ideaLogger.trace(message.toString() +
                                             (throwable?.let { ExceptionUtil.getThrowableText(it) } ?: ""))
          LogLevel.Debug -> ideaLogger.debug(message.toString(), throwable)
          LogLevel.Info -> ideaLogger.info(message.toString(), throwable)
          LogLevel.Warn -> ideaLogger.warn(message.toString(), throwable)
          LogLevel.Error, LogLevel.Fatal -> ideaLogger.error(message.toString(), throwable)
        }
      }
    }
  }
}