package com.jetbrains.rd.platform.util

import com.intellij.openapi.diagnostic.Logger
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.diagnostic.runAndLogException
import com.intellij.openapi.diagnostic.trace
import com.intellij.openapi.progress.ProcessCanceledException
import com.jetbrains.rd.util.lifetime.Lifetime
import java.util.concurrent.CancellationException

/* Historically the patter in rd-platform is 'val logger = getLogger<T>()'
* a lot of 'val logger' will be shown in completion if we use platform API directly.
* it's better to stick with the platform 'val LOG = logger<Smth>()'
 */
inline fun <reified T : Any> getLogger(): Logger = logger<T>()

inline fun Logger.debug(printer: () -> String) {
  if (isDebugEnabled) debug(printer())
}

@Deprecated("Use platform API", ReplaceWith("trace", "com.intellij.openapi.diagnostic.trace"))
inline fun Logger.trace(printer: () -> String) = trace(printer)

@Deprecated("Use platform API", ReplaceWith("runAndLogException(runnable)", "com.intellij.openapi.diagnostic.runAndLogException"))
inline fun <T> Logger.catchAndLog(runnable: () -> T): T? = runAndLogException(runnable)

fun Logger.infoBracket(lifetime: Lifetime, start: String, end: String) {
  lifetime.bracket({ info(start) }, { info(end) })
}

fun Logger.debugBracket(lifetime: Lifetime, start: String, end: String) {
  if (isDebugEnabled) lifetime.bracket({ debug(start) }, { debug(end) })
}

@Deprecated("Use overload with lazy parameters")
fun Logger.traceBracket(lifetime: Lifetime, start: String, end: String) {
  if (isTraceEnabled) lifetime.bracket({ trace(start) }, { trace(end) })
}

inline fun Logger.traceBracket(lifetime: Lifetime, crossinline lazyStart: () -> String, crossinline lazyEnd: () -> String) {
  if (isTraceEnabled) lifetime.bracket({ trace(lazyStart()) }, { trace(lazyEnd()) })
}

inline fun <T> Logger.catchWarn(message: String? = null, action: () -> T): T? {
  return try {
    action()
  } catch (e: ProcessCanceledException) {
    throw e
  } catch (e: CancellationException) {
    throw e
  } catch (e: Throwable) {
    warn(message, e)
    null
  }
}