// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

import com.intellij.util.ReflectionUtil
import com.intellij.util.io.URLUtil
import com.intellij.util.lang.UrlClassLoader
import java.io.File
import java.net.URL
import java.net.URLClassLoader

object RdClassLoaderUtil {
  fun ClassLoader.getClasspathFiles(): List<File> {
    @Suppress("UNCHECKED_CAST")
    val urls = when {
      this is URLClassLoader -> urLs.toList()
      this.javaClass.name == UrlClassLoader::class.java.name ->
        (ReflectionUtil.getMethod(javaClass, "getUrls")!!.invoke(this) as List<URL>).toList()
      else -> error("Unknown classloader type: ${javaClass.name}")
    }

    return urls.map { URLUtil.urlToFile(it) }
  }
}