package com.jetbrains.rd.platform.util

import com.jetbrains.rd.util.reactive.Property
import kotlin.reflect.KMutableProperty0

fun noReenter(flag: Property<Boolean>, body: () -> Unit) {
  if (flag.value) return
  flag.set(true)
  try {
    body()
  }
  finally {
    flag.set(false)
  }
}

inline fun noReenter(flag: KMutableProperty0<Boolean>, body: () -> Unit) {
  if (flag.get()) return
  flag.set(true)
  try {
    body()
  }
  finally {
    flag.set(false)
  }
}

private var currentLocks = hashSetOf<Any>()
fun noReenter(uniqueLock: Any, body: () -> Unit) {
  application.assertIsDispatchThread()
  if (currentLocks.contains(uniqueLock))
    return

  try {
    currentLocks.add(uniqueLock)
    body()
  } finally {
    currentLocks.remove(uniqueLock)
  }
}

inline fun <T,R> usingNullableFlag(flag: KMutableProperty0<T?>, value: T, action: () -> R) : R {
  require(flag.get() == null)

  try {
    flag.set(value)
    return action()
  } finally {
    flag.set(null)
  }
}
