// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.util.reactive.ISource
import com.jetbrains.rd.util.reactive.Signal
import java.util.concurrent.locks.ReentrantLock
import kotlin.concurrent.withLock

class ThreadSafeProperty<T>(defaultValue: T) : IProperty<T> {
  private val lock = ReentrantLock()

  override fun set(newValue: T) {
    value = newValue
  }

  override var value: T = defaultValue
    get() = lock.withLock { field }
    set(newValue) {
      lock.withLock {
        if (field == newValue) return
        field = newValue
      }

      _change.fire(newValue)
    }

  private val _change = Signal<T>()
  override val change: ISource<T> get() = _change
}

