package com.jetbrains.rd.platform.util

import java.time.Duration

class TimeoutTracker(val timeout: Duration) {
  private val startTimeNs: Long

  init {
    assert(timeout.toNanos() > 0) { "The number of nanoseconds (${timeout.toNanos()}) in timeout should be positive." }
    startTimeNs = getCurrentTimeNs()
  }

  val isExpired: Boolean
    get() {
      if (timeout.isNegative) return false
      return getCurrentDeltaNs() > timeout.toNanos()
    }

  fun throwIfExpired(message: () -> String) {
    if (isExpired) throw Exception(message())
  }

  val delta: Duration get() = Duration.ofNanos(getCurrentDeltaNs())

  private fun getCurrentDeltaNs() = getCurrentTimeNs() - startTimeNs
  private fun getCurrentTimeNs() = System.nanoTime()
}