// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util

object TreeTraverse {
  fun <T> pathToRoot(me: T, getParent: (T) -> T?): Sequence<T> = sequence {
    yield(me)
    val parent = getParent(me)
    if (parent != null)
      yieldAll(pathToRoot(parent, getParent))
  }

  fun <T> thisAndDescendantsDfs(root: T, getChildren: (T) -> Sequence<T>): Sequence<T> = sequence {
    yield(root)
    getChildren(root)
      .forEach {
        yieldAll(thisAndDescendantsDfs(it, getChildren))
      }
  }
  fun <T> descendantsDfs(root: T, getChildren: (T) -> Sequence<T>): Sequence<T> = sequence {
    getChildren(root)
      .forEach {
        yield(it)
        yieldAll(descendantsDfs(it, getChildren))
      }
  }
}