// Copyright 2000-2020 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.platform.util.idea

import com.intellij.openapi.Disposable
import com.intellij.openapi.project.Project
import com.intellij.openapi.rd.defineNestedLifetime
import com.jetbrains.rd.platform.util.application

/**
 * Marker class that a component has only protocol-related or TOPIC subscriptions in its constructor
 * and DOESN'T call any services
 */
abstract class ProtocolSubscribedProjectComponent(val project: Project): Disposable {
  private val lifetimeDefinition = project.defineNestedLifetime()
  protected val projectComponentLifetime get() = lifetimeDefinition.lifetime

  @Deprecated(message = "Use projectComponentLifetime", replaceWith = ReplaceWith("projectComponentLifetime"))
  val componentLifetime get() = projectComponentLifetime

  override fun dispose() {
    lifetimeDefinition.terminate()
  }
}

open class LifetimedProjectService(val project: Project): Disposable {
  private val lifetimeDefinition = project.defineNestedLifetime()
  protected val projectServiceLifetime get() =  lifetimeDefinition.lifetime

  @Deprecated(message = "Use projectServiceLifetime", replaceWith = ReplaceWith("projectServiceLifetime"))
  val componentLifetime get() =  projectServiceLifetime

  override fun dispose() {
    lifetimeDefinition.terminate()
  }
}

open class LifetimedApplicationService : Disposable {
  private val lifetimeDefinition = application.defineNestedLifetime()
  val componentLifetime get() = lifetimeDefinition.lifetime

  override fun dispose() {
    lifetimeDefinition.terminate()
  }
}
