package com.jetbrains.rd.ui.bedsl

import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ide.model.IconModel
import com.jetbrains.rd.ui.bedsl.dsl.beInit
import com.jetbrains.rd.ui.bedsl.dsl.label
import com.jetbrains.rd.ui.bedsl.dsl.labelWithShortcut
import com.jetbrains.rd.ui.bedsl.dsl.showDialog
import com.jetbrains.rd.ui.bindable.BeActionManager
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.advise

private fun createButton(
  control: BeAbstractText,
  beButtonAction: BeUserAction? = null,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = ""
): BeButton {
  val button = BeButton(beButtonAction ?: BeUserAction(), control, style)
  button.beInit(id)
  return button
}

fun actionButton(actionId: String, lifetime: Lifetime): BeControl {
  val action = ActionManager.getInstance().getAction(actionId)
  if (action == null) return BeBackendIdeAction(null, actionId).apply { beInit() }
  return actionButton(action, lifetime)
}

fun actionButton(action: AnAction, lifetime: Lifetime): BeControl {
  return BeActionManager.EP_NAME.extensions.firstOrNull()?.getBeAction(action, lifetime) ?: label("action can not be created")
}

private fun createPredefinedButton(
  control: BeAbstractText,
  behaviour: BeCommonBehavior,
  beButtonAction: BeUserAction? = null,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = ""
): BePredefinedBehaviorButton {
  val button = BePredefinedBehaviorButton(behaviour, beButtonAction ?: BeUserAction(), control, style)
  button.beInit(id)
  return button
}

private fun toBeUserAction(lifetime: Lifetime, onClick: () -> Unit): BeUserAction {
  val action = BeUserAction()
  action.click.advise(lifetime, onClick)
  return action
}

fun button(
  control: BeAbstractText,
  lifetime: Lifetime,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = "",
  onClick: () -> Unit
): BeButton {
  return createButton(control, toBeUserAction(lifetime, onClick), style, id)
}

fun button(
  icon: IconModel?,
  lifetime: Lifetime,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = "",
  onClick: () -> Unit
): BeButton {
  IconModel
  return button(label("", icon), lifetime, style, id, onClick)
}

fun button(
  text: String,
  lifetime: Lifetime,
  icon: IconModel? = null,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = "",
  onClick: () -> Unit
): BeButton {
  return button(labelWithShortcut(text, icon), lifetime, style, id, onClick)
}

fun buttonWithDialog(
  control: BeAbstractText,
  lifetime: Lifetime,
  project: Project,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = "",
  getDialog: (Lifetime) -> BeDialog
): BeButton {
  return createButton(control, toBeUserAction(lifetime) {
    showDialog(project, lifetime, getDialog)
  }, style, id)
}

fun buttonWithDialog(
  text: String,
  lifetime: Lifetime,
  project: Project,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = "",
  getDialog: (Lifetime) -> BeDialog
): BeButton {
  return createButton(label(text), toBeUserAction(lifetime) {
    showDialog(project, lifetime, getDialog) //TODO:doesn't work as expected!!
  }, style, id)
}


fun predefinedBehaviorButton(
  control: BeAbstractText,
  lifetime: Lifetime,
  behaviour: BeCommonBehavior,
  style: BeButtonStyle = BeButtonStyle.DEFAULT,
  id: String? = "",
  onClick: () -> Unit
): BePredefinedBehaviorButton {
  return createPredefinedButton(control, behaviour, toBeUserAction(lifetime, onClick), style, id)
}

fun link(
  linkText: String,
  lifetime: Lifetime,
  prefix: String = "",
  postfix: String = "",
  onClick: () -> Unit
): BeControl {
  return BeLink(label(prefix), label(postfix), toBeUserAction(lifetime, onClick)).apply {
    this.beInit()
    content.set(linkText)
  }
}

fun pathSelectionButton(control: BeAbstractText, lifetime: Lifetime): BeButton {
  return predefinedBehaviorButton(control, lifetime, BeCommonBehavior.OPEN_FILE_DIALOG) {}
}
