package com.jetbrains.rd.ui.bedsl.dsl

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.ide.model.uiautomation.BeCommonBehavior
import com.jetbrains.rd.ui.bedsl.dsl.util.BeControlSizes
import com.jetbrains.rd.ui.bedsl.extensions.bind
import com.jetbrains.rd.ui.bedsl.extensions.compose
import com.jetbrains.rd.ui.bedsl.extensions.descendantsOfType
import com.jetbrains.rd.ui.bedsl.extensions.valueOrEmpty
import com.jetbrains.rd.ui.bedsl.predefinedBehaviorButton
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IOptProperty
import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.viewNotNull
fun showDialog(project: Project, parentLifetime: Lifetime? = null, dialog: (Lifetime) -> BeDialog) {
  val lifetimeDef = (parentLifetime ?: project.lifetime).createNested()
  val lifetime = lifetimeDef.lifetime
  val beDialog = dialog(lifetime)

  beDialog.result.advise(lifetime) {
    if (it != DialogResult.NONE)
      lifetimeDef.terminate(true)
  }
  //todo: use dialog host and model
}


//HACK: this function should be removed
fun showDialog(project: Project, parentLifetime: Lifetime? = null, realShow: (BeDialog, Lifetime) -> Unit, dialog: (Lifetime) -> BeDialog) {
  val lifetimeDef = (parentLifetime ?: project.lifetime).createNested()
  val lifetime = lifetimeDef.lifetime
  val beDialog = dialog(lifetime)

  beDialog.result.advise(lifetime) {
    if (it != DialogResult.NONE)
      lifetimeDef.terminate(true)
  }
  realShow(beDialog, lifetime)
}

fun okCancelDialog(
  title: String,
  id: String,
  lifetime: Lifetime,
  ok: (() -> Unit)? = null,
  cancel: (() -> Unit)? = null,
  disableWhenInvalid: Boolean = false,
  modality: DialogModality = DialogModality.NON_MODAL,
  size: BeControlSize? = null,
  scrollbarPolicy: BeScrollbarPolicy = BeScrollbarPolicy.BOTH,
  layoutPersistenceMode: DialogLayoutPersistenceMode = DialogLayoutPersistenceMode.SIZE_AND_POSITION,
  controlBox: BeDialogControlBox = BeDialogControlBox.MINIMIZE_MAXIMIZE_AND_CLOSE, dialogContent: () -> BeControl
)
  : BeDialog {
  return dialog(title, id, modality, size, layoutPersistenceMode, controlBox, dialogContent)
    .withOKButton(lifetime, ok, disableWhenInvalid = disableWhenInvalid).withCancelButton(lifetime, cancel)
}

fun dialog(
  title: String,
  id: String,
  modality: DialogModality = DialogModality.NON_MODAL,
  size: BeControlSize? = null,
  layoutPersistenceMode: DialogLayoutPersistenceMode = DialogLayoutPersistenceMode.SIZE_AND_POSITION,
  controlBox: BeDialogControlBox = BeDialogControlBox.MINIMIZE_MAXIMIZE_AND_CLOSE,
  dialogContent: () -> BeControl
): BeDialog {
  val dialog = BeDialog(
    modality,
    title,
    size ?: BeControlSizes.getDefaultSize(),
    layoutPersistenceMode,
    controlBox
  )
  dialog.content.set(dialogContent())
  dialog.helpButton.value = null
  dialog.acceptButton.value = null
  dialog.cancelButton.value = null
  dialog.loaded.set(false)
  dialog.beInit(id)
  dialog.buttons.set(listOf())
  dialog.extraFooterContent.set(spacer())
  return dialog
}

fun BeControl.bindValidation(lifetime: Lifetime, enabled: IProperty<Boolean>) {
  val rdControlPresentations = this.descendantsOfType(BeControlWithValidation::class.java)
  val rdProperties = rdControlPresentations.mapNotNull { it.validationResult }
  rdProperties.toList().bindValidation(lifetime, null, enabled)
}

private fun Iterable<IOptProperty<ValidationResult>>.bindValidation(lifetime: Lifetime,
                                                                    continueEnabled: Property<Boolean>?,
                                                                    rdContinueEnabled: IProperty<Boolean>?) {
  if (rdContinueEnabled == null && continueEnabled == null) return
  val property = Property<Boolean?>(false)
  property.compose(this, lifetime) {
    it.any { it?.state == ValidationStates.validationError }
  }
  property.viewNotNull(lifetime
  ) { _, value ->
    if (continueEnabled != null)
      continueEnabled.value = !value
    if (rdContinueEnabled != null)
      rdContinueEnabled.value = !value
  }
}

fun BeDialog.withCancelButton(
  lifetime: Lifetime,
  cancel: (() -> Unit)? = null): BeDialog {
  return withCustomButton("Cancel", lifetime, cancel, BeCommonBehavior.DIALOG_CANCEL_BUTTON, DialogResult.CANCEL)
}

fun BeDialog.withOKButton(
  lifetime: Lifetime,
  ok: (() -> Unit)? = null,
  buttonEnabled: Property<Boolean>? = null,
  disableWhenInvalid: Boolean = false,
  text: String = "OK"): BeDialog {
  return withCustomButton(text, lifetime, ok, BeCommonBehavior.DIALOG_ACCEPT_BUTTON, DialogResult.OK, buttonEnabled, disableWhenInvalid)
}

fun BeDialog.withCustomFooter(footer: BeControl): BeDialog = this.apply { extraFooterContent.set(footer) }
fun BeDialog.withCustomFooter(footer: () -> BeControl): BeDialog = this.apply { extraFooterContent.set(footer()) }

fun BeDialog.withCustomButton(
  text: String,
  lifetime: Lifetime,
  onClick: (() -> Unit)? = null,
  dialogButtonAction: BeCommonBehavior = BeCommonBehavior.NONE,
  resultAfterClick: DialogResult = DialogResult.CLOSE,
  buttonEnabled: Property<Boolean>? = null,
  disableWhenInvalid: Boolean = false,
  finish: (() -> Unit)? = null): BeDialog {
  return withCustomButton(label(text), lifetime, onClick, dialogButtonAction, resultAfterClick, buttonEnabled, disableWhenInvalid, finish)
}

fun BeDialog.withCustomButton(
  text: IProperty<String>,
  lifetime: Lifetime,
  onClick: (() -> Unit)? = null,
  dialogButtonAction: BeCommonBehavior = BeCommonBehavior.NONE,
  resultAfterClick: DialogResult = DialogResult.CLOSE,
  buttonEnabled: Property<Boolean>?,
  disableWhenInvalid: Boolean = false,
  finish: (() -> Unit)? = null): BeDialog {
  return withCustomButton(label(text, lifetime), lifetime, onClick, dialogButtonAction, resultAfterClick, buttonEnabled, disableWhenInvalid,
                          finish)
}

fun BeDialog.withCustomButton(
  text: BeAbstractText,
  lifetime: Lifetime,
  onClick: (() -> Unit)? = null,
  dialogButtonAction: BeCommonBehavior = BeCommonBehavior.NONE,
  resultAfterClick: DialogResult = DialogResult.CLOSE,
  buttonEnabled: Property<Boolean>?,
  disableWhenInvalid: Boolean = false,
  finish: (() -> Unit)? = null): BeDialog {
  val predefinedBehaviorButton =
    getPredefinedBehaviorButton(text, lifetime, dialogButtonAction, onClick, resultAfterClick, finish, disableWhenInvalid)
  buttonEnabled?.bind(lifetime, predefinedBehaviorButton.enabled)

  addDialogButton(this, dialogButtonAction, predefinedBehaviorButton)
  return this
}

fun BeDialog.withCustomButton(
  button: BeButton,
  lifetime: Lifetime,
  onClick: (() -> Unit)? = null,
  dialogButtonAction: BeCommonBehavior = BeCommonBehavior.NONE,
  resultAfterClick: DialogResult = DialogResult.CLOSE,
  disableWhenInvalid: Boolean = false,
  finish: (() -> Unit)? = null): BeDialog {
  val predefinedBehaviorButton =
    getPredefinedBehaviorButton(button.content, lifetime, dialogButtonAction, onClick, resultAfterClick, finish, disableWhenInvalid)
  button.enabled.bind(lifetime, predefinedBehaviorButton.enabled)
  button.visible.bind(lifetime, predefinedBehaviorButton.visible)

  addDialogButton(this, dialogButtonAction, predefinedBehaviorButton)
  return this
}

private fun BeDialog.getPredefinedBehaviorButton(
  text: BeAbstractText,
  lifetime: Lifetime,
  dialogButtonAction: BeCommonBehavior,
  onClick: (() -> Unit)?,
  resultAfterClick: DialogResult,
  finish: (() -> Unit)?,
  disableWhenInvalid: Boolean
): BePredefinedBehaviorButton {
  val predefinedBehaviorButton = predefinedBehaviorButton(
    text,
    lifetime,
    dialogButtonAction,
    id = "${this.controlId.value}.${text.getText()}"
  )
  {
    onClick?.invoke()
    if (resultAfterClick != DialogResult.NONE)
      this.result.set(resultAfterClick)
    finish?.invoke()
  }

  if (disableWhenInvalid) {
    this.bindValidation(lifetime, predefinedBehaviorButton.enabled)
  }
  return predefinedBehaviorButton
}

private fun addDialogButton(
  dialog: BeDialog,
  dialogButtonAction: BeCommonBehavior,
  button: BePredefinedBehaviorButton
) {
  when (dialogButtonAction) {
    BeCommonBehavior.DIALOG_ACCEPT_BUTTON -> {
      //Assertion.Assert(dialog.AcceptButton.Maybe.Value == null, "Dialog can't have two accept buttons")
      dialog.acceptButton.value = button
    }
    BeCommonBehavior.DIALOG_CANCEL_BUTTON -> {
      //Assertion.Assert(dialog.CancelButton.Maybe.Value == null, "Dialog can't have two cancel buttons")
      dialog.cancelButton.value = button
    }
    else -> {
      val buttons = dialog.buttons.valueOrEmpty().toMutableList()
      buttons.add(button)
      dialog.buttons.set(buttons)
    }
  }
}

fun BeDialog.show(project: Project, parentLifetime: Lifetime? = null) {
  showDialog(project, parentLifetime) { this }
}