package com.jetbrains.rd.ui.bedsl.dsl

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.pathSelectionButton
import com.jetbrains.rd.ui.bedsl.predefinedBehaviorButton
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.ui.bedsl.extensions.bind
import com.jetbrains.rd.ui.bedsl.extensions.getBeLabel

fun textBox(
  lifetime: Lifetime,
  isReadonly: Boolean = false,
  id: String = "",
  trigger: UpdateSourceTrigger = UpdateSourceTrigger.TextChanged
): BeTextBox {
  val beTextBox = BeTextBox(BeTextBoxSettings(trigger))
  beTextBox.beInit(id)
  beTextBox.validationResult.set(ValidationResult().apply { icon.value = null })
  beTextBox.handledByParent.value = false
  beTextBox.revalidate.advise(lifetime) { }

  beTextBox.completionItems.set(mutableListOf())

  beTextBox.settings.caret.value = 0
  beTextBox.settings.selectionLength.value = beTextBox.text.value.length

  beTextBox.settings.readonly.value = isReadonly
  beTextBox.settings.textSettings.fontStyle.set(BeFontStyle.DEFAULT)
  beTextBox.settings.textSettings.fontSize.set(BeFontSize.DEFAULT)
  beTextBox.settings.useCamelHumps.value = false
  return beTextBox
}

fun textBox(
  property: IProperty<String>,
  lifetime: Lifetime,
  isReadonly: Boolean = false,
  id: String = "",
  trigger: UpdateSourceTrigger = UpdateSourceTrigger.TextChanged
  //propagateValueWhenInvalid : BeInvalidaValuePropagation //todo
): BeTextBox {
  val beTextBox = textBox(lifetime, isReadonly, id, trigger)
  property.bind(lifetime, beTextBox.text)
  beTextBox.settings.selectionLength.value = beTextBox.text.value.length
  return beTextBox
}

fun BeTextBox.withMonospaceFont(): BeTextBox {
  return this.apply { settings.textSettings.fontStyle.set(BeFontStyle.MONOSPACE) }
}

fun BeTextBox.withReadOnly(): BeTextBox {
  return this.apply { settings.readonly.set(true) }
}

fun spinner(
  min: Int = Int.MIN_VALUE,
  max: Int = Int.MAX_VALUE,
  id: String = ""): BeSpinner {
  val spinner = BeSpinner(min, max)
  spinner.beInit(id)
  return spinner
}

fun spinner(
  property: IProperty<Int>,
  lifetime: Lifetime,
  min: Int = Int.MIN_VALUE,
  max: Int = Int.MAX_VALUE,
  id: String = ""): BeSpinner {
  return spinner(min, max, id).apply {
    property.bind(lifetime, this.value)
  }
}

//todo: problems with open dialog calls
fun pathSelector(property: IProperty<String>,
                 lifetime: Lifetime,
                 id: String = "",
                 trigger: UpdateSourceTrigger = UpdateSourceTrigger.TextChanged
): BeControlWithAction {
  val buttons = mutableListOf(
    pathSelectionButton("Browse".getBeLabel(), lifetime),
    predefinedBehaviorButton("Reset".getBeLabel(), lifetime, BeCommonBehavior.RESET_VALUE) {}
  )
  return BeControlWithAction(textBox(property, lifetime, false, id, trigger), buttons)
}