package com.jetbrains.rd.ui.bedsl.dsl

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.dsl.util.BeControlSizes
import com.jetbrains.rd.ui.bedsl.dsl.util.BeMarginsBuilder
import com.jetbrains.rd.ide.model.AbstractColor
import com.jetbrains.rd.ide.model.RgbaColor
import java.awt.Color

private fun BeControl.getOrCreateStyleControl(): BeStyleControl {
  if (this is BeStyleControl) return this
  return BeStyleControl(this).apply {
    beInit()
    backgroundColor.value = null
    sizes.clear()
    margin.value = null
    opacity.value = null
  }
}


fun BeControl.withMargin(getMargin: BeMarginsBuilder.() -> BeMargin): BeControl {
  val beMarginsBuilder = BeMarginsBuilder() //todo: add force create option, cause margins should be combined, see c# ext
  val margin = beMarginsBuilder.getMargin()
  return getOrCreateStyleControl().apply { this.margin.set(BeMarginModifier(margin)) }
}

fun BeControl.withOpacity(opacity: Float): BeControl {
  return getOrCreateStyleControl().apply { this.opacity.set(BeOpacityModifier(opacity)) }
}
fun BeControl.withCursor(cursor: BeCursor): BeControl {
  return getOrCreateStyleControl().apply { this.cursor.set(BeCursorModifier(cursor)) }
}
fun BeControl.withColor(color: Color): BeControl {
  return getOrCreateStyleControl().apply { this.backgroundColor.set(BeBackgroundColorModifier(color.toAbstractColor())) }
}
fun BeControl.withMinSize(width:Int = -1, height:Int = -1): BeControl {
  return getOrCreateStyleControl().apply {
    this.sizes.add(BeSizeModifier(BeBoundsType.MIN_SIZE, BeControlSizes.getCustomSize(width, height))) }
}

private fun Color.toAbstractColor(): AbstractColor {
  //todo
  return RgbaColor(this.red, this.green, this.blue, this.alpha)
}
