package com.jetbrains.rd.ui.bedsl.dsl

import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeDock
import com.jetbrains.ide.model.uiautomation.BeToolbar
import com.jetbrains.ide.model.uiautomation.BeTreeGrid
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.util.reactive.ViewableList
import com.jetbrains.rd.ui.bedsl.extensions.extend

fun toolbar(dock: BeDock = BeDock.TOP, toolbarID: String = "", content: BeControl, init: BeToolbarBuilder.() -> Unit): BeToolbar {
  return BeToolbar(content, dock).apply {
    BeToolbarBuilder(this).init()
    controlId.value = if (toolbarID.isEmpty()) "BeToolBar.ActionBar" else toolbarID
  }
}

fun BeToolbar.init(init: BeToolbarBuilder.() -> Unit): BeToolbar {
  return this.apply { BeToolbarBuilder(this).init() }
}

fun <T : Any> singleSelectionTableWithToolbar(
  resultProperty: IProperty<T?>,
  initialList: ViewableList<T>,
  lifetime: Lifetime,
  columnsAndSizes: List<String> = emptyList(),
  hasHeader: Boolean = true,
  dock: BeDock = BeDock.TOP,
  toolbarID: String = "",
  tableID: String = "",
  presentLine: presentTableLine<T>): BeToolbar {
  val list = singleSelectionTable(resultProperty, initialList,
                                  lifetime,
                                  columnsAndSizes,
                                  tableID,
                                  hasHeader,
                                  presentLine = presentLine)
  val toolbarTableActionData = TreeGridActionData(resultProperty, initialList, lifetime, list)
  val beToolbar = toolbar(dock, toolbarID, list) {}
  beToolbar.extend(toolbarTableActionData, "ToolbarTableActionData")
  return beToolbar
}

data class TreeGridActionData<T : Any>(
  val selectedElement: IProperty<T?>,
  val collection: ViewableList<T>,
  val lifetime: Lifetime,
  val list: BeTreeGrid
)