package com.jetbrains.rd.ui.bedsl.dsl

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.util.reactive.flowInto

import com.jetbrains.rd.ide.model.IconModel
import com.jetbrains.rd.ide.model.RgbColor
import com.jetbrains.rd.ide.model.RichStringModel
import com.jetbrains.rd.ide.model.RichTextModel
import java.awt.Color

fun getRichTextModel(string: String): RichTextModel {
  val part = RichStringModel(string)
  val parts = mutableListOf<RichStringModel>()
  parts.add(part)
  val model = RichTextModel(parts)
  return model
}

fun BeAbstractText.tryGetSettings(): BeTextSettings? {
  return when (this) {
    is BeRichText -> this.settings
    is BeLabel -> this.settings
    else -> null
  }
}

private fun getShortcutIndex(text: String): Pair<String, Int> {
  var previousIsUnderline = false
  var resultText = text
  val shortcutIndex = text.indexOfFirst()
  { character ->
    val isUnderline = character.equals('_')

    // Skip double underline
    if (previousIsUnderline && isUnderline)
      return@indexOfFirst false

    if (previousIsUnderline) {
      return@indexOfFirst true
    }

    previousIsUnderline = isUnderline
    return@indexOfFirst false
  } - 1

  if (shortcutIndex >= 0)
    resultText = text.removeRange(shortcutIndex, shortcutIndex + 1)
  return Pair(resultText, shortcutIndex)
}

fun BeAbstractText.setText(text: String, withShortcut: Boolean) {
  var resultText = text
  if (withShortcut && text.isNotBlank()) {
    val result: Pair<String, Int> = getShortcutIndex(text)
    resultText = result.first
    shortcutIndex.set(result.second)
  }
  setText(resultText)
}

fun BeAbstractText.setText(text: String) = when (this) {
  is BeLabel -> this.text.set(text)
  is BeRichText -> this.text.set(getRichTextModel(text))
  else -> {
  }
}

fun BeControl.getText(): String? = when (this) {
  is BeAbstractText -> this.getText()
  is BeTextBox -> this.text.value
  else -> null
}

fun BeAbstractText.getText(): String = when (this) {
  is BeLabel -> this.text.value
  is BeRichText -> this.text.valueOrNull?.parts?.joinToString("") { it.text } ?: ""
  else -> ""
}

fun richText(wrap: Boolean = false): BeRichText {
  val beRichText = BeRichText(wrap)
  beRichText.beInit()
  beRichText.icon.value = null
  beRichText.text.set(RichTextModel(mutableListOf()))
  beRichText.shortcutIndex.value = -1
  beRichText.target.value = ""
  beRichText.settings.fontSize.set(BeFontSize.DEFAULT)
  beRichText.settings.fontStyle.set(BeFontStyle.DEFAULT)
  return beRichText
}


fun label(wrap: Boolean = false): BeLabel {
  val beLabel = BeLabel(wrap)
  beLabel.beInit()
  beLabel.icon.value = null
  beLabel.text.value = ""
  beLabel.shortcutIndex.value = -1
  beLabel.target.value = ""
  beLabel.settings.fontSize.set(BeFontSize.DEFAULT)
  beLabel.settings.fontStyle.set(BeFontStyle.DEFAULT)
  return beLabel
}

fun label(text: String, iconId: IconModel? = null, wrap: Boolean = false): BeLabel {
  val beLabel = label(wrap)
  beLabel.setText(text)
  if (iconId != null)
    beLabel.icon.set(iconId)
  return beLabel
}

fun labelWithShortcut(text: String, iconId: IconModel? = null, capitalize: Boolean = true, wrapText: Boolean = false): BeLabel {
  var resultText = text
  if (capitalize) {
    val capitalizeIndex = text.indexOfFirst { it.isLetter() }
    if (capitalizeIndex > 0) {
      val prefix = text.substring(0, capitalizeIndex)
      resultText = "${prefix}${text.substring(capitalizeIndex).capitalize()}"
    }
  }
  val label = label(resultText, iconId, wrapText)
  label.setText(text, true)
  return label
}

fun labelWithShortcut(text: IProperty<String>, lifetime: Lifetime, icon: IconModel? = null, wrapText: Boolean = false): BeLabel {
  val beLabel = labelWithShortcut(text.value, icon, wrapText)
  text.change.advise(lifetime) {
    beLabel.setText(it, true)
  }
  return beLabel
}

fun label(text: IProperty<String>, lifetime: Lifetime, icon: IconModel? = null, wrapText: Boolean = false): BeLabel {
  val beLabel = label(text.value, icon, wrapText)
  text.flowInto(lifetime, beLabel.text)
  return beLabel
}

fun BeAbstractText.withCustomTextSize(size: BeFontSize): BeAbstractText {
  tryGetSettings()?.fontSize?.set(size)
  return this
}

fun BeAbstractText.withCustomTextStyle(style: BeFontStyle): BeAbstractText {
  tryGetSettings()?.fontStyle?.set(style)
  return this
}

fun descriptionLabel(description: String, control: BeControl, customTargetId: String? = null): BeLabel {
  val label = labelWithShortcut(description)
  label.target.value = customTargetId ?: control.controlId.value
  return label
}

fun header(text: String, iconId: IconModel? = null): BeHeader {
  return BeHeader().apply {
    this.beInit()
    this.text.value = text
    if (iconId != null)
      this.icon.value = iconId
  }
}
  fun arrow(leftColor: Color , rightColor: Color, leftContent:String, rightContent:String) : BeControl{
    return BeArrowControl(leftColor.toRgbColor(), rightColor.toRgbColor(), leftContent, rightContent)
  }
  private fun Color.toRgbColor(): RgbColor {
    //if (this == null) return null
    return RgbColor(red, green, blue)
  }
