@file:Suppress("unused")

package com.jetbrains.rd.ui.bedsl.dsl

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.ui.bedsl.dsl.util.BeLocalCookie
import com.jetbrains.rd.ui.bedsl.extensions.bind
import com.jetbrains.rd.ui.bedsl.extensions.descendants

fun toggleButton(
  beControl: BeAbstractText,
  lifetime: Lifetime,
  id: String = "", style: BeButtonStyle, onClick: (Boolean) -> Unit) {

}

fun checkbox(text: BeAbstractText, initialValue: Boolean = false, enabled: Boolean = true, id: String? = ""): BeCheckbox {
  val beCheckbox = BeCheckbox(text)
  beCheckbox.beInit(id, enabled = enabled)
  beCheckbox.property.value = initialValue
  return beCheckbox
}

fun checkbox(text: String,
             initialValue: Boolean = false,
             enabled: Boolean = true,
             id: String? = "",
             wrapText: Boolean = false): BeCheckbox {
  return checkbox(labelWithShortcut(text, wrapText = wrapText), initialValue, enabled, id)
}

fun checkbox(
  text: String,
  property: IProperty<Boolean?>,
  lifetime: Lifetime,
  enabled: Boolean = true,
  id: String? = "",
  wrapText: Boolean = false
): BeCheckbox {
  return checkbox(labelWithShortcut(text, wrapText = wrapText), property.value ?: false, enabled, id).apply {
    property.bind(lifetime, this.property) //todo:create bind to bool?
  }
}

fun checkbox(
  text: BeAbstractText,
  property: IProperty<Boolean?>,
  lifetime: Lifetime,
  enabled: Boolean = true,
  id: String? = ""
): BeCheckbox {
  return checkbox(text, property.value ?: false, enabled, id).apply {
    property.bind(lifetime, this.property) //todo:create bind to bool?
  }
}

fun radiobutton(text: BeAbstractText, initialValue: Boolean = false, enabled: Boolean = true, id: String = ""): BeRadioButton {
  val radioButton = BeRadioButton(text)
  radioButton.beInit(id, enabled = enabled)
  radioButton.property.value = initialValue
  return radioButton
}

fun <T> radiobutton(value: T,
                    initialValue: Boolean = false,
                    enabled: Boolean = true,
                    id: String = "",
                    wrapText: Boolean = false,
                    convert: (T) -> String): BeRadioButton {
  return radiobutton(labelWithShortcut(convert(value), wrapText = wrapText), initialValue, enabled, id)
}

fun radiobutton(text: String,
                initialValue: Boolean = false,
                enabled: Boolean = true,
                id: String = "",
                wrapText: Boolean = false): BeRadioButton {
  return radiobutton(labelWithShortcut(text, wrapText = wrapText), initialValue, enabled, id)
}

fun radiobutton(
  text: String,
  property: IProperty<Boolean>,
  lifetime: Lifetime,
  initialValue: Boolean = false,
  enabled: Boolean = true,
  id: String = "",
  wrapText: Boolean = false
): BeRadioButton {
  return radiobutton(text, initialValue, enabled, id, wrapText).apply {
    property.bind(lifetime, this.property)
  }
}

fun radiobutton(text: BeAbstractText,
                property: IProperty<Boolean>,
                lifetime: Lifetime,
                initialValue: Boolean = false,
                enabled: Boolean = true,
                id: String = ""): BeRadioButton {
  return radiobutton(text, initialValue, enabled, id).apply {
    property.bind(lifetime, this.property)
  }
}

fun radioGroup(text: String,
               selected: String,
               nonSelected: String,
               property: IProperty<Boolean>,
               lifetime: Lifetime,
               enabled: Boolean = true): BeControl {
  val description = labelWithShortcut(text)
  val selectedRadio = radiobutton(selected, property, lifetime) // id
  val nonSelectedRadio = radiobutton(nonSelected, !property.value) // id
  property.change.advise(lifetime) { if (it) nonSelectedRadio.property.value = false }
  nonSelectedRadio.property.advise(lifetime) { if (it) property.value = false }
  val hasMargin = text.isNotBlank()
  return verticalGrid {
    row { description }
    row { if (hasMargin) selectedRadio.withMargin { checkBoxSizeMargin } else selectedRadio }
    row { if (hasMargin) nonSelectedRadio.withMargin { checkBoxSizeMargin } else nonSelectedRadio }
  }.apply {
    //todo: bind descendants with enabled to one method
    this.enabled.advise(lifetime) {
      description.enabled.value = it
      selectedRadio.enabled.value = it
      nonSelectedRadio.enabled.value = it
    }
    this.enabled.value = enabled
  }

}

fun <T> radioGroup(
  text: String,
  property: IProperty<T>,
  lifetime: Lifetime,
  selected: List<T>,
  getRadioButton: (T) -> BeRadioButton
): BeControl {
  val cookie = BeLocalCookie()
  return radioGroup(text, lifetime) {
    selected.map {
      getRadioButton(it).apply {
        this.property.advise(lifetime) { v -> if (v) cookie.Execute { property.value = it } }
        property.change.advise(lifetime) { v: T ->
          cookie.Execute {
            if (it == v) {
              this.property.value = true
            }
          }
        }
      }
    }
  }
}

inline fun <reified T : Enum<T>> radioGroup(description: String,
                                            property: IProperty<T>,
                                            lifetime: Lifetime,
                                            crossinline getText: (T) -> String = { it.name }): BeControl {
  return radioGroup(description, property, lifetime, enumValues<T>().toList()) { radiobutton(getText(it)) }
}

fun radioGroup(description: String, lifetime: Lifetime, radioButtons: () -> List<BeRadioButton>): BeControl {
  val localCookie = BeLocalCookie()
  val buttons = radioButtons()
  val hasMargin = description.isNotBlank()

  val selectedIndex = Property(buttons.indexOfFirst { it.enabled.value })
  return verticalGrid {
    row { labelWithShortcut(description) }
    rows {
      buttons.mapIndexed { i, rb ->
        selectedIndex.advise(lifetime) {
          if (i != it) rb.property.value = false
          else localCookie.Execute { rb.property.value = true }
        }
        rb.property.advise(lifetime) { localCookie.Execute { if (it) selectedIndex.value = i } }
        if (hasMargin) rb.withMargin { checkBoxSizeMargin } else rb
      }
    }
  }.apply {
    this.enabled.advise(lifetime) {
      this.descendants().forEach { d -> if (d != this) d.enabled.value = it }
    }
  }
}



