package com.jetbrains.rd.ui.bedsl.demo

import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.project.DumbAwareAction
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.dsl.okCancelDialog
import com.jetbrains.rd.ui.bedsl.dsl.showDialog
import com.jetbrains.rd.ui.bedsl.dsl.tabbedControl
import com.jetbrains.rd.ui.bedsl.dsl.util.BeControlSizes
import com.jetbrains.rd.util.lifetime.Lifetime
import org.jetbrains.annotations.Nullable

abstract class BeControlsDemoAction(text:String) : DumbAwareAction(text) {

  fun getContent(lifetime: Lifetime, project: Project): BeControl {
    val model = DemoUITestModel(lifetime)
    return tabbedControl(lifetime) {
      tab("Labels") { DemoLabelsTab(model).content(lifetime) }
      tab("Toggles") { DemoTogglesTab(model).content(lifetime) }
      tab("Buttons") { DemoButtonsTab(model, project).content(lifetime) }
      tab("Comboboxes") { DemoComboTab(model).content(lifetime) }
      tab("Editable") { DemoEditableControlsTab(model).content(lifetime) }
      tab("Tree grids") { DemoTreeGridTab(model).content(lifetime) }
      //psi
    }
  }
  override fun actionPerformed(e: AnActionEvent) {
    val project = e.project ?: return
    //ClientId
    //todo: fix me, convert to proper protocol interaction model call
    showDialog(project, null, { dialog, lifetime -> this.realShow(dialog, project, lifetime) }) { getDialog(it, project) }
  }

  abstract fun realShow(it: BeDialog, project: @Nullable Project, lifetime: Lifetime)

  private fun getDialog(lifetime: Lifetime,
                        project: @Nullable Project) = okCancelDialog(
    "Showcase for be controls dsl",
    "DemoAction.MainDialog",
    lifetime,
    modality = DialogModality.NON_MODAL,
    size = BeControlSizes.getSize(BeControlSizeType.HUGE, BeControlSizeType.LARGE),
    scrollbarPolicy = BeScrollbarPolicy.NONE
  ) {
    getContent(lifetime, project)
  }

}