package com.jetbrains.rd.ui.bedsl.demo

import com.jetbrains.ide.model.uiautomation.BeButton
import com.jetbrains.ide.model.uiautomation.BeComboBox
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.ui.bedsl.button
import com.jetbrains.rd.ui.bedsl.dsl.*
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.ViewableList

class DemoComboTab(private val model: DemoUITestModel) {
  fun content(lifetime: Lifetime): BeControl {
    val strings = mutableListOf("one", "two", "three")
    val ints = mutableListOf(1, 2, 3)
    val mutableStrings = ViewableList(mutableListOf("1", "2", "3"))
    val mutableInts = ViewableList(mutableListOf(1, 2, 3))

    val labelWithResult = label("Nothing selected now")
    return verticalGrid {
      row {
        horizontalGrid {
          column {
            verticalGrid {
              row { combobox(lifetime, strings) { labelWithResult.setText(it) } }
              row {
                combobox(lifetime, ints, handleSelected = { labelWithResult.setText(it.toString()) })
                { label("$it number").withMargin { checkBoxInTreeMargin } }
              }
            }
          }
          column { labelWithResult }
        }
      } // simple combobox with handler from strings, and T

      row {
        horizontalGrid {
          column {
            verticalGrid {
              row { combobox(model.StringProperty, lifetime, strings) }
              row { combobox(model.IntProperty, lifetime, ints) { label("$it number") } }
            }
          }
          column { label(model.StringProperty, lifetime) }
        }
      } //simple comboboxes with values from property
      row {
        comboboxFromEnum(model.EnumProperty, lifetime)
      } // combobox with values from enum

      row {
        val comboboxWithStrings = combobox(model.StringProperty, lifetime, mutableStrings)
        val comboboxWithInts = combobox(model.IntProperty, lifetime, mutableInts) { label("$it number") }

        horizontalGrid {
          column {
            verticalGrid {
              row { comboboxWithStrings }
              row { comboboxWithInts }
            }
          }
          column { getCheckButton(lifetime, mutableStrings, mutableInts, comboboxWithStrings, comboboxWithInts, strings, ints) }
        }
      }
      // comboboxes with mutable content
      row {
        verticalGrid("auto,auto") {
          val stringProperty = Property("Test")
          val labelWithResult2 = label("")

          newRowChild(columnSpan = 1) { editableCombobox(stringProperty, lifetime, strings) }
          existingRowChild(columnSpan = 1) { label(stringProperty, lifetime) }

          newRowChild(columnSpan = 1) { editableCombobox(lifetime, strings) { labelWithResult2.setText(it) } }
          existingRowChild(columnSpan = 1) { labelWithResult2 }
        }
      } //editable combo with property and setter
      row {
        val editableCombobox = editableCombobox(model.StringProperty, lifetime, ints,
                                                presentation = { label(it.toString()) }) { Integer.getInteger(it) }
        verticalGrid {
          row {
            editableCombobox
          }
          row {
            editableCombobox(model.StringProperty, lifetime, mutableInts, presentation = { label(it.toString()) }) {
              Integer.getInteger(it)
            }
          }
        } //editable from different type
      }
    }
  }

  private fun getCheckButton(
    lifetime: Lifetime,
    mutableStrings: ViewableList<String>,
    mutableInts: ViewableList<Int>,
    comboboxWithStrings: BeComboBox,
    comboboxWithInts: BeComboBox,
    strings: MutableList<String>,
    ints: MutableList<Int>
  ): BeButton {
    return button("Check", lifetime) {
      mutableStrings.clear()
      mutableInts.clear()
      mutate(mutableStrings, mutableInts, comboboxWithStrings, comboboxWithInts, -1)
      mutate(mutableStrings, mutableInts, comboboxWithStrings, comboboxWithInts, 0)
      mutableStrings.addAll(strings)
      mutableInts.addAll(ints)

      comboboxWithStrings.selectedIndex.value = 1
      comboboxWithInts.selectedIndex.value = 1
    }
  }

  private fun mutate(
    mutableStrings: ViewableList<String>,
    mutableInts: ViewableList<Int>,
    comboboxWithStrings: BeComboBox,
    comboboxWithInts: BeComboBox,
    removeAt: Int
  ) {
    for (i in 0..4) {
      for (j in 0..4) {
        mutableStrings.add(j.toString())
        mutableInts.add(j)
      }

      comboboxWithStrings.selectedIndex.value = i
      comboboxWithInts.selectedIndex.value = i

      for (j in 4 downTo 0) {
        val removeIndex = if (removeAt < 0) j else removeAt
        mutableStrings.removeAt(removeIndex)
        mutableInts.removeAt(removeIndex)
      }
    }
  }
}



