package com.jetbrains.rd.ui.bedsl.demo

import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.ui.bedsl.dsl.*
import com.jetbrains.rd.ui.bedsl.extensions.bind

class DemoTogglesTab(private val model: DemoUITestModel) {
  fun content(lifetime: Lifetime): BeControl {
    return tabbedControl(lifetime) {
      tab("Checkboxes") { checkBoxesTab(lifetime) }
      tab("Radio buttons") { radioButtonsTab(lifetime) }
      tab("Radio groups") { radioGroupsTab(lifetime) }
      //tab("Toggle buttons") { togglesTab(lifetime) }
    }
  }

  private fun checkBoxesTab(lifetime: Lifetime): BeControl {
    return verticalGrid {
      val selectionProperty = Property(true)
      row { radioGroup("Make all controls below ", "Enabled", "Disabled", selectionProperty, lifetime) }
      row {
        checkbox("C_heckbox with shortcut").apply {
          selectionProperty.bind(lifetime, this.enabled)
        }
      }
      row {
        checkbox(
          "`This is checkbox with ${model.LongText} text with wrap checking: _wrap",
          model.BoolQProperty, lifetime, wrapText = true
        ).apply {
          selectionProperty.bind(lifetime, this.enabled)
        }
      }

      row {
        checkbox(
          "`This is checkbox with kinda ${model.LongText.take(50)} text with wrap checking: _no wrap",
          model.BoolQProperty, lifetime
        ).apply {
          selectionProperty.bind(lifetime, this.enabled)
        }
      }

      row {
        description("Checkbox with no text", lifetime) {
          checkbox("").apply { selectionProperty.bind(lifetime, this.enabled) }
        }
      }
    }
  }

  private fun radioButtonsTab(lifetime: Lifetime): BeControl {
    return verticalGrid {
      val selectionProperty = Property(true)
      row { radioGroup("Make all controls below ", "Enabled", "Disabled", selectionProperty, lifetime) }
      row {
        radiobutton("R_adiobutton with shortcut").apply {
          selectionProperty.bind(lifetime, this.enabled)
        }
      }
      row {
        radiobutton(
          "`This is radiobutton with ${model.LongText} text with wrap checking: _wrap",
          model.BoolProperty, lifetime, wrapText = true
        ).apply {
          selectionProperty.bind(lifetime, this.enabled)
        }
      }

      row {
        radiobutton(
          "`This is radiobutton with kinda ${model.LongText.take(50)} text with wrap checking: _no wrap",
          model.BoolProperty, lifetime
        ).apply {
          selectionProperty.bind(lifetime, this.enabled)
        }
      }

      row {
        description("Radiobutton with no text", lifetime) {
          radiobutton("").apply { selectionProperty.bind(lifetime, this.enabled) }
        }
      }
    }
  }

  private fun radioGroupsTab(lifetime: Lifetime): BeControl {
    val stringValues = listOf("one", "two", "three")
    return verticalGrid {
      val selectionProperty = Property(true)
      row { radioGroup("Make all controls below ", "Enabled", "Disabled", selectionProperty, lifetime) }
      row {
        radioGroup("Select one:", model.StringProperty, lifetime, stringValues) {
          radiobutton(it)
        }
      }
    }
  }
}

