package com.jetbrains.rd.ui.bedsl.demo

import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.DefaultActionGroup
import com.intellij.openapi.diagnostic.thisLogger
import com.intellij.openapi.diagnostic.trace
import com.intellij.openapi.project.DumbAwareAction
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeToolbar
import com.jetbrains.ide.model.uiautomation.BeTreeGrid
import com.jetbrains.ide.model.uiautomation.BeTreeSelection
import com.jetbrains.rd.ui.bedsl.dsl.*
import com.jetbrains.rd.ui.bedsl.dsl.util.TreeConfiguration
import com.jetbrains.rd.ui.bedsl.dsl.util.TreeSettings
import com.jetbrains.rd.util.lifetime.Lifetime

class DemoTreeGridTab(val model: DemoUITestModel) {
  fun content(lifetime: Lifetime): BeControl {
    return singleSelectionTable3(lifetime)
    /* tabbedControl(lifetime) {
        tab("Table 1") {
          singleSelectionTable(lifetime)
        }
      }
        tab("Table 2") {
          verticalGrid {
            row(BeSizingType.Fill) { checkboxMultiSelectionTable(lifetime) }
          }
        }
        tab("Table 2") {
          singleSelectionTable2(lifetime)
        }
        tab("Table with toolbar 1") {
          singleSelectionTable3(lifetime)

        }
        tab("Tree 1") {
          verticalGrid {
            row(BeSizingType.Fill) { singleSelectionTree(lifetime) }
          }
        }
        tab("Tree 2") {
          verticalGrid {
            row(BeSizingType.Fill) { checkboxMultiSelectionTree(lifetime) }
          }
        }
      }*/
  }

  private fun singleSelectionTable(lifetime: Lifetime): BeTreeGrid {
    return table(model.SingleSelectionTableElements, lifetime, TreeConfiguration.oneColumn()) { _, it, _ ->
      return@table listOf(label(it.name).withMargin { margin(10, 3, 10, 1) })
    }
  }

  private fun singleSelectionTable2(lifetime: Lifetime): BeTreeGrid {
    return singleSelectionTable(model.SingleSelectionTableElement,
                                model.SingleSelectionTableElements,
                                lifetime,
                                listOf("a, auto", "b, *")
    ) { lt, it, lp ->
      return@singleSelectionTable listOf(label(it.name), label(it.index.toString()))
    }
  }

  fun singleSelectionTable3(lifetime: Lifetime): BeToolbar {
    return singleSelectionTableWithToolbar(
      model.SingleSelectionTableElement,
      model.ViewableSingleSelectionTableElements,
      lifetime,
      listOf("a, auto", "b, *")
    ) { lt, it, lp ->
      return@singleSelectionTableWithToolbar listOf(label(it.name), label(it.index.toString()))
    }.init {
      buttonWithAction(BeTableAddAction.ADD, { DemoTestElement("T1", 0) })
      buttonWithAction<DemoTestElement>(BeTableAction.REMOVE)
      actionButton("BackendBeControlsDSLShowCase", lifetime)
      actionButton("RdClient.internal", lifetime)

      val id = "DemoTreeGridTab.CreatedAction"
      val action = object : DumbAwareAction("My Custom Action") {
        override fun actionPerformed(e: AnActionEvent) = thisLogger().trace { "Received execute request for $id" }
        override fun update(e: AnActionEvent) {
          val presentation = e.presentation
          presentation.description = if (model.SingleSelectionTableElements.count() % 2 == 0) "Action 0" else "Action 1"
          presentation.text = if (model.SingleSelectionTableElements.count() % 2 == 0) "Action text 0" else "Action text 1"
          //presentation.icon = if(model.SingleSelectionTableElements.count()%2 == 0) "Action 0" else "Action 1"
          thisLogger().trace { "Received update request for $id" }
        }
      }
      //action.templatePresentation.text = "My Custom Action"

      val action2 = object : DumbAwareAction("My Custom Action 2") {
        override fun actionPerformed(e: AnActionEvent) = thisLogger().trace { "Received execute request for $id" }
        override fun update(e: AnActionEvent) {
          val presentation = e.presentation
          presentation.description = if (model.SingleSelectionTableElements.count() % 2 == 0) "Action 0" else "Action 2"
          presentation.text = if (model.SingleSelectionTableElements.count() % 2 == 0) "Action text 0" else "Action text 2"
          //presentation.icon = if(model.SingleSelectionTableElements.count()%2 == 0) "Action 0" else "Action 1"
          thisLogger().trace { "Received update request for $id" }
        }
      }
      //action.templatePresentation.text = "My Custom Action"
      var group = DefaultActionGroup(action, action2)
      var group2 = DefaultActionGroup(action, action2)
      group2.isPopup = true
      group.templatePresentation.text = "MyGroupFromBackend 1"
      group.templatePresentation.text = "MyGroupFromBackend 2"
      actionButton(group, lifetime)
      actionButton(group2, lifetime)
    }
  }

  fun singleSelectionTree(lifetime: Lifetime): BeTreeGrid {
    return tree(model.SingleSelectionTableElements, lifetime, TreeConfiguration.oneColumn(),
                { t -> t?.children ?: mutableListOf<DemoAnotherTestElement>() })
    { lt, it, lp, tp ->
      return@tree listOf(label(it?.name ?: "empty"))
    }
  }

  fun checkboxMultiSelectionTree(lifetime: Lifetime): BeTreeGrid {
    return treeWithCheckBoxes(model.MultiSelectionTableElements, lifetime, TreeConfiguration.oneColumn(),
                              { t -> t?.children ?: mutableListOf<DemoAnotherTestElement>() })
    { lt, it, lp, tp ->
      return@treeWithCheckBoxes listOf(label(it!!.name), label(it.index.toString()))
    }
  }

  fun checkboxMultiSelectionTable(lifetime: Lifetime): BeTreeGrid {
    return tableWithCheckBoxes(
      model.MultiSelectionTableElements,
      lifetime,
      TreeConfiguration.columnNames(listOf("name,*", "index,*"),
                                    TreeSettings(selection = BeTreeSelection.Multi)
      )
    )
    { lt, it, lp ->
      return@tableWithCheckBoxes listOf(label(it.name), label(it.index.toString()))
    }
  }
}
