package com.jetbrains.rd.ui.bedsl.extensions

import com.jetbrains.ide.model.uiautomation.BeCollapsiblePanel
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.LifetimeDefinition
import com.jetbrains.rd.util.lifetime.isAlive
import com.jetbrains.rd.util.lifetime.isNotAlive
import com.jetbrains.rd.util.reactive.*

fun <TValue> IProperty<TValue>.bind(lifetime: Lifetime, point2: IProperty<TValue>) {
  this.change.advise(lifetime) { args ->
    if (args == null) return@advise
    if (lifetime.isNotAlive) return@advise
    if (args != point2.value) point2.set(args)
  }

  point2.change.advise(lifetime
  ) { args ->
    if (args == null) return@advise
    if (lifetime.isNotAlive) return@advise
    if (args != this.value) this.set(args)
  }
  point2.set(this.value)
}

fun <TValue : Any> IProperty<TValue>.bind(lifetime: Lifetime, point2: IOptProperty<TValue>) {
  this.change.advise(lifetime) { args ->
    if (!lifetime.isAlive) return@advise
    point2.set(args)
  }

  point2.change.advise(lifetime) { args ->
    if (lifetime.isNotAlive) return@advise
    if (args != this.value) this.set(args)
  }

  point2.set(this.value)
}

fun <TValue : Any> IOptProperty<TValue>.bind(lifetime: Lifetime, point2: IOptProperty<TValue>) {
  this.change.advise(lifetime) { args ->
    if (lifetime.isNotAlive) return@advise
    if (args != point2.valueOrNull) point2.set(args)
  }

  point2.change.advise(
    lifetime
  ) { args ->
    if (lifetime.isNotAlive) return@advise
    if (args != this.valueOrNull) this.set(args)
  }
  if (this.valueOrNull == null) return
  point2.set(this.valueOrNull!!)
}

fun Collection<IProperty<Boolean>>.setSingleSelection(lifetime: Lifetime) {
  val rdProperties = this.toList()
  //foreach (var element in rdProperties) element.Value = false;
  var selectedIndex = 0
  rdProperties.forEachIndexed { index, element ->
    //var modelElement = element;
    element.view(lifetime) { _, e ->

      if (!e || selectedIndex == index) return@view
      for (property in rdProperties)
        if (property != element)
          property.set(false)
      selectedIndex = index
    }
  }


  if (rdProperties.any() && !rdProperties.any({ it.value }))
    rdProperties.first().set(true)
}

fun BeControl.inCollapsiblePanel(
  title: String,
  initiallyExpanded: Boolean = true
): BeCollapsiblePanel {
  val panel = BeCollapsiblePanel(this)
  panel.headerText.set(title)
  panel.isExpanded.set(initiallyExpanded)
  return panel
}

inline fun <reified T : Any> BeControl.extend(value: T, id: String) {
  this.getOrCreateExtension("BeUtil.$id") { value }
}

inline fun <reified T : Any> BeControl.getExtended(id: String): T? {
  return this.getOrCreateExtension("BeUtil.$id") { Any() } as? T
}
/*
@ExperimentalStdlibApi
inline fun <reified T : Any> BeControl.extend(value: T) {
  this.getOrCreateExtension("BeUtil.${typeOf<T>().javaClass.name}", { value })
}

@ExperimentalStdlibApi
inline fun <reified T : Any> BeTreeConfiguration.extend(value: T) {
  this.getOrCreateExtension("BeUtil.${typeOf<T>().javaClass.name}", { value })
}

@ExperimentalStdlibApi
inline fun <reified T : Any> BeControl.getSafeExtended(value: T, default: T) {
  this.getOrCreateExtension("BeUtil.${typeOf<T>().javaClass.name}", { default })
}*/

fun <V : Any> ViewableList<V>.view(lifetime: Lifetime, handler: (Lifetime?, Int, V, AddRemove) -> Unit) {
  val lifetimes = mutableListOf<LifetimeDefinition>()
  adviseAddRemove(lifetime) { kind, idx, value ->
    when (kind) {
      AddRemove.Add -> {
        val ldef = lifetime.createNested()
        lifetimes.add(idx, ldef)
        handler(ldef.lifetime, idx, value, kind)

      }
      AddRemove.Remove -> {
        handler(null, idx, value, kind)
        lifetimes.removeAt(idx).terminate()
      }
    }

  }
}

fun <T1 : Any, T> Property<T?>.compose(all: Iterable<IOptProperty<T1>>, lifetime: Lifetime,
                                       composer: (Iterable<T1?>) -> T?) {
  for (readonlyProperty in all)
    readonlyProperty.advise(lifetime) { this.value = composer(all.map { it.valueOrNull }) }
}