package com.jetbrains.rd.ui.bedsl.extensions

import com.intellij.openapi.wm.IdeFocusManager
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JComponent
import javax.swing.SwingUtilities


fun <TModel : BeControl, TControl : JComponent> TModel.bindFocus(
  lifetime: Lifetime,
  component: TControl,
  afterFocused: ((TModel, TControl) -> Unit)? = null
) {
  focus.advise(lifetime) {
    val focusManager = IdeFocusManager.getGlobalInstance()
    focusManager.doWhenFocusSettlesDown {
      SwingUtilities.invokeLater {
        focusManager.getFocusTargetFor(component)?.requestFocusInWindow()
        afterFocused?.invoke(this, component)
      }
    }
  }
}