package com.jetbrains.rd.ui.bedsl.extensions

import com.intellij.util.ui.JBScalableIcon
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IOptProperty
import com.jetbrains.rd.util.reactive.ViewableList


fun BeComboBox.withValidationRule(lifetime: Lifetime,
                                  validate: (String) -> Boolean,
                                  errorMessage: String,
                                  shouldBeGuarded: Boolean,
                                  validationState: ValidationStates = ValidationStates.validationError,
                                  customIcon: JBScalableIcon? = null): BeComboBox {
  return this.withValidationRule<BeComboBox, String>(lifetime,
                                                     { it -> if (!validate(it)) errorMessage else null }, shouldBeGuarded, validationState,
                                                     customIcon)
}

fun BeComboBox.withValidationRule(lifetime: Lifetime,
                                  validate: (String) -> String,
                                  shouldBeGuarded: Boolean,
                                  validationState: ValidationStates = ValidationStates.validationError,
                                  customIcon: JBScalableIcon? = null): BeComboBox {
  return this.withValidationRule<BeComboBox, String>(lifetime, validate, shouldBeGuarded, validationState, customIcon)
}

fun BeTextBox.withValidationRule(lifetime: Lifetime,
                                 validate: (String) -> Boolean,
                                 errorMessage: String,
                                 shouldBeGuarded: Boolean,
                                 validationState: ValidationStates = ValidationStates.validationError,
                                 customIcon: JBScalableIcon? = null): BeTextBox {
  return this.withValidationRule<BeTextBox, String>(lifetime,
                                                    { it -> if (!validate(it)) errorMessage else null }, shouldBeGuarded, validationState,
                                                    customIcon)
}

fun BeTextBox.withValidationRule(lifetime: Lifetime,
                                 validate: (String) -> String,
                                 shouldBeGuarded: Boolean,
                                 validationState: ValidationStates = ValidationStates.validationError,
                                 customIcon: JBScalableIcon? = null): BeTextBox {
  return this.withValidationRule<BeTextBox, String>(lifetime, validate, shouldBeGuarded, validationState, customIcon)
}

fun <TControlType : BeControlWithValidation, T : Any> TControlType.withValidationRule(lifetime: Lifetime,
                                                                                      validate: (T) -> String?,
                                                                                      shouldBeGuarded: Boolean,
                                                                                      validationState: ValidationStates = ValidationStates.validationError,
                                                                                      customIcon: JBScalableIcon? = null): TControlType {
  /*var icon = (customIcon == null)
  ? GetIcon(validationState)
  : customIcon;*/
  val rule = CustomValidationRule<T>(validate, validationState, null, shouldBeGuarded)
  return this.withValidationRule(lifetime, rule)
}

fun <TControlType : BeControlWithValidation, T : Any> TControlType.withValidationRule(lifetime: Lifetime,
                                                                                      rule: IValidationRule<T>): TControlType {
  tryAddRule(this, lifetime, rule)
  return this
}

private fun <T : Any> tryAddRule(control: BeControlWithValidation, lifetime: Lifetime, rule: IValidationRule<T>) {
  if (control is BeComboBox)
    control.getOrCreateExtension("BeComboBoxValidation",
                                 { ValidationRules(control, { comboBox -> comboBox.text as IOptProperty<T> }, lifetime).Rules.add(rule) })

  if (control is BeTextBox)
    control.getOrCreateExtension("BeTextBoxValidation",
                                 { ValidationRules(control, { textBox -> textBox.text as IOptProperty<T> }, lifetime).Rules.add(rule) })
}

interface IValidationRule<in T> {
  val state: ValidationStates
  val icon: JBScalableIcon?
  val shouldBeGuarded: Boolean
  fun validate(text: T): String?
}

class CustomValidationRule<in T>(private val myValidate: (T) -> String?,
                                 override val state: ValidationStates,
                                 override val icon: JBScalableIcon?,
                                 override val shouldBeGuarded: Boolean) :
  IValidationRule<T> {
  override fun validate(text: T): String? {
    return myValidate(text)
  }
}


class ValidationRules<ControlType : BeControlWithValidation, T : Any> //, "ValidationRules.Rules");
(controlWithValidation: ControlType, getValue: (ControlType) -> IOptProperty<T>, lifetime: Lifetime) {
  var Rules: ViewableList<IValidationRule<T>> = ViewableList()

  private fun validate(canBeGuarded: Boolean, textValue: T?, textBoxValidationResult: IOptProperty<ValidationResult>) {
    val text = textValue
    if (text == null) {
      textBoxValidationResult.set(ValidationResult(ValidationStates.validationError, "Current value is null"))
      return
    }
    //todo: if error and warning exits altogether - error should be shown
    var result = ValidationResult(ValidationStates.validationPassed, "")
    //todo
    //result.icon.{Icon = {Value = null}, HasIcon = {Value = true}};
    for (validationRule in Rules) {
      if (!canBeGuarded && validationRule.shouldBeGuarded) continue
      var localResult: ValidationResult? = null
      /* if (validationRule.shouldBeGuarded *//*&& !Shell.Instance.IsTestShell*//*)
            {
            //todo change guarded execution
                var shellLocks = Shell.Instance.GetComponent<IShellLocks>();
                shellLocks.ReentrancyGuard.ExecuteOrQueue("TextDataModel.Validate",
                    () => ReadLockCookie.Execute(() =>
                {
                    localResult = ValidationResult(text, validationRule);
                    if (localResult == null) return;

                    switch (result.State)
                    {
                        case ValidationStates.validationPassed:
                        result = localResult;
                        break;
                        case ValidationStates.validationWarning:
                        if (localResult.State == ValidationStates.validationError)
                            result = localResult;
                        break;
                        case ValidationStates.validationError:
                        break;
                        default:
                        throw new ArgumentOutOfRangeException();
                    }
                    textBoxValidationResult.Value = result;
                }));
            }
            else
            {*/
      localResult = validationResult(text, validationRule)
      //}

      if (localResult == null) continue

      if (result.state == ValidationStates.validationPassed)
        result = localResult
      else if (result.state == ValidationStates.validationWarning)
        if (localResult.state == ValidationStates.validationError)
          result = localResult
      /*else
      *//*case ValidationStates.validationError:*//*
                break;*/
    }
    textBoxValidationResult.set(result)
  }

  private fun validationResult(text: T, validationRule: IValidationRule<T>): ValidationResult? {
    var validateResult: ValidationResult? = null
    val validationError = validationRule.validate(text)
    if (validationError != null) {
      /*var iconHost = Shell.Instance.GetComponent<IconHostBase>();
      var icon = iconHost.Transform(validationRule.Icon ?? validationRule.State.GetIcon());*/

      validateResult = ValidationResult(validationRule.state, validationError)
      //{Icon = {Value = icon}, HasIcon = {Value = true}};

    }

    return validateResult
  }

  init {
    val textValue = getValue(controlWithValidation)
    textValue.advise(lifetime, { validate(true, it, controlWithValidation.validationResult) })
    controlWithValidation.enabled.advise(lifetime, { v ->
      if (v)
        validate(true, getValue(controlWithValidation).valueOrNull, controlWithValidation.validationResult)
      else
        controlWithValidation.validationResult.set(ValidationResult(ValidationStates.validationPassed, ""))
    })
    this.Rules.adviseAddRemove(lifetime, { e, i, r ->
      validate(true, getValue(controlWithValidation).valueOrNull, controlWithValidation.validationResult)
    })
  }
}
