package com.jetbrains.rd.ui.bedsl.dsl.util

import com.jetbrains.ide.model.uiautomation.BeMargin
import com.jetbrains.ide.model.uiautomation.BeMarginItem
import com.jetbrains.ide.model.uiautomation.BeMarginType

typealias BeDslMargin = Pair<BeMarginType, Int>

class BeMarginsBuilder {
  /*public static BeMargin DEFAULT = Create()
  public static BeMargin EMPTY = Create()*/


  fun margin(
    left: BeMarginType = BeMarginType.None,
    top: BeMarginType = BeMarginType.None,
    right: BeMarginType = BeMarginType.None,
    bottom: BeMarginType = BeMarginType.None): BeMargin {
    return margin(
      BeDslMargin(left, 1),
      BeDslMargin(top, 1),
      BeDslMargin(right, 1),
      BeDslMargin(bottom, 1)
    )
  }

  fun margin(left: Int = 0, top: Int = 0, right: Int = 0, bottom: Int = 0): BeMargin {
    return margin(
      BeDslMargin(BeMarginType.OnePx, left),
      BeDslMargin(BeMarginType.OnePx, top),
      BeDslMargin(BeMarginType.OnePx, right),
      BeDslMargin(BeMarginType.OnePx, bottom)
    )
  }

  fun margin(
    left: BeDslMargin,
    top: BeDslMargin = emptyMargin,
    right: BeDslMargin = emptyMargin,
    bottom: BeDslMargin = emptyMargin): BeMargin {

    return BeMargin(
      BeMarginItem(left.first, right.second),
      BeMarginItem(top.first, top.second),
      BeMarginItem(right.first, right.second),
      BeMarginItem(bottom.first, bottom.second))
  }

  fun BeMargin.isEmpty(): Boolean {
    return this.bottom.unit == BeMarginType.None
           && this.left.unit == BeMarginType.None
           && this.right.unit == BeMarginType.None
           && this.top.unit == BeMarginType.None
  }

  val emptyMargin = BeDslMargin(BeMarginType.None, 1)
  val treeLineMargin = margin(
    BeDslMargin(BeMarginType.OnePx, 1),
    BeDslMargin(BeMarginType.OnePx, 1),
    BeDslMargin(BeMarginType.OnePx, 1),
    BeDslMargin(BeMarginType.OnePx, 1)
  )

  val popupMargin = margin(
    BeDslMargin(BeMarginType.OnePx, 5),
    BeDslMargin(BeMarginType.OnePx, 5),
    BeDslMargin(BeMarginType.OnePx, 5),
    BeDslMargin(BeMarginType.OnePx, 5)
  )

  val checkBoxSizeMargin = margin(
    BeDslMargin(BeMarginType.Checkbox, 1)
  )

  val checkBoxInTreeMargin = margin(
    BeDslMargin(BeMarginType.OnePx, 2),
    right = BeDslMargin(BeMarginType.OnePx, 2)
  )
}