package com.jetbrains.rd.ui.bedsl.dsl.util

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.dsl.getUnitSize

data class ColumnDefinition(val name: String, val size: BeUnitSize, val alignment: BeAlignment, var comparer: Any? = null) {
  companion object {
    fun parse(sizeAndName: String): ColumnDefinition {
      val headerAndSize = sizeAndName.split(',')
      val size = if (headerAndSize.count() == 2) headerAndSize[1] else "auto"
      val beSizingType = if (size.trim(' ') == "*") BeSizingType.Fill else BeSizingType.Fit
      return ColumnDefinition(
        headerAndSize[0],
        getUnitSize(beSizingType, 1),
        BeAlignment.Stretch
      )
    }
  }
}

data class TreeSettings(
  val hasHeader: Boolean = true,
  val childOffsetSize: ChildOffsetSize = ChildOffsetSize.DEFAULT,
  val autoScrollToEnd: BeScrollSettings = BeScrollSettings.DO_NOT_AUTOSCROLL,
  var showAsList: Boolean = false,
  val showBorders: BeShowBorders = BeShowBorders.All,
  val selection: BeTreeSelection = BeTreeSelection.Single,
  val autoStartEditing: Boolean = false,
  val backendContextMenu: BeBackendContextMenu? = null,
  val selectOnMouseOver: Boolean  = false
)

data class TreeConfiguration private constructor(
  val settings: TreeSettings = TreeSettings(),
  val columnElements: List<ColumnDefinition>?,
  private val initialComparer: Any? = null
) {

  //val DefaultSortKey: SortKey? = null //todo

  fun getBeConfiguration(): BeTreeConfiguration {
    val hasContextMenu = settings.backendContextMenu != null
    val columnDefinitions = if (columnElements.isNullOrEmpty())
      mutableListOf<BeColumnDefinition>()
        .apply {
          add(BeColumnDefinition(" ",
                                 getUnitSize(BeSizingType.Fill, 1),
                                 BeAlignment.Stretch, initialComparer != null))
        }
    else columnElements.map { BeColumnDefinition(it.name, it.size, it.alignment, it.comparer != null) }
    val beTreeConfiguration = BeTreeConfiguration(
      columnDefinitions, settings.selection, settings.hasHeader, settings.childOffsetSize,
      settings.showAsList, settings.autoScrollToEnd == BeScrollSettings.AUTOSCROLL_TO_END, settings.showBorders, settings.autoStartEditing, settings.backendContextMenu
    ).apply{
      this.autoScroll.set(settings.autoScrollToEnd)
      this.selectOnMouseOver.set(settings.selectOnMouseOver)
    }
    return beTreeConfiguration
  }

  companion object {
    fun columnNames(columnNamesAndSizes: List<String>?, treeSettings: TreeSettings = TreeSettings()): TreeConfiguration {
      return TreeConfiguration(
        treeSettings,
        columnNamesAndSizes?.map {
          ColumnDefinition.parse(it)
        })
    }

    fun oneColumn(treeSettings: TreeSettings = TreeSettings(), comparer: Any? = null): TreeConfiguration {
      return TreeConfiguration(treeSettings, null, comparer)
    }

    fun columnDefinitions(columnDefinitions: List<ColumnDefinition>?, treeSettings: TreeSettings = TreeSettings()): TreeConfiguration {
      return TreeConfiguration(treeSettings, columnDefinitions)
    }
  }
}