package com.jetbrains.rd.ui.bedsl.dsl.util

import com.jetbrains.ide.model.uiautomation.ValidationResult
import com.jetbrains.ide.model.uiautomation.ValidationStates
import com.jetbrains.rd.util.reactive.IProperty
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.ide.model.IconModel

data class ValidationCheckResult(val text: String = "",
                                 val iconModel: IconModel? = null,
                                 val states: ValidationStates = ValidationStates.validationError)

open class ValidationRule(validationCheck: () -> ValidationCheckResult, private var firstValueCheck: Boolean) {
  protected var myValidate: () -> Unit = { }
  protected val passedValidationResult: ValidationResult = ValidationResult().apply { icon.value = null }
  val validationResult = Property(passedValidationResult)
  val isGuarded: Boolean = false //todo: guarded validation rules?
  val validated = Property(false)

  init {
    myValidate = { getResult(validationCheck) }
  }

  fun validate() {
    validated.value = false
    myValidate.invoke()
  }

  protected fun getResult(validate: () -> ValidationCheckResult) {
    if (firstValueCheck) {
      val (text, iconModel, state) = validate()
      val hasNoError = text.isBlank()
      validationResult.value = if (hasNoError) passedValidationResult
      else ValidationResult(state, text).apply { icon.value = iconModel }
    }
    else {
      validationResult.value = passedValidationResult
      firstValueCheck = true
    }

    validated.value = true
  }
}


class ValidationRuleWithProperty<TValue>(propertyToValidate: IProperty<TValue>,
                                         validationCheck: (TValue) -> ValidationCheckResult,
                                         firstValueCheck: Boolean) : ValidationRule(
  { validationCheck(propertyToValidate.value) }, firstValueCheck)
