package com.jetbrains.rd.ui.bedsl.dsl.util

import com.jetbrains.ide.model.uiautomation.BeControlWithValidation
import com.jetbrains.ide.model.uiautomation.ValidationResult
import com.jetbrains.ide.model.uiautomation.ValidationStates
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.ViewableList

internal class ValidationRules<TControlType : BeControlWithValidation> //, "ValidationRules.Rules");
(controlWithValidation: TControlType, lifetime: Lifetime) {
  var rules: ViewableList<ValidationRule> = ViewableList()
  private var validatedRulesNumber = Property(0)

  private val myPassedResult = ValidationResult().apply { icon.value = null }
  private val validationResult = Property<ValidationResult?>(myPassedResult)

  init {
    rules.view(lifetime) { lt, _, r ->
      getValidationResult(lt, controlWithValidation, r)
    }
    validatedRulesNumber.advise(lifetime) { if (it == 0) controlWithValidation.validationResult.set(myPassedResult) }

    controlWithValidation.revalidate.advise(lifetime) { if (controlWithValidation.enabled.value) validate() }
    controlWithValidation.enabled.advise(lifetime) { v ->
      if (v)
        controlWithValidation.revalidate.fire(Unit)
      else
        validatedRulesNumber.value = 0
    }
    lifetime.onTerminationIfAlive { rules.clear() }
  }

  private fun validate() {
    validatedRulesNumber.value = 0
    validationResult.value = myPassedResult
    for (rule in rules) {
      rule.validate()
    }
  }

  private fun getValidationResult(
    lifetime: Lifetime,
    controlWithValidation: TControlType,
    rule: ValidationRule
  ) {
    rule.validated.advise(lifetime)
    {
      if (!it) return@advise
      val localResult = rule.validationResult.value
      validationResult.value = if (validationResult.value != null) getValidationResult(localResult) else localResult
      validatedRulesNumber.value++
      if (validatedRulesNumber.value == rules.count())
        controlWithValidation.validationResult.set(validationResult.value!!)
    }
  }


  private fun getValidationResult(localResult: ValidationResult): ValidationResult? {
    return when (localResult.state) {
      ValidationStates.validationPassed -> validationResult.value
      ValidationStates.validationWarning -> when (validationResult.value?.state) {
        ValidationStates.validationPassed -> validationResult.value
        else -> validationResult.value
      }
      ValidationStates.validationError -> localResult
      else -> validationResult.value
    }

  }
}