// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable

import com.intellij.ui.EditorTextField
import com.intellij.ui.table.JBTable
import com.intellij.ui.treeStructure.Tree
import com.intellij.ui.treeStructure.treetable.TreeTable
import javax.swing.*

fun findFirstFocusableControl(component: JComponent?): JComponent? {
  if (component == null) return null
  if (!component.isEnabled) return null
  if (!component.isFocusable) return null
  when (component) {
    is JTextField -> return component
    is EditorTextField -> return component.focusTarget
    is JCheckBox, is JRadioButton -> return component
    is JComboBox<*> -> return component
    is JBTable -> return component
    is Tree -> return component
    is TreeTable -> return component
    else -> {
      for (child in component.components.filterIsInstance<JComponent>()) {
        val childResult = findFirstFocusableControl(child)
        if (childResult != null) return childResult
      }
      return null
    }
  }
}