// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeArrowControl
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeArrowRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdMultiRenderer
import com.jetbrains.rdclient.util.idea.ui.toColor
import java.awt.*
import javax.swing.JComponent
import javax.swing.JPanel

class ArrowViewControl(private val project: Project) : ViewBinder<BeArrowControl>, IRendererOwner<BeArrowControl> {
  override fun getRenderer(viewModel: BeArrowControl): RdMultiRenderer<JPanel, BeArrowControl> = BeArrowRenderer()

  override fun bind(viewModel: BeArrowControl, lifetime: Lifetime): JComponent {
    //not finished yet
    return object : JPanel() {
      override fun paintComponent(g: Graphics?) {
        super.paintComponent(g)

        val graphics = g?.create() as Graphics2D
        try {
          //graphics.fillRect(0, 0, getWidth(), getHeight());
          //graphics.drawString(viewModel.leftContent, 5, height)
          drawRightArrow(graphics, width, height, viewModel.leftColor.toColor())
        }
        finally {
          graphics.dispose()
        }
      }
    }
  }

  private fun drawRightArrow(graphics: Graphics2D, width: Int, height: Int, color: Color) {
    graphics.color = color
    graphics.fillRect(0, 0, width - 4, height)
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON)
    val arrowPolygon = Polygon()
    arrowPolygon.addPoint(width - 4, 0)
    arrowPolygon.addPoint(width, height / 2)
    arrowPolygon.addPoint(width - 4, height)
    graphics.fillPolygon(arrowPolygon)
    graphics.drawString("ASASDASDAS", 0, height)
  }

}