// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.CustomShortcutSet
import com.intellij.openapi.actionSystem.Shortcut
import com.intellij.openapi.keymap.KeymapManager
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeActionIdListener
import com.jetbrains.ide.model.uiautomation.BeBehaviorControl
import com.jetbrains.ide.model.uiautomation.BePredefinedShortcutListener
import com.jetbrains.ide.model.uiautomation.BeShortcutListener
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.utils.getCustomShortCut
import com.jetbrains.rd.ui.util.getShortcut
import com.jetbrains.rd.ui.util.registerShortcuts
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JComponent

class BehaviorControlViewControl(private val project: Project) : ViewBinder<BeBehaviorControl> {

  override fun bind(viewModel: BeBehaviorControl, lifetime: Lifetime): JComponent {
    val component = ViewRegistry.getInstance(project).getView(viewModel.content, lifetime)
    return component.apply {
        viewModel.behavior.advise(lifetime){
          when(val actionListener = it.newValueOpt) {
            is BeShortcutListener -> {
              val shortcuts = actionListener.shortcuts.mapNotNull { getShortcut(it, this, actionListener.overrideDefault) }
              if (shortcuts.isNotEmpty()) {
                val shortcutSet = CustomShortcutSet(*shortcuts.toTypedArray())
                registerShortcuts(shortcutSet, actionListener)
              }
            }
            is BeActionIdListener ->{
              val manager : KeymapManager ? = KeymapManager.getInstance()
              val shortcuts = if (manager != null) manager.activeKeymap.getShortcuts(actionListener.actionId) else Shortcut.EMPTY_ARRAY
              if (shortcuts.isNotEmpty()) {
                val shortcutSet = CustomShortcutSet(*shortcuts)
                registerShortcuts(shortcutSet, actionListener)
              }
            }
            is BePredefinedShortcutListener -> {
              val shortcuts = actionListener.shortcut.getCustomShortCut()
              registerShortcuts(shortcuts, actionListener)
            }
          }
        }
    }
  }
}

