// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.intellij.ui.HideableDecorator
import com.intellij.util.ui.components.BorderLayoutPanel
import com.jetbrains.ide.model.uiautomation.BeCollapsiblePanel
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import javax.swing.JComponent

class CollapsiblePanelControl(private val project: Project) : ViewBinder<BeCollapsiblePanel> {
  override fun bind(viewModel: BeCollapsiblePanel, lifetime: Lifetime): JComponent {
    val panel = BorderLayoutPanel().apply { this.isOpaque = false }
    val decorator = HideableDecorator(panel, viewModel.headerText.valueOrNull ?: "", true)

    val binder = ViewRegistry.getInstance(project)
    decorator.setContentComponent(binder.getView(viewModel.content, lifetime))

    // todo: fix HideableDecorator to support events and flow expansion the other way
    viewModel.headerText.change.advise(lifetime) { decorator.title = it }
    viewModel.isExpanded.advise(lifetime) { decorator.setOn(it) }

    return panel
  }
}
