// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeComboBox
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.ComponentActionWrapper
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.customControls.CustomComboBoxControl
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeComboRenderer
import javax.swing.JComponent

/**
 * @author Alisa.Afonina
 */
class ComboBoxViewControl(private val project: Project) : ViewBinder<BeComboBox>,
                                                          IRendererOwner<BeComboBox>, IActionOwner<BeComboBox> {
  override fun getAction(lifetime: Lifetime, viewModel: BeComboBox): AnAction = ComponentActionWrapper {
    bind(viewModel, lifetime)
  }

  override fun bind(viewModel: BeComboBox, lifetime: Lifetime): JComponent =
    CustomComboBoxControl(lifetime, false, project).apply { init(viewModel) }

  override fun getRenderer(viewModel: BeComboBox): ITreeGridRenderer<BeComboBox> = BeComboRenderer(project)
}

