// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.intellij.ui.components.fields.ExtendableTextComponent
import com.intellij.ui.layout.*
import com.intellij.util.IconUtil
import com.jetbrains.ide.model.uiautomation.BeButton
import com.jetbrains.ide.model.uiautomation.BeControlWithAction
import com.jetbrains.ide.model.uiautomation.BeTextBox
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.getView
import com.jetbrains.rd.ui.bindable.views.customControls.TextBoxControlJb
import com.jetbrains.rd.ui.bindable.views.utils.getButtonActionText
import com.jetbrains.rd.ui.bindable.views.utils.tryGetPredefinedIcon
import javax.swing.JComponent

/**
 * @author Alisa.Afonina
 */
class ControlWithActionsViewControl(private val project: Project) : ViewBinder<BeControlWithAction> {
  //IRendererOwner<BeTextBoxWithAction>, IActionOwner<BeTextBoxWithAction> {
  /* override fun getAction(lifetime: Lifetime, viewModel: BeTextBoxWithAction): AnAction = ComponentActionWrapper {
       bind(viewModel, lifetime)
   }*/
  override fun bind(viewModel: BeControlWithAction, lifetime: Lifetime): JComponent {
    return when (viewModel.content) {
      is BeTextBox -> {
        val textBoxControl = TextBoxControlJb(lifetime, viewModel.content as BeTextBox, project)
        for (button in viewModel.actions.reversed()) {
          textBoxControl.addExtension(getExtension(button))
        }
        return textBoxControl
        /*
          KeyStroke keyStroke = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, InputEvent.SHIFT_DOWN_MASK);
String tooltip = UIBundle.message("component.with.browse.button.browse.button.tooltip.text") + " (" + KeymapUtil.getKeystrokeText(keyStroke) + ")";

ExtendableTextComponent.Extension browseExtension =
ExtendableTextComponent.Extension.create(AllIcons.General.OpenDisk, AllIcons.General.OpenDiskHover, tooltip, action);

new DumbAwareAction() {
@Override
public void actionPerformed(@NotNull AnActionEvent e) {
action.run();
}
}.registerCustomShortcutSet(new CustomShortcutSet(keyStroke), this, parentDisposable);
addExtension(browseExtension);


        * */


      }
      else -> {
        panel {
          row {
            component(viewModel.content.getView(lifetime, project))
            for (button in viewModel.actions)
              component(button.getView(lifetime, project))
          }
        }
      }
    }

  }

  private fun getExtension(button: BeButton): ExtendableTextComponent.Extension {
    val icon = button.tryGetPredefinedIcon(project) ?: IconUtil.getEmptyIcon(true)
    return ExtendableTextComponent.Extension.create(icon, button.getButtonActionText()) { button.action.click.fire(Unit) }
  }

  //override fun getRenderer(viewModel: BeTextBoxWithAction): ITreeGridRenderer<BeTextBoxWithAction> = BeComboRenderer(project)
}
