// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.EmptyAction
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeDynamicControlContainer
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeDynamicContentRenderer
import com.jetbrains.rd.ui.bindable.views.utils.BeControlHost
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JComponent


class DynamicComponentViewControl(private val project: Project) : ViewBinder<BeDynamicControlContainer>,
                                                                  IRendererOwner<BeDynamicControlContainer>,
                                                                  IActionOwner<BeDynamicControlContainer> {
  override fun bind(viewModel: BeDynamicControlContainer, lifetime: Lifetime): JComponent {
    return BeControlHost(project).apply {
      bind(lifetime, viewModel.content)
    }
  }

  override fun getRenderer(viewModel: BeDynamicControlContainer): ITreeGridRenderer<BeDynamicControlContainer> {
    return BeDynamicContentRenderer(project)
  }

  override fun getAction(lifetime: Lifetime, viewModel: BeDynamicControlContainer): AnAction {
    val registry = ViewRegistry.getInstance(project)
    val control = viewModel.content.valueOrNull
    val failedActionText = "Dynamic Content Action Failed"
    return if (control == null) EmptyAction.createEmptyAction(failedActionText, null, true)
    else registry.getAction(control, lifetime) ?: EmptyAction.createEmptyAction(failedActionText, null, true)
  }
}
