// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.intellij.ui.SimpleColoredComponent
import com.intellij.ui.SimpleTextAttributes
import com.intellij.ui.TitledSeparator
import com.jetbrains.ide.model.uiautomation.BeHeader
import com.jetbrains.ide.model.uiautomation.HeaderStyle
import com.jetbrains.rd.ui.ComponentActionWrapper
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdMultiRenderer
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.BeControlHost
import java.awt.BorderLayout
import javax.swing.JComponent
import javax.swing.JPanel

class HeaderViewControl(val project:Project)
  : ViewBinder<BeHeader>
    , IRendererOwner<BeHeader>
    , IActionOwner<BeHeader> {
  override fun getAction(lifetime: Lifetime, viewModel: BeHeader): AnAction = ComponentActionWrapper {
    bind(viewModel, lifetime)
  }
  override fun getRenderer(viewModel: BeHeader): ITreeGridRenderer<BeHeader> {
    return object : RdMultiRenderer<JPanel, BeHeader>() {
      override fun reset(component: JComponent?, value: BeHeader, settings: RdCellSettings): JComponent {
        val panel = getElement(settings.isEditor, value)
        when (val firstElement = panel.components.firstOrNull()) {
           is TitledSeparator -> {
             firstElement.text = value.text.value
          }
          is SimpleColoredComponent -> {
            firstElement.clear()
            firstElement.append(value.text.value, SimpleTextAttributes.REGULAR_BOLD_ATTRIBUTES) }
          }
        return panel
      }

      override fun getNewInstance(isEditor: Boolean, model: BeHeader): JPanel {
        return BeComponents.getNewGridInstance().apply {
          this.isOpaque = false
          when (model.headerStyle) {
            HeaderStyle.BOLD -> {
              add(SimpleColoredComponent())
            }
            HeaderStyle.SEPARATOR ->{
              add(TitledSeparator(""), "push, grow, gap 0")
            }
          }
        }
      }

      override fun isEditable(): Boolean = false
      override fun containsEditor(model: BeHeader): Boolean = false
    }
  }

  override fun bind(viewModel: BeHeader, lifetime: Lifetime): JComponent {
    when(viewModel.headerStyle) {
      HeaderStyle.SEPARATOR -> {
        val result = TitledSeparator(viewModel.text.value)
        viewModel.text.advise(lifetime) { result.text = it }
        return BeComponents.getNewGridInstance().apply {
          this.add(result, BorderLayout.NORTH)
        }
      }
      HeaderStyle.BOLD -> {
        return BeControlHost(project).
        apply {
          bind(lifetime, viewModel.text){ SimpleColoredComponent().apply{append(it, SimpleTextAttributes.REGULAR_BOLD_ATTRIBUTES)}}
        }
      }
    }

  }
}