// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.ui.ColorUtil
import com.intellij.util.ui.UIUtil
import com.jetbrains.ide.model.uiautomation.BeHtmlViewer
import com.jetbrains.rd.ui.bedsl.extensions.valueOrEmpty
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.util.lifetime.Lifetime
import java.net.URL
import javax.swing.JComponent
import javax.swing.JEditorPane


class HtmlViewerViewControl : ViewBinder<BeHtmlViewer> {
  override fun bind(viewModel: BeHtmlViewer, lifetime: Lifetime): JComponent {
    val jEditorPane = JEditorPane()
    jEditorPane.isEditable = false
    val kit = UIUtil.JBWordWrapHtmlEditorKit()
    jEditorPane.editorKit = kit
    jEditorPane.background = UIUtil.getPanelBackground()
    val doc = kit.createDefaultDocument()
    jEditorPane.document = doc
    val htmlString = viewModel.text.value
    jEditorPane.contentType = "text/html"
    val css = kit.styleSheet
    //default color
    css.addRule("body {color:#${ColorUtil.toHex(UIUtil.getLabelForeground())} }")

    css.addRule("body {font:${UIUtil.getLabelFont().family} }")
    css.addRule("body {word-break: break-all;}")

    viewModel.cssRules.valueOrEmpty().forEach{
       css.addRule(it)
     }
      if (htmlString != null) {
      viewModel.text.advise(lifetime) { v -> jEditorPane.text = v }
    }
    else {
      viewModel.url.advise(lifetime) { v -> jEditorPane.page = URL(v) }
      jEditorPane.text = "<html> ${viewModel.textOnError}</html>"
    }
    return jEditorPane
  }
}
