// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeIcon
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.icons.toIdeaIcon
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeIconRenderer
import com.jetbrains.rd.ui.bindable.views.utils.getCustomIcon
import javax.swing.JComponent
import javax.swing.JLabel

class IconViewControl(private val project: Project) : ViewBinder<BeIcon>,
                                                      IRendererOwner<BeIcon> {
  override fun getRenderer(viewModel: BeIcon): ITreeGridRenderer<BeIcon> = BeIconRenderer(project)

  override fun bind(viewModel: BeIcon, lifetime: Lifetime): JComponent {
    return JLabel().apply {
      icon = viewModel.iconId.valueOrNull?.getCustomIcon()
             ?: viewModel.icon.valueOrNull?.toIdeaIcon(RdIconHost.getInstance(project))
    }
  }
}