// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeLabel
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.ComponentActionWrapper
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeLabelRenderer
import com.jetbrains.rd.ui.bindable.views.utils.allowWrapped
import com.jetbrains.rd.ui.bindable.views.utils.getLabelForAbstractText
import javax.swing.JComponent

class LabelViewControl(private val project: Project)
  : ViewBinder<BeLabel>
    , IRendererOwner<BeLabel>
    , IActionOwner<BeLabel> {
  override fun getAction(lifetime: Lifetime, viewModel: BeLabel): AnAction = ComponentActionWrapper {
    bind(viewModel, lifetime)
  }

  override fun bind(viewModel: BeLabel, lifetime: Lifetime): JComponent {
    val iconHost = RdIconHost.getInstance(project)
    return viewModel.allowWrapped(viewModel.getLabelForAbstractText(lifetime, iconHost)).apply { isOpaque = false }
  }

  override fun getRenderer(viewModel: BeLabel): ITreeGridRenderer<BeLabel> = BeLabelRenderer(project)
}


