// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.ui.HyperlinkAdapter
import com.intellij.ui.HyperlinkLabel
import com.jetbrains.ide.model.uiautomation.BeLink
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeLinkRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdMultiRenderer
import com.jetbrains.rd.ui.bindable.views.utils.getSimpleText
import javax.swing.JComponent
import javax.swing.event.HyperlinkEvent

class LinkViewControl : ViewBinder<BeLink>, IRendererOwner<BeLink> {
  override fun getRenderer(viewModel: BeLink): RdMultiRenderer<HyperlinkLabel, BeLink> = BeLinkRenderer()

  override fun bind(viewModel: BeLink, lifetime: Lifetime): JComponent {
    return HyperlinkLabel().apply {
      setHyperlinkText(viewModel.prefix.getSimpleText(), viewModel.content.valueOrNull, viewModel.postfix.getSimpleText())
      viewModel.content.advise(lifetime) {
        setHyperlinkText(viewModel.prefix.getSimpleText(), it, viewModel.postfix.getSimpleText())
      }
      this.addHyperlinkListener(object : HyperlinkAdapter() {
        override fun hyperlinkActivated(e: HyperlinkEvent?) {
          viewModel.action.click.fire(Unit)
        }
      })
    }
  }
}
