// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.*
import com.intellij.openapi.actionSystem.impl.ActionButton
import com.intellij.openapi.project.DumbAwareAction
import com.intellij.openapi.project.Project
import com.intellij.ui.components.JBMenu
import com.jetbrains.ide.model.uiautomation.BeAbstractText
import com.jetbrains.ide.model.uiautomation.BeMenuItem
import com.jetbrains.ide.model.uiautomation.BeSpacer
import com.jetbrains.rd.ui.RdUiBundle
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.valueOrDefault
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeMenuItemRenderer
import com.jetbrains.rd.ui.bindable.views.utils.getLabelText
import javax.swing.JComponent
import javax.swing.JLabel
import javax.swing.JMenu

class MenuItemViewControl(private var project: Project) : ViewBinder<BeMenuItem>, IActionOwner<BeMenuItem>, IRendererOwner<BeMenuItem> {
  override fun getRenderer(viewModel: BeMenuItem): ITreeGridRenderer<BeMenuItem> = BeMenuItemRenderer(project)

  override fun bind(viewModel: BeMenuItem, lifetime: Lifetime): JComponent {
    val action = getAction(lifetime, viewModel)
    return ActionButton(action, action.templatePresentation, ActionPlaces.UNKNOWN, ActionToolbar.DEFAULT_MINIMUM_BUTTON_SIZE);
  }

  override fun getAction(lifetime: Lifetime, viewModel: BeMenuItem): AnAction {
    val registry = ViewRegistry.getInstance(project)
    val mainAction = registry.getAction(viewModel.header.valueOrDefault(BeSpacer()), lifetime) ?: object : DumbAwareAction(
      RdUiBundle.message("becontrols.not.action.owner")) {
      override fun actionPerformed(e: AnActionEvent) {}
    }

    val items = viewModel.innerMenuItems.valueOrNull?.map { registry.getAction(it, lifetime) }
    if (items == null || items.isEmpty())
      return mainAction

    val labelText = (viewModel.header as? BeAbstractText)?.getLabelText()
    return object : DefaultActionGroup(labelText ?: RdUiBundle.message("becontrols.header"), true) {
      init {
        addAll(items)

        templatePresentation.text = mainAction.templatePresentation.text
        templatePresentation.icon = mainAction.templatePresentation.icon
      }


      override fun displayTextInToolbar(): Boolean = mainAction.displayTextInToolbar()
    }

  }

}