// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeMigLayout
import com.jetbrains.ide.model.uiautomation.BeMigLayoutItem
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.viewNotNull
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import net.miginfocom.swing.MigLayout
import javax.swing.JComponent
import javax.swing.JPanel

class MigLayoutViewControl(val project: Project) : ViewBinder<BeMigLayout> {
  override fun bind(viewModel: BeMigLayout, lifetime: Lifetime): JComponent {
    val panel = JPanel(MigLayout(viewModel.layoutConstraints, viewModel.colConstraints, viewModel.rowConstraints))

    viewModel.items.viewNotNull(lifetime) { itemsLifetime, items ->
      panel.removeAll()
      items.forEach { item ->
        addItem(panel, itemsLifetime, item)
      }
    }

    return panel
  }

  private fun addItem(panel: JPanel, lifetime: Lifetime, item: BeMigLayoutItem) {
    val content = item.content
    val constraints = item.constraints

    val component = ViewRegistry.getInstance(project).getView(content, lifetime)
    panel.add(component, constraints)
  }
}