// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.ui.components.JBPasswordField
import com.jetbrains.ide.model.uiautomation.BePasswordTextBox
import com.jetbrains.ide.model.uiautomation.ValidationStates
import com.jetbrains.rd.swing.textProperty
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.fire
import com.jetbrains.rd.ui.bindable.ViewBinder
import javax.swing.JComponent


class PasswordTextBoxViewControl : ViewBinder<BePasswordTextBox> {
  override fun bind(viewModel: BePasswordTextBox, lifetime: Lifetime): JComponent {
    val textBox = JBPasswordField()
    textBox.textProperty().change.advise(lifetime) {
      viewModel.text.value = it
    }

    viewModel.validationResult.view(lifetime) { _, it ->
      when (it.state) {
        ValidationStates.validationError -> textBox.putClientProperty("JComponent.outline", "error")
        ValidationStates.validationWarning -> textBox.putClientProperty("JComponent.outline", "warning")
        ValidationStates.validationPassed -> textBox.putClientProperty("JComponent.outline", null)
      }
      textBox.toolTipText = it.resultMessage
      textBox.repaint()
    }
    viewModel.enabled.advise(lifetime) {
      textBox.isEnabled = it
      viewModel.revalidate.fire()
    }

    return textBox
  }
}