// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.dsl.label
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.bindDefaultProperties
import net.miginfocom.swing.MigLayout
import javax.swing.JComponent
import javax.swing.JProgressBar
import kotlin.math.roundToInt

class ProgressViewControl(private val project: Project) :
  ViewBinder<BeProgress> {
  override fun bind(viewModel: BeProgress, lifetime: Lifetime): JComponent {
    val resultPanel = BeComponents.getNewGridInstance { it.rowConstraints = "[pref!]" }
    viewModel.execute.advise(lifetime) {

      resultPanel.removeAll()
      if (it != null) {
        when (it) {
          is IndefiniteProgress -> {
            val progressBar = JProgressBar()
            progressBar.isIndeterminate = true
            viewModel.bindDefaultProperties(lifetime, progressBar)
            resultPanel.add(progressBar, "growx")
          }
          is CancelableProgress -> {

          }
          is DefiniteProgress -> {
            val progressPanel = BeComponents.getNewGridInstance()
            //        constraints.insets = JBUI.insets(8, 0, 4, 1) // for button panel


            var rowConstraints = "[pref!]"
            if (it.taskName != null) {
              val progressLabel = ViewRegistry.getInstance(project).getView(it.taskName!!, lifetime)
              progressPanel.add(progressLabel, "growx, wrap")
              rowConstraints = "[20:pref:pref]2${rowConstraints}"
            }
            val progressBar = JProgressBar()
            progressPanel.add(progressBar, "growx, push, wrap")
            progressBar.isBorderPainted = true
            it.fraction.advise(lifetime) { fraction -> progressBar.value = fraction.roundToInt() }

            if (it.stepName != null) {
              val stepLabel = ViewRegistry.getInstance(project).getView(it.stepName!!, lifetime)
              progressPanel.add(stepLabel, "growx, wrap")
              rowConstraints = "${rowConstraints}2[20:pref:pref]"
            }
            val migLayout = progressPanel.layout as? MigLayout
            if (migLayout != null)
              migLayout.rowConstraints = rowConstraints


            //                        viewModel.bindDefaultProperties(lifetime, progressPanel)
            resultPanel.add(progressPanel, "growx")

            viewModel.bindDefaultProperties(lifetime, progressPanel)
          }
          is TaskProgress -> {
            val progressPanel = BeComponents.getNewGridInstance()
            //        constraints.insets = JBUI.insets(8, 0, 4, 1) // for button panel


            var rowConstraints = "[pref!]"
            if (it.taskModel.header != null) {
              val header = it.taskModel.header
              if (header.value != null) {
                val beLabel = label(header.value!!)
                header.advise(lifetime) {
                  beLabel.text.set(it ?: "")
                }

                val progressLabel = ViewRegistry.getInstance(project).getView(beLabel, lifetime)
                progressPanel.add(progressLabel, "growx, wrap")
                rowConstraints = "[20:pref:pref]2${rowConstraints}"
              }
            }
            val progressBar = JProgressBar()
            progressPanel.add(progressBar, "growx, push, wrap")
            progressBar.isBorderPainted = true
            it.taskModel.progress.advise(lifetime) { fraction -> progressBar.value = (fraction * 99 + 0.5).roundToInt() }

            if (it.taskModel.description != null) {
              val description = it.taskModel.description
              if(description.value != null) {
                val beLabel = label(description.value!!)
                description.advise(lifetime) {
                  beLabel.text.set(it ?: "")
                }

                val stepLabel = ViewRegistry.getInstance(project).getView(beLabel, lifetime)
                progressPanel.add(stepLabel, "growx, wrap")
                rowConstraints = "${rowConstraints}2[20:pref:pref]"
              }
            }
            val migLayout = progressPanel.layout as? MigLayout
            if (migLayout != null)
              migLayout.rowConstraints = rowConstraints


          }
        }
      }
      //?
      resultPanel.revalidate()
      resultPanel.repaint()
    }
    return resultPanel
  }
}

