// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.intellij.ui.components.JBRadioButton
import com.jetbrains.ide.model.uiautomation.BeAbstractText
import com.jetbrains.ide.model.uiautomation.BeRadioButton
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.initFromText
import com.jetbrains.rd.ui.bindable.views.utils.shouldBeWrappedIntoHtml
import java.awt.event.MouseAdapter
import java.awt.event.MouseEvent
import javax.swing.JComponent

class RadioButtonViewControl(private val project: Project) : ViewBinder<BeRadioButton> {
  override fun bind(viewModel: BeRadioButton, lifetime: Lifetime): JComponent {
    val result = JBRadioButton()
    viewModel.enabled.advise(lifetime) {
      viewModel.description.enabled.value = it
      result.isEnabled = it
      result.repaint()
    }
    viewModel.tooltip.advise(lifetime) { result.toolTipText = it }
    viewModel.visible.advise(lifetime) {
      result.isVisible = it == ControlVisibility.Visible
      result.repaint()
    }

    viewModel.property.value.let { result.isSelected = it }

    viewModel.property.view(lifetime) { _, it ->
      result.isSelected = it
      result.repaint()
    }

    result.addItemListener {
      if (result.isSelected)
        viewModel.property.set(result.isSelected)
      else {
        result.isSelected = viewModel.property.value
        result.repaint()
      }
    }


    return if (viewModel.description.shouldBeWrappedIntoHtml) initWrapped(result, viewModel.description, lifetime)
    else result.initFromText(lifetime, viewModel.description, RdIconHost.getInstance(project), false)

  }

  private fun initWrapped(checkbox: JBRadioButton, viewModel: BeAbstractText, lifetime: Lifetime): JComponent {
    val label = ViewRegistry.getInstance(project).getView(viewModel, lifetime)
    label.addMouseListener(object : MouseAdapter() {
      override fun mousePressed(e: MouseEvent?) {
        if (checkbox.isEnabled)
          checkbox.isSelected = !checkbox.isSelected
      }
    })

    return BeComponents.getNewGridInstance({ it.columnConstraints = "[min!]5[fill,grow]" }).apply {
      add(checkbox, "aligny top")
      add(label)
    }
  }
}
