// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.project.Project
import com.intellij.ui.SimpleColoredComponent
import com.intellij.util.ui.UIUtil
import com.jetbrains.ide.model.uiautomation.BeRichText
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.adviseNotNull
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.icons.toIdeaIcon
import com.jetbrains.rd.ui.ComponentActionWrapper
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeRichTextRenderer
import com.jetbrains.rd.ui.bindable.views.utils.*
import com.jetbrains.rd.ui.fillWidthButDoNotOverflow
import javax.swing.JComponent

class RichTextViewControl(private val project: Project)
  : ViewBinder<BeRichText>
    , IRendererOwner<BeRichText>
    , IActionOwner<BeRichText> {
  override fun getAction(lifetime: Lifetime, viewModel: BeRichText): AnAction = ComponentActionWrapper {
    bind(viewModel, lifetime)
  }

  override fun getRenderer(viewModel: BeRichText): ITreeGridRenderer<BeRichText> = BeRichTextRenderer(project)

  override fun bind(viewModel: BeRichText, lifetime: Lifetime): JComponent {
    val iconHost = RdIconHost.getInstance(project)
    return if (viewModel.shouldBeWrappedIntoHtml)
    //|| viewModel.settings.fontSize.valueOrDefault(BeFontSize.DEFAULT) != BeFontSize.DEFAULT) //todo: may be parsing of shortcut should be done on rich text processing (replace shortcut char to underlined text part)
      viewModel.allowWrapped(viewModel.getLabelForAbstractText(lifetime, iconHost))
    else {
      viewModel.getSimpleColoredComponentForRichText(lifetime, iconHost).fillWidthButDoNotOverflow(false)
    }.apply { isOpaque = false }
  }
}

internal fun BeRichText.getSimpleColoredComponentForRichText(lifetime: Lifetime, iconHost: RdIconHost): SimpleColoredComponent {
  val component = BeComponents.getNewSCCInstance(false)
  this.text.advise(lifetime) { component.applyModel(this, UIUtil.getLabelBackground()) }
  this.enabled.advise(lifetime) { component.isEnabled = it }
  this.icon.advise(lifetime) { component.icon = this.icon.value?.toIdeaIcon(iconHost) }
  this.visible.adviseNotNull(lifetime) { component.isVisible = it == ControlVisibility.Visible }
  component.font = settings.getFont()

  //size = textSettings.getFontSize()
  return component
}






