// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.intellij.ui.components.JBScrollPane
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeScrollablePanel
import com.jetbrains.ide.model.uiautomation.BeScrollbarPolicy
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.applyToContent
import javax.swing.JComponent

class ScrollablePanelViewControl(private val project: Project) :
  ViewBinder<BeScrollablePanel> {
    override fun bind(viewModel: BeScrollablePanel, lifetime: Lifetime): JComponent {
        val grid = BeComponents.getNewGridInstance { it.rowConstraints = "[]" }
        viewModel.content.apply{
            val content = ViewRegistry.getInstance(project).getView(this, lifetime)
            grid.removeAll()
            grid.add(when (val policy = viewModel.scrollbarPolicy) {
                BeScrollbarPolicy.NONE -> content
                else -> JBScrollPane(
                    content,
                    if (policy == BeScrollbarPolicy.VERTICAL || policy == BeScrollbarPolicy.BOTH) JBScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED else JBScrollPane.VERTICAL_SCROLLBAR_NEVER,
                    if (policy == BeScrollbarPolicy.HORIZONTAL || policy == BeScrollbarPolicy.BOTH) JBScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED else JBScrollPane.HORIZONTAL_SCROLLBAR_NEVER
                ).apply {
                    border = JBUI.Borders.empty()
                }
            }.apply {
                viewModel.size.applyToContent(this)
            }, "grow, push"
            )
            grid.revalidate()
            grid.repaint()
        }
        return grid
    }
}