// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.ui.JBColor
import com.intellij.ui.SearchTextField
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeSearchBox
import com.jetbrains.ide.model.uiautomation.Place
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.ComponentActionWrapper
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bedsl.extensions.UpdateFlag
import com.jetbrains.rd.ui.bedsl.extensions.bindFocus
import com.jetbrains.rd.ui.bedsl.extensions.execute
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import java.awt.Dimension
import java.awt.event.KeyEvent
import javax.swing.JComponent

class SearchBoxViewControl : ViewBinder<BeSearchBox>, IActionOwner<BeSearchBox> {
  override fun getAction(lifetime: Lifetime, viewModel: BeSearchBox): AnAction = ComponentActionWrapper {
    bind(viewModel, lifetime)
  }

  override fun bind(viewModel: BeSearchBox, lifetime: Lifetime): JComponent {
    val changeFromUI = UpdateFlag()
    val searchTextField = object : SearchTextField(false) {
      private fun updateModel() {
        changeFromUI.execute {
          viewModel.text.set(this.text)
        }
      }

      override fun preprocessEventForTextField(e: KeyEvent?): Boolean {
        updateModel()
        return super.preprocessEventForTextField(e)
      }

      override fun onFieldCleared() {
        super.onFieldCleared()
        updateModel()
      }
    }.apply {
      // https://jetbrains.github.io/ui/controls/search_field/
      if (viewModel.place == Place.ToolbarLeft) {
        textEditor.border = JBUI.Borders.customLine(JBColor.border(), 0, 0, 0, 1)
        minimumSize = Dimension(200, 0)
      }
      else {
        textEditor.border = JBUI.Borders.customLine(JBColor.border(), 1, 1, 0, 1)
        border = JBUI.Borders.customLine(JBColor.border(), 0, 0, 0, 0)
      }
      minimumSize = Dimension(minimumSize.width, 26)

      viewModel.bindFocus(lifetime, this)
    }
    viewModel.enabled.advise(lifetime) { searchTextField.isEnabled = it }
    viewModel.text.advise(lifetime) {
      if (!changeFromUI.happening)
        searchTextField.text = it
    }
    return searchTextField
  }
}