// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.Separator
import com.jetbrains.ide.model.uiautomation.BeSeparator
import com.jetbrains.ide.model.uiautomation.GridOrientation
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.customControls.SimpleSeparator
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import java.awt.BorderLayout
import javax.swing.JComponent
import javax.swing.JSeparator
import javax.swing.SwingConstants

class SeparatorViewControl : ViewBinder<BeSeparator>, IActionOwner<BeSeparator> {
  override fun getAction(lifetime: Lifetime, viewModel: BeSeparator): AnAction = Separator()

  override fun bind(viewModel: BeSeparator, lifetime: Lifetime): JComponent {
    return BeComponents.getNewGridInstance().apply {
      val result: JComponent =
        if (viewModel.orientation == GridOrientation.Horizontal)
          SimpleSeparator()
        else
          JSeparator(SwingConstants.VERTICAL)
      add(result, BorderLayout.NORTH)
    }
  }
}

