// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.ui.components.panels.NonOpaquePanel
import com.intellij.util.ui.UIUtil
import com.jetbrains.ide.model.uiautomation.BeSpacer
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdMultiRenderer
import javax.swing.JComponent
import javax.swing.JPanel

class SpacerViewControl : ViewBinder<BeSpacer>, IRendererOwner<BeSpacer> {
  companion object {
    private fun createSpacer(): JPanel {
      return NonOpaquePanel()
    }
  }

  private class RdSpacerRenderer : RdMultiRenderer<JPanel, BeSpacer>() {
    override fun getNewInstance(isEditor: Boolean, viewModel: BeSpacer): JPanel {
      return createSpacer()
    }

    override fun isEditable(): Boolean = false
    override fun containsEditor(model: BeSpacer) = false

    override fun reset(component: JComponent?, value: BeSpacer, settings: RdCellSettings): JPanel {
      return getElement(settings.isEditor, value)
        .apply {
          background = UIUtil.getTableBackground(settings.isSelected, settings.hasFocus)
        }
    }
  }

  override fun getRenderer(viewModel: BeSpacer): ITreeGridRenderer<BeSpacer> = RdSpacerRenderer()
  override fun bind(viewModel: BeSpacer, lifetime: Lifetime): JComponent = createSpacer()
}
 