// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.ui.JBIntSpinner
import com.jetbrains.ide.model.uiautomation.BeSpinner
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.listControl.IRendererOwner
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.BeSpinnerRenderer
import javax.swing.JComponent

class SpinnerViewControl : ViewBinder<BeSpinner>, IRendererOwner<BeSpinner> {
  override fun getRenderer(viewModel: BeSpinner): ITreeGridRenderer<BeSpinner> = BeSpinnerRenderer()

  override fun bind(viewModel: BeSpinner, lifetime: Lifetime): JComponent {
    val jbIntSpinner = JBIntSpinner(viewModel.value.value, viewModel.minValue, viewModel.maxValue)
    viewModel.value.advise(lifetime) { jbIntSpinner.value = it }
    viewModel.enabled.advise(lifetime) { jbIntSpinner.isEnabled = it }
    viewModel.tooltip.advise(lifetime) { jbIntSpinner.toolTipText = it }
    return jbIntSpinner.apply {
      this.addChangeListener { viewModel.value.set(jbIntSpinner.number) }
    }
  }
}