// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.project.Project
import com.intellij.ui.JBSplitter
import com.jetbrains.ide.model.uiautomation.BeSplitControl
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.ViewRegistry
import javax.swing.JComponent

class SplitViewControl(private val project: Project) : ViewBinder<BeSplitControl> {
  override fun bind(viewModel: BeSplitControl, lifetime: Lifetime): JComponent {
    var proportion = viewModel.proportion
    if (proportion > 1) proportion /= (proportion + 1)
    return (if (viewModel.controlId.value.isNotEmpty())
      JBSplitter(viewModel.isVertical, viewModel.controlId.value, proportion)
    else
      JBSplitter(viewModel.isVertical, proportion)).apply {
      val viewRegistry = ViewRegistry.getInstance(project)
      firstComponent = viewRegistry.getView(viewModel.firstPanel, lifetime)
      secondComponent = viewRegistry.getView(viewModel.secondPanel, lifetime)
    }
  }
}