// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.ActionPlaces
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.DataProvider
import com.intellij.openapi.project.Project
import com.intellij.util.ui.JBEmptyBorder
import com.jetbrains.ide.model.uiautomation.BeControlConstant
import com.jetbrains.ide.model.uiautomation.BeDock
import com.jetbrains.ide.model.uiautomation.BePredefinedBehaviorButton
import com.jetbrains.ide.model.uiautomation.BeToolbar
import com.jetbrains.rd.ui.bindable.BeControlsConstantProvider
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.getAction
import com.jetbrains.rd.ui.bindable.getView
import com.jetbrains.rd.ui.bindable.views.listControl.IActionOwner
import com.jetbrains.rd.ui.bindable.views.utils.LiveActionGroup
import com.jetbrains.rd.ui.bindable.views.utils.tryAddCustomShortCut
import com.jetbrains.rd.util.lifetime.Lifetime
import java.awt.BorderLayout
import java.awt.LayoutManager2
import javax.swing.JComponent
import javax.swing.JPanel

class ToolbarViewControl(private val project: Project) : ViewBinder<BeToolbar>,
                                                         IActionOwner<BeToolbar> {
  override fun getAction(lifetime: Lifetime, viewModel: BeToolbar): AnAction {
    val actionGroup = LiveActionGroup()
    viewModel.leftItems.view(lifetime) { lt, _, control ->
      control.getAction(lt, project)?.let {
        actionGroup.add(lt, it)
      }
    }
    return actionGroup
  }

  override fun bind(viewModel: BeToolbar, lifetime: Lifetime): JComponent {
    val dock = viewModel.dock
    val actionGroup = LiveActionGroup()
    val actionToolbar =
      ActionManager.getInstance().createActionToolbar(
        ActionPlaces.TOOLBAR, actionGroup,
        dock == BeDock.TOP || dock == BeDock.BOTTOM
      ).apply {
        component.isOpaque = false
        // By default our BeGrid shrink toolbar to its minimum size which is 26, but all toolbars
        // in IDEA are 28 pixels height, so we add remaining pixels via border to make minHeight match prefHeight
        component.border = JBEmptyBorder(2, 2, 4, 2)
      }

    val panel = BeToolbarPanel(BorderLayout(), viewModel).apply { isOpaque = false }

    val view = viewModel.content.getView(lifetime, project)
    view.putClientProperty("view", "view")
    panel.add(view, BorderLayout.CENTER)
    actionToolbar.setTargetComponent(view)
    viewModel.leftItems.view(lifetime) { lt, _, control ->
      val action = control.getAction(lt, project)
      if (action != null)
        actionGroup.add(lt, action)
      if (control is BePredefinedBehaviorButton) {
        control.tryAddCustomShortCut(action, /*view*/panel, lifetime)
      }
    }
    panel.add(actionToolbar.component, when (dock) {
      BeDock.TOP -> BorderLayout.NORTH
      BeDock.BOTTOM -> BorderLayout.SOUTH
      BeDock.LEFT -> BorderLayout.WEST
      BeDock.RIGHT -> BorderLayout.EAST
      BeDock.DEFAULT_IDE_POSITION -> BorderLayout.WEST
      BeDock.PARENT_DEPENDENT -> BorderLayout.WEST //todo
    })
    /* viewModel.visible.advise(lifetime) {
       when (it) {
           ControlVisibility.Visible -> panel.add(actionToolbar.component, when (dock) {
             BeDock.TOP -> BorderLayout.NORTH
             BeDock.BOTTOM -> BorderLayout.SOUTH
             BeDock.LEFT -> BorderLayout.WEST
             BeDock.RIGHT -> BorderLayout.EAST
           })
         ControlVisibility.Collapsed -> panel.remove(actionToolbar.component)
         ControlVisibility.Hidden -> panel.remove(actionToolbar.component)
       }
       panel.repaint()
     }*/
    return panel
  }

  class BeToolbarPanel(layoutManager: LayoutManager2, private val viewModel: BeToolbar) : JPanel(layoutManager), DataProvider {
    override fun getData(dataId: String): Any? = when {
      dataId != BeControlsConstantProvider.dataId -> null
      else -> listOf(BeControlConstant(viewModel.controlId.value, listOf()))
    }
  }
}