// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views

import com.intellij.openapi.ui.ComponentValidator
import com.intellij.openapi.ui.ValidationInfo
import com.jetbrains.ide.model.uiautomation.BeValidationLabel
import com.jetbrains.ide.model.uiautomation.ValidationStates
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.ViewBinder
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import javax.swing.JComponent

class ValidationLabelViewControl : ViewBinder<BeValidationLabel> {
  override fun bind(viewModel: BeValidationLabel, lifetime: Lifetime): JComponent {
    val jPanel = BeComponents.getNewGridInstance()
    viewModel.validationResult.view(lifetime) { _, it ->
      val error =
        when (it.state) {
          ValidationStates.validationError -> ValidationInfo(it.resultMessage)
          ValidationStates.validationWarning -> ValidationInfo(it.resultMessage).asWarning()
          ValidationStates.validationPassed -> null
          else -> null
        }

      jPanel.removeAll()
      error?.apply {
        ComponentValidator.createPopupBuilder(error) { errorHint ->
          jPanel.add(errorHint)
          jPanel.revalidate()
        }
      }
    }

    return jPanel
  }
}