// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl

import com.intellij.openapi.actionSystem.DataProvider
import com.intellij.openapi.actionSystem.LangDataKeys
import com.intellij.ui.treeStructure.treetable.ListTreeTableModelOnColumns
import com.intellij.ui.treeStructure.treetable.TreeTable
import com.intellij.ui.treeStructure.treetable.TreeTableCellRenderer
import com.intellij.ui.treeStructure.treetable.TreeTableModel
import com.jetbrains.ide.model.uiautomation.BeTreeGrid
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdclient.util.idea.ui.bindToBeEmptyText
import javax.swing.table.JTableHeader
import javax.swing.table.TableCellEditor
import javax.swing.table.TableCellRenderer

class BeTreeTable(
  lifetime: Lifetime,
  tableModel: ListTreeTableModelOnColumns,
  private val showAsList: Boolean,
  private val columns: Array<TreeGridListColumn>,
  private val viewModel: BeTreeGrid

) : TreeTable(tableModel), DataProvider {
  init {
    bindProgress(lifetime, viewModel.progress) {
      setPaintBusy(it)
    }
    //todo:setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN)
    this.emptyText.bindToBeEmptyText(lifetime, viewModel.emptyText)
  }

  override fun createDefaultTableHeader(): JTableHeader {
    return JBTableHeader()
  }
  //End

  private val logger = getLogger<BeTreeTable>()

  override fun createTableRenderer(treeTableModel: TreeTableModel): TreeTableCellRenderer {
    val tableRenderer = super.createTableRenderer(treeTableModel)
    tableRenderer.setRootVisible(false)
    return tableRenderer
  }

  override fun getCellRenderer(row: Int, column: Int): TableCellRenderer {
    val treePath = tree.getPathForRow(row) ?: return super.getCellRenderer(row, column)
    //special case for working with expanders
    if (column == 0 && !showAsList) return super.getCellRenderer(row, column)
    val node = treePath.lastPathComponent
    val columnInfo = columns[column]
    val renderer = columnInfo.getRenderer(node)
    return renderer ?: super.getCellRenderer(row, column)
  }

  override fun getCellEditor(row: Int, column: Int): TableCellEditor {
    val treePath = tree.getPathForRow(row) ?: return super.getCellEditor(row, column)
    val node = treePath.lastPathComponent
    val columnInfo = columns[column]
    val editor = columnInfo.getEditor(node)
    val tableCellEditor = editor ?: super.getCellEditor(row, column)
    if (column == 0 && !showAsList)
      return super.getCellEditor(row, column)
    return tableCellEditor
  }

  override fun getData(dataId: String): Any? {
    return when {
      LangDataKeys.NO_NEW_ACTION.`is`(dataId) -> true
      else -> tableGetData(this, viewModel.controlId.value, dataId, logger)
    }
  }
}