// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl

import com.intellij.openapi.project.Project
import com.intellij.ui.treeStructure.Tree
import com.intellij.ui.treeStructure.treetable.TreeTableModel
import com.intellij.util.ui.ColumnInfo
import com.jetbrains.ide.model.uiautomation.BeAlignment
import com.jetbrains.ide.model.uiautomation.BeColumnDefinition
import com.jetbrains.ide.model.uiautomation.BeSpacer
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdCellEditor
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdCellRenderer
import javax.swing.table.TableCellEditor
import javax.swing.table.TableCellRenderer
import javax.swing.table.TableModel
import javax.swing.tree.DefaultMutableTreeNode

class TreeGridListColumn(
  val project: Project,
  private val renderer: RdCellRenderer,
  val columnDefinition: BeColumnDefinition,
  private val index: Int,
  private val columnAlignments: List<BeAlignment>
) : ColumnInfo<Any, Any>(columnDefinition.columnName) {

  override fun getColumnClass(): Class<*> {
    return if (index == 0) TreeTableModel::class.java else TableModel::class.java
  }

  override fun getRenderer(item: Any?): TableCellRenderer {
    return renderer
  }

  override fun getEditor(item: Any?): TableCellEditor {
    return RdCellEditor(project, null, columnAlignments)
  }

  override fun isCellEditable(item: Any?): Boolean {
    val node = item as? DefaultMutableTreeNode ?: return false
    val userObject = node.treeGridUserObject()
    val line = userObject.line
    if (line.columnElements.count() <= index)
      return false
    return renderer.isCellEditable(line.columnElements[index])
  }

  override fun valueOf(o: Any): Any {
    val node = o as DefaultMutableTreeNode
    val userObject = node.treeGridUserObject()
    val line = userObject.line
    if (line.columnElements.count() <= index)
      return BeSpacer()
    return line.columnElements[index]
  }

  val canBeSorted: Boolean
    get() {
      return columnDefinition.isSortable
    }
}