// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl

import javax.swing.JTree
import javax.swing.event.TreeModelEvent
import javax.swing.tree.DefaultMutableTreeNode


fun TreeModelEvent.toLog(): String {
  return "childIndices: ${this.childIndices?.contentToString()}, treePath.last: ${lastPathComponent()}"
}

private fun TreeModelEvent.lastPathComponent(): Any? {
  val lastPathComponent = treePath.lastPathComponent
  return when (lastPathComponent) {
    is DefaultMutableTreeNode -> "${lastPathComponent.userObject}"
    else -> lastPathComponent.toString()
  }
}

/**
 * Compute real table row indices of nodes that are affected by the event.
 *
 * @param e Event description.
 * @return Indices of rows in the table where the nodes (affected by the
 * event) are displayed, or null if they are not available.
 */
fun computeRowIndices(table: JTree, e: TreeModelEvent): IntArray? {
  val rowIndices: IntArray?
  val parentRow = table.getRowForPath(e.treePath)
  if (e.children != null) {
    rowIndices = IntArray(e.children.size)
    for (i in 0 until e.children.size) {
      val childPath = e.treePath.pathByAddingChild(e.children[i])
      val index = table.getRowForPath(childPath)
      rowIndices[i] = if (index < 0)
        parentRow + e.childIndices[i] + 1
      else
        index // child node is not shown yet, compute child row from
      // parent row and index of the child
    }
  }
  else {
    rowIndices = null
  }
  return rowIndices
}
