package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeDynamicControlContainer
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.ui.bedsl.dsl.label
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.RdRendererCache
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.valueOrDefault
import javax.swing.JComponent
import javax.swing.JPanel

class BeDynamicContentRenderer(private val project: Project) : RdMultiRenderer<JPanel, BeDynamicControlContainer>() {
  private val myCache = RdRendererCache(project)

  override fun isEditable(): Boolean {
    return false
  }

  override fun containsEditor(model: BeDynamicControlContainer): Boolean {
    return model.content.valueOrNull?.run {
      this.visible.valueOrDefault(ControlVisibility.Visible) == ControlVisibility.Visible &&
      this.enabled.value &&
      myCache.isEditable(this)
    } ?: false
  }

  override fun getNewInstance(isEditor: Boolean, model: BeDynamicControlContainer): JPanel {
    return BeComponents.getNewGridInstance().apply { isOpaque = false }
  }

  override fun addUpdate(element: BeDynamicControlContainer, lifetime: Lifetime, update: () -> Unit) {
    element.content.advise(lifetime) {
      it.visible.change.advise(lifetime) { update() }
      it.enabled.change.advise(lifetime) { update() }
      ViewRegistry.getInstance(project).getRenderer(it)?.addUpdate(it, lifetime, update)
      update()
    }
  }

  override fun reset(component: JComponent?, value: BeDynamicControlContainer, settings: RdCellSettings): JComponent {
    return getElement(settings.isEditor, value).apply {
      removeAll()
      val constraints = "grow, push"
      val jComponent = myCache.getComponent(value.content.valueOrDefault(label("Invalid dynamic content")), component, settings)
      add(jComponent, constraints)
    }
  }

  override fun resetCounter() {
    super.resetCounter()
    myCache.resetCounter()
  }
}
