// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.intellij.ui.SimpleColoredComponent
import com.intellij.ui.speedSearch.SpeedSearchUtil
import com.jetbrains.ide.model.uiautomation.BeRichText
import com.jetbrains.ide.model.uiautomation.BeTextSettings
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.icons.toIdeaIcon
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.BeGridUI
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.applyModel
import com.jetbrains.rd.ui.bindable.views.utils.getFont
import com.jetbrains.rd.ide.model.IconModel
import com.jetbrains.rd.ide.model.RichTextModel
import javax.swing.JComponent

class BeRichTextRenderer(private val project: Project) : RdMultiRenderer<SimpleColoredComponent, BeRichText>() {
  override fun getNewInstance(isEditor: Boolean, model: BeRichText): SimpleColoredComponent {
    return BeComponents.getNewSCCInstance(true)
  }

  override fun addUpdate(element: BeRichText, lifetime: Lifetime, update: () -> Unit) {
    element.text.change.advise(lifetime) { update() }
  }

  override fun reset(
    component: JComponent?,
    value: BeRichText,
    settings: RdCellSettings
  ): JComponent {
    return getElement(settings.isEditor, value).apply {
      resetRichText(this,
                    value.text.valueOrNull,
                    value.icon.value,
                    settings,
                    RdIconHost.getInstance(project),
                    value.settings,
                    value.tooltip.value,
                    component
      )
    }
  }

  override fun isEditable() = false
  override fun containsEditor(model: BeRichText) = false


  companion object {
    fun resetRichText(
      coloredComponent: SimpleColoredComponent,
      modelRichText: RichTextModel?,
      icon: IconModel?,
      settings: RdCellSettings,
      iconHost: RdIconHost,
      textSettings: BeTextSettings?,
      tooltip: String?,
      component: JComponent? = null
    ) {
      val bg = BeGridUI.getBackground(settings.isSelected, settings.hasFocus)
      val fgOverride = BeGridUI.getForegroundOverride(settings.isSelected, settings.hasFocus)

      with(coloredComponent) {
        applyModel(modelRichText, bg, fgOverride)
        apply {
          this.icon = icon?.toIdeaIcon(iconHost)

          if (settings.applyHighlighting && component != null){
            SpeedSearchUtil.applySpeedSearchHighlighting(component, this, false, settings.isSelected)
          } else {
            background = bg
            foreground = fgOverride ?: BeGridUI.getForeground(settings.isSelected, settings.hasFocus)
          }
          isOpaque = false
        }
        toolTipText = tooltip
        if (textSettings == null) return
        coloredComponent.font = textSettings.getFont()

      }
    }


  }
}



