// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.ui.SimpleColoredComponent
import com.intellij.ui.SimpleTextAttributes
import com.intellij.util.ui.UIUtil
import com.jetbrains.ide.model.uiautomation.ArrowOrientation
import com.jetbrains.ide.model.uiautomation.BeArrowControl
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.fire
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.BeGridUI
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.util.getContrastedForeground
import com.jetbrains.rdclient.util.idea.ui.toColor
import java.awt.Color
import java.awt.event.MouseAdapter
import java.awt.event.MouseEvent
import javax.swing.JComponent
import javax.swing.JPanel


class BeArrowRenderer : RdMultiRenderer<JPanel, BeArrowControl>() {
  override fun isEditable(): Boolean = true
  override fun containsEditor(model: BeArrowControl) = true

  override fun getNewInstance(isEditor: Boolean, model: BeArrowControl): JPanel {
    val component = BeComponents.getNewSCCInstance(false)
    return BeComponents.getNewGridInstance().apply {
      this.isOpaque = false
      this.add(component)
    }
  }

  override fun addUpdate(element: BeArrowControl, lifetime: Lifetime, update: () -> Unit) {
    element.click.advise(lifetime) { update() }
    element.arrowOrientation.advise(lifetime) { update() }
  }

  override fun reset(
    component: JComponent?,
    value: BeArrowControl,
    settings: RdCellSettings
  ): JPanel {
    return getElement(settings.isEditor, value).apply {
      isOpaque = false
      resetLabel(this.components.first() as SimpleColoredComponent, value, settings)

      if (settings.isEditor && value.enabled.value)
        this.addMouseListener(object : MouseAdapter() {
          override fun mousePressed(e: MouseEvent?) {
            value.click.fire()
            stopEditing(component)
          }
        })
    }
  }

  companion object {
    fun resetLabel(
      label: SimpleColoredComponent,
      arrowControl: BeArrowControl,
      settings: RdCellSettings) {
      with(label) {
        isEnabled = settings.isEnabled
        this.clear()
        size = minimumSize
        val isNotRight = arrowControl.arrowOrientation.valueOrNull != ArrowOrientation.RIGHT
        val isNotLeft = arrowControl.arrowOrientation.valueOrNull != ArrowOrientation.LEFT
        //  val bg = UIUtil.TRANSPARENT_COLOR
        val bg = BeGridUI.getBackground(settings.isSelected, settings.hasFocus)
        var localBgColor: Color = if (isNotRight) arrowControl.leftColor.toColor() else bg
        var fgColor = if (isNotRight) getContrastedForeground(localBgColor, UIUtil.getLabelForeground()) else bg
        var attributes = SimpleTextAttributes(
          localBgColor,
          fgColor, null,
          SimpleTextAttributes.STYLE_PLAIN)

        this.append("  ${arrowControl.leftContent}  ", attributes)

        localBgColor = if (isNotLeft) arrowControl.rightColor.toColor() else bg
        fgColor = if (isNotLeft) getContrastedForeground(localBgColor, UIUtil.getLabelForeground()) else bg
        attributes = SimpleTextAttributes(
          localBgColor,
          fgColor, null,
          SimpleTextAttributes.STYLE_PLAIN)

        this.append("  ${arrowControl.rightContent}  ", attributes)
        /*     foreground = if (!isEnabled) JBUI.CurrentTheme.Label.disabledForeground(isSelected) else BeGridUI.getForeground(
                 isSelected,
                 hasFocus
             )*/
        toolTipText = arrowControl.tooltip.value
        isOpaque = true
        background = bg
      }

    }
  }
}




