package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.intellij.ui.treeStructure.treetable.TreeTableTree
import com.jetbrains.ide.model.uiautomation.BeAlignment
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.ui.bedsl.extensions.getBeLabel
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.treeGridUserObject
import java.awt.Component
import javax.swing.*
import javax.swing.table.DefaultTableCellRenderer
import javax.swing.tree.DefaultMutableTreeNode
import javax.swing.tree.TreeCellRenderer

class RdCellRenderer(
  project: Project,
  backgroundIsVisible: Boolean = false,
  private val columnAlignments: List<BeAlignment>,
  private val highlight: Boolean = false
) : DefaultTableCellRenderer(), TreeCellRenderer, ListCellRenderer<Any?> {
  private val renderer = BeGridRenderer(project, backgroundIsVisible)

  override fun getListCellRendererComponent(
    p0: JList<out Any?>?,
    value: Any?,
    row: Int,
    isSelected: Boolean,
    hasFocus: Boolean
  ): Component {
    return renderer.apply {
      reset(
        value as? BeControl ?: "".getBeLabel(),
        RdCellSettings(true, isSelected, true, false, applyHighlighting = highlight),
        null,
        columnAlignments[0]
      )
      resetCounter()
    }
  }

  fun isCellEditable(model: BeControl?): Boolean {
    return renderer.isCellEditable(model)
  }

  override fun getTreeCellRendererComponent(
    tree: JTree?,
    value: Any?,
    isSelected: Boolean,
    isExpanded: Boolean,
    isLeaf: Boolean,
    row: Int,
    hasFocus: Boolean
  ): Component {
    val node = value as DefaultMutableTreeNode
    if (node.isRoot)
      return JPanel()
    val userObject = node.treeGridUserObject()
    val hasFocusInTree = if (tree is TreeTableTree) true else tree?.hasFocus() ?: hasFocus
    return renderer.apply {
      reset(
        userObject.line.columnElements[0],
        RdCellSettings(true, isSelected, hasFocusInTree, false, applyHighlighting = highlight),
        tree,
        columnAlignments[0]
      )
      resetCounter()
    }
  }

  override fun getTableCellRendererComponent(
    tree: JTable?,
    value: Any?,
    isSelected: Boolean,
    hasFocus: Boolean,
    row: Int,
    column: Int
  ): Component {
    return renderer.apply {
      reset(
        value as BeControl,
        RdCellSettings(true, isSelected, true, false, applyHighlighting = highlight),
        tree,
        columnAlignments[column]
      )
      resetCounter()
    }
  }
}
