// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.util.ui.JBUI
import com.intellij.util.ui.ThreeStateCheckBox
import com.jetbrains.ide.model.uiautomation.BeCheckbox
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.valueOrDefault
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.BeGridUI
import com.jetbrains.rd.ui.bindable.views.utils.getLabelText
import java.awt.Component
import javax.swing.JComponent
import javax.swing.SwingConstants


class BeCheckBoxRenderer : RdMultiRenderer<ThreeStateCheckBox, BeCheckbox>() {
  override fun isEditable() = true
  override fun containsEditor(model: BeCheckbox) = true

  override fun addUpdate(element: BeCheckbox, lifetime: Lifetime, update: () -> Unit) {
    element.property.change.advise(lifetime, { update() })
    element.enabled.change.advise(lifetime, { update() })
  }

  override fun getNewInstance(isEditor: Boolean, model: BeCheckbox): ThreeStateCheckBox {
    val checkBox = ThreeStateCheckBox()
    checkBox.isThirdStateEnabled
    checkBox.border = JBUI.Borders.empty(0, 0, 0, 0)
    checkBox.isOpaque = true
    return checkBox
  }

  fun getState(it: Boolean?): ThreeStateCheckBox.State? = when (it) {
    java.lang.Boolean.TRUE -> ThreeStateCheckBox.State.SELECTED
    java.lang.Boolean.FALSE -> ThreeStateCheckBox.State.NOT_SELECTED
    else -> ThreeStateCheckBox.State.DONT_CARE
  }

  override fun reset(
    component: JComponent?,
    value: BeCheckbox,
    settings: RdCellSettings
  ): ThreeStateCheckBox {
    return getElement(settings.isEditor, value).apply {
      this.isEnabled = settings.isEnabled && value.enabled.value
      val description = value.description
      //            text = description?.getHtmlLabelFromBeRichText(description.text.valueOrDefault(RichTextModel(listOf(ModelRichString(""))))) ?:"invalid""
      val result = description.getLabelText()
      text = result
      state = getState(value.property.value)
      isThirdStateEnabled = false
      isRequestFocusEnabled = false
      horizontalAlignment = SwingConstants.CENTER
      verticalAlignment = SwingConstants.CENTER
      alignmentY = Component.CENTER_ALIGNMENT
      foreground = BeGridUI.getForeground(settings.isSelected, settings.hasFocus)
      background = BeGridUI.getBackground(settings.isSelected, settings.hasFocus)
      toolTipText = value.tooltip.value
      if (settings.isEditor && value.visible.valueOrDefault(ControlVisibility.Visible) == ControlVisibility.Visible) {
        addItemListener {
          value.property.set(state == ThreeStateCheckBox.State.SELECTED)
          Companion.stopEditing(component)
        }
      }
    }
  }

}