// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.intellij.ui.SimpleColoredComponent
import com.intellij.ui.speedSearch.SpeedSearchUtil
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeLabel
import com.jetbrains.ide.model.uiautomation.BeTextSettings
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.icons.toIdeaIcon
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.BeGridUI
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.getFont
import com.jetbrains.rd.ide.model.IconModel
import javax.swing.JComponent


class BeLabelRenderer(private val project: Project) : RdMultiRenderer<SimpleColoredComponent, BeLabel>() {
  override fun isEditable(): Boolean = false
  override fun containsEditor(model: BeLabel) = false

  override fun getNewInstance(isEditor: Boolean, model: BeLabel): SimpleColoredComponent {
    return BeComponents.getNewSCCInstance(true)
  }

  override fun addUpdate(element: BeLabel, lifetime: Lifetime, update: () -> Unit) {
    element.text.change.advise(lifetime) { update() }
  }

  override fun reset(
    component: JComponent?,
    value: BeLabel,
    settings: RdCellSettings
  ): SimpleColoredComponent {
    return getElement(settings.isEditor, value).apply {
      resetLabel(
        this,
        value.text.value,
        value.icon.value,
        settings,
        RdIconHost.getInstance(project),
        value.settings,
        value.tooltip.value
      )

    }
  }


  companion object {
    fun resetLabel(
      label: SimpleColoredComponent,
      text: String?,
      icon: IconModel?,
      settings: RdCellSettings,
      iconHost: RdIconHost?,
      textSettings: BeTextSettings?,
      tooltip: String?,
      component: JComponent? = null
    ) {
      with(label) {
        this.clear()
        this.append(text ?: "")
        val backgroundColor = BeGridUI.getBackground(settings.isSelected, settings.hasFocus)
        val foregroundColor = if (!isEnabled) JBUI.CurrentTheme.Label.disabledForeground(settings.isSelected)
                              else BeGridUI.getForeground( settings.isSelected, settings.hasFocus)

        if (textSettings != null) font = textSettings.getFont()
        if (settings.applyHighlighting && component != null){
          SpeedSearchUtil.applySpeedSearchHighlighting(component, this, false, settings.isSelected)
        } else {
          background = backgroundColor
          foreground = foregroundColor
        }
        this.icon = if (icon != null && iconHost != null) icon.toIdeaIcon(iconHost) else null
        isOpaque = false
        toolTipText = tooltip
        isEnabled = settings.isEnabled
      }
    }
  }
}




