// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.ui.HyperlinkAdapter
import com.intellij.ui.HyperlinkLabel
import com.jetbrains.ide.model.uiautomation.BeLink
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.BeGridUI
import com.jetbrains.rd.ui.bindable.views.utils.getSimpleText
import javax.swing.JComponent
import javax.swing.event.HyperlinkEvent

class BeLinkRenderer : RdMultiRenderer<HyperlinkLabel, BeLink>() {
  override fun isEditable() = true
  override fun containsEditor(model: BeLink) = true

  override fun getNewInstance(isEditor: Boolean, model: BeLink): HyperlinkLabel {
    return HyperlinkLabel()
  }

  override fun reset(
    component: JComponent?,
    value: BeLink,
    settings: RdCellSettings
  ): JComponent {
    val panel = getElement(settings.isEditor, value)
    panel.background = BeGridUI.getBackground(settings.isSelected, settings.hasFocus)
    return panel.apply {
      setHyperlinkText(value.prefix.getSimpleText(), value.content.valueOrNull, value.postfix.getSimpleText())
      if (settings.isEditor)
        this.addHyperlinkListener(object : HyperlinkAdapter() {
          override fun hyperlinkActivated(e: HyperlinkEvent?) {
            value.action.click.fire(Unit)
            Companion.stopEditing(component)
          }
        })
      toolTipText = value.tooltip.value
    }
  }
}


