// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeMenuItem
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.popupControls.MenuViewControlInRenderer
import javax.swing.JComponent
import javax.swing.JPanel
import javax.swing.JTable

class BeMenuItemRenderer(private val project: Project) : RdMultiRenderer<MenuViewControlInRenderer, BeMenuItem>() {
  override fun isEditable(): Boolean = true
  override fun containsEditor(model: BeMenuItem) = true
  var clickCount = 0

  override fun getNewInstance(isEditor: Boolean, model: BeMenuItem): MenuViewControlInRenderer =
    MenuViewControlInRenderer(project, model)

  override fun addUpdate(element: BeMenuItem, lifetime: Lifetime, update: () -> Unit) = element.header.advise(lifetime) { update() }
  override fun reset(
    component: JComponent?,
    value: BeMenuItem,
    settings: RdCellSettings
  ): JPanel {
    val control = getElement(settings.isEditor, value)
    control.isEnabled = value.enabled.value
    if (settings.isEditor)
      clickCount++
    control.init(value, component as? JTable, clickCount, settings)
    return control
  }

  override fun resetCounter() {
    super.resetCounter()
    clickCount = 0
  }


}
