// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import javax.swing.JComponent
import javax.swing.JTable
import javax.swing.JTree

object BeControls {
  const val isEditableKey = "BeControls.isEditable"
}


abstract class RdMultiRenderer<T, U> : ITreeGridRenderer<U> where T : JComponent, U : BeControl {
  private val myElements = mutableListOf<T>()
  private var currentElementIndex = -1

  abstract fun getNewInstance(isEditor: Boolean, model: U): T

  private fun getNewInstanceAndPutCanHaveEditor(isEditor: Boolean, model: U): T =
    getNewInstance(isEditor, model).apply {
      putClientProperty(BeControls.isEditableKey, isEditable())
    }

  override fun addUpdate(element: U, lifetime: Lifetime, update: () -> Unit) = Unit


  abstract override fun reset(
    component: JComponent?,
    value: U,
    settings: RdCellSettings
  ): JComponent

  protected fun getElement(isEditor: Boolean, model: U): T {
    if (isEditor)
      return getNewInstanceAndPutCanHaveEditor(true, model)

    var element: T? = null
    currentElementIndex++
    if (myElements.isNotEmpty() && myElements.count() > currentElementIndex) {
      element = myElements[currentElementIndex]
    }
    if (element == null) {
      element = getNewInstanceAndPutCanHaveEditor(false, model)
      myElements.add(element)
      if (myElements.size > 100) {
        getLogger<RdMultiRenderer<T, U>>().error("Possible leak in renderer detected. Fix it or increase threshold.")
      }
    }
    return element
  }

  override fun resetCounter() {
    currentElementIndex = -1
  }

  companion object {
    fun stopEditing(component: JComponent?) {
      when (component) {
        is JTable -> {
          if (component.isEditing)
            component.cellEditor.cancelCellEditing()
        }
        is JTree -> {
          component.cancelEditing()
        }
      }
    }
  }
}