// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.ide.ui.UINumericRange
import com.intellij.ui.JBIntSpinner
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeSpinner
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import javax.swing.JComponent


class BeSpinnerRenderer : RdMultiRenderer<JBIntSpinner, BeSpinner>() {
  override fun isEditable() = true
  override fun containsEditor(model: BeSpinner) = true

  override fun getNewInstance(isEditor: Boolean, model: BeSpinner): JBIntSpinner {
    val jbIntSpinner = JBIntSpinner(UINumericRange(0, Int.MIN_VALUE, Int.MAX_VALUE))
    jbIntSpinner.border = JBUI.Borders.empty(0, 0, 0, 0)
    return jbIntSpinner
  }

  override fun addUpdate(element: BeSpinner, lifetime: Lifetime, update: () -> Unit) {
    element.value.change.advise(lifetime, { update() })
  }

  override fun reset(
    component: JComponent?,
    value: BeSpinner,
    settings: RdCellSettings
  ): JBIntSpinner {
    val spinner = getElement(settings.isEditor, value)
    spinner.max = value.maxValue
    spinner.min = value.minValue
    spinner.number = value.value.value
    if (settings.isEditor)
      spinner.addChangeListener { value.value.set(spinner.number) }
    spinner.toolTipText = value.tooltip.value
    //isEnabled = model.enabled.value
    return spinner
  }
}

