// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.GraphicsConfig
import com.jetbrains.ide.model.uiautomation.BeStyleControl
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.utils.BeGridUI
import com.jetbrains.rd.ui.bindable.views.listControl.utils.RdRendererCache
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.marginToMig
import com.jetbrains.rd.ide.model.RgbColor
import com.jetbrains.rd.ide.model.RgbaColor
import com.jetbrains.rd.ui.themes.rgbToColor
import com.jetbrains.rd.ui.themes.rgbaToColor
import java.awt.AlphaComposite
import java.awt.Color
import java.awt.Graphics
import java.awt.Graphics2D
import javax.swing.JComponent
import javax.swing.JPanel


class BeStyleRenderer(val project: Project) : RdMultiRenderer<JPanel, BeStyleControl>() {
  private val myCache = RdRendererCache(project)

  override fun getNewInstance(isEditor: Boolean, model: BeStyleControl): JPanel {

    val opacity = model.opacity.value
    if (opacity != null) {
      val panel = object : JPanel(BeComponents.getMigLayout()) {
        override fun paint(graphics: Graphics) {
          val config = GraphicsConfig(graphics)
          val g2d = graphics as Graphics2D
          g2d.composite = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, opacity.opacity)
          super.paintChildren(g2d)
          config.restore()
        }
      }
      panel.isOpaque = false
      return panel
    }
    return BeComponents.getNewGridInstance().apply { isOpaque = false }

  }

  override fun addUpdate(element: BeStyleControl, lifetime: Lifetime, update: () -> Unit) {
    myCache.getByKey(element.content, project)!!.addUpdate(element.content, lifetime, update)
  }

  override fun isEditable() = false
  override fun containsEditor(model: BeStyleControl): Boolean =
    myCache.isEditable(model.content)

  override fun reset(
    component: JComponent?,
    value: BeStyleControl,
    settings: RdCellSettings
  ): JPanel {
    return getElement(settings.isEditor, value).apply {
      removeAll()
      var constraints = "grow, push"
      val jComponent = myCache.getComponent(value.content, component, settings)
      if (value.margin.value != null) {
        val it = value.margin.value!!
        constraints = marginToMig(it.margin) + ", ${constraints}"
      }

      if (value.backgroundColor.value != null) {
        val it = value.backgroundColor.value!!
        isOpaque = true
        var color : Color? = null
        val rgbaColor = it.color as? RgbaColor
        if (rgbaColor != null) {
          color = rgbaColor.rgbaToColor()
        }
        val rgbColor = it.color as? RgbColor
        if (rgbColor != null) {
          color = rgbColor.rgbToColor()
        }
        if(color != null)
          background = if(settings.isSelected) BeGridUI.getBackground(true, true) else color
        /*if(settings.isSelected)
        foreground = BeGridUI.getForeground(settings.isSelected, true)
        */
        /*TreeTraverse.descendantsDfs(jComponent){
           it.components.filterIsInstance(JComponent::class.java).asSequence()
         }.forEach {
           it.isOpaque = true
           it.background = if(settings.isSelected) BeGridUI.getBackground(true, true) else rgbaColor.rgbaToColor()
           it.foreground = BeGridUI.getForeground(settings.isSelected, true)
         }*/
      }
      add(jComponent, constraints)
    }
  }


  override fun resetCounter() {
    super.resetCounter()
    myCache.resetCounter()
  }
}