// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers

import com.intellij.openapi.project.Project
import com.intellij.ui.SimpleColoredComponent
import com.jetbrains.ide.model.uiautomation.BeToggleButton
import com.jetbrains.rd.ui.bedsl.dsl.tryGetSettings
import com.jetbrains.rd.ui.icons.RdIconHost
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import com.jetbrains.rd.ui.bindable.views.utils.getLabelText
import java.awt.event.MouseAdapter
import java.awt.event.MouseEvent
import javax.swing.JComponent

class BeToggleButtonLabelRenderer(private val project: Project) : RdMultiRenderer<SimpleColoredComponent, BeToggleButton>() {
  override fun isEditable() = true
  override fun containsEditor(model: BeToggleButton) = true

  override fun getNewInstance(isEditor: Boolean, model: BeToggleButton): SimpleColoredComponent = BeComponents.getNewSCCInstance(false)

  override fun reset(
    component: JComponent?,
    value: BeToggleButton,
    settings: RdCellSettings
  ): SimpleColoredComponent {
    return getElement(settings.isEditor, value).apply {
      BeLabelRenderer.resetLabel(
        this,
        value.content.getLabelText(),
        value.content.icon.value,
        settings,
        RdIconHost.getInstance(project),
        value.content.tryGetSettings(),
        value.tooltip.value
      )
      if (settings.isEditor) {
        addMouseListener(object : MouseAdapter() {
          override fun mousePressed(e: MouseEvent?) {
            value.isChecked.set(!value.isChecked.value)
          }
        })
      }
    }
  }
}