// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers.popupControls

import com.intellij.icons.AllIcons
import com.intellij.openapi.project.Project
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeComboBox
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdMultiRenderer
import net.miginfocom.swing.MigLayout
import javax.swing.Icon
import javax.swing.JLabel
import javax.swing.JTable

class ComboBoxViewControlInRenderer(project: Project, model: BeComboBox) : ControlWithPopupInRenderer(
  project,
  MigLayout(
    "ins 0, gap 0, fill",
    "${JBUI.scale(1)}[grow][min!]${JBUI.scale(5)}"
  ),
  model
) {

  init {
    isOpaque = false
  }

  private val myIcon = JLabel(AllIcons.General.ArrowDown)

  fun setIcon(icon: Icon) {
    myIcon.icon = icon
  }

  fun getIcon(): Icon {
    return myIcon.icon
  }

  override fun setValue(value: BeControl, settings: RdCellSettings) {
    super.setValue(value, settings)
    add(myIcon)
  }

  fun init(model: BeComboBox, table: JTable?, clickCount: Int, settings: RdCellSettings) {
    mySetItemChosen = {
      model.selectedIndex.set(myList.selectedIndex)
      val element = model.values[model.selectedIndex.value]
      setValue(element, RdCellSettings(false, true, true, settings.isEditor, applyHighlighting = false))
      RdMultiRenderer.stopEditing(table)

    }
    myList.selectedIndex = model.selectedIndex.value
    init(model, settings.isEditor, clickCount, model.values)
    if (model.selectedIndex.value < model.values.count()) {
      val value = model.values[model.selectedIndex.value]
      setValue(value, settings)
    }
  }
}