// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers.popupControls

import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.popup.JBPopup
import com.intellij.openapi.ui.popup.JBPopupFactory
import com.intellij.openapi.ui.popup.JBPopupListener
import com.intellij.openapi.ui.popup.LightweightWindowEvent
import com.intellij.ui.awt.RelativePoint
import com.intellij.ui.components.JBList
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdCellRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.utils.RdRendererCache
import net.miginfocom.swing.MigLayout
import java.awt.Dimension
import java.awt.Point
import java.awt.event.MouseAdapter
import java.awt.event.MouseEvent
import javax.swing.JComponent
import javax.swing.JPanel

open class ControlWithPopupInRenderer(
  project: Project,
  migLayout: MigLayout,
  private val container: BeControl
) : JPanel(migLayout) {
  private var popup: JBPopup? = null
  protected val myList = JBList<Any>()
  protected lateinit var mySetItemChosen: () -> Unit
  private val myCache = RdRendererCache(project)

  private val myCellRenderer = RdCellRenderer(
    project,
    false,
    arrayListOf(BeAlignment.Left)
  )

  private fun initPopup(list: List<BeControl>?) {
    val toTypedArray = list?.toTypedArray() ?: return
    myList.model = toTypedArray.let { JBList.createDefaultListModel<Any>(*it) }
    myList.cellRenderer = myCellRenderer.apply {
      myCache.resetCounter()
    } //ComboBoxRendererWrapper(listCellRenderer, true)
    // todo:  myList.cellRenderer = RdCellRenderer(lifetime, project, false, arrayListOf(BeAlignment.Left))//ComboBoxRendererWrapper(listCellRenderer, true)
    /*val surrendersFocusOnKeystrokeOldValue =
        if (table is JBTable) table.surrendersFocusOnKeyStroke() else table.surrendersFocusOnKeystroke*/
  }


  protected fun createAndShowPopup(
    controlWithPopupInRenderer: ControlWithPopupInRenderer) {
    if (!isEnabled) return
    if (popup != null) {
      popup!!.cancel()
      popup = null
      return
    }
    popup = JBPopupFactory.getInstance()
      .createListPopupBuilder(myList)

      .setItemChoosenCallback {
        if (myList.selectedIndex >= 0) {
          val beControl = myList.model.getElementAt(myList.selectedIndex) as BeControl
          if (beControl is BeAbstractButton) {
            when (beControl) {
              is BeButton -> beControl.action.click.fire(Unit)
              is BeToggleButton -> beControl.isChecked.value = !beControl.isChecked.value
            }
          }
        }

        mySetItemChosen()
        myClickCount = 0
      }
      .setCancelCallback {
        //TableUtil.stopEditing(table)
        true
      }
      .addListener(object : JBPopupListener {

        override fun onClosed(event: LightweightWindowEvent) {
          // table.setSurrendersFocusOnKeystroke(surrendersFocusOnKeystrokeOldValue)
          super.onClosed(event)
          popup = null
        }
      })
      .setMinSize(Dimension(controlWithPopupInRenderer.width, -1))
      .createPopup()
    val point = Point(0, controlWithPopupInRenderer.y + controlWithPopupInRenderer.height)
    popup?.show(RelativePoint(controlWithPopupInRenderer, point))
  }

  private var myClickCount: Int = 0

  protected fun init(
    model: BeControl,
    editor: Boolean,
    clickCount: Int,
    values: List<BeControl>?
  ) {
    //myClickCount = 0
    isEnabled = model.enabled.value
    toolTipText = model.tooltip.value
    if (editor && isEnabled && container.visible.valueOrNull == ControlVisibility.Visible) {
      initPopup(values)
      addMouseListener(object : MouseAdapter() {
        override fun mousePressed(p0: MouseEvent?) {
          myClickCount++
          if (myClickCount % 2 == 0) return
          createAndShowPopup(this@ControlWithPopupInRenderer)

        }
      })

    }
  }

  protected open fun setValue(value: BeControl, settings: RdCellSettings) {
    removeAll()

    val isEnabled = value.enabled.value
    val component: JComponent = myCache.getComponent(value, null, settings)
    myCache.resetCounter()
    if (settings.isEditor && isEnabled && container.visible.valueOrNull == ControlVisibility.Visible) {
      val listener = object : MouseAdapter() {
        override fun mousePressed(p0: MouseEvent?) {
          myClickCount++
          if (myClickCount % 2 == 0) return
          createAndShowPopup(this@ControlWithPopupInRenderer)

        }
      }
      when (component) {
        is JPanel -> component.components.forEach { it.addMouseListener(listener) }
        else -> component.addMouseListener(listener)
      }
    }
    add(component, "growx")
  }
}