// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.renderers.popupControls

import com.intellij.icons.AllIcons
import com.intellij.openapi.project.Project
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeMenuItem
import com.jetbrains.ide.model.uiautomation.BeSpacer
import com.jetbrains.rd.util.reactive.valueOrDefault
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import com.jetbrains.rd.ui.bindable.views.listControl.renderers.RdMultiRenderer
import com.jetbrains.rd.ui.bindable.views.utils.BeComponents
import javax.swing.JLabel
import javax.swing.JTable

class MenuViewControlInRenderer(
  project: Project,
  model: BeMenuItem
) : ControlWithPopupInRenderer(project,
                               BeComponents.getMigLayout({
                                                           it.columnConstraints = "${JBUI.scale(1)}[grow][min!]${JBUI.scale(5)}"
                                                         }), model) {

  init {
    isOpaque = false
  }

  private val myIcon = JLabel(AllIcons.General.ArrowDown)

  override fun setValue(value: BeControl, settings: RdCellSettings) {
    super.setValue(value, settings)
    add(myIcon)
  }

  fun init(model: BeMenuItem, table: JTable?, clickCount: Int, settings: RdCellSettings) {
    mySetItemChosen = {
      val index = myList.selectedIndex
      setValue(model.header.valueOrDefault(BeSpacer()),
               RdCellSettings(settings.isEnabled, settings.isSelected, settings.hasFocus, settings.isEditor))
      RdMultiRenderer.stopEditing(table)
    }

    init(model, settings.isEditor, clickCount, model.innerMenuItems.valueOrNull)
    setValue(model.header.valueOrDefault(BeSpacer()), settings)
  }


}