// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.utils

import com.intellij.ide.IdeBundle
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.ui.bindable.ViewRegistry
import com.jetbrains.rd.ui.bindable.views.listControl.ITreeGridRenderer
import com.jetbrains.rd.ui.bindable.views.listControl.RdCellSettings
import java.util.concurrent.ConcurrentHashMap
import javax.swing.JComponent
import javax.swing.JLabel

class RdRendererCache(private val project: Project) {
  private val myCache = ConcurrentHashMap<String, ITreeGridRenderer<BeControl>>()

  private fun getKey(key: Class<*>): String {
    return key.name
  }

  fun getByKey(control: BeControl, project: Project): ITreeGridRenderer<BeControl>? {
    val key = getKey(control.javaClass)
    var cached = myCache[key]
    if (cached != null) return cached

    cached = ViewRegistry.getInstance(project).getRenderer(control)

    if (cached != null) myCache[key] = cached
    return cached
  }

  fun getComponent(
    control: BeControl,
    table: JComponent?,
    settings: RdCellSettings
  ): JComponent {
    val renderer = getByKey(control, project)
    val jComponent = renderer?.reset(table, control, settings)
    val controlVisibility = control.visible.valueOrNull ?: ControlVisibility.Visible
    jComponent?.isVisible = controlVisibility == ControlVisibility.Visible

    return jComponent ?: JLabel(IdeBundle.message("name.invalid"))
  }

  fun resetCounter() {
    myCache.forEach {
      it.value.resetCounter()
    }
  }

  fun isEditable(model: BeControl?): Boolean {
    return if (model != null) getByKey(model, project)?.containsEditor(model) ?: false else false
  }
}