// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.listControl.utils

import com.jetbrains.rd.platform.util.TreeTraverse
import java.awt.Component
import java.awt.Container
import java.awt.Rectangle
import java.awt.event.MouseEvent
import javax.swing.JComponent
import javax.swing.SwingUtilities

fun getDeepestRendererComponentByEvent(
  container: Container,
  rendererComponent: Component,
  nodeBounds: Rectangle,
  e: MouseEvent
): JComponent? {
  container.add(rendererComponent)
  //NOTE(Korovin): If mousePressed event happens on a node row but X coordinate is outside of nodeBounds, IDEA normalizes coordinate to be in
  // [nodeBounds.x..nodeBounds.x+nodeBounds.width-1] (see com.intellij.util.ui.MouseEventAdapter.convert)
  // This means any click to the left of the node will activate leftmost editor in a node if it is placed first, and same story with the right side.
  // This hack just treat all clicks that happen on `nodeBounds.x` or `nodeBounds.width - 1` coordinate as misses.
  val shiftedXBound = if (nodeBounds.width > 0) nodeBounds.x + 1 else nodeBounds.x
  val shiftedWidth = if (nodeBounds.width > 0) nodeBounds.width - 1 else nodeBounds.width
  rendererComponent.setBounds(shiftedXBound, nodeBounds.y, shiftedWidth, nodeBounds.height)
  rendererComponent.validate()
  val componentPoint = SwingUtilities.convertPoint(e.component, e.point, rendererComponent)
  val activeComponent = SwingUtilities.getDeepestComponentAt(rendererComponent, componentPoint.x, componentPoint.y) as? JComponent
  container.remove(rendererComponent)
  return activeComponent
}

fun Component.pathToRoot(root: Component? = null): Sequence<Component> =
  TreeTraverse.pathToRoot(this) { it.parent }
    .takeWhile { it != root }