// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.utils

import com.intellij.ui.SimpleColoredComponent
import com.intellij.util.ui.JBUI
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.util.lifetime.Lifetime
import net.miginfocom.swing.MigLayout
import javax.swing.JComponent
import javax.swing.JPanel

class BeComponents {
  companion object {
    fun getNewSCCInstance(withRendererMargin: Boolean = false): SimpleColoredComponent {
      return SimpleColoredComponent().apply {
        if (withRendererMargin)
          ipad = JBUI.insets(1, 0, 3, 0)
        border = JBUI.Borders.empty()
        isOpaque = false
      }
    }

    fun getNewGridInstance(tuneDefaultLayout: ((MigLayout) -> Unit)? = null): JPanel {
      return JPanel(getMigLayout(tuneDefaultLayout)).apply {
        isOpaque = true
        border = JBUI.Borders.empty(0, 0, 0, 0)
      }
    }

    fun getMigLayout(tuneDefaultLayout: ((MigLayout) -> Unit)? = null): MigLayout {
      return MigLayout(getDefaultMigLayoutString()).apply {
        tuneDefaultLayout?.invoke(this)
      }
    }

    fun getDefaultMigLayoutString() = "insets 0, gap 0, hidemode 2, novisualpadding, fill, left"
  }
}

fun BeControl.generateName(): String {
  return javaClass.simpleName + "@" + controlId.value.ifBlank { "hashCode:" + hashCode() }
}

fun BeControl.bindDefaultProperties(lifetime: Lifetime, component: JComponent) {
  this.enabled.advise(lifetime) { component.isEnabled = it }
  this.tooltip.advise(lifetime) { component.toolTipText = it }
  this.visible.advise(lifetime) {
    component.isVisible = it == ControlVisibility.Visible
    component.parent?.revalidate()
  }
}