package com.jetbrains.rdclient.ui.bindableUi.views.utils

import com.jetbrains.ide.model.uiautomation.*
import javax.swing.JComponent

fun BeAlignment.toDock(): String {
  return when (this) {
    BeAlignment.Bottom -> "south"
    BeAlignment.Center -> "center"
    BeAlignment.Stretch -> "west, push, grow"
    BeAlignment.Default -> "west"
    BeAlignment.JustifiedDefault -> "west"
    else -> "west, push, grow"
  }
}

  internal fun BeControl.shouldHaveCustomFocusMargin(parentOrientation: GridOrientation? = null) : Boolean {
      return when(this){
          is BeGrid -> {
              this.checkGrid(parentOrientation)
            }
          is BeSpanGrid -> {
              this.checkSpanGrid(parentOrientation)
            }
          else -> false
        }
    }
  
  private fun BeGrid.checkGrid(parentOrientation: GridOrientation? = null): Boolean {
      var hasFocusable = false;
      var hasNonFocusable = false;
      items.valueOrNull?.forEachIndexed() { idx, it ->
          if (hasFocusable && hasNonFocusable) return true
          if (parentOrientation != this.orientation && idx == 1) return hasFocusable
          when (getFocusability(it.content)) {
              BeControlFocusability.FOCUSABLE -> {
                  hasFocusable = true;
                  return@forEachIndexed
                }
              BeControlFocusability.NON_FOCUSABLE -> {
                  hasNonFocusable = true; return@forEachIndexed
                }
              BeControlFocusability.UNDEFINED -> {
                 // if (it.content.shouldHaveCustomFocusMargin(this.orientation)) {
                    return true
                  //}
                }
            }
        }
      if (hasFocusable && hasNonFocusable) return true
      return false
    }
  
  private fun BeSpanGrid.checkSpanGrid(parentOrientation: GridOrientation? = null): Boolean {
      var hasFocusable = false;
      var hasNonFocusable = false;
      items.valueOrNull?.mapNotNull{it.content.valueOrNull?.first()}?.forEachIndexed() { idx, it ->
          if (hasFocusable && hasNonFocusable) return true
        /*  if (parentOrientation == GridOrientation.Horizontal && idx == 1) return hasFocusable */
          when (getFocusability(it.content.content)) {
              BeControlFocusability.FOCUSABLE -> {
                  hasFocusable = true;
                  return@forEachIndexed
                }
              BeControlFocusability.NON_FOCUSABLE -> {
                  hasNonFocusable = true; return@forEachIndexed
                }
              BeControlFocusability.UNDEFINED -> {
                  //if (it.content.content.shouldHaveCustomFocusMargin(GridOrientation.Vertical)) {
                    return true
                 // }
                }
            }
        }
      if (hasFocusable && hasNonFocusable) return true
      return false
    }
  
  
  private enum class BeControlFocusability{FOCUSABLE, NON_FOCUSABLE, UNDEFINED}
  private fun getFocusability(it: BeControl):   BeControlFocusability{
      return when (it) {
          is BeGrid, is BeSpanGrid -> {
              return BeControlFocusability.UNDEFINED
            }
          is BeAbstractText -> {
              return BeControlFocusability.NON_FOCUSABLE
            }
          else -> {
              BeControlFocusability.NON_FOCUSABLE
            }
        }
    }
  
  fun BeControl.contentHasFocusBorder(): Boolean {
      return when (this) {
          //is BeAbstractText, is BeHeader -> false
          is BeStyleControl -> this.content.contentHasFocusBorder()
          is BeBorder -> this.content.contentHasFocusBorder() ?: true
      
          //is BeGrid, is BeSpanGrid, is BeSplitControl,
          //is BeBehaviorControl, is BeCollapsiblePanel, is BeToolbar  -> true//this.orientation != GridOrientation.Horizontal //&& this.items.valueOrNull?.count() == 1
          // is BeSpanGrid -> this.items.valueOrNull?.count() == 1
          is BeControlWithEditableText, is BeSpinner -> true
          is BeAbstractButton -> true
          is BeRadioButton, is BeCheckbox -> true
          else -> return false
    }
  }
