// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.bindable.views.utils

import com.intellij.openapi.actionSystem.ActionGroup
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.DefaultActionGroup
import com.intellij.openapi.actionSystem.Separator
import com.jetbrains.rd.framework.impl.RdProperty
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination

/**
 * NOTE(Korovin): This is an attempt to simulate "live" action group
 * Note that update signal is only fired if you added action via overloads with lifetime
 * In the perfect world we either should make add methods not "final" in DefaultActionGroup or reimplement
 * this collection over ActionGroup base class
 */
class LiveActionGroup : DefaultActionGroup() {
  val updated = RdProperty(Any())

  fun add(lifetime: Lifetime, action: AnAction) {
    add(action)
    lifetime.onTermination {
      remove(action)
      updated.value = Any()
    }
    updated.value = Any()
  }

  fun addSeparator(lifetime: Lifetime) {
    add(lifetime, Separator.create())
  }

  fun addAll(lifetime: Lifetime, actionGroup: ActionGroup) {
    actionGroup.getChildren(null).forEach {
      add(lifetime, it)
    }
  }
}