// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.icons

import com.intellij.openapi.components.service
import com.intellij.openapi.util.ModificationTracker
import com.intellij.openapi.vfs.VirtualFileManager
import com.intellij.openapi.vfs.newvfs.BulkFileListener
import com.intellij.openapi.vfs.newvfs.events.VFileEvent
import com.intellij.psi.util.PsiModificationTracker
import com.jetbrains.rd.platform.util.application
import javax.swing.Icon

class DefaultIconInModificationTracker : ModificationTracker {
  companion object {
    fun getInstance() = service<DefaultIconInModificationTracker>()

    val invalidator by lazy {
      val instance = getInstance()
      return@lazy { _: Icon -> instance.modificationCount }
    }
  }

  private var counter = 0L

  init {
    val connection = application.messageBus.connect()

    connection.subscribe(PsiModificationTracker.TOPIC, PsiModificationTracker.Listener {
      invalidate()
    })
    connection.subscribe(VirtualFileManager.VFS_CHANGES, object : BulkFileListener {
      override fun after(events: List<VFileEvent?>) {
        invalidate()
      }
    })
  }

  // todo pull increments from the backend
  private fun invalidate() {
    counter++
  }

  override fun getModificationCount(): Long = counter
}