// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.icons

import com.intellij.openapi.components.service
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rd.platform.util.idea.LifetimedProjectService
import com.jetbrains.rd.ui.icons.diagnostic.MarkBackendFeaturesAction
import java.util.concurrent.ConcurrentHashMap
import javax.swing.Icon

open class RdIconHost(project: Project) : LifetimedProjectService(project) {
  companion object {
    fun getInstance(project: Project): RdIconHost = project.service()
  }

  internal val imageIconCache = ConcurrentHashMap<Long, Icon>()

  open fun createWrapper(iconNode: IconModel): IconWrapper = when (iconNode) {
    is CompositionIconModel -> CompositionIconWrapper(iconNode, this)
    is RowIconModel -> RowIconWrapper(iconNode, this)
    is ImageIconModel -> ImageIconWrapper(iconNode, this)
    is RotatedIconModel -> RotatedIconWrapper(iconNode, this)
    is ColorIconModel -> ColorIconWrapper(iconNode)
    is IdeaIconModel -> IdeaIconWrapper(iconNode.iconUrl)
    is ColorizedIconModel -> ColorizedIconWrapper(iconNode, this)
    else -> throw UnsupportedOperationException("Unsupported node type! $iconNode")
  }

  fun toIdeaIcon(iconId: IconModel): Icon {
    val icon = createWrapper(iconId).createIcon()
    return MarkBackendFeaturesAction.decorateIconIfNeeded(icon)
  }
}

fun IconModel.toIdeaIcon(iconHost: RdIconHost): Icon {
  return iconHost.toIdeaIcon(this)
}

fun IconModel.toIdeaIcon(project: Project): Icon {
  return RdIconHost.getInstance(project).toIdeaIcon(this)
}
