// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.icons.diagnostic

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.project.DumbAware
import com.intellij.ui.JBColor
import com.intellij.ui.LayeredIcon
import com.intellij.util.ui.ColorIcon
import javax.swing.Icon
import kotlin.math.min

class MarkBackendFeaturesAction : AnAction(), DumbAware {
  companion object {
    private var isEnabled = false

    fun decorateIconIfNeeded(icon: Icon): Icon {
      return if (isEnabled) {
        val colorIcon = ColorIcon(min(icon.iconWidth, icon.iconHeight), JBColor.GREEN, true)
        val markedIcon = LayeredIcon(2)
        markedIcon.setIcon(colorIcon, 0)
        markedIcon.setIcon(icon, 1)
        return markedIcon
      }
      else {
        icon
      }
    }
  }

  override fun update(e: AnActionEvent) {
    val presentation = e.presentation
    if (isEnabled) {
      presentation.text = "Disable debugging background for backend icons and etc."
    }
    else {
      presentation.text = "Enable debugging background for backend icons and etc."
    }
  }

  override fun actionPerformed(e: AnActionEvent) {
    isEnabled = !isEnabled
  }

}