// Copyright 2000-2021 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rd.ui.themes;

import com.intellij.openapi.application.ApplicationManager
import com.intellij.openapi.diagnostic.logger
import com.intellij.ui.Gray
import com.intellij.ui.JBColor
import com.intellij.util.ui.UIUtil
import com.jetbrains.rd.ide.model.ColorId
import com.jetbrains.rd.ide.model.RgbColor
import com.jetbrains.rd.ide.model.RgbaColor
import com.jetbrains.rd.ide.model.ThemeColorId
import java.awt.Color

class ColorHost {
  companion object {
    fun getInstance(): ColorHost = ApplicationManager.getApplication().getService(ColorHost::class.java)
  }

  private val logger = logger<ColorHost>()

  private val enumMap = mapOf<ColorId, Color>(
    ColorId.DefaultForeground to JBColor.black,
    ColorId.Gray to JBColor.gray,
    ColorId.LabelForeground to UIUtil.getLabelForeground(),
    ColorId.LabelForegroundBrighter to JBColor(Gray.x00, Gray.xFF),
    ColorId.LabelDisabled to UIUtil.getLabelDisabledForeground(),
    ColorId.ListBackground to UIUtil.getListBackground()
  )

  fun getColor(colorId: ThemeColorId): Color {
    return enumMap[colorId.id] ?: run {
      logger.error("Can't resolve color by enum: $colorId")
      return JBColor.BLACK
    }
  }
}

fun RgbaColor.rgbaToColor() = Color(red, green, blue, alpha)

fun RgbColor.rgbToColor() = Color(red, green, blue)