package com.jetbrains.rd.ui.util

import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.KeyboardShortcut
import com.intellij.openapi.actionSystem.ShortcutSet
import com.intellij.openapi.project.DumbAwareAction
import com.jetbrains.ide.model.uiautomation.BeActionListener
import com.jetbrains.rd.util.reactive.fire
import javax.swing.JComponent
import javax.swing.KeyStroke
import javax.swing.SwingUtilities

fun ShortcutSet.getStringPresentation(): List<String> {
  return shortcuts.filterIsInstance(
    KeyboardShortcut::class.java).mapNotNull {
    val firstShortcutPresentation = it.firstKeyStroke.toString()
    if (firstShortcutPresentation.isEmpty()) null else firstShortcutPresentation
  }
}

fun JComponent.registerShortcuts(shortcuts: ShortcutSet?,
                                 actionListener: BeActionListener) {
  shortcuts?.let {
    object : DumbAwareAction() {
      override fun actionPerformed(e: AnActionEvent) {
        actionListener.reaction.fire()
      }

      override fun displayTextInToolbar(): Boolean = false //9todo: may be this is needed&

      override fun update(e: AnActionEvent) {}
    }.registerCustomShortcutSet(it, this)
  }
}

fun getShortcut(shortcutTextPresentation: String, component: JComponent, overrideDefault: Boolean): KeyboardShortcut? {
  val keyStroke = KeyStroke.getKeyStroke(shortcutTextPresentation)
  if (keyStroke == null) return null
  if (!overrideDefault &&
      (SwingUtilities.getUIInputMap(component, JComponent.WHEN_IN_FOCUSED_WINDOW)?.get(keyStroke) != null ||
       SwingUtilities.getUIInputMap(component, JComponent.WHEN_FOCUSED)?.get(keyStroke) != null ||
       SwingUtilities.getUIInputMap(component, JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT)?.get(keyStroke) != null))
    return null
  return KeyboardShortcut(keyStroke, null)
}