package com.jetbrains.rdserver.unattendedHost

import com.intellij.ide.CliResult
import com.intellij.openapi.application.ApplicationStarter
import com.intellij.openapi.diagnostic.Logger
import com.intellij.ui.IconManager
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rdserver.joinLinks.permanent.CwmPermanentLinkManager
import java.util.concurrent.CompletableFuture
import java.util.concurrent.Future

abstract class UnattendedHostStarterBase : ApplicationStarter {
  companion object {
    protected val logger = Logger.getInstance(UnattendedHostStarterBase::class.java)
  }

  abstract override fun getCommandName(): String
  override fun isHeadless(): Boolean = false // idea headless does not suit CWM at all

  override fun premain(args: MutableList<String>) {
  }

  override fun main(args: MutableList<String>) {
    UnattendedHostManager.getInstance().setUnattendedMode()

    if (application.isHeadlessEnvironment)
      IconManager.activate()

    if (args.size <= 1) {
      return
    }

    val cliResult = processExternalCommandLineAsync(args, null).get()
    logger.info(cliResult.message)
    println(cliResult.message)
    if (cliResult.exitCode != 0)
      application.exit(true, true, false)
  }

  override fun canProcessExternalCommandLine() = true

  override fun processExternalCommandLineAsync(args: MutableList<String>, currentDirectory: String?): Future<CliResult> {
    if (args.size <= 0) {
      logger.error("No project path provided")
      return CompletableFuture.completedFuture(CliResult(1, "No project path provided"))
    }
    return UnattendedHostManager.openProjectAndGetJoinLink(args.last()).handle { pair, ex ->
      if (ex != null) {
        logger.error(ex)
        return@handle CliResult(1, ex.message)
      }
      if (pair == null) {
        logger.error("Failed to get join link")
        return@handle CliResult(1, "Failed to get join link")
      }
      val (project, link) = pair
      logger.info("Join link: $link")
      println("Join link: $link")
      println("Control socket port: ${UnattendedHostControlSocketManager.getInstance().currentControlSocketPort}")
      println("Unattended host PID: ${ProcessHandle.current().pid()}")

      if (args.contains("permalink")) {
        val permalinkHost = CwmPermanentLinkManager.getInstance(project)
        permalinkHost.enableAndGetPermanentLink().handle handle2@{ permalink, ex2 ->
          if (ex2 != null) {
            logger.error("Exception creating permalink", ex2)
            return@handle2
          }

          logger.info("Permanent link: $permalink")
          println("Permanent link: $permalink")
        }
      }

      return@handle CliResult(0, link)
    }.toCompletableFuture()
  }
}