package com.jetbrains.rdserver.unattendedHost

import com.intellij.ide.CliResult
import com.intellij.openapi.application.ApplicationInfo
import com.intellij.openapi.application.ApplicationStarter
import com.intellij.openapi.diagnostic.Logger
import com.intellij.openapi.project.ProjectManager
import com.jetbrains.cwm.unattended.common.UnattendedHostConstants
import com.jetbrains.rd.platform.util.withFragment
import com.jetbrains.rdserver.joinLinks.permanent.CwmPermanentLinkManager
import com.jetbrains.rdserver.joinLinks.RdServerJoinLinkManager
import java.util.concurrent.CompletableFuture
import java.util.concurrent.Future
import kotlin.system.exitProcess

class UnattendedHostStatusStarter : ApplicationStarter {
  companion object {
    private val logger = Logger.getInstance(UnattendedHostStatusStarter::class.java)
  }

  override fun getCommandName(): String = "cwmHostStatus"
  override fun isHeadless(): Boolean = true

  override fun premain(args: MutableList<String>) {}

  override fun main(args: MutableList<String>) {
    val result = processExternalCommandLineAsync(args, null).get()
    exitProcess(result.exitCode)
  }

  override fun canProcessExternalCommandLine() = true

  override fun processExternalCommandLineAsync(args: MutableList<String>, currentDirectory: String?): Future<CliResult> {
    val statusInfo = StringBuilder()

    statusInfo.appendLine("${UnattendedHostConstants.currentlyRunningVersion}${ApplicationInfo.getInstance().build}")
    statusInfo.appendLine("Unattended host PID: ${ProcessHandle.current().pid()}")
    if (!UnattendedHostManager.getInstance().isUnattendedMode) {
      if (ProjectManager.getInstance().openProjects.any { !it.isDefault })
        statusInfo.appendLine(UnattendedHostConstants.notRunningInUnattendedMode) // only report non-unattended mode if any projects are open
      else
        statusInfo.appendLine(UnattendedHostConstants.noProjectsOpen)
    } else {
      val port = UnattendedHostControlSocketManager.getInstance().currentControlSocketPort
      statusInfo.appendLine("${UnattendedHostConstants.controlPortText}$port")
      ProjectManager.getInstance().openProjects.forEach {
        val permaLink = CwmPermanentLinkManager.getInstance(it).lastPermanentLink
        if (permaLink != null) {
          statusInfo.appendLine("${UnattendedHostConstants.permaLinkPrefixText}${it.name}: $permaLink")
        }
        val joinLink = RdServerJoinLinkManager.getInstance(it).getExistingJoinLink()
        if (joinLink != null) {
          val joinTokens = UnattendedHostProjectService.getInstance(it)
          statusInfo.appendLine("${UnattendedHostConstants.controlLinkPrefixText}${it.name}: ${joinLink.withFragment(joinLink.fragment + "&jt=" + joinTokens.createOrGetSingleUseConnectionToken())}")
        }
      }
    }

    println(statusInfo.toString())
    return CompletableFuture.completedFuture(CliResult(0, statusInfo.toString()))
  }
}