@file:Suppress("EXPERIMENTAL_API_USAGE","EXPERIMENTAL_UNSIGNED_LITERALS","PackageDirectoryMismatch","UnusedImport","unused","LocalVariableName","CanBeVal","PropertyName","EnumEntryName","ClassName","ObjectPropertyName","UnnecessaryVariable","SpellCheckingInspection")
package com.jetbrains.rd.ide.model

import com.jetbrains.rd.framework.*
import com.jetbrains.rd.framework.base.*
import com.jetbrains.rd.framework.impl.*

import com.jetbrains.rd.util.lifetime.*
import com.jetbrains.rd.util.reactive.*
import com.jetbrains.rd.util.string.*
import com.jetbrains.rd.util.*
import kotlin.reflect.KClass



/**
 * This provider should go after the provider with specified ID
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensSettingsModel private constructor(
    private val _providers: RdMap<String, CodeLensProviderModel>,
    private val _showUnavailableLenses: RdOptionalProperty<Boolean>,
    private val _providersLoaded: RdProperty<Boolean>
) : RdExtBase() {
    //companion
    
    companion object : ISerializersOwner {
        
        override fun registerSerializersCore(serializers: ISerializers)  {
            serializers.register(CodeLensAnchorKind.marshaller)
            serializers.register(CodeLensRelativeOrderingFirst)
            serializers.register(CodeLensRelativeOrderingLast)
            serializers.register(CodeLensRelativeOrderingBefore)
            serializers.register(CodeLensRelativeOrderingAfter)
            serializers.register(CodeLensProviderModel)
            serializers.register(CodeLensProviderClickedArguments)
            serializers.register(CodeLensExtraActionClickedArguments)
            serializers.register(CodeLensPopupModel)
            serializers.register(CodeLensRelativeOrdering_Unknown)
        }
        
        
        fun create(lifetime: Lifetime, protocol: IProtocol): CodeLensSettingsModel  {
            IdeRoot.register(protocol.serializers)
            
            return CodeLensSettingsModel().apply {
                identify(protocol.identity, RdId.Null.mix("CodeLensSettingsModel"))
                bind(lifetime, protocol, "CodeLensSettingsModel")
            }
        }
        
        
        const val serializationHash = 2586525817940286064L
        
    }
    override val serializersOwner: ISerializersOwner get() = CodeLensSettingsModel
    override val serializationHash: Long get() = CodeLensSettingsModel.serializationHash
    
    //fields
    
    /**
     * Code Lens providers available on this backend. Key is the unique ID of the provider - settings are tied to it
     */
    val providers: IMutableViewableMap<String, CodeLensProviderModel> get() = _providers
    
    /**
     * Show SWEA-dependent lenses if SWEA if disabled
     */
    val showUnavailableLenses: IOptProperty<Boolean> get() = _showUnavailableLenses
    
    /**
     * Shows if all providers are loaded
     */
    val providersLoaded: IProperty<Boolean> get() = _providersLoaded
    //methods
    //initializer
    init {
        _showUnavailableLenses.optimizeNested = true
        _providersLoaded.optimizeNested = true
    }
    
    init {
        bindableChildren.add("providers" to _providers)
        bindableChildren.add("showUnavailableLenses" to _showUnavailableLenses)
        bindableChildren.add("providersLoaded" to _providersLoaded)
    }
    
    //secondary constructor
    private constructor(
    ) : this(
        RdMap<String, CodeLensProviderModel>(FrameworkMarshallers.String, CodeLensProviderModel),
        RdOptionalProperty<Boolean>(FrameworkMarshallers.Bool),
        RdProperty<Boolean>(false, FrameworkMarshallers.Bool)
    )
    
    //equals trait
    //hash code trait
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensSettingsModel (")
        printer.indent {
            print("providers = "); _providers.print(printer); println()
            print("showUnavailableLenses = "); _showUnavailableLenses.print(printer); println()
            print("providersLoaded = "); _providersLoaded.print(printer); println()
        }
        printer.print(")")
    }
    //deepClone
    override fun deepClone(): CodeLensSettingsModel   {
        return CodeLensSettingsModel(
            _providers.deepClonePolymorphic(),
            _showUnavailableLenses.deepClonePolymorphic(),
            _providersLoaded.deepClonePolymorphic()
        )
    }
    //contexts
}


/**
 * Describes the position of entry relative to target range
 * @property Top Above the target range
 * @property Right After end of line with target range
 * @property NearScroll On the same line as target range, near the scrollbar
 * @property EmptySpace In any empty space near the target range
 * @property Default Use the global default value from settings
 * #### Generated from [CodeLensModel.kt]
 */
enum class CodeLensAnchorKind {
    
    /**
     * Above the target range
     */
    Top, 
    
    /**
     * After end of line with target range
     */
    Right, 
    
    /**
     * On the same line as target range, near the scrollbar
     */
    NearScroll, 
    
    /**
     * In any empty space near the target range
     */
    EmptySpace, 
    
    /**
     * Use the global default value from settings
     */
    Default;
    
    companion object {
        val marshaller = FrameworkMarshallers.enum<CodeLensAnchorKind>()
        
    }
}


/**
 * @property documentId The document in which an extra action was clicked
 * @property range The range on which the clicked lens resides
 * @property actionId The action that was clicked
 * #### Generated from [CodeLensModel.kt]
 */
data class CodeLensExtraActionClickedArguments (
    val documentId: RdDocumentId,
    val range: RdTextRange,
    val actionId: String
) : IPrintable {
    //companion
    
    companion object : IMarshaller<CodeLensExtraActionClickedArguments> {
        override val _type: KClass<CodeLensExtraActionClickedArguments> = CodeLensExtraActionClickedArguments::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensExtraActionClickedArguments  {
            val documentId = ctx.serializers.readPolymorphic<RdDocumentId>(ctx, buffer, RdDocumentId)
            val range = RdTextRange.read(ctx, buffer)
            val actionId = buffer.readString()
            return CodeLensExtraActionClickedArguments(documentId, range, actionId)
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensExtraActionClickedArguments)  {
            ctx.serializers.writePolymorphic(ctx, buffer, value.documentId)
            RdTextRange.write(ctx, buffer, value.range)
            buffer.writeString(value.actionId)
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensExtraActionClickedArguments
        
        if (documentId != other.documentId) return false
        if (range != other.range) return false
        if (actionId != other.actionId) return false
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        __r = __r*31 + documentId.hashCode()
        __r = __r*31 + range.hashCode()
        __r = __r*31 + actionId.hashCode()
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensExtraActionClickedArguments (")
        printer.indent {
            print("documentId = "); documentId.print(printer); println()
            print("range = "); range.print(printer); println()
            print("actionId = "); actionId.print(printer); println()
        }
        printer.print(")")
    }
    //deepClone
    //contexts
}


/**
 * @property control The control to show in a popup
 * @property documentId The document in which to show the popup
 * @property range The range of the popup
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensPopupModel (
    val control: com.jetbrains.ide.model.uiautomation.BeControl,
    val documentId: RdDocumentId,
    val range: RdTextRange
) : RdBindableBase() {
    //companion
    
    companion object : IMarshaller<CodeLensPopupModel> {
        override val _type: KClass<CodeLensPopupModel> = CodeLensPopupModel::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensPopupModel  {
            val _id = RdId.read(buffer)
            val control = ctx.serializers.readPolymorphic<com.jetbrains.ide.model.uiautomation.BeControl>(ctx, buffer, com.jetbrains.ide.model.uiautomation.BeControl)
            val documentId = ctx.serializers.readPolymorphic<RdDocumentId>(ctx, buffer, RdDocumentId)
            val range = RdTextRange.read(ctx, buffer)
            return CodeLensPopupModel(control, documentId, range).withId(_id)
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensPopupModel)  {
            value.rdid.write(buffer)
            ctx.serializers.writePolymorphic(ctx, buffer, value.control)
            ctx.serializers.writePolymorphic(ctx, buffer, value.documentId)
            RdTextRange.write(ctx, buffer, value.range)
        }
        
        
    }
    //fields
    //methods
    //initializer
    init {
        bindableChildren.add("control" to control)
    }
    
    //secondary constructor
    //equals trait
    //hash code trait
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensPopupModel (")
        printer.indent {
            print("control = "); control.print(printer); println()
            print("documentId = "); documentId.print(printer); println()
            print("range = "); range.print(printer); println()
        }
        printer.print(")")
    }
    //deepClone
    override fun deepClone(): CodeLensPopupModel   {
        return CodeLensPopupModel(
            control.deepClonePolymorphic(),
            documentId,
            range
        )
    }
    //contexts
}


/**
 * @property documentId The document in which a lens is clicked
 * @property range The range on which the clicked lens resides
 * #### Generated from [CodeLensModel.kt]
 */
data class CodeLensProviderClickedArguments (
    val documentId: RdDocumentId,
    val range: RdTextRange
) : IPrintable {
    //companion
    
    companion object : IMarshaller<CodeLensProviderClickedArguments> {
        override val _type: KClass<CodeLensProviderClickedArguments> = CodeLensProviderClickedArguments::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensProviderClickedArguments  {
            val documentId = ctx.serializers.readPolymorphic<RdDocumentId>(ctx, buffer, RdDocumentId)
            val range = RdTextRange.read(ctx, buffer)
            return CodeLensProviderClickedArguments(documentId, range)
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensProviderClickedArguments)  {
            ctx.serializers.writePolymorphic(ctx, buffer, value.documentId)
            RdTextRange.write(ctx, buffer, value.range)
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensProviderClickedArguments
        
        if (documentId != other.documentId) return false
        if (range != other.range) return false
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        __r = __r*31 + documentId.hashCode()
        __r = __r*31 + range.hashCode()
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensProviderClickedArguments (")
        printer.indent {
            print("documentId = "); documentId.print(printer); println()
            print("range = "); range.print(printer); println()
        }
        printer.print(")")
    }
    //deepClone
    //contexts
}


/**
 * @property displayName User-visible provider name
 * @property orderingConstraints Ordering constraints; these will be used when initially composing lens list of the user
 * @property anchor Default anchor
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensProviderModel private constructor(
    val displayName: String,
    val orderingConstraints: List<CodeLensRelativeOrdering>,
    val anchor: CodeLensAnchorKind,
    private val _isEnabled: RdOptionalProperty<Boolean>,
    private val _clicked: RdSignal<CodeLensProviderClickedArguments>,
    private val _extraActionClicked: RdSignal<CodeLensExtraActionClickedArguments>,
    private val _popup: RdOptionalProperty<CodeLensPopupModel>
) : RdBindableBase() {
    //companion
    
    companion object : IMarshaller<CodeLensProviderModel> {
        override val _type: KClass<CodeLensProviderModel> = CodeLensProviderModel::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensProviderModel  {
            val _id = RdId.read(buffer)
            val displayName = buffer.readString()
            val orderingConstraints = buffer.readList { ctx.serializers.readPolymorphic<CodeLensRelativeOrdering>(ctx, buffer, CodeLensRelativeOrdering) }
            val anchor = buffer.readEnum<CodeLensAnchorKind>()
            val _isEnabled = RdOptionalProperty.read(ctx, buffer, FrameworkMarshallers.Bool)
            val _clicked = RdSignal.read(ctx, buffer, CodeLensProviderClickedArguments)
            val _extraActionClicked = RdSignal.read(ctx, buffer, CodeLensExtraActionClickedArguments)
            val _popup = RdOptionalProperty.read(ctx, buffer, CodeLensPopupModel)
            return CodeLensProviderModel(displayName, orderingConstraints, anchor, _isEnabled, _clicked, _extraActionClicked, _popup).withId(_id)
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensProviderModel)  {
            value.rdid.write(buffer)
            buffer.writeString(value.displayName)
            buffer.writeList(value.orderingConstraints) { v -> ctx.serializers.writePolymorphic(ctx, buffer, v) }
            buffer.writeEnum(value.anchor)
            RdOptionalProperty.write(ctx, buffer, value._isEnabled)
            RdSignal.write(ctx, buffer, value._clicked)
            RdSignal.write(ctx, buffer, value._extraActionClicked)
            RdOptionalProperty.write(ctx, buffer, value._popup)
        }
        
        
    }
    //fields
    
    /**
     * Whether or not this provider is enabled by the user. Will be false if Code Lens is disabled
     */
    val isEnabled: IOptProperty<Boolean> get() = _isEnabled
    
    /**
     * Fired when this entry is clicked in editor or 'More' menu
     */
    val clicked: ISignal<CodeLensProviderClickedArguments> get() = _clicked
    
    /**
     * Fired when an extra action is clicked on a lens
     */
    val extraActionClicked: ISignal<CodeLensExtraActionClickedArguments> get() = _extraActionClicked
    
    /**
     * Shows a popup when set
     */
    val popup: IOptProperty<CodeLensPopupModel> get() = _popup
    //methods
    //initializer
    init {
        _isEnabled.optimizeNested = true
    }
    
    init {
        bindableChildren.add("isEnabled" to _isEnabled)
        bindableChildren.add("clicked" to _clicked)
        bindableChildren.add("extraActionClicked" to _extraActionClicked)
        bindableChildren.add("popup" to _popup)
    }
    
    //secondary constructor
    constructor(
        displayName: String,
        orderingConstraints: List<CodeLensRelativeOrdering>,
        anchor: CodeLensAnchorKind
    ) : this(
        displayName,
        orderingConstraints,
        anchor,
        RdOptionalProperty<Boolean>(FrameworkMarshallers.Bool),
        RdSignal<CodeLensProviderClickedArguments>(CodeLensProviderClickedArguments),
        RdSignal<CodeLensExtraActionClickedArguments>(CodeLensExtraActionClickedArguments),
        RdOptionalProperty<CodeLensPopupModel>(CodeLensPopupModel)
    )
    
    //equals trait
    //hash code trait
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensProviderModel (")
        printer.indent {
            print("displayName = "); displayName.print(printer); println()
            print("orderingConstraints = "); orderingConstraints.print(printer); println()
            print("anchor = "); anchor.print(printer); println()
            print("isEnabled = "); _isEnabled.print(printer); println()
            print("clicked = "); _clicked.print(printer); println()
            print("extraActionClicked = "); _extraActionClicked.print(printer); println()
            print("popup = "); _popup.print(printer); println()
        }
        printer.print(")")
    }
    //deepClone
    override fun deepClone(): CodeLensProviderModel   {
        return CodeLensProviderModel(
            displayName,
            orderingConstraints,
            anchor,
            _isEnabled.deepClonePolymorphic(),
            _clicked.deepClonePolymorphic(),
            _extraActionClicked.deepClonePolymorphic(),
            _popup.deepClonePolymorphic()
        )
    }
    //contexts
}


/**
 * Base class for relative ordering types
 * #### Generated from [CodeLensModel.kt]
 */
abstract class CodeLensRelativeOrdering (
) : IPrintable {
    //companion
    
    companion object : IAbstractDeclaration<CodeLensRelativeOrdering> {
        override fun readUnknownInstance(ctx: SerializationCtx, buffer: AbstractBuffer, unknownId: RdId, size: Int): CodeLensRelativeOrdering  {
            val objectStartPosition = buffer.position
            val unknownBytes = ByteArray(objectStartPosition + size - buffer.position)
            buffer.readByteArrayRaw(unknownBytes)
            return CodeLensRelativeOrdering_Unknown(unknownId, unknownBytes)
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    //hash code trait
    //pretty print
    //deepClone
    //contexts
}


/**
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensRelativeOrderingAfter (
    val id: String
) : CodeLensRelativeOrdering (
) {
    //companion
    
    companion object : IMarshaller<CodeLensRelativeOrderingAfter> {
        override val _type: KClass<CodeLensRelativeOrderingAfter> = CodeLensRelativeOrderingAfter::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensRelativeOrderingAfter  {
            val id = buffer.readString()
            return CodeLensRelativeOrderingAfter(id)
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensRelativeOrderingAfter)  {
            buffer.writeString(value.id)
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensRelativeOrderingAfter
        
        if (id != other.id) return false
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        __r = __r*31 + id.hashCode()
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensRelativeOrderingAfter (")
        printer.indent {
            print("id = "); id.print(printer); println()
        }
        printer.print(")")
    }
    
    override fun toString() = PrettyPrinter().singleLine().also { print(it) }.toString()
    //deepClone
    //contexts
}


/**
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensRelativeOrderingBefore (
    val id: String
) : CodeLensRelativeOrdering (
) {
    //companion
    
    companion object : IMarshaller<CodeLensRelativeOrderingBefore> {
        override val _type: KClass<CodeLensRelativeOrderingBefore> = CodeLensRelativeOrderingBefore::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensRelativeOrderingBefore  {
            val id = buffer.readString()
            return CodeLensRelativeOrderingBefore(id)
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensRelativeOrderingBefore)  {
            buffer.writeString(value.id)
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensRelativeOrderingBefore
        
        if (id != other.id) return false
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        __r = __r*31 + id.hashCode()
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensRelativeOrderingBefore (")
        printer.indent {
            print("id = "); id.print(printer); println()
        }
        printer.print(")")
    }
    
    override fun toString() = PrettyPrinter().singleLine().also { print(it) }.toString()
    //deepClone
    //contexts
}


/**
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensRelativeOrderingFirst (
) : CodeLensRelativeOrdering (
) {
    //companion
    
    companion object : IMarshaller<CodeLensRelativeOrderingFirst> {
        override val _type: KClass<CodeLensRelativeOrderingFirst> = CodeLensRelativeOrderingFirst::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensRelativeOrderingFirst  {
            return CodeLensRelativeOrderingFirst()
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensRelativeOrderingFirst)  {
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensRelativeOrderingFirst
        
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensRelativeOrderingFirst (")
        printer.print(")")
    }
    
    override fun toString() = PrettyPrinter().singleLine().also { print(it) }.toString()
    //deepClone
    //contexts
}


/**
 * #### Generated from [CodeLensModel.kt]
 */
class CodeLensRelativeOrderingLast (
) : CodeLensRelativeOrdering (
) {
    //companion
    
    companion object : IMarshaller<CodeLensRelativeOrderingLast> {
        override val _type: KClass<CodeLensRelativeOrderingLast> = CodeLensRelativeOrderingLast::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensRelativeOrderingLast  {
            return CodeLensRelativeOrderingLast()
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensRelativeOrderingLast)  {
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensRelativeOrderingLast
        
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensRelativeOrderingLast (")
        printer.print(")")
    }
    
    override fun toString() = PrettyPrinter().singleLine().also { print(it) }.toString()
    //deepClone
    //contexts
}


class CodeLensRelativeOrdering_Unknown (
    override val unknownId: RdId,
    val unknownBytes: ByteArray
) : CodeLensRelativeOrdering (
), IUnknownInstance {
    //companion
    
    companion object : IMarshaller<CodeLensRelativeOrdering_Unknown> {
        override val _type: KClass<CodeLensRelativeOrdering_Unknown> = CodeLensRelativeOrdering_Unknown::class
        
        @Suppress("UNCHECKED_CAST")
        override fun read(ctx: SerializationCtx, buffer: AbstractBuffer): CodeLensRelativeOrdering_Unknown  {
            throw NotImplementedError("Unknown instances should not be read via serializer")
        }
        
        override fun write(ctx: SerializationCtx, buffer: AbstractBuffer, value: CodeLensRelativeOrdering_Unknown)  {
            buffer.writeByteArrayRaw(value.unknownBytes)
        }
        
        
    }
    //fields
    //methods
    //initializer
    //secondary constructor
    //equals trait
    override fun equals(other: Any?): Boolean  {
        if (this === other) return true
        if (other == null || other::class != this::class) return false
        
        other as CodeLensRelativeOrdering_Unknown
        
        
        return true
    }
    //hash code trait
    override fun hashCode(): Int  {
        var __r = 0
        return __r
    }
    //pretty print
    override fun print(printer: PrettyPrinter)  {
        printer.println("CodeLensRelativeOrdering_Unknown (")
        printer.print(")")
    }
    
    override fun toString() = PrettyPrinter().singleLine().also { print(it) }.toString()
    //deepClone
    //contexts
}
