package com.jetbrains.rdserver.completion

import com.intellij.openapi.project.Project
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.ISignal
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.daemon.suppressMarkupSynchronization
import com.jetbrains.rdserver.editors.BackendEditorHost
import com.jetbrains.rdserver.requests.BackendAsyncRequestDelegate
import com.jetbrains.rdserver.requests.BackendIdeGateway
import com.jetbrains.rd.ide.model.RdCompleteItemRequest
import com.jetbrains.rd.ide.model.RdFailureResponse
import com.jetbrains.rd.ide.model.RdPatchResponse
import com.jetbrains.rd.ide.model.RdResponse

class BackendCompleteItemExecutorDelegate : BackendAsyncRequestDelegate<RdCompleteItemRequest> {
  override suspend fun execute(request: RdCompleteItemRequest, project: Project, consumer: (RdResponse) -> Unit) {
    val requestId = request.id
    val completionHost = RdServerPerClientComponents.getService(project, BackendCompletionHost::class.java)
    val editorHost = BackendEditorHost.getInstance(project)
    val item = request.item
    val char = request.char
    val editorId = request.editorId
    val editor = editorHost.tryGetOwnRemoteEditor(editorId) ?: return consumer(RdFailureResponse("Not found editor with $editorId id", requestId))

    val autoPopupController = RdServerAutoPopupController.getInstance(project)
    Lifetime.using { lt ->
      editor.suppressMarkupSynchronization(lt, "Complete item#$requestId")
      autoPopupController.suppressor.suppress(lt)
      val ideGateway = RdServerPerClientComponents.getService<BackendIdeGateway>(project)
      val patch = ideGateway.executeBulkUpdate(editor) {
        if (!completionHost.completeItem(item, char, editor)) {
          error("Failed to complete")
        }
      }
      consumer(RdPatchResponse(patch, requestId))
    }
  }
}