package com.jetbrains.rdserver.completion

import com.intellij.codeInsight.completion.impl.CamelHumpMatcher
import com.intellij.codeInsight.lookup.*
import com.intellij.codeInsight.template.impl.LiveTemplateLookupElement
import com.intellij.openapi.progress.ProgressManager
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rdserver.completionItemText
import com.jetbrains.rdserver.toProtocolModel
import java.util.concurrent.atomic.AtomicInteger

private val idCounter = AtomicInteger()

fun List<LookupElement?>.attachProtocolItems(arranger: LookupArranger? = null): List<Pair<CompletionItem, LookupElement>> {
  return mapIndexed { index, item ->
    ProgressManager.checkCanceled()
    if (item == null) return@mapIndexed null
    val presentation = LookupElementPresentation()
    val renderer = item.expensiveRenderer
    var speculativeInsert: String? = null
    @Suppress("UNCHECKED_CAST") // a peculiarity of IJ completion API
    if (renderer as? LookupElementRenderer<LookupElement> != null) {
      speculativeInsert = item.lookupString
      renderer.renderElement(item, presentation)
    }
    else {
      item.renderElement(presentation)
    }
    val prefixMatcher = arranger?.itemMatcher(item)
    val prefix = (prefixMatcher as? CamelHumpMatcher)?.toString()
    val isInteractive = item.getUserData(LookupElementInteractivity.KEY)?.isInsertHandlerInteractive(item) ?: false
    val type = if (item is LiveTemplateLookupElement) CompletionItemType.TemplateItem else CompletionItemType.Default
    val resultItem = CompletionItem(presentation.itemText ?: item.lookupString,
                                    presentation.toCompletionPresentation(), presentation.typeText, presentation.icon?.toProtocolModel(),
                                    idCounter.incrementAndGet(),
                                    (size - index).toLong(), type, listOf(), prefix, 0, !isInteractive,
                                    item.isWorthShowingInAutoPopup, speculativeInsert
    )
    resultItem to item
  }.filterNotNull()
}

private fun LookupElementPresentation.toCompletionPresentation(): CompletionPresentationModel {
  return completionItemText {
    itemText?.let {
      append(CompletionPresentationPart(
        it, isTypeGrayed,
        when {
          isStrikeout -> EffectStyle.StrikeOut
          isItemTextUnderlined -> EffectStyle.StraightUnderline
          else -> EffectStyle.None
        },
        if (isItemTextBold) FontStyle.Bold else if (isItemTextItalic) FontStyle.Italic else FontStyle.Regular
      ))
    }
    for (tailFragment in tailFragments) {
      append(
        CompletionPresentationPart(
          tailFragment.text,
          isTypeGrayed,
          if (isStrikeout) EffectStyle.StrikeOut else EffectStyle.None,
          if (tailFragment.isItalic) FontStyle.Italic else FontStyle.Regular
        )
      )
    }
  }
}
