package com.jetbrains.rdserver.completion

import com.intellij.codeInsight.editorActions.TypedHandlerDelegate
import com.intellij.codeInsight.lookup.CharFilter
import com.intellij.codeInsight.lookup.LookupManager
import com.intellij.codeInsight.lookup.impl.LookupImpl
import com.intellij.codeInsight.lookup.impl.LookupTypedHandler
import com.intellij.codeInsight.lookup.impl.actions.ChooseItemAction
import com.intellij.codeInsight.template.impl.TemplateSettings
import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.project.Project
import com.intellij.psi.PsiFile

class BackendLookupTypedHandler : TypedHandlerDelegate() {
  override fun charTyped(c: Char, project: Project, editor: Editor, file: PsiFile): Result {
    if (!ClientId.isCurrentlyUnderLocalId) {
      executeForClient(editor, c)
    }
    return Result.CONTINUE
  }

  private fun executeForClient(editor: Editor, c: Char) {
    val lookup = LookupManager.getActiveLookup(editor) as LookupImpl?
    if (lookup == null || lookup.isLookupDisposed) {
      return
    }
    if (c == ' ' && ChooseItemAction.hasTemplatePrefix(lookup, TemplateSettings.SPACE_CHAR)) {
      return
    }
    val result = LookupTypedHandler.getLookupAction(c, lookup)
    if (result == CharFilter.Result.ADD_TO_PREFIX) {
      lookup.fireBeforeAppendPrefix(c)
      lookup.appendPrefix(c)
    }
  }
}