package com.jetbrains.rdserver.conflicts

import com.intellij.codeInsight.AutoPopupController
import com.intellij.ide.plugins.cl.PluginClassLoader
import com.intellij.openapi.components.service
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.project.Project
import com.intellij.openapi.startup.StartupActivity
import com.intellij.ui.AppUIUtil
import com.jetbrains.rdserver.completion.RdServerAutoPopupController

class BackendConflictsResolverActivity : StartupActivity.DumbAware {

  companion object {
    private val LOG = logger<BackendConflictsResolverActivity>()
  }

  override fun runActivity(project: Project) {
    project.service<AutoPopupController>().tryCastOrShowConflictDialog<RdServerAutoPopupController>()
  }

  private inline fun <reified T : Any> Any.tryCastOrShowConflictDialog() {
    if (this is T) return

    val classLoader: ClassLoader = javaClass.classLoader
    val exception = Exception("Can not cast $javaClass to ${T::class.java}")
    if (classLoader is PluginClassLoader) {
      LOG.warn(exception)
      AppUIUtil.invokeOnEdt {
        BackendPluginConflictDialog(classLoader.pluginId).show()
      }
    }
    else {
      LOG.error(exception)  // => Code With Me is incompatible with IDE
    }
  }

}