package com.jetbrains.rdserver.connection

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.jetbrains.codeWithMe.model.*
import com.jetbrains.rd.ide.model.Solution
import com.jetbrains.rd.platform.codeWithMe.connection.CwmConnectionStatusService
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.flowInto
import com.jetbrains.rd.util.reactive.valueOrThrow

@Suppress("NonDefaultConstructor")
class BackendConnectionStatusSynchronizer(
  clientId: ClientId,
  lifetime: Lifetime,
  solution: Solution,
  project: Project
) {
  init {
    val statusService = CwmConnectionStatusService.getInstance(project)
    val connection = solution.codeWithMeSolutionExtModel.connection.apply { set(ConnectionModel()) }.valueOrThrow

    statusService.sessionStatus.flowInto(lifetime, connection.sessionStatus)

    statusService.freeSessionRemainingTimeMs.flowInto(lifetime, connection.freeSessionRemainingTimeMs)

    statusService.terminated.advise(lifetime) {
      // to make it possible to terminate only particular guests
      if (ClientId.current == clientId) {
        connection.terminated.fire(Terminated(it.reason, it.message))
      }
    }
  }
}