package com.jetbrains.rdserver.connection

import com.intellij.openapi.components.service
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rdserver.followMe.FollowMeManager
import com.jetbrains.rdserver.joinLinks.RdServerLobbySessionManager

class BackendGuestLimitNotificator(val project: Project) {
  companion object {
    const val MAX_CONCURRENT_GUESTS_FOR_NO_LICENSE = 3
  }

  val guestCount: Property<Int> = Property(0)
  private val lifetime = project.lifetime.createNested().lifetime

  init {
    val users = project.service<FollowMeManager>().users
    users.change.advise(lifetime) {
      guestCount.value = users.size - 1
    }
  }

  val isAtConcurrentGuestLimit: Boolean
    get() {
      if (RdServerLobbySessionManager.isLicensed()) return false

      return guestCount.value >= MAX_CONCURRENT_GUESTS_FOR_NO_LICENSE
    }
}