package com.jetbrains.rdserver.daemon

import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.ex.RangeMarkerEx
import com.intellij.openapi.extensions.ProjectExtensionPointName
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.RdDocumentVersion

/**
 * You can inject project in the constructor
 */
interface BackendHighlighterModelCreator : BackendRangeMarkerModelCreator {
  companion object {
    val EP_NAME = ProjectExtensionPointName<BackendHighlighterModelCreator>("com.intellij.backend.highlighterModelCreator")
  }

  fun accept(highlighter: RangeHighlighterEx): Boolean
  fun createModel(highlighter: RangeHighlighterEx, documentVersion: RdDocumentVersion): HighlighterModel?

  override fun accept(rangeMarker: RangeMarkerEx): Boolean {
    return rangeMarker is RangeHighlighterEx && accept(rangeMarker)
  }

  override fun createModel(rangeMarker: RangeMarkerEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    return createModel(rangeMarker as RangeHighlighterEx, documentVersion)
  }
}

interface BackendRangeMarkerModelCreator {
  companion object {
    val EP_NAME = ProjectExtensionPointName<BackendRangeMarkerModelCreator>("com.intellij.backend.highlighterModelCreator")
  }

  fun accept(rangeMarker: RangeMarkerEx): Boolean
  fun createModel(rangeMarker: RangeMarkerEx, documentVersion: RdDocumentVersion): HighlighterModel?

  //fun getProperties()
}