package com.jetbrains.rdserver.daemon

import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.intersect
import com.jetbrains.rd.util.lifetime.isNotAlive
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rd.util.reactive.whenTrue

class BackendMarkupSuppressor(private val lifetime: Lifetime) {
  private var suppressRequests = 0
  private var isSuppressed = Property(false)

  fun shouldSuppress() = isSuppressed.value

  fun onRelease(action: () -> Unit) {
    isSuppressed.change.advise(lifetime) { value -> if (!value) action() }
  }

  fun onSuppress(lt: Lifetime, action: (Lifetime) -> Unit) {
    isSuppressed.whenTrue(lt) { ltLt -> action(ltLt) }
  }

  fun suppress(suppressLifetime: Lifetime, reason: String) {
    if (suppressLifetime.isNotAlive) return

    logger.trace("Started suppression: $reason")
    suppressRequests++
    if (suppressRequests > 0) {
      isSuppressed.set(true)
    }
    suppressLifetime.intersect(lifetime).onTermination {
      suppressRequests--
      if (suppressRequests == 0) {
        isSuppressed.set(false)
      }
    }
  }

  companion object {
    private val logger = getLogger<BackendMarkupSuppressor>()
  }

}