package com.jetbrains.rdserver.daemon

import com.intellij.openapi.project.Project
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.RdServerPerClientComponents

@Suppress("NonDefaultConstructor")
class BackendMarkupSuppressors(lifetime: Lifetime) {
  companion object {

    suspend inline fun fullSuppressMarkupSynchronization(project: Project, reason: String, crossinline block: suspend () -> Unit) {
      val service = RdServerPerClientComponents.getService<BackendMarkupSuppressors>(project)
      Lifetime.using { lifetime ->
        service.getGlobalSuppressor().suppress(lifetime, reason)
        block()
      }
    }
  }

  private val globalSuppressor = BackendMarkupSuppressor(lifetime)

  fun getGlobalSuppressor(): BackendMarkupSuppressor {
    return globalSuppressor
  }

  fun createSuppressor(lifetime: Lifetime, parentId: String? = null, parent: BackendMarkupSuppressor? = null): BackendMarkupSuppressor {
    val childSuppressor = BackendMarkupSuppressor(lifetime)
    val reason = parentId ?: "global markup suppressor"
    val parent = parent ?: globalSuppressor
    parent.onSuppress(lifetime) { childSuppressor.suppress(it, reason) }
    return childSuppressor
  }
}