package com.jetbrains.rdserver.daemon

import com.intellij.openapi.editor.ex.RangeMarkerEx
import it.unimi.dsi.fastutil.longs.Long2ObjectOpenHashMap

class BackendRangeMarkerStorage {
  private val rangeMarkerMap = Long2ObjectOpenHashMap<RangeMarkerEx>()
  private val queuedForRegistration = Long2ObjectOpenHashMap<RangeMarkerEx>()
  private val queuedForUpdate = mutableSetOf<RangeMarkerEx>()
  private val queuedForDeletion = mutableSetOf<RangeMarkerEx>()

  fun add(rangeMarker: RangeMarkerEx) {
    rangeMarkerMap.put(rangeMarker.id, rangeMarker)
  }

  fun get(id: Long): RangeMarkerEx? {
    return rangeMarkerMap.get(id)
  }

  fun remove(rangeMarker: RangeMarkerEx): Boolean {
    if (queuedForRegistration.remove(rangeMarker.id) != null) return false
    return rangeMarkerMap.remove(rangeMarker.id) != null
  }

  fun flushPendingToAdd(): List<RangeMarkerEx> {
    val flush = queuedForRegistration.values.map { it as RangeMarkerEx }
    queuedForRegistration.clear()
    return flush
  }

  fun flushPendingToRemove(): List<RangeMarkerEx> {
    val flush = queuedForDeletion.toList()
    queuedForDeletion.clear()
    return flush
  }

  fun flushPendingToUpdate(): List<RangeMarkerEx> {
    val flush = queuedForUpdate.toList()
    queuedForUpdate.clear()
    return flush
  }

  fun contains(rangeMarker: RangeMarkerEx): Boolean {
    return rangeMarkerMap.contains(rangeMarker.id)
  }

  fun queueRegistration(rangeMarker: RangeMarkerEx) {
    queuedForRegistration.put(rangeMarker.id, rangeMarker)
  }

  fun queueDeletion(rangeMarker: RangeMarkerEx) {
    if (queuedForRegistration.remove(rangeMarker.id) != null) return
    queuedForDeletion.add(rangeMarker)
  }

  fun queueUpdate(rangeMarker: RangeMarkerEx) {
    if (!queuedForRegistration.contains(rangeMarker.id)) {
      queuedForUpdate.add(rangeMarker)
    }
  }

  fun getAllRegistered(): List<RangeMarkerEx> {
    return rangeMarkerMap.values.map { it as RangeMarkerEx }
  }
}