package com.jetbrains.rdserver.daemon.folding

import com.intellij.openapi.editor.FoldRegion
import com.intellij.openapi.editor.RangeMarker
import com.intellij.openapi.editor.ex.FoldingListener
import com.intellij.openapi.editor.ex.FoldingModelEx
import com.intellij.openapi.rd.createNestedDisposable
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.daemon.BackendEditorMarkupModelContributor
import com.jetbrains.rdserver.daemon.BackendMarkupModelContributor
import com.jetbrains.rdserver.daemon.BackendProtocolMarkupModelExtensionHandler
import com.jetbrains.rd.ide.model.MarkupModelExtension

class BackendFoldingModelBinder : BackendProtocolMarkupModelExtensionHandler {
  override fun consume(lifetime: Lifetime, extensions: List<MarkupModelExtension>, contributor: BackendMarkupModelContributor) {
    if (contributor !is BackendEditorMarkupModelContributor) return
    val editor = contributor.editor
    contributor.bind(lifetime, editor.foldingModel as FoldingModelEx)
  }

  fun BackendEditorMarkupModelContributor.bind(lifetime: Lifetime, foldingModelEx: FoldingModelEx) {
    val listener = object : FoldingListener {
      private val toAddOrUpdate = mutableSetOf<RangeMarker>()
      private val toRemove = mutableSetOf<RangeMarker>()

      override fun onFoldRegionStateChange(region: FoldRegion) {
        toAddOrUpdate.add(region)
      }

      override fun beforeFoldRegionRemoved(region: FoldRegion) {
        toAddOrUpdate.remove(region)
        toRemove.add(region)
      }

      override fun onFoldProcessingEnd() {
        try {
          removeRangeMarkers(toRemove)
          addOrUpdateRangeMarkers(toAddOrUpdate)
        }
        finally {
          toRemove.clear()
          toAddOrUpdate.clear()
        }
      }
    }

    // it's crucial to update all folding at once
    // since partial overlapping isn't supported by the platform
    // but possible due to frontend<->backend asynchrony
    addOrUpdateRangeMarkers(foldingModelEx.allFoldRegions.asIterable())

    foldingModelEx.addListener(listener, lifetime.createNestedDisposable())
  }
}