package com.jetbrains.rdserver.daemon.highlighters.defaults

import com.intellij.codeInsight.daemon.impl.HighlightInfo
import com.intellij.codeInsight.daemon.impl.HighlightInfoType
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.markup.HighlighterTargetArea
import com.intellij.openapi.editor.markup.TextAttributes
import com.jetbrains.ide.model.highlighterRegistration.IdeaHighlightInfoType
import com.jetbrains.ide.model.highlighterRegistration.IdeaTextAttributesKey
import com.jetbrains.rd.ide.model.DefaultHighlighterModel
import com.jetbrains.rd.ide.model.HighlightInfoModel
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rdserver.daemon.BackendHighlighterModelCreator
import com.jetbrains.rdserver.daemon.BackendHighlighterRegistrationsHost

class DefaultProtocolHighlighterModelCreator : BackendHighlighterModelCreator {
  companion object {
    private val LOG = logger<DefaultProtocolHighlighterModelCreator>()
  }

  override fun accept(highlighter: RangeHighlighterEx): Boolean {
    return highlighter.textAttributesKey != null
  }

  override fun createModel(highlighter: RangeHighlighterEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    var key = highlighter.textAttributesKey
    while (key != null && BackendHighlighterRegistrationsHost.getAttributeId(key) == null) {
      LOG.warn("TextAttributesKey($key, ${key.fallbackAttributeKey}) has not been registered, falling back")
      key = key.fallbackAttributeKey
    }

    if (key == null) {
      LOG.error("Ignoring highlighter which has not been registered: $highlighter.textAttributesKey")
      return null
    }

    val attributeId = BackendHighlighterRegistrationsHost.getAttributeId(key)!!

    val info = HighlightInfo.fromRangeHighlighter(highlighter)

    return highlighter.run {
      DefaultHighlighterModel(
        info?.toModel(),
        isAfterEndOfLine,
        info?.needUpdateOnTyping(),
        forcedTextAttributes === TextAttributes.ERASE_MARKER,
        layer,
        targetArea == HighlighterTargetArea.EXACT_RANGE,
        documentVersion,
        isGreedyToLeft,
        isGreedyToRight,
        isThinErrorStripeMark,
        null,
        id,
        attributeId,
        startOffset,
        endOffset
      )
    }
  }
}

private fun HighlightInfo.toModel(): HighlightInfoModel {
  return HighlightInfoModel(type.toModel(), toolTip, description, false)
}

private fun HighlightInfoType.toModel(): IdeaHighlightInfoType {
  val needsUpdateOnTyping = this is HighlightInfoType.UpdateOnTypingSuppressible && this.needsUpdateOnTyping()
  val key = IdeaTextAttributesKey(attributesKey.externalName, attributesKey.fallbackAttributeKey?.externalName)
  return IdeaHighlightInfoType(getSeverity(null).myName, key, needsUpdateOnTyping)
}