package com.jetbrains.rdserver.daemon.highlighters.diagnostics

import com.intellij.openapi.util.Key
import com.jetbrains.rd.framework.impl.RdTask
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.daemon.BackendMarkupModelContributor
import com.jetbrains.rdserver.daemon.BackendProtocolMarkupModelExtensionHandler
import com.jetbrains.rd.ide.model.*

class MarkupDiagnosticsExtensionHandler : BackendProtocolMarkupModelExtensionHandler {
  companion object {
    private val KEY = Key<MarkupDiagnostics>("DIAGNOSTICS_KEY")

    fun isDebugMode(contributor: BackendMarkupModelContributor): Boolean {
      return contributor.getExtension(KEY)?.isDebugMode?.valueOrNull ?: false
    }
  }

  override fun consume(lifetime: Lifetime, extensions: List<MarkupModelExtension>, contributor: BackendMarkupModelContributor) {
    val diagnostics = contributor.getExtension(KEY) ?: return

    diagnostics.dumpAllHighlighters.set { _, _ ->
      val allRegistered = contributor.storage.getAllRegistered()
      val currentVersion = contributor.currentDocumentVersion()
      val elements = allRegistered.mapNotNull { contributor.createModel(it, currentVersion) }
      RdTask.fromResult(HighlighterList(elements))
    }

    diagnostics.dumpMarkupInfo.set { _, _ ->
      val allRegistered = contributor.storage.getAllRegistered().map { it.id }
      val dumpInfo = MarkupDumpInfo(HighlighterIdList(allRegistered), allRegistered.size, contributor.currentDocumentVersion())
      RdTask.fromResult(dumpInfo)
    }
  }
}