package com.jetbrains.rdserver.daemon.highlighters.gutterMarks

import com.intellij.openapi.actionSystem.ActionPlaces
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.Presentation
import com.intellij.openapi.editor.EditorFactory
import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.util.ui.UIUtil
import com.jetbrains.ide.model.popups.RdGutterMarkPopupWindowContext
import com.jetbrains.rd.ide.editor.markup.gutterMarkAction
import com.jetbrains.rd.ide.editor.markup.gutterMarkTooltipProviderModel
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.actions.dataContextForEditor
import com.jetbrains.rdserver.daemon.BackendMarkupModelContributor
import com.jetbrains.rdserver.daemon.BackendProtocolMarkupModelExtensionHandler
import com.jetbrains.rdserver.ui.popup.bePopups.impl.popupContextKey
import com.jetbrains.rd.ide.model.MarkupModelExtension
import com.jetbrains.rd.ide.model.StringTooltipModel
import java.awt.event.MouseEvent

internal class BackendGutterMarkMarkupModelExtensionHandler : BackendProtocolMarkupModelExtensionHandler {
  override fun consume(lifetime: Lifetime, extensions: List<MarkupModelExtension>, contributor: BackendMarkupModelContributor) {
    val gutterMarkActionModel = contributor.protocolMarkupModel.gutterMarkAction
    gutterMarkActionModel.leftClick.advise(lifetime) { req ->
      val id = req.gutterId
      val highlighter = contributor.getRangeMarker(id) as? RangeHighlighterEx ?: return@advise
      val gutterIconRenderer = highlighter.gutterIconRenderer ?: return@advise
      val clickAction = gutterIconRenderer.clickAction ?: return@advise
      val editor = EditorFactory.getInstance().editors(contributor.document).findFirst().orElse(null) ?: return@advise
      UIUtil.putClientProperty(editor.component, popupContextKey, RdGutterMarkPopupWindowContext(highlighter.startOffset))
      val mouseEvent = MouseEvent(
        editor.component, MouseEvent.MOUSE_CLICKED, System.currentTimeMillis(),
        MouseEvent.BUTTON1_MASK, 0, 0, 1, false, MouseEvent.BUTTON1
      )
      val actionEvent = AnActionEvent.createFromInputEvent(
        mouseEvent, ActionPlaces.UNKNOWN, Presentation(),
        dataContextForEditor(contributor.project, editor)
      )
      clickAction.actionPerformed(actionEvent)
    }

    val tooltipProviderModel = contributor.protocolMarkupModel.gutterMarkTooltipProviderModel
    tooltipProviderModel.getTooltip.set { id ->
      val highlighter = contributor.getRangeMarker(id) as? RangeHighlighterEx ?: return@set StringTooltipModel(null)
      val renderer = highlighter.gutterIconRenderer ?: return@set StringTooltipModel(null)
      // todo pass attribute keys instead of hard-coded colors in HTML, action id instead of hard-coded host's hotkey, action instead of href
      return@set StringTooltipModel(renderer.tooltipText)
    }
  }
}