package com.jetbrains.rdserver.daemon.highlighters.gutterMarks

import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.markup.HighlighterTargetArea
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.popups.RdPopupMenuItemKind
import com.jetbrains.rd.ide.model.GutterMarkHighlighterModel
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rdserver.daemon.BackendHighlighterModelCreator
import com.jetbrains.rdserver.daemon.BackendHighlighterRegistrationsHost
import com.jetbrains.rdserver.toProtocolModel

class GutterMarkProtocolHighlighterModelCreator(val project: Project) : BackendHighlighterModelCreator {
  override fun accept(highlighter: RangeHighlighterEx): Boolean {
    return highlighter.gutterIconRenderer != null
  }

  override fun createModel(highlighter: RangeHighlighterEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    val gutterIconRenderer = highlighter.gutterIconRenderer!!
    val icon = gutterIconRenderer.icon.toProtocolModel() ?: return null
    val popupActions = if (gutterIconRenderer.popupMenuActions != null) listOf("TODO: REPLACE WITH BOOLEAN") else emptyList()
    return GutterMarkHighlighterModel(
      icon,
      listOf(),
      RdPopupMenuItemKind.Gutter,
      gutterIconRenderer.tooltipText ?: "",
      gutterIconRenderer.isNavigateAction,
      gutterIconRenderer.clickAction != null,
      gutterIconRenderer.middleButtonClickAction != null,
      gutterIconRenderer.rightButtonClickAction != null,
      popupActions, // todo remove after 2021.1 release
      highlighter.layer,
      highlighter.targetArea == HighlighterTargetArea.EXACT_RANGE,
      documentVersion,
      highlighter.isGreedyToLeft,
      highlighter.isGreedyToRight,
      highlighter.isThinErrorStripeMark,
      null,
      highlighter.id,
      BackendHighlighterRegistrationsHost.GUTTER_MARK_ID,
      highlighter.startOffset,
      highlighter.endOffset
    )
  }
}