package com.jetbrains.rdserver.daemon.highlighters.lineMarkers

import com.intellij.openapi.editor.EditorFactory
import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.markup.ActiveGutterRenderer
import com.intellij.openapi.project.Project
import com.intellij.openapi.util.Key
import com.intellij.util.ui.UIUtil
import com.jetbrains.ide.model.popups.RdGutterMarkPopupWindowContext
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.daemon.BackendMarkupModelContributor
import com.jetbrains.rdserver.daemon.BackendProtocolMarkupModelExtensionHandler
import com.jetbrains.rdserver.ui.popup.bePopups.impl.popupContextKey
import com.jetbrains.rd.ide.model.LineMarkerActionModel
import com.jetbrains.rd.ide.model.MarkupModelExtension
import com.jetbrains.rd.ide.model.StringTooltipModel
import com.jetbrains.rd.ide.model.TooltipProviderModel
import java.awt.event.MouseEvent

class BackendLineMarkerMarkupModelExtensionHandler(private val project: Project) :
  BackendProtocolMarkupModelExtensionHandler {

  companion object {
    val LINE_MARKER_TOOLTIPS_KEY = Key<TooltipProviderModel>("LINE_MARKER_TOOLTIPS_KEY")
    val LINE_MARKER_ACTIONS_KEY = Key<LineMarkerActionModel>("LINE_MARKER_ACTIONS_KEY")
  }

  override fun consume(lifetime: Lifetime, extensions: List<MarkupModelExtension>, contributor: BackendMarkupModelContributor) {
    val lineMarkerActionModel = extensions.lineMarkerActionsModel

    lineMarkerActionModel.click.advise(lifetime) { event ->
      val highlighter = contributor.getRangeMarker(event.id) as? RangeHighlighterEx ?: return@advise
      val lineMarkerRenderer = highlighter.lineMarkerRenderer as? ActiveGutterRenderer ?: return@advise
      val editor = EditorFactory.getInstance().editors(contributor.document, project).findFirst().orElse(null)?: return@advise
      UIUtil.putClientProperty(editor.component, popupContextKey, RdGutterMarkPopupWindowContext(highlighter.startOffset))
      val mouseEvent = MouseEvent(
        editor.component, MouseEvent.MOUSE_CLICKED, System.currentTimeMillis(),
        MouseEvent.BUTTON1_MASK, 0, 0, 1, false, MouseEvent.BUTTON1
      )

      if (lineMarkerRenderer.canDoAction(editor, mouseEvent)) {
        lineMarkerRenderer.doAction(editor, mouseEvent)
      }
    }

    val tooltipProviderModel = extensions.lineMarkerTooltipProviderModel

    tooltipProviderModel.getTooltip.set { id ->
      val highlighter = contributor.getRangeMarker(id) as? RangeHighlighterEx ?: return@set StringTooltipModel(null)
      val lineMarkerRenderer = highlighter.lineMarkerRenderer as? ActiveGutterRenderer ?: return@set StringTooltipModel(null)
      return@set StringTooltipModel(lineMarkerRenderer.tooltipText)
    }
  }

  @Suppress("UNCHECKED_CAST")
  private fun <T> List<MarkupModelExtension>.getExtension(key: Key<T>) = firstOrNull { it.key == key.toString() } as T

  private val List<MarkupModelExtension>.lineMarkerTooltipProviderModel get() = getExtension(LINE_MARKER_TOOLTIPS_KEY)
  private val List<MarkupModelExtension>.lineMarkerActionsModel get() = getExtension(LINE_MARKER_ACTIONS_KEY)
}