package com.jetbrains.rdserver.daemon.highlighters.lineMarkers

import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.markup.*
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.LineMarkerHighlighterModel
import com.jetbrains.rd.ide.model.LineMarkerPosition
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rdserver.daemon.BackendHighlighterModelCreator
import com.jetbrains.rdserver.daemon.BackendHighlighterRegistrationsHost

class BackendLineMarkerModelCreator : BackendHighlighterModelCreator {
  override fun accept(highlighter: RangeHighlighterEx): Boolean {
    return highlighter.lineMarkerRenderer != null
  }

  private val DEFAULT_RENDERER_ID = "DefaultLineMarkerRendererProvider"

  override fun createModel(highlighter: RangeHighlighterEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    val renderer = highlighter.lineMarkerRenderer!!

    return if (renderer is DefaultLineMarkerRenderer) {
      createDefaultModel(renderer, highlighter, documentVersion)
    }
    else {
      null // todo support more line renderers
      //val rendererId = renderer::class.java.canonicalName
      //createCustomRendererModel(rendererId, renderer, highlighter, documentVersion)
    }
  }

  private fun createDefaultModel(
    renderer: DefaultLineMarkerRenderer,
    highlighter: RangeHighlighterEx,
    documentVersion: RdDocumentVersion
  ): LineMarkerHighlighterModel {
    return LineMarkerHighlighterModel(
      null,
      BackendHighlighterRegistrationsHost.getAttributeId(renderer.attributesKey),
      renderer.thickness,
      renderer.depth,
      renderer.position.toModel(),
      renderer is ActiveGutterRenderer,
      highlighter.layer,
      highlighter.targetArea == HighlighterTargetArea.EXACT_RANGE,
      documentVersion,
      highlighter.isGreedyToLeft,
      highlighter.isGreedyToRight,
      highlighter.isThinErrorStripeMark,
      null,
      highlighter.id,
      BackendHighlighterRegistrationsHost.LINE_MARKER_ID,
      highlighter.startOffset,
      highlighter.endOffset
    )
  }

  private fun createCustomRendererModel(
    rendererId: String?,
    renderer: LineMarkerRenderer,
    highlighter: RangeHighlighterEx,
    documentVersion: RdDocumentVersion
  ): LineMarkerHighlighterModel {
    return LineMarkerHighlighterModel(
      rendererId,
      null,
      1,
      0,
      if (renderer is LineMarkerRendererEx) renderer.position.toModel() else LineMarkerPosition.RIGHT,
      renderer is ActiveGutterRenderer,
      highlighter.layer,
      highlighter.targetArea == HighlighterTargetArea.EXACT_RANGE,
      documentVersion,
      highlighter.isGreedyToLeft,
      highlighter.isGreedyToRight,
      highlighter.isThinErrorStripeMark,
      null,
      highlighter.id,
      BackendHighlighterRegistrationsHost.LINE_MARKER_ID,
      highlighter.startOffset,
      highlighter.endOffset
    )
  }
}

private fun LineMarkerRendererEx.Position.toModel() = when (this) {
  LineMarkerRendererEx.Position.LEFT -> LineMarkerPosition.LEFT
  LineMarkerRendererEx.Position.RIGHT -> LineMarkerPosition.RIGHT
  LineMarkerRendererEx.Position.CUSTOM -> LineMarkerPosition.CUSTOM
}