package com.jetbrains.rdserver.daemon.highlighters.occurrenceHighlighters

import com.intellij.codeInsight.highlighting.HighlightManagerImpl.HIGHLIGHT_FLAGS_KEY
import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.markup.HighlighterTargetArea
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.OccurrenceHighlighterModel
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rdserver.daemon.BackendHighlighterModelCreator
import com.jetbrains.rdserver.daemon.BackendHighlighterRegistrationsHost

class OccurrenceHighlighterModelCreator(val project: Project) : BackendHighlighterModelCreator {

  override fun accept(highlighter: RangeHighlighterEx): Boolean {
    val flags = highlighter.getUserData(HIGHLIGHT_FLAGS_KEY)
    return flags != null
  }

  override fun createModel(highlighter: RangeHighlighterEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    val key = highlighter.textAttributesKey ?: return null
    val attributeId = BackendHighlighterRegistrationsHost.getAttributeId(key) ?: return null
    val flags = highlighter.getUserData(HIGHLIGHT_FLAGS_KEY) ?: return null
    return with(highlighter) {
      OccurrenceHighlighterModel(
        flags,
        layer,
        targetArea == HighlighterTargetArea.EXACT_RANGE,
        documentVersion,
        isGreedyToLeft,
        isGreedyToRight,
        isThinErrorStripeMark,
        null,
        id,
        attributeId,
        startOffset,
        endOffset
      )
    }
  }
}