package com.jetbrains.rdserver.daemon.inlays

import com.intellij.codeInsight.daemon.impl.HintRenderer
import com.intellij.openapi.editor.Inlay
import com.intellij.openapi.editor.ex.RangeMarkerEx
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.InlayHighlighterModel
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rd.ide.model.RichTextModel
import com.jetbrains.rdserver.daemon.BackendHighlighterRegistrationsHost
import com.jetbrains.rdserver.daemon.BackendRangeMarkerModelCreator
import com.jetbrains.rdserver.toRichText

class BackendInlayModelCreator : BackendRangeMarkerModelCreator {
  override fun accept(rangeMarker: RangeMarkerEx): Boolean {
    return rangeMarker is Inlay<*>
  }

  override fun createModel(rangeMarker: RangeMarkerEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    val inlay = rangeMarker as Inlay<*>
    // only hints are supported for now
    val renderer = inlay.renderer as? HintRenderer ?: return null
    val text = renderer.text?.toRichText() ?: return null
    return InlayHighlighterModel(
      text,
      RichTextModel(emptyList()),
      0,
      null,
      false,
      false,
      0,
      true,
      documentVersion,
      false,
      false,
      false,
      null,
      (inlay as RangeMarkerEx).id,
      BackendHighlighterRegistrationsHost.INLAY_ID,
      inlay.startOffset,
      inlay.startOffset
    )
  }
}