package com.jetbrains.rdserver.daemon.syntax

import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.intellij.openapi.editor.ex.util.LayeredTextAttributes
import com.intellij.openapi.editor.markup.HighlighterTargetArea
import com.jetbrains.codeWithMe.model.SyntaxHighlighterModel
import com.jetbrains.rd.ide.model.HighlighterModel
import com.jetbrains.rd.ide.model.RdDocumentVersion
import com.jetbrains.rdserver.daemon.BackendHighlighterModelCreator
import com.jetbrains.rdserver.daemon.BackendHighlighterRegistrationsHost

class SyntaxHighlighterModelCreator : BackendHighlighterModelCreator {
  override fun accept(highlighter: RangeHighlighterEx): Boolean {
    // syntax highlighting or highlighting from injections
    return highlighter.textAttributesKeys != null || highlighter.forcedTextAttributes is LayeredTextAttributes
  }

  override fun createModel(highlighter: RangeHighlighterEx, documentVersion: RdDocumentVersion): HighlighterModel? {
    val keys = highlighter.textAttributesKeys ?: (highlighter.forcedTextAttributes as LayeredTextAttributes).keys
    val ids = keys.mapNotNull { BackendHighlighterRegistrationsHost.getAttributeId(it) }
    if (ids.isEmpty()) return null

    return SyntaxHighlighterModel(
      ids,
      highlighter.layer,
      highlighter.targetArea == HighlighterTargetArea.EXACT_RANGE,
      documentVersion,
      highlighter.isGreedyToLeft,
      highlighter.isGreedyToRight,
      highlighter.isThinErrorStripeMark,
      null,
      highlighter.id,
      ids.last(),
      highlighter.startOffset,
      highlighter.endOffset
    )
  }
}