package com.jetbrains.rdserver.daemon.tooltips

import com.intellij.codeInsight.daemon.impl.HighlightInfo
import com.intellij.openapi.editor.ex.RangeHighlighterEx
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.daemon.BackendMarkupModelContributor
import com.jetbrains.rdserver.daemon.BackendProtocolMarkupModelExtensionHandler
import com.jetbrains.rd.ide.model.HtmlStringTooltipModel
import com.jetbrains.rd.ide.model.MarkupModelExtension
import com.jetbrains.rd.ide.model.StringTooltipModel
import com.jetbrains.rd.ide.model.TooltipProviderModel

class BackendTooltipMarkupModelExtensionHandler :
  BackendProtocolMarkupModelExtensionHandler {
  override fun consume(lifetime: Lifetime, extensions: List<MarkupModelExtension>, contributor: BackendMarkupModelContributor) {
    val tooltipProviderModel = extensions.find { it.key == "TOOLTIP_PROVIDER_MODEL_KEY" } as? TooltipProviderModel
                               ?: return

    tooltipProviderModel.getTooltip.set { id ->
      val highlighter = contributor.getRangeMarker(id) as? RangeHighlighterEx
                        ?: return@set StringTooltipModel(null)
      val highlightInfo = HighlightInfo.fromRangeHighlighter(highlighter)
      if (highlightInfo != null) {
        return@set HtmlStringTooltipModel(highlightInfo.toolTip, highlightInfo.description)
      }
      return@set StringTooltipModel(null)
    }
  }
}