package com.jetbrains.rdserver.debugger

import com.intellij.openapi.editor.Document
import com.intellij.openapi.project.Project
import com.intellij.xdebugger.XDebuggerManager
import com.intellij.xdebugger.evaluation.EvaluationMode
import com.intellij.xdebugger.impl.XSourcePositionImpl
import com.intellij.xdebugger.impl.breakpoints.XExpressionImpl
import com.jetbrains.codeWithMe.model.CodeWithMeSourcePositionModel
import com.jetbrains.rd.ide.model.RdDocumentId
import com.jetbrains.rd.ide.model.RdDocumentModel
import com.jetbrains.rd.platform.codeWithMe.debugger.debuggerInfo
import com.jetbrains.rdserver.document.BackendDocumentProvider
import com.jetbrains.rdserver.vfs.BackendVirtualFileSystemHost

/**
 * Provides in-memory documents for various debugger features as watches, evaluate, etc.
 */
class BackendDebuggerDocumentProvider(val project: Project) : BackendDocumentProvider {
  override fun accept(documentId: RdDocumentId, documentModel: RdDocumentModel): Boolean {
    return documentModel.debuggerInfo != null
  }

  override fun document(documentId: RdDocumentId, documentModel: RdDocumentModel): Document? {
    // TODO[moklev]: create watches without debugger session?
    val session = XDebuggerManager.getInstance(project).currentSession ?: return null

    val watchesInfo = documentModel.debuggerInfo!!
    val sourcePosition = watchesInfo.sourcePosition?.let {
      it as CodeWithMeSourcePositionModel
      val contextFile = BackendVirtualFileSystemHost.getInstance(project).findVirtualFile(it.fileId)
      XSourcePositionImpl.createByOffset(contextFile, it.offset)
    }

    val document = session.debugProcess.editorsProvider.createDocument(
      project,
      XExpressionImpl(watchesInfo.initialText, null, null),
      sourcePosition,
      EvaluationMode.EXPRESSION
    )
    return document
  }
}