package com.jetbrains.rdserver.debugger

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.xdebugger.frame.XExecutionStack
import com.intellij.xdebugger.frame.XStackFrame
import com.jetbrains.codeWithMe.model.StackFrameModelBase
import com.jetbrains.rd.framework.RdTaskResult
import com.jetbrains.rd.framework.impl.RdTask
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isNotAlive

class CodeWithMeFrameContainer(
  val clientId: ClientId,
  val lifetime: Lifetime,
  val project: Project,
  val stack: XExecutionStack,
  val task: RdTask<List<StackFrameModelBase>>
) : XExecutionStack.XStackFrameContainer {
  private val stackFrameModels = mutableListOf<StackFrameModelBase>()

  override fun isObsolete(): Boolean {
    return lifetime.isNotAlive
  }

  override fun errorOccurred(errorMessage: String) {
    application.invokeLater {
      ClientId.withClientId(clientId) {
        task.result.set(RdTaskResult.Fault(Exception(errorMessage)))
      }
    }
  }

  override fun addStackFrames(stackFrames: MutableList<out XStackFrame>, last: Boolean) {
    application.invokeLater {
      stackFrameModels.addAll(stackFrames.map { xFrame ->
        xFrame.createStackFrameModel(project, clientId, stack)
      })
      if (last) {
        ClientId.withClientId(clientId) {
          task.set(stackFrameModels)
        }
      }
    }
  }
}