package com.jetbrains.rdserver.debugger

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.xdebugger.frame.XExecutionStack
import com.intellij.xdebugger.frame.XSuspendContext
import com.jetbrains.codeWithMe.model.ExecutionStackModelBase
import com.jetbrains.rd.framework.RdTaskResult
import com.jetbrains.rd.framework.impl.RdTask
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isNotAlive

class CodeWithMeStackContainer(
  val clientId: ClientId,
  val lifetime: Lifetime,
  val project: Project,
  val task: RdTask<List<ExecutionStackModelBase>>
) : XSuspendContext.XExecutionStackContainer {
  private val executionStackModels = mutableListOf<ExecutionStackModelBase>()

  override fun isObsolete(): Boolean {
    return lifetime.isNotAlive
  }

  override fun errorOccurred(errorMessage: String) {
    ClientId.withClientId(clientId) {
      task.result.set(RdTaskResult.Fault(Exception(errorMessage)))
    }
  }

  override fun addExecutionStack(executionStacks: MutableList<out XExecutionStack>, last: Boolean) {
    executionStacks.mapTo(executionStackModels) { xStack ->
      xStack.createExecutionStackModel(project, clientId)
    }
    if (last) {
      ClientId.withClientId(clientId) {
        task.set(executionStackModels)
      }
    }
  }
}