package com.jetbrains.rdserver.debugger

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.xdebugger.frame.XExecutionStack
import com.jetbrains.codeWithMe.model.CodeWithMeExecutionStackModel
import com.jetbrains.codeWithMe.model.CodeWithMeStackFrameModel
import com.jetbrains.codeWithMe.model.StackFrameModelBase
import com.jetbrains.rd.framework.impl.RdTask

fun XExecutionStack.createExecutionStackModel(project: Project, clientId: ClientId): CodeWithMeExecutionStackModel {
  return CodeWithMeExecutionStackModel(displayName, safeCreateTopFrameModel(project, clientId)).apply {
    computeFrames.set { lifetime, arg ->
      RdTask<List<StackFrameModelBase>>().apply {
        computeStackFrames(
          arg.firstFrameIndex,
          CodeWithMeFrameContainer(clientId, lifetime, project, this@createExecutionStackModel, this)
        )
      }
    }
  }
}

private fun XExecutionStack.safeCreateTopFrameModel(project: Project, clientId: ClientId): CodeWithMeStackFrameModel? {
  return try {
    topFrame?.createStackFrameModel(project, clientId, this)
  } catch (e: Throwable) {
    null
  }
}