package com.jetbrains.rdserver.debugger

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.util.concurrency.annotations.RequiresEdt
import com.intellij.xdebugger.XDebuggerUtil
import com.intellij.xdebugger.XSourcePosition
import com.jetbrains.codeWithMe.model.CodeWithMeSourcePositionModel
import com.jetbrains.codeWithMe.model.SourcePositionBase
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rdserver.vfs.BackendVirtualFileSystemHost

@RequiresEdt
fun XSourcePosition.toModel(clientId: ClientId, project: Project): CodeWithMeSourcePositionModel? {
  application.assertIsDispatchThread()
  val fileId = ClientId.withClientId(clientId) {
    BackendVirtualFileSystemHost.getInstance(project).bindFile(file)
  } ?: return null

  // todo remove path
  return CodeWithMeSourcePositionModel(fileId, "INVALID", line, offset)
}

fun SourcePositionBase.fromModel(project: Project): XSourcePosition? {
  this as CodeWithMeSourcePositionModel

  val file = BackendVirtualFileSystemHost.getInstance(project).findVirtualFile(fileId)
  return XDebuggerUtil.getInstance().run {
    if (offset != 0) {
      createPositionByOffset(file, offset)
    }
    else {
      createPosition(file, line)
    }
  }
}