@file:Suppress("NonDefaultConstructor")

package com.jetbrains.rdserver.diagnostics

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.jetbrains.rd.framework.IProtocol
import com.jetbrains.rd.ide.model.FileTransmissionModel
import com.jetbrains.rd.ide.model.FileTransmissionRequest
import com.jetbrains.rd.platform.codeWithMe.diagnostics.FileTransmission
import com.jetbrains.rd.platform.codeWithMe.diagnostics.FileTransmissionHandler
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.RdServerPerClientComponents
import java.io.File
import java.util.concurrent.CompletableFuture

class BackendFileTransmissionHost(
  lifetime: Lifetime,
  private val clientId: ClientId,
  protocol: IProtocol
) : FileTransmission {

  init {
    // Always initialize models on dispatch thread
    application.assertIsDispatchThread()
  }

  private val model = FileTransmissionModel.create(lifetime, protocol)
  private val handler = FileTransmissionHandler(lifetime, model)

  companion object {
    fun getInstance(project: Project, clientId: ClientId): BackendFileTransmissionHost =
      RdServerPerClientComponents.getService(project, clientId, BackendFileTransmissionHost::class.java)
  }

  override fun sendFileAsync(file: File, request: FileTransmissionRequest): CompletableFuture<Unit> =
    ClientId.withClientId(clientId) {
      handler.sendFileAsync(file, request)
    }

  override fun receiveFileAsync(request: FileTransmissionRequest): CompletableFuture<File> =
    ClientId.withClientId(clientId) {
      handler.receiveFileAsync(request)
    }

}
