package com.jetbrains.rdserver.diagnostics

import com.intellij.build.BuildProgressListener
import com.intellij.build.BuildViewManager
import com.intellij.build.events.FinishBuildEvent
import com.intellij.build.events.StartBuildEvent
import com.intellij.openapi.components.service
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.platform.codeWithMe.users.broadcastToAllGuests
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rdserver.startup.serverSolution
import com.jetbrains.rd.ide.model.ReadonlyModeReasons

class BackendPerformanceHost(project: Project) {

  companion object{
    private const val usersOverflowCount = 100
  }

  private val logger = logger<BackendPerformanceHost>()

  init {
    project.service<BuildViewManager>().addListener(BuildProgressListener
    {
      _, event ->
      broadcastToAllGuests(project) {
        if (event is StartBuildEvent) {
          logger.info("setting ON readonly mode due to the start of the build")
          project.serverSolution.readonlyMode.value = ReadonlyModeReasons.Build
        }
        else if (event is FinishBuildEvent) {
          logger.info("setting OFF readonly mode due to the end of the build")
          project.serverSolution.readonlyMode.value = null
        }
      }
    }, project)

    val userManager = UserManager.getInstance(project)
    userManager.users.view(project.lifetime)
    {
      _, _ ->

      application.invokeLater {
        broadcastToAllGuests(project) {
          if (userManager.users.count() > usersOverflowCount) {
            logger.info("setting ON readonly mode cause there are more than $usersOverflowCount users")
            project.serverSolution.readonlyMode.value = ReadonlyModeReasons.TooManyUser
          }
          else {
            logger.info("setting OFF readonly mode cause there are less than $usersOverflowCount users")
            project.serverSolution.readonlyMode.value = null
          }
        }
      }
    }
  }

}