package com.jetbrains.rdserver.diagnostics

import com.intellij.openapi.editor.impl.CaretModelImpl
import com.intellij.openapi.fileEditor.FileDocumentManager
import com.intellij.openapi.fileEditor.FileEditorManager
import com.intellij.openapi.fileEditor.TextEditor
import com.intellij.openapi.project.Project
import com.intellij.troubleshooting.TroubleInfoCollector
import com.jetbrains.rdserver.editors.BackendFollowMeEditorsHost
import com.jetbrains.rdserver.editors.clientId
import com.jetbrains.rdserver.editors.isLocal

class RemoteEditorsTroubleInfoCollector : TroubleInfoCollector {

  override fun collectInfo(project: Project): String = collect(project)

  companion object {
    fun collect(project: Project): String {
      val allEditors = FileEditorManager.getInstance(project).allEditors.mapNotNull { (it as? TextEditor)?.editor }

      val fileDocumentManager = FileDocumentManager.getInstance()
      val stringBuilder = StringBuilder()

      stringBuilder.appendln("=== Code With Me Editors ===")

      val editorFollowMeHost = BackendFollowMeEditorsHost.getInstance(project)
      stringBuilder.appendln("FOLLOW ME:")

      val followMeDump = editorFollowMeHost.dump()
      stringBuilder.appendln(followMeDump)
      stringBuilder.appendln("ALL EDITORS:")

      for (editor in allEditors) {

        val file = fileDocumentManager.getFile(editor.document) ?: continue
        stringBuilder.appendln("          |||${file.name}|||")

        if (editor.isLocal()) {
          stringBuilder.appendln("PHYSICAL EDITOR")
        }
        else {
          stringBuilder.appendln("PHANTOM EDITOR. CLIENT ID: ${editor.clientId()}")
        }

        stringBuilder.append("Caret dump: ")
        stringBuilder.appendln((editor.caretModel as? CaretModelImpl)?.dumpState())

        stringBuilder.appendln("=============================")
      }

      return stringBuilder.toString()
    }
  }
}