package com.jetbrains.rdserver.document

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.diagnostic.trace
import com.jetbrains.rd.ide.document.DocumentLog
import com.jetbrains.rd.ide.model.RdDocumentVersion

private val logger = logger<BackendDocumentLog>()

class BackendDocumentLog : DocumentLog() {
  private val clientToVersions = mutableMapOf<ClientId, RdDocumentVersion>()

  fun clientNumber() = clientToVersions.size


  fun registerClient(clientId: ClientId, version: RdDocumentVersion) {
    logger.trace { "Register consumer: $clientId, with last known version: $lastKnownVersion" }
    clientToVersions[clientId] = version
  }

  fun unregisterClient(clientId: ClientId) {
    logger.trace { "Unregister consumer: $clientId" }
    clientToVersions.remove(clientId)
  }

  override fun updateLastKnownVersion(newVersion: RdDocumentVersion) {
    val clientId = ClientId.current
    val oldVersion = clientToVersions[clientId]
    if (oldVersion == null) {
      logger.error("Got ack from unregistered client: $clientId")
      return
    }
    if (oldVersion.value >= newVersion.value) {
      logger.error("old version(${oldVersion.value} >= new version (${newVersion.value})) ")
      return
    }

    clientToVersions[clientId] = newVersion

    val minLastKnownVersion = clientToVersions.values.minBy { it.value } ?: return

    super.updateLastKnownVersion(minLastKnownVersion)
  }

  override fun rollbackHistoryToVersion(v: RdDocumentVersion) {
    throw UnsupportedOperationException()
  }
}