package com.jetbrains.rdserver.document

import com.intellij.codeWithMe.ClientId
import com.jetbrains.rd.ide.document.DocumentLog
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rd.ide.model.RdDocumentId
import com.jetbrains.rd.ide.model.RdDocumentVersion

class BackendDocumentLogManager {
  private val documentsLogs = mutableMapOf<RdDocumentId, BackendDocumentLog>()

  fun register(lifetime: Lifetime, clientId: ClientId, documentId: RdDocumentId, version: RdDocumentVersion): DocumentLog {
    val documentLog = documentsLogs.getOrPut(documentId) { BackendDocumentLog() }
    documentLog.registerClient(clientId, version)

    lifetime.onTermination {
      documentLog.unregisterClient(clientId)
      if (documentLog.clientNumber() == 0) {
        documentsLogs.remove(documentId)
      }
    }

    return documentLog
  }
}