package com.jetbrains.rdserver.document

import com.intellij.openapi.editor.Document
import com.intellij.openapi.extensions.ProjectExtensionPointName
import com.jetbrains.rd.ide.model.RdDocumentId
import com.jetbrains.rd.ide.model.RdDocumentModel

/**
 * An extension for providing suitable documents for various features and subsystems.
 * Documents can be created out of thin air, associated with some context,
 * belong to [com.intellij.openapi.fileEditor.TextEditor]s in the tab,
 * or to arbitrary [com.intellij.openapi.editor.Editor]s.
 *
 * [RdDocumentModel] is passed to be able to introspect [RdDocumentModel.extensions].
 * Take a look at [BackendDocumentExtensionsProvider] and a corresponding type on the frontend
 */
interface BackendDocumentProvider {
  companion object {
    val EP_NAME = ProjectExtensionPointName<BackendDocumentProvider>("com.intellij.rdserver.documentProvider")
  }

  fun accept(documentId: RdDocumentId, documentModel: RdDocumentModel): Boolean

  fun document(documentId: RdDocumentId, documentModel: RdDocumentModel): Document?
}