package com.jetbrains.rdserver.document

import com.jetbrains.rd.ide.document.DocumentVersionHolder
import com.jetbrains.rd.ide.document.DocumentVersionHolderImpl
import com.jetbrains.rd.ide.document.InitialDocVersion
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rd.ide.model.RdDocumentId

private data class DocVersionHolderWithConsumerCounter(val holder: DocumentVersionHolder, var consumerCounter: Int)

class BackendDocumentVersions {
  private val docId2versions = mutableMapOf<RdDocumentId, DocVersionHolderWithConsumerCounter>()

  fun createDocumentVersionHolder(lifetime: Lifetime, documentId: RdDocumentId): DocumentVersionHolder {
    application.assertIsDispatchThread()
    val pair = docId2versions.getOrPut(documentId) {
      DocVersionHolderWithConsumerCounter(DocumentVersionHolderImpl(InitialDocVersion), 0)
    }
    pair.consumerCounter++

    lifetime.onTermination {
      application.assertIsDispatchThread()
      pair.consumerCounter--
      if (pair.consumerCounter == 0)
        docId2versions.remove(documentId)
    }
    return pair.holder
  }
}