package com.jetbrains.rdserver.document

import com.intellij.openapi.editor.Document
import com.intellij.openapi.fileEditor.FileDocumentManager
import com.intellij.openapi.project.Project
import com.intellij.openapi.vfs.VirtualFile
import com.intellij.openapi.vfs.VirtualFileManager
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rdserver.vfs.BackendVirtualFileSystemHost

class BackendFileBasedDocumentProvider(private val project: Project) : BackendDocumentProvider {
  override fun accept(documentId: RdDocumentId, documentModel: RdDocumentModel): Boolean {
    return documentId is RdDocumentIdWithFileId || documentId is RdFileDocumentId
  }

  override fun document(documentId: RdDocumentId, documentModel: RdDocumentModel): Document? {
    var url: String? = null
    var file: VirtualFile? = null

    if (documentId is RdDocumentIdWithFileId) {
      val fileId = documentId.fileId
      if (fileId is RdFileIdNew) {
        file = VirtualFileManager.getInstance().findFileById(fileId.id)
      }
      if (fileId is RdFileDocumentId) {
        url = VirtualFileManager.constructUrl(fileId.protocol, fileId.filePath)
      }
    }

    if (documentId is RdFileDocumentId) {
      url = VirtualFileManager.constructUrl(documentId.protocol, documentId.filePath)
    }

    if (url != null) {
      file = VirtualFileManager.getInstance().refreshAndFindFileByUrl(url)
    }

    if (file == null) return null

    BackendVirtualFileSystemHost.getInstance(project).bindFile(file, sendProperties = true, sendContent = true)

    return FileDocumentManager.getInstance().getDocument(file)
  }
}
