package com.jetbrains.rdserver.editors

import com.intellij.openapi.fileEditor.FileDocumentManager
import com.intellij.openapi.fileEditor.FileEditorManager
import com.intellij.openapi.fileEditor.TextEditor
import com.jetbrains.rd.ide.editor.component1
import com.jetbrains.rd.ide.editor.component2

internal fun FileEditorManager.dumpBackendTextControls(): String {
  return buildString {
    appendln("IDEA backend text controls:")
    for (fileEditor in allEditors.sortedBy { it.file?.path }) {
      appendln("Name: ${fileEditor.name}")
      val editor = (fileEditor as? TextEditor)?.editor
      if (editor == null) {
        appendln("no Editor associated with this FileEditor")
        continue
      }

      val virtualFile = FileDocumentManager.getInstance().getFile(editor.document)
      appendln("Virtual file: ${virtualFile?.name ?: "null file"}")

      val (logicalLine, logicalColumn) = editor.caretModel.logicalPosition
      appendln("Current caret: (${logicalLine}, ${logicalColumn})")

      val selection = editor.selectionModel
      val (startLine, startColumn) = selection.selectionStartPosition
      val (endLine, endColumn) = selection.selectionEndPosition
      appendln("Current selection:  startPosition = (${startLine}, ${startColumn}) endPosition = (${endLine}, ${endColumn})")
    }
  }
}