package com.jetbrains.rdserver.editors

import com.intellij.codeWithMe.ClientId
import com.intellij.injected.editor.EditorWindow
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.fileEditor.FileEditor
import com.jetbrains.rdserver.editors.BackendEditorHost.Companion.synchronizer
import com.jetbrains.rdserver.editors.BackendServerFileEditorManager.Companion.editorClientId
import com.jetbrains.rdserver.editors.BackendServerFileEditorManager.Companion.fileEditorClientId
import com.intellij.openapi.vfs.VirtualFile
import com.jetbrains.rd.framework.base.RdExtBase
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rdserver.editors.BackendEditorHost.Companion.textControlModel

fun Editor.getTextControlId() = getSynchronizer()?.textControlId

fun Editor.getSynchronizer() : BackendEditorSynchronizer? {
  if (this is EditorWindow)
    logger<BackendEditorHost>().error("trying to get model synchronizer for editor window")

  return this.synchronizer
}

fun Editor.clientId() = editorClientId ?: ClientId.localId

fun Editor.isLocal() = editorClientId == null

fun FileEditor.clientId() = fileEditorClientId ?: ClientId.localId

fun FileEditor.isLocal() = fileEditorClientId == null

fun <TExt : RdExtBase> Editor.getTextControlExtension(selector: TextControlModel.() -> TExt) : TExt? =
  this.textControlModel?.selector()

val Editor.parameterInfoModel : ParameterInfoModel? get() = this.getTextControlExtension { parameterInfoModel }

val Editor.completionModel : CompletionModel? get() = this.getTextControlExtension { completionModel }