package com.jetbrains.rdserver.execution

import com.intellij.openapi.components.service
import com.intellij.openapi.externalSystem.model.task.ExternalSystemTaskId
import com.intellij.openapi.externalSystem.model.task.ExternalSystemTaskType
import com.intellij.openapi.externalSystem.model.task.event.*
import com.intellij.openapi.project.Project
import com.jetbrains.rd.ide.model.*
import com.jetbrains.rd.platform.codeWithMe.ThreadSafeIdHolder

fun ExternalSystemTaskType.toModel() = when (this) {
  ExternalSystemTaskType.RESOLVE_PROJECT -> ExternalSystemTaskTypeModel.ResolveProject
  ExternalSystemTaskType.REFRESH_TASKS_LIST -> ExternalSystemTaskTypeModel.RefreshTasksList
  ExternalSystemTaskType.EXECUTE_TASK -> ExternalSystemTaskTypeModel.ExecuteTask
}

fun ExternalSystemTaskId.toModel(project: Project): ExternalSystemTaskIdModel {
  val id = project.service<ThreadSafeIdHolder>().getId(this)
  return ExternalSystemTaskIdModel(projectSystemId.id, type.toModel(), id)
}

fun ExternalSystemProgressEvent<*>.toModel(): ExternalSystemTaskExecutionEventModel? {
  val descriptorModel = OperationDescriptorModel(descriptor.displayName, descriptor.eventTime)
  return when (this) {
    is ExternalSystemStartEvent -> ExternalSystemStartEventModel(eventId, parentEventId, descriptorModel)
    is ExternalSystemStatusEvent -> ExternalSystemStatusEventModel(eventId, parentEventId, descriptorModel)
    is ExternalSystemFinishEvent -> {
      val resultModel = when (operationResult) {
        is SuccessResult -> OperationResultModel.Success
        is FailureResult -> OperationResultModel.Failure
        is SkippedResult -> OperationResultModel.Skipped
        else -> TODO()
      }
      ExternalSystemFinishEventModel(resultModel, eventId, parentEventId, descriptorModel)
    }
    else -> null
  }
}