package com.jetbrains.rdserver.extension

import com.intellij.serviceContainer.BaseKeyedLazyInstance
import com.intellij.util.KeyedLazyInstance
import com.intellij.util.xmlb.annotations.Attribute
import java.lang.reflect.ParameterizedType

/**
 * @author Alex Plate
 * @date 2019-01-16
 *
 * Class extension point for classes that implement generic interface
 *
 * E.g.
 * class Child: IParent<GenericType>
 *
 * First generic type is used as key for extension point
 */
class GenericClassExtensionPoint<T : Any> : BaseKeyedLazyInstance<T>(), KeyedLazyInstance<T> {
  @Attribute("implementationClass")
  var implementationClass: String? = null

  override fun getImplementationClassName() = implementationClass

  override fun getKey(): String {
    val type = (instance.javaClass.genericInterfaces[0] as ParameterizedType).actualTypeArguments[0]
    if (type is ParameterizedType) {
      return type.rawType.typeName
    }
    return type.typeName
  }
}