package com.jetbrains.rdserver.followMe

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.jetbrains.codeWithMe.model.UserFocusModel
import com.jetbrains.rd.framework.IProtocol
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.RdServerPerClientComponents

@Suppress("NonDefaultConstructor")
class BackendUserFocusHost(
  lifetime: Lifetime,
  project: Project,
  protocol: IProtocol
) {

  companion object {
    fun getInstance(project: Project, clientId: ClientId): BackendUserFocusHost {
      return RdServerPerClientComponents.getService(project, clientId, BackendUserFocusHost::class.java)
    }
    fun tryGetInstance(project: Project, clientId: ClientId): BackendUserFocusHost? {
      return RdServerPerClientComponents.tryGetService(project, clientId, BackendUserFocusHost::class.java)
    }
  }

  private val model = UserFocusModel.create(lifetime, protocol)
  private val clientId = ClientId.current

  init {
    model.focusChanged.advise(lifetime) {
      val focusManager = BackendUserFocusManager.getBackendInstance(project)
      focusManager.refreshActiveToolWindow(clientId, FocusChangeSource.FrontendFocusChanges)
    }
  }

  fun getCurrentActiveToolWindow(): String? {
    return model.currentActiveToolWindowId.value
  }

  fun setLeadActiveToolWindow(toolWindowId: String?) {
    model.leadActiveToolWindowId.set(toolWindowId)
  }
}