package com.jetbrains.rdserver.followMe

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.Separator
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.platform.codeWithMe.users.clientId
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.addUnique
import com.jetbrains.rd.util.reactive.setValue
import com.jetbrains.rdserver.joinLinks.RdServerJoinLinkManager
import com.jetbrains.codeWithMe.model.RdUserModel
import com.jetbrains.rd.util.reactive.IPropertyView
import com.jetbrains.rd.util.reactive.Property

class BackendUserManager(project: Project) : UserManager(project) {

  init {
    application.invokeLater {
      if (project.isDisposed) return@invokeLater
      val followMeManager = FollowMeManager.getInstance(project)

      followMeManager.users.view(lifetime) { lt, _, user ->
        registerUser(lt, user)
      }

      followMeManager.followers.view(lifetime) { lt, who, whom ->
        if (who == ClientId.localId) {
          val user = followMeManager.getUser(whom)
          leadProperty.setValue(lt, user)
        }
        if (whom == ClientId.localId) {
          val user = followMeManager.getUser(who)
          followersList.addUnique(lt, user)
        }
      }

      val manager = RdServerJoinLinkManager.getInstance(project)
      sessionEnabledProperty.set(manager.getExistingJoinLink() != null)
      manager.joinLinksUpdated.advise(lifetime) {
        application.invokeLater {
          if (project.isDisposed) return@invokeLater
          sessionEnabledProperty.set(manager.getExistingJoinLink() != null)
        }
      }
    }
  }

  override fun jumpTo(user: RdUserModel) {
    BackendUserOperations.getInstance(project).jump(ClientId.localId, user)
  }

  override fun startFollowing(user: RdUserModel) {
    FollowMeManager.getInstance(project).registerFollower(ClientId.localId, user.clientId)
    recentLead = Property(lead.value)
  }

  override fun stopFollowing() {
    val tmpLead: IPropertyView<RdUserModel?> = Property(lead.value)
    FollowMeManager.getInstance(project).unregisterFollower(ClientId.localId)
    recentLead = tmpLead
  }

  override fun requestFollowing(user: RdUserModel, force: Boolean) {
    BackendUserOperations.getInstance(project).requestFollowing(ClientId.localId, user, force)
  }

  override fun requestStopFollowing(user: RdUserModel) {
    BackendUserOperations.getInstance(project).requestStopFollowing(ClientId.localId, user)
  }

  override fun createUserActions(user: RdUserModel): List<AnAction> {
    return super.createUserActions(user).union(listOf(
      Separator(),
      createPermissionsAction(project, user),
      createKickAction(project, user)
    )).toList()
  }
}