package com.jetbrains.rdserver.followMe

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.components.service
import com.intellij.openapi.fileEditor.FileEditorManager
import com.intellij.openapi.fileEditor.OpenFileDescriptor
import com.intellij.openapi.project.Project
import com.jetbrains.codeWithMe.model.RdUserModel
import com.jetbrains.rd.platform.codeWithMe.users.UserNotifications
import com.jetbrains.rd.platform.codeWithMe.users.clientId
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rdserver.editors.BackendFileEditorHandler

class BackendUserOperations(private val project: Project) {

  companion object {
    private val LOG = getLogger<BackendUserOperations>()

    fun getInstance(project: Project): BackendUserOperations = project.service()
  }

  fun jump(who: ClientId, toWhom: RdUserModel) {
    LOG.info("[$who] Jump to user ${toWhom.id}:'${toWhom.name}' [clientId ${toWhom.clientId}]...")
    val fileEditorManager = FileEditorManager.getInstance(project)

    val selectedEditor = ClientId.withClientId(toWhom.clientId) {
      fileEditorManager.selectedEditor ?: return
    }

    val fileToNavigate = selectedEditor.file ?: return

    val navigatable = BackendFileEditorHandler.EP_NAME.computeSafeIfAny { it.createFollowNavigatable(project, selectedEditor) }
                      ?: OpenFileDescriptor(project, fileToNavigate)

    LOG.info("jumping from $who to ${toWhom}. File: $fileToNavigate")

    ClientId.withClientId(who) {
      fileEditorManager.openFileEditor(navigatable, true)
    }
  }

  fun requestFollowing(who: ClientId, whom: RdUserModel, force: Boolean) {
    val whoUser = FollowMeManager.getInstance(project).getUser(who)

    // If someone asks host to follow - show notification
    if (whom.clientId == ClientId.localId) {
      UserNotifications.askForFollowOtherUser(project, whoUser, force) {
        FollowMeManager.getInstance(project).registerFollower(ClientId.localId, who)
      }
      return
    }

    // In other case translate request to a client
    LOG.info("[$who] Ask user ${whom.id}:'${whom.name}' for following [clientId ${whom.clientId}]...")
    val host = BackendUsersHost.getInstance(project, whom.clientId)
    host.requestFollowing(whoUser, force)
  }

  fun requestStopFollowing(who: ClientId, whom: RdUserModel) {
    // Stop following if the request goes to host (us)
    if (whom.clientId == ClientId.localId) {
      FollowMeManager.getInstance(project).unregisterFollower(whom.clientId)
      return
    }

    // In other case translate request to a client
    LOG.info("[$who] Tell user ${whom.id}:'${whom.name}' to stop following user [clientId ${who}]...")
    val host = BackendUsersHost.getInstance(project, whom.clientId)
    host.requestStopFollowing()
  }
}