package com.jetbrains.rdserver.followMe.popup

import com.intellij.codeWithMe.ClientId
import com.intellij.codeWithMe.ClientId.Companion.isLocal
import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.ex.ActionManagerEx
import com.intellij.openapi.project.Project
import com.jetbrains.codeWithMe.model.PopupsModel
import com.jetbrains.ide.model.uiautomation.BeAbstractPopup
import com.jetbrains.rd.ide.model.Solution
import com.jetbrains.rd.ui.bindable.showPopup
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.viewNotNull
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.followMe.FollowMeManager
import com.jetbrains.rdserver.startup.serverSolutionFor
import com.jetbrains.rdserver.ui.popup.BackendPopupModelHost

@Suppress("NonDefaultConstructor")
class BackendPopupsHost(lifetime: Lifetime, val project: Project, solution: Solution) {

  private val popupsModel: PopupsModel
    get() = RdServerPerClientComponents.getService(project, BackendPopupModelHost::class.java).popupsModel

  init {
    popupsModel.bePopups.viewNotNull(lifetime, ::fireClientPopupWasShown)
  }

  private fun fireClientPopupWasShown(lifetime: Lifetime, popupWithId: Map.Entry<Int, BeAbstractPopup>) {
    val actionId = (ActionManager.getInstance() as ActionManagerEx).lastPreformedActionId
    if (FollowMeManager.getInstance(project).canReflectPopupToFollowers(actionId)) {
      fireLeadsPopupWasShown(popupWithId.value, lifetime, project)
    }
  }

  companion object {
    fun fireLeadsPopupWasShown(popupModel: BeAbstractPopup,
                               lifetime: Lifetime,
                               project: Project) {
      val followers = FollowMeManager.getInstance(project).getFollowers(ClientId.current)
      for (follower in followers) {
        if (follower.isLocal) {
          popupModel.showPopup(project, lifetime, true)
          continue
        }

        val solutionForFollower = project.serverSolutionFor(follower)

        val clonedBeAbstractPopup = popupModel.deepClone()
        // note: this method changes the content(submodels) of argument-model and left AS-IS the content of model on which this method was called
        popupModel.synchronizeWith(lifetime, clonedBeAbstractPopup)
        solutionForFollower.leadsPopupModel.set(clonedBeAbstractPopup)
      }
    }
  }
}
