package com.jetbrains.rdserver.internal

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.application.ApplicationListener
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.project.Project
import com.intellij.openapi.rd.createNestedDisposable
import com.intellij.openapi.startup.StartupManager
import com.jetbrains.rd.ide.model.Solution
import com.jetbrains.rd.ide.model.WriteLockInfo
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.isAlive
import com.jetbrains.rdserver.followMe.FollowMeManager

@Suppress("NonDefaultConstructor")
class BackendWriteLocksMonitor(
  private val clientId: ClientId,
  lifetime: Lifetime,
  project: Project,
  solution: Solution
) {
  init {
    StartupManager.getInstance(project).runWhenProjectIsInitialized {
      if (!lifetime.isAlive) return@runWhenProjectIsInitialized
      application.addApplicationListener(object : ApplicationListener {
        override fun writeActionStarted(action: Any) {
          try {
            val writeLockClientId = ClientId.current
            if (writeLockClientId == clientId) return

            val writeLockUserName = FollowMeManager.getInstance(project).getUser(writeLockClientId).name
            solution.writeLockAcquired.fire(WriteLockInfo(writeLockUserName, action.toString()))
          }
          catch (e: Exception) {
            logger<BackendWriteLocksMonitor>().warn(e)
          }
        }
      }, lifetime.createNestedDisposable())
    }
  }
}