package com.jetbrains.rdserver.internal

import com.intellij.ide.gdpr.AgreementUi
import com.intellij.ide.gdpr.EndUserAgreement
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.util.ResourceUtil
import com.jetbrains.rdserver.CodeWithMePluginBundle
import java.util.*

object CwmEULA {
  private val commonEulaBundle
    get() = ResourceBundle.getBundle("messages.AgreementsBundle")

  private val eula by lazy {
    // we don't have our own legal document yet, so we're using common IJ EAP one
    val docName = "cwmHostEua"
    CwmEULA::class.java.getResourceAsStream("/$docName.html").use { input ->
      if (input == null) error("Can't find CWM Host EULA text")
      EndUserAgreement.Document(docName, ResourceUtil.loadText(input))
    }
  }

  fun isAccepted(): Boolean {
    val isAccepted = eula.isAccepted
    if (isAccepted)
      logger<CwmEULA>().info("EULA is accepted")
    else
      logger<CwmEULA>().info("EULA is not accepted")
    return isAccepted
  }

  fun showEulaAcceptanceDialog(): Boolean {
    val agreementUi = AgreementUi.create(eula.text, exitOnCancel = false, useRtfPane = false)
    val dialog = agreementUi.apply {
      setTitle(CodeWithMePluginBundle.message("license.terms.title"))
      setDeclineButton(CodeWithMePluginBundle.message("userAgreement.dialog.decline")) {
        it.close(DialogWrapper.CANCEL_EXIT_CODE)
      }
      setAcceptButton(commonEulaBundle.getString("userAgreement.dialog.continue"), false) { dialogWrapper: DialogWrapper ->
        EndUserAgreement.setAccepted(eula)
        dialogWrapper.close(DialogWrapper.OK_EXIT_CODE)
      }
      addCheckBox(commonEulaBundle.getString("userAgreement.dialog.checkBox")) { checkBox ->
        this.enableAcceptButton(checkBox.isSelected)
        if (checkBox.isSelected) focusToAcceptButton()
      }
    }.pack()

    return dialog.showAndGet()
  }
}