package com.jetbrains.rdserver.internal

import com.intellij.codeInsight.hint.HintManager
import com.intellij.openapi.fileEditor.FileEditorManager
import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.DialogWrapperPeerFactory
import com.intellij.openapi.ui.messages.MessagesService
import com.intellij.openapi.ui.popup.JBPopupFactory
import com.intellij.ui.ColorChooserService
import com.intellij.usages.UsageViewManager
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rdserver.codeInsight.BackendHintManager
import com.jetbrains.rdserver.editors.BackendServerFileEditorManager
import com.jetbrains.rdserver.findUsages.BackendUsageViewManager
import com.jetbrains.rdserver.ui.BackendColorHost
import com.jetbrains.rdserver.ui.BackendDialogWrapperPeerFactory
import com.jetbrains.rdserver.ui.BackendMessagesService
import com.jetbrains.rdserver.ui.popup.bePopups.impl.BackendBePopupFactory

class RdServerOverridesVerifier(project: Project) {
  companion object {
    private val logger = getLogger<RdServerOverridesVerifier>()
  }

  init {
    if (application.isInternal) {
      assertComponentInstance<BackendUsageViewManager>(UsageViewManager.getInstance(project))
      assertComponentInstance<BackendServerFileEditorManager>(FileEditorManager.getInstance(project))
      assertComponentInstance<BackendBePopupFactory>(JBPopupFactory.getInstance())
      assertComponentInstance<BackendDialogWrapperPeerFactory>(DialogWrapperPeerFactory.getInstance())
      assertComponentInstance<BackendColorHost>(ColorChooserService.getInstance())
      assertComponentInstance<BackendMessagesService>(MessagesService.getInstance())
      assertComponentInstance<BackendHintManager>(HintManager.getInstance())
    }
  }

  private inline fun <reified T> assertComponentInstance(component: Any) {
    if (component !is T) {
      logger.error("Service/component ${component.javaClass} should be derived from ${T::class.java} in order to work with Live Share")
    }
  }
}

