package com.jetbrains.rdserver.internal

import com.intellij.icons.AllIcons
import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.editor.impl.EditorImpl
import com.jetbrains.rd.platform.util.application

class TypeCharsAction(private val times: Int = 100, private val char: Char = '(', private val delayMillis: Long = 300) : AnAction() {
  companion object {
    private const val typeCharsText = "Type chars"
    private const val cancelTypeCharsText = "Cancel typing"
  }

  @Volatile
  private var isRunning = false

  override fun update(e: AnActionEvent) {
    val presentation = e.presentation
    if (isRunning) {
      presentation.icon = AllIcons.Actions.StopRefresh
      presentation.text = cancelTypeCharsText
    }
    else {
      presentation.icon = AllIcons.RunConfigurations.TestState.Run
      presentation.text = typeCharsText
    }
  }

  override fun actionPerformed(e: AnActionEvent) {
    if (isRunning) {
      isRunning = false
      return
    }
    val editor = CommonDataKeys.EDITOR.getData(e.dataContext) ?: return
    val typeSession = TypeSession(editor as EditorImpl)
    Thread(typeSession::runLoop, "TypeCharsAction").start()
  }

  override fun displayTextInToolbar(): Boolean = true

  private inner class TypeSession(private val editor: EditorImpl) {
    fun runLoop() {
      isRunning = true
      try {
        repeat(times) {
          if (!isRunning || editor.project?.isDisposed == true)
            return

          application.invokeLater {
            editor.type(char.toString())
          }
          Thread.sleep(delayMillis)
        }
      }
      catch (e: InterruptedException) {

      }
      finally {
        isRunning = false
      }
    }
  }
}
