package com.jetbrains.rdserver.joinLinks

import com.intellij.openapi.actionSystem.AnAction
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.application.ApplicationInfo
import com.intellij.openapi.diagnostic.Logger
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.ui.Messages
import com.jetbrains.codeWithMe.lobby.SessionInfoRequest
import com.jetbrains.rd.platform.codeWithMe.CodeWithMeClientDownloadsManager
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.connection.LobbyServerApi
import com.jetbrains.rd.platform.codeWithMe.util.CodeWithMePlatformUtil
import com.jetbrains.rd.platform.codeWithMe.util.UrlUtil
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.handleWithUnwrapException
import com.jetbrains.rdserver.CodeWithMePluginBundle
import org.jetbrains.annotations.Nullable

class ConnectToRemoteFromHostInstallAction : AnAction(), DumbAware {

  override fun actionPerformed(e: AnActionEvent) {
    val url = Messages.showInputDialog(
      CodeWithMePluginBundle.message("message.enter.join.link"),
      CodeWithMePlatformBundle.message("product.name"),
      null) ?: return
    connectToUserProvidedUrl(url)
  }

  fun connectToUserProvidedUrl(url: @Nullable String) {
    connectToUserProvidedUrl(url) { }
  }

  private fun connectToUserProvidedUrl(url: @Nullable String, onDone: () -> Unit) {
    val uri = UrlUtil.parseOrShowError(url) ?: return

    application.executeOnPooledThread {
      try {
        val (sessionInfoResponse, _) = LobbyServerApi.sessionInfo(uri, SessionInfoRequest(
          clientBuildNumber = ApplicationInfo.getInstance().build.toString(),
          clientPlatform = CodeWithMePlatformUtil.currentPlatform()
        ))

        val downloadManager = CodeWithMeClientDownloadsManager.getInstance()
        val downloadFuture = downloadManager.downloadClientAndJdk(sessionInfoResponse)
        downloadFuture.handleWithUnwrapException { pair, throwable ->
          if (throwable != null) {
            // TODO: Report it to the user
            Logger.getInstance(javaClass).warn(throwable)

            application.invokeLater {
              Messages.showErrorDialog(
                CodeWithMePlatformBundle.message("error.url.issue", throwable.message ?: "Unknown"),
                CodeWithMePlatformBundle.message("product.name"))
            }

            return@handleWithUnwrapException
          }

          if (pair == null) return@handleWithUnwrapException
          // todo: offer to connect as-is?
          downloadManager.runCwmGuestProcessFromDownload(url, pair.first, pair.second)
          application.invokeLater {
            onDone()
          }
        }
      } catch (t: Throwable) {
        application.invokeLater {
          Messages.showErrorDialog(
            CodeWithMePlatformBundle.message("error.url.issue", t.message ?: "Unknown"),
            CodeWithMePlatformBundle.message("product.name"))
        }
      }
    }
  }
}