package com.jetbrains.rdserver.joinLinks

import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.actionSystem.Presentation
import com.intellij.openapi.application.ApplicationManager
import com.intellij.openapi.application.ModalityState
import com.intellij.openapi.application.impl.ApplicationInfoImpl
import com.intellij.ui.LicensingFacade
import com.jetbrains.codeWithMe.license.CertKey
import com.jetbrains.codeWithMe.license.CertificateDecoder
import java.nio.charset.StandardCharsets
import java.security.Signature
import java.security.cert.X509Certificate
import java.util.*
import kotlin.math.abs

@Suppress("NOTHING_TO_INLINE")
object LicenseStampCheck {
  @JvmStatic
  val CWM_PRODUCT_CODE = "PCWMP"

  @JvmStatic
  val ROOT_CERTIFICATE_1 =
    "-----BEGIN CERTIFICATE-----\n" +
      "MIIFOzCCAyOgAwIBAgIJANJssYOyg3nhMA0GCSqGSIb3DQEBCwUAMBgxFjAUBgNV\n" +
      "BAMMDUpldFByb2ZpbGUgQ0EwHhcNMTUxMDAyMTEwMDU2WhcNNDUxMDI0MTEwMDU2\n" +
      "WjAYMRYwFAYDVQQDDA1KZXRQcm9maWxlIENBMIICIjANBgkqhkiG9w0BAQEFAAOC\n" +
      "Ag8AMIICCgKCAgEA0tQuEA8784NabB1+T2XBhpB+2P1qjewHiSajAV8dfIeWJOYG\n" +
      "y+ShXiuedj8rL8VCdU+yH7Ux/6IvTcT3nwM/E/3rjJIgLnbZNerFm15Eez+XpWBl\n" +
      "m5fDBJhEGhPc89Y31GpTzW0vCLmhJ44XwvYPntWxYISUrqeR3zoUQrCEp1C6mXNX\n" +
      "EpqIGIVbJ6JVa/YI+pwbfuP51o0ZtF2rzvgfPzKtkpYQ7m7KgA8g8ktRXyNrz8bo\n" +
      "iwg7RRPeqs4uL/RK8d2KLpgLqcAB9WDpcEQzPWegbDrFO1F3z4UVNH6hrMfOLGVA\n" +
      "xoiQhNFhZj6RumBXlPS0rmCOCkUkWrDr3l6Z3spUVgoeea+QdX682j6t7JnakaOw\n" +
      "jzwY777SrZoi9mFFpLVhfb4haq4IWyKSHR3/0BlWXgcgI6w6LXm+V+ZgLVDON52F\n" +
      "LcxnfftaBJz2yclEwBohq38rYEpb+28+JBvHJYqcZRaldHYLjjmb8XXvf2MyFeXr\n" +
      "SopYkdzCvzmiEJAewrEbPUaTllogUQmnv7Rv9sZ9jfdJ/cEn8e7GSGjHIbnjV2ZM\n" +
      "Q9vTpWjvsT/cqatbxzdBo/iEg5i9yohOC9aBfpIHPXFw+fEj7VLvktxZY6qThYXR\n" +
      "Rus1WErPgxDzVpNp+4gXovAYOxsZak5oTV74ynv1aQ93HSndGkKUE/qA/JECAwEA\n" +
      "AaOBhzCBhDAdBgNVHQ4EFgQUo562SGdCEjZBvW3gubSgUouX8bMwSAYDVR0jBEEw\n" +
      "P4AUo562SGdCEjZBvW3gubSgUouX8bOhHKQaMBgxFjAUBgNVBAMMDUpldFByb2Zp\n" +
      "bGUgQ0GCCQDSbLGDsoN54TAMBgNVHRMEBTADAQH/MAsGA1UdDwQEAwIBBjANBgkq\n" +
      "hkiG9w0BAQsFAAOCAgEAjrPAZ4xC7sNiSSqh69s3KJD3Ti4etaxcrSnD7r9rJYpK\n" +
      "BMviCKZRKFbLv+iaF5JK5QWuWdlgA37ol7mLeoF7aIA9b60Ag2OpgRICRG79QY7o\n" +
      "uLviF/yRMqm6yno7NYkGLd61e5Huu+BfT459MWG9RVkG/DY0sGfkyTHJS5xrjBV6\n" +
      "hjLG0lf3orwqOlqSNRmhvn9sMzwAP3ILLM5VJC5jNF1zAk0jrqKz64vuA8PLJZlL\n" +
      "S9TZJIYwdesCGfnN2AETvzf3qxLcGTF038zKOHUMnjZuFW1ba/12fDK5GJ4i5y+n\n" +
      "fDWVZVUDYOPUixEZ1cwzmf9Tx3hR8tRjMWQmHixcNC8XEkVfztID5XeHtDeQ+uPk\n" +
      "X+jTDXbRb+77BP6n41briXhm57AwUI3TqqJFvoiFyx5JvVWG3ZqlVaeU/U9e0gxn\n" +
      "8qyR+ZA3BGbtUSDDs8LDnE67URzK+L+q0F2BC758lSPNB2qsJeQ63bYyzf0du3wB\n" +
      "/gb2+xJijAvscU3KgNpkxfGklvJD/oDUIqZQAnNcHe7QEf8iG2WqaMJIyXZlW3me\n" +
      "0rn+cgvxHPt6N4EBh5GgNZR4l0eaFEV+fxVsydOQYo1RIyFMXtafFBqQl6DDxujl\n" +
      "FeU3FZ+Bcp12t7dlM4E0/sS1XdL47CfGVj4Bp+/VbF862HmkAbd7shs7sDQkHbU=\n" +
      "-----END CERTIFICATE-----\n"

  @JvmStatic
  val ROOT_CERTIFICATE_2 =
    "-----BEGIN CERTIFICATE-----\n" +
      "MIIFTDCCAzSgAwIBAgIJAMCrW9HV+hjZMA0GCSqGSIb3DQEBCwUAMB0xGzAZBgNV\n" +
      "BAMMEkxpY2Vuc2UgU2VydmVycyBDQTAgFw0xNjEwMTIxNDMwNTRaGA8yMTE2MTIy\n" +
      "NzE0MzA1NFowHTEbMBkGA1UEAwwSTGljZW5zZSBTZXJ2ZXJzIENBMIICIjANBgkq\n" +
      "hkiG9w0BAQEFAAOCAg8AMIICCgKCAgEAoT7LvHj3JKK2pgc5f02z+xEiJDcvlBi6\n" +
      "fIwrg/504UaMx3xWXAE5CEPelFty+QPRJnTNnSxqKQQmg2s/5tMJpL9lzGwXaV7a\n" +
      "rrcsEDbzV4el5mIXUnk77Bm/QVv48s63iQqUjVmvjQt9SWG2J7+h6X3ICRvF1sQB\n" +
      "yeat/cO7tkpz1aXXbvbAws7/3dXLTgAZTAmBXWNEZHVUTcwSg2IziYxL8HRFOH0+\n" +
      "GMBhHqa0ySmF1UTnTV4atIXrvjpABsoUvGxw+qOO2qnwe6ENEFWFz1a7pryVOHXg\n" +
      "P+4JyPkI1hdAhAqT2kOKbTHvlXDMUaxAPlriOVw+vaIjIVlNHpBGhqTj1aqfJpLj\n" +
      "qfDFcuqQSI4O1W5tVPRNFrjr74nDwLDZnOF+oSy4E1/WhL85FfP3IeQAIHdswNMJ\n" +
      "y+RdkPZCfXzSUhBKRtiM+yjpIn5RBY+8z+9yeGocoxPf7l0or3YF4GUpud202zgy\n" +
      "Y3sJqEsZksB750M0hx+vMMC9GD5nkzm9BykJS25hZOSsRNhX9InPWYYIi6mFm8QA\n" +
      "2Dnv8wxAwt2tDNgqa0v/N8OxHglPcK/VO9kXrUBtwCIfZigO//N3hqzfRNbTv/ZO\n" +
      "k9lArqGtcu1hSa78U4fuu7lIHi+u5rgXbB6HMVT3g5GQ1L9xxT1xad76k2EGEi3F\n" +
      "9B+tSrvru70CAwEAAaOBjDCBiTAdBgNVHQ4EFgQUpsRiEz+uvh6TsQqurtwXMd4J\n" +
      "8VEwTQYDVR0jBEYwRIAUpsRiEz+uvh6TsQqurtwXMd4J8VGhIaQfMB0xGzAZBgNV\n" +
      "BAMMEkxpY2Vuc2UgU2VydmVycyBDQYIJAMCrW9HV+hjZMAwGA1UdEwQFMAMBAf8w\n" +
      "CwYDVR0PBAQDAgEGMA0GCSqGSIb3DQEBCwUAA4ICAQCJ9+GQWvBS3zsgPB+1PCVc\n" +
      "oG6FY87N6nb3ZgNTHrUMNYdo7FDeol2DSB4wh/6rsP9Z4FqVlpGkckB+QHCvqU+d\n" +
      "rYPe6QWHIb1kE8ftTnwapj/ZaBtF80NWUfYBER/9c6To5moW63O7q6cmKgaGk6zv\n" +
      "St2IhwNdTX0Q5cib9ytE4XROeVwPUn6RdU/+AVqSOspSMc1WQxkPVGRF7HPCoGhd\n" +
      "vqebbYhpahiMWfClEuv1I37gJaRtsoNpx3f/jleoC/vDvXjAznfO497YTf/GgSM2\n" +
      "LCnVtpPQQ2vQbOfTjaBYO2MpibQlYpbkbjkd5ZcO5U5PGrQpPFrWcylz7eUC3c05\n" +
      "UVeygGIthsA/0hMCioYz4UjWTgi9NQLbhVkfmVQ5lCVxTotyBzoubh3FBz+wq2Qt\n" +
      "iElsBrCMR7UwmIu79UYzmLGt3/gBdHxaImrT9SQ8uqzP5eit54LlGbvGekVdAL5l\n" +
      "DFwPcSB1IKauXZvi1DwFGPeemcSAndy+Uoqw5XGRqE6jBxS7XVI7/4BSMDDRBz1u\n" +
      "a+JMGZXS8yyYT+7HdsybfsZLvkVmc9zVSDI7/MjVPdk6h0sLn+vuPC1bIi5edoNy\n" +
      "PdiG2uPH5eDO6INcisyPpLS4yFKliaO4Jjap7yzLU9pbItoWgCAYa2NpxuxHJ0tB\n" +
      "7tlDFnvaRnQukqSG+VqNWg==\n" +
      "-----END CERTIFICATE-----"

  @JvmStatic private val SECOND = 1000L
  @JvmStatic private val MINUTE = 60 * SECOND
  @JvmStatic private val HOUR = 60 * MINUTE
  @JvmStatic val TIMESTAMP_VALIDITY_PERIOD_MS = 48 * HOUR // match standard FLS absence tolerance period

  @JvmStatic val KEY_PREFIX = "key:"
  @JvmStatic val STAMP_PREFIX = "stamp:"
  @JvmStatic val EVAL_PREFIX = "eval:"

  inline fun hasLicenseStamp(productCode: String): Boolean = LicensingFacade.getInstance()?.getConfirmationStamp(productCode) != null
  
  inline fun isLicensed(productCode: String): Boolean = runCatchingOrFalse {
    val cstamp = LicensingFacade.getInstance()?.getConfirmationStamp(productCode) ?: return false
    when {
      cstamp.startsWith(KEY_PREFIX) -> isKeyValid(cstamp.substring(KEY_PREFIX.length), productCode)
      cstamp.startsWith(STAMP_PREFIX) -> isLicenseServerStampValid(cstamp.substring(STAMP_PREFIX.length))
      cstamp.startsWith(EVAL_PREFIX) -> ApplicationInfoImpl.getShadowInstance().isEAP && isEvaluationValid(cstamp.substring(EVAL_PREFIX.length))
      else -> false
    }
  }

  inline fun isKeyValid(key: String, productCode: String) = runCatchingOrFalse {
    return !CertKey(key).isExpired(productCode)
  }

  inline fun isEvaluationValid(expirationTime: String): Boolean = runCatchingOrFalse {
    Date().before(Date(expirationTime.toLong()))
  }

  inline fun isLicenseServerStampValid(serverStamp: String) = runCatchingOrFalse {
    val parts = serverStamp.split(":")
    val base64 = Base64.getMimeDecoder()
    val expectedMachineId = parts[0]
    val timeStamp = parts[1].toLong()
    val machineId = parts[2]
    val signatureType = parts[3]
    val signatureBytes = base64.decode(parts[4].toByteArray(StandardCharsets.UTF_8))
    val certBytes = base64.decode(parts[5].toByteArray(StandardCharsets.UTF_8))
    val intermediate: MutableCollection<ByteArray> = ArrayList()
    for (idx in 6 until parts.size) {
      intermediate.add(base64.decode(parts[idx].toByteArray(StandardCharsets.UTF_8)))
    }
    val sig = Signature.getInstance(signatureType)

    sig.initVerify(createCertificate(certBytes, intermediate, true))
    sig.update("$timeStamp:$machineId".toByteArray(StandardCharsets.UTF_8))
    if (sig.verify(signatureBytes)) {
      expectedMachineId == machineId && abs(System.currentTimeMillis() - timeStamp) < TIMESTAMP_VALIDITY_PERIOD_MS
    }
    else false
  }

  inline fun createCertificate(certBytes: ByteArray, intermediateCertsBytes: Collection<ByteArray>, checkValidityAtCurrentDate: Boolean): X509Certificate {
    return CertificateDecoder(listOf(ROOT_CERTIFICATE_1.toByteArray(StandardCharsets.UTF_8), ROOT_CERTIFICATE_2.toByteArray(StandardCharsets.UTF_8))).createCertificate(certBytes, intermediateCertsBytes, checkValidityAtCurrentDate).certificate
  }

  fun requestLicense(message: String?) {
    // ensure the dialog appears from UI thread and in a non-modal context
    ApplicationManager.getApplication()?.invokeLater({ showRegisterDialog(CWM_PRODUCT_CODE, message) }, ModalityState.NON_MODAL)
  }

   private fun showRegisterDialog(productCode : String, message: String?) {
     val actionManager = com.intellij.openapi.actionSystem.ActionManager.getInstance()
     if (actionManager != null) {
       // first, assume we are running inside the opensource version, otherwise assume running inside commercial IDE distribution
       (actionManager.getAction("RegisterPlugins") ?: actionManager.getAction("Register"))?.actionPerformed(AnActionEvent.createFromDataContext("", Presentation(), asDataContext(productCode, message)))
     }
   }

  private fun asDataContext(productCode: String, message : String?) : DataContext {
    return DataContext {
      when (it) {
        "register.product-descriptor.code" -> return@DataContext productCode // the same code as registered in plugin.xml, 'product-descriptor' tag
        "register.message" -> return@DataContext message                     // optional message to be shown in the registration dialog that appears
        else -> return@DataContext null
      }
    }
  }

  inline fun runCatchingOrFalse(action: () -> Boolean) = runCatching(action).getOrElse { false }
}
