package com.jetbrains.rdserver.joinLinks.permanent

import com.intellij.codeWithMe.ClientId
import com.intellij.core.CoreBundle
import com.intellij.ide.util.PropertiesComponent
import com.intellij.notification.Notification
import com.intellij.notification.NotificationType
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.ide.CopyPasteManager
import com.intellij.openapi.project.DumbAware
import com.intellij.openapi.project.DumbAwareAction
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.openapi.ui.MessageDialogBuilder
import com.intellij.util.ui.TextTransferable
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.CwmNotifications.CWM_NOTIFICATION_GROUP_ID
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.util.catch
import com.jetbrains.rdserver.CodeWithMePluginBundle

class GetPermanentJoinLinkAction : DumbAwareAction(), DumbAware {
  companion object {
    private const val dontShowSecurityWarningPref = "cwm.hide.permanent.link.warning.1"
  }

  override fun update(e: AnActionEvent) {
    val project = e.project
    e.presentation.isEnabledAndVisible = project != null && application.isInternal
    if (project != null)
      CwmPermanentLinkManager.getInstance(project) // just create the instance
  }

  override fun actionPerformed(e: AnActionEvent) {
    val project = e.project ?: return

    if (!PropertiesComponent.getInstance().getBoolean(dontShowSecurityWarningPref, false)) {
      val builder = MessageDialogBuilder.yesNo(CodeWithMePlatformBundle.message("product.name"), CodeWithMePluginBundle.message("message.permanent.link.security.warning"))
        .yesText(CodeWithMePluginBundle.message("button.continue.anyway"))
        .noText(CoreBundle.message("button.cancel"))
        .doNotAsk(object : DialogWrapper.DoNotAskOption.Adapter() {
          override fun rememberChoice(isSelected: Boolean, exitCode: Int) {
            if (exitCode != 0) return
            PropertiesComponent.getInstance().setValue(dontShowSecurityWarningPref, isSelected)
          }
        })
        .asWarning()

      if (!builder.ask(project)) return
    }

    val futureLink = CwmPermanentLinkManager.getInstance(project).enableAndGetPermanentLink()

    futureLink.whenCompleteAsync { uri, exception ->
      ClientId.withClientId(ClientId.localId) {
        if (exception == null) {
          if (uri == null) error("wrong state, both uri and exception are null")

          fun copyLink() = CopyPasteManager.getInstance().setContents(TextTransferable(uri.toString() as String?))

          catch { copyLink() }

          Notification(CWM_NOTIFICATION_GROUP_ID,
                       CodeWithMePluginBundle.message("action.GetPermanentCwmJoinLink.notification.title"),
                       uri.toString(),
                       NotificationType.INFORMATION)
            .addAction(object : DumbAwareAction(CodeWithMePluginBundle.message("action.copy.link.text")) {
              override fun actionPerformed(e: AnActionEvent) {
                copyLink()
              }
            })
            .notify(project)
        }
        else {
          // TODO More friendly?
          // TODO Logs link?
          Notification(CWM_NOTIFICATION_GROUP_ID,
                       CodeWithMePluginBundle.message("action.GetPermanentCwmJoinLink.notification.title"),
                       exception.message + "<br>See logs for more information",
                       NotificationType.ERROR)
            .notify(project)
        }
      }
    }
  }
}