package com.jetbrains.rdserver.navigation

import com.intellij.codeWithMe.ClientId
import com.intellij.codeWithMe.ClientId.Companion.isLocal
import com.intellij.ide.projectView.impl.ProjectViewPane
import com.intellij.ide.util.PsiNavigationSupportImpl
import com.intellij.psi.PsiDirectory
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rdserver.projectView.BackendProjectViewHost
import java.io.File

class BackendPsiNavigationSupport: PsiNavigationSupportImpl() {

  companion object {
    private val logger = getLogger<BackendPsiNavigationSupport>()
  }

  override fun navigateToDirectory(psiDirectory: PsiDirectory, requestFocus: Boolean) {
    val clientId = ClientId.currentOrNull
    if (clientId == null || clientId.isLocal) {
      super.navigateToDirectory(psiDirectory, requestFocus)
    } else {
      val host = BackendProjectViewHost.getInstance(psiDirectory.project, clientId)
      host.select(ProjectViewPane.ID, psiDirectory, null, requestFocus)
    }
  }

  override fun openDirectoryInSystemFileManager(file: File) {
    val clientId = ClientId.currentOrNull
    if (clientId == null || clientId.isLocal) {
      super.openDirectoryInSystemFileManager(file)
    }
    else {
      logger.warn("Ignore BackendPsiNavigationSupport.openDirectoryInSystemFileManager call for file '$file' from client $clientId")
    }
  }
}