package com.jetbrains.rdserver.permissions

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.editor.Document
import com.intellij.openapi.project.Project
import com.intellij.openapi.vfs.VirtualFile

/**
 * Providers core API for different permissions granted to clients
 */
interface PermissionManager {

  companion object {
    const val NOTIFICATION_GROUP_ID = "CWM PermissionManager"

    fun getInstance(project: Project): PermissionManager = project.getService(PermissionManager::class.java)
  }

  /**
   * Returns true if client can modify anything in the project
   */
  fun hasFileWriteAccess(client: ClientId): Boolean

  /**
   * Returns true if client can view existence of virtual file and read its content
   */
  fun isVisible(client: ClientId, virtualFile: VirtualFile): Boolean

  fun isVisible(client: ClientId, document: Document): Boolean

  /**
   * Returns true if client can modify the particular file
   */
  fun isWritable(client: ClientId, virtualFile: VirtualFile): Boolean

  fun isWritable(client: ClientId, document: Document): Boolean

  fun getTerminalAccess(client: ClientId): TerminalAccessLevel
  fun getRunAccess(client: ClientId): RunAccessLevel
  fun getToolWindowAccess(client: ClientId): ToolWindowAccessLevel
}

enum class FileAccessLevel {
  Readonly,
  FullAccess
}

enum class TerminalAccessLevel {
  Disabled,
  Readonly,
  Request,
  FullAccess
}

enum class RunAccessLevel {
  Disabled,
  Readonly,
  FullAccess
}

enum class ToolWindowAccessLevel {
  Disabled,
  Readonly,
  FullAccess
}