package com.jetbrains.rdserver.permissions.impl

import com.intellij.codeWithMe.ClientId
import com.intellij.notification.Notification
import com.intellij.notification.NotificationType
import com.intellij.openapi.project.Project
import com.intellij.openapi.vfs.LocalFileOperationsHandler
import com.intellij.openapi.vfs.LocalFileSystem
import com.intellij.openapi.vfs.VirtualFile
import com.intellij.util.ThrowableConsumer
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rdserver.permissions.PermissionManager
import com.jetbrains.rdserver.ui.BackendNotificationsHost
import java.io.File
import java.io.IOException

class BackendFileOperationsHandler(private val project: Project) : LocalFileOperationsHandler {

  init {
    // TODO: find a way to register this handler before others (VCS's)
    LocalFileSystem.getInstance().registerAuxiliaryFileOperationsHandler(this)
    project.lifetime.onTermination {
      LocalFileSystem.getInstance().unregisterAuxiliaryFileOperationsHandler(this)
    }
  }

  override fun delete(file: VirtualFile) = assertAccess { "Delete" }
  override fun move(file: VirtualFile, toDir: VirtualFile) = assertAccess { "Move" }
  override fun rename(file: VirtualFile, newName: String) = assertAccess { "Rename" }
  override fun createFile(dir: VirtualFile, name: String) = assertAccess { "Create File" }
  override fun createDirectory(dir: VirtualFile, name: String) = assertAccess { "Create Directory" }

  override fun copy(file: VirtualFile, toDir: VirtualFile, copyName: String): File? {
    assertAccess { "Copy" }
    return null
  }

  override fun afterDone(invoker: ThrowableConsumer<LocalFileOperationsHandler, IOException>) = Unit

  private fun assertAccess(actionPresentableName: () -> String) : Boolean {
    if (ClientId.isCurrentlyUnderLocalId) return false

    if (!PermissionManager.getInstance(project).hasFileWriteAccess(ClientId.current)) {
      BackendNotificationsHost.getInstance(project).notifyClient(Notification(
        PermissionManager.NOTIFICATION_GROUP_ID, "Can not execute ${actionPresentableName()} action",
        "You do not have write permissions.",
        NotificationType.INFORMATION))
      throw Exception("Write access denied")
    }
    return false
  }

}