package com.jetbrains.rdserver.permissions.impl

import com.intellij.codeWithMe.ClientId
import com.intellij.codeWithMe.ClientId.Companion.isLocal
import com.intellij.openapi.components.service
import com.intellij.openapi.project.Project
import com.jetbrains.codeWithMe.model.UserPermissionModel
import com.jetbrains.rd.framework.IProtocol
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.followMe.FollowMeManager
import com.jetbrains.rdserver.permissions.FileAccessLevel
import com.jetbrains.rdserver.permissions.RunAccessLevel

@Suppress("NonDefaultConstructor")
class BackendPermissionHost(
  lifetime: Lifetime,
  project: Project,
  protocol: IProtocol
) {

  companion object {
    fun update(project: Project, client: ClientId?, model: PermissionModel) {
      if (client != null) {
        updateFor(project, client, model)
      }
      else {
        for (clientId in FollowMeManager.getInstance(project).users.keys) {
          updateFor(project, clientId, model)
        }
      }
    }

    private fun updateFor(project: Project, client: ClientId, model: PermissionModel) {
      if (client.isLocal) return
      RdServerPerClientComponents.getService(project, client, BackendPermissionHost::class.java).update(model)
    }
  }

  private val userPermissionModel = UserPermissionModel.create(lifetime, protocol)

  init {
    val model = project.service<PermissionStore>().model
    update(model)
  }

  private fun update(permissionModel: PermissionModel) {
    userPermissionModel.hasWriteAccess.set(permissionModel.fileAccess == FileAccessLevel.FullAccess)
    userPermissionModel.hasRunAccess.set(permissionModel.runAccess == RunAccessLevel.FullAccess)
  }
}