package com.jetbrains.rdserver.permissions.impl

import com.intellij.openapi.project.Project
import com.intellij.project.stateStore
import com.jetbrains.rdserver.permissions.FileAccessLevel
import com.jetbrains.rdserver.permissions.RunAccessLevel
import com.jetbrains.rdserver.permissions.TerminalAccessLevel
import com.jetbrains.rdserver.permissions.ToolWindowAccessLevel
import java.nio.file.Path

interface PermissionModel {
  val excludedFiles: List<Path>
  val fileAccess: FileAccessLevel
  val terminalAccess: TerminalAccessLevel
  val runAccess: RunAccessLevel
  val toolWindowAccess: ToolWindowAccessLevel

  companion object {

    val defaultFileAccessLevel = FileAccessLevel.Readonly
    val defaultTerminalAccessLevel = TerminalAccessLevel.Readonly
    val defaultRunAccessLevel = RunAccessLevel.Readonly
    val defaultToolWindowAccessLevel = ToolWindowAccessLevel.Disabled

    fun createDefault(project: Project): PermissionModel {
      val storePath = project.stateStore.directoryStorePath
      val excludedFiles = if (storePath == null) mutableListOf() else mutableListOf(storePath)
      return MutablePermissionModel(excludedFiles, defaultFileAccessLevel, defaultTerminalAccessLevel, defaultRunAccessLevel,
                                    defaultToolWindowAccessLevel)
    }
  }
}