package com.jetbrains.rdserver.permissions.ui

import com.intellij.openapi.fileChooser.FileChooser
import com.intellij.openapi.fileChooser.FileChooserDescriptor
import com.intellij.openapi.project.Project
import com.intellij.openapi.project.guessProjectDir
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.ui.AddDeleteListPanel
import com.jetbrains.rdserver.CodeWithMePluginBundle
import java.nio.file.Path
import javax.swing.JComponent

class PermissionsFilesDialog(project: Project, val hiddenFiles: MutableList<Path>) : DialogWrapper(project, true) {

  private val hiddenFilesList = object : AddDeleteListPanel<Path>("", hiddenFiles) {
    override fun findItemToAdd(): Path? {
      val descriptor = FileChooserDescriptor(
        true,
        true,
        false,
        false,
        false,
        false)
        .withHideIgnored(false)
      return FileChooser.chooseFile(descriptor, this, project, project.guessProjectDir())?.toNioPath()
    }
  }

  init {
    title = CodeWithMePluginBundle.message("permission.dialog.hidden.files")
    init()
  }

  override fun createCenterPanel(): JComponent {
    return hiddenFilesList
  }

  override fun doOKAction() {
    hiddenFiles.clear()
    hiddenFiles.addAll(hiddenFilesList.listItems.filterIsInstance<Path>())
    super.doOKAction()
  }
}