package com.jetbrains.rdserver.projectView

import com.intellij.ide.projectView.ProjectView
import com.intellij.ide.projectView.impl.*
import com.intellij.openapi.project.Project
import com.intellij.openapi.rd.createNestedDisposable
import com.intellij.openapi.vfs.VirtualFile
import com.intellij.psi.PsiManager
import com.intellij.ui.tree.AsyncTreeModel
import com.intellij.ui.tree.TreeVisitor
import com.jetbrains.rd.util.lifetime.Lifetime

class BackendProjectViewPaneProvider(private val project: Project) : BackendProjectPaneProvider {

  private val projectView = ProjectView.getInstance(project) as ProjectViewImpl

  override fun isAvailable(): Boolean {
    return projectView.getProjectViewPaneById(ProjectViewPane.ID) != null
  }

  override fun getProjectViewPane(): AbstractProjectViewPane {
    return projectView.getProjectViewPaneById(ProjectViewPane.ID)
  }

  override fun createAsyncTreeModel(lifetime: Lifetime): AsyncTreeModel {
    val comparator = GroupByTypeComparator(project, ProjectViewPane.ID)
    val pane = projectView.getProjectViewPaneById(ProjectViewPane.ID) as AbstractProjectViewPSIPane
    val asyncSupport = pane.createAsyncSupport(lifetime.createNestedDisposable(), comparator)
    return asyncSupport.treeModel
  }

  override fun createSelectVisitor(file: VirtualFile): TreeVisitor? {
    val element = PsiManager.getInstance(project).findFile(file)
    return AbstractProjectViewPane.createVisitor(element, file)
  }
}