package com.jetbrains.rdserver.projectView

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rdserver.followMe.BackendUserFocusHost
import com.jetbrains.rdserver.followMe.BackendUserFocusManager
import com.jetbrains.rdserver.followMe.FocusChangeSource
import com.jetbrains.rdserver.followMe.FollowMeManager

/**
 * Separate component to sync project view selection on following started
 */
class BackendProjectViewSync(project: Project) {

  private val ID = "Project"

  init {
    val userFocusManager = BackendUserFocusManager.getBackendInstance(project)
    userFocusManager.changeSignal.advise(project.lifetime) { event ->
      if (event.source == FocusChangeSource.FollowingStarted) {
        val leadId = event.leadId

        if (leadId == ClientId.localId) {
          // Someone follows the host
          if (userFocusManager.currentActiveToolWindowId.value == ID) {
            syncProjectViewSelection(project) { it.firePaneSelectionChanged(true) }
          }
        }
        else {
          // Someone follows a client
          if (BackendUserFocusHost.getInstance(project, leadId).getCurrentActiveToolWindow() == ID) {
            syncProjectViewSelection(project) { it.fireModelSelectionChanged(true) }
          }
        }
      }
    }
  }

  private fun syncProjectViewSelection(project: Project, action: (BackendProjectViewHost) -> Unit) {
    for ((userId, _) in FollowMeManager.getInstance(project).users) {
      if (userId == ClientId.localId) continue
      action(BackendProjectViewHost.getInstance(project, userId))
    }
  }
}