package com.jetbrains.rdserver.startup

import com.intellij.ide.IdeEventQueue
import com.jetbrains.rd.platform.protocol.IRdDispatcher
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.IScheduler
import com.jetbrains.rd.util.threading.SingleThreadScheduler
import javax.swing.SwingUtilities

object EdtScheduler : IRdDispatcher {
  override fun flush() {
    IdeEventQueue.getInstance().flushQueue()
  }

  override fun pumpProtocolQueue(condition: () -> Boolean) = throw UnsupportedOperationException()

  override val backgroundScheduler: IScheduler = SingleThreadScheduler(
    application.lifetime, "BackgroundScheduler")

  override fun suppressPumping(lifetime: Lifetime) = throw UnsupportedOperationException()

  override fun repost() = throw UnsupportedOperationException()

  override fun allowBackgroundThreadAndSuppressPumping(lifetime: Lifetime) = throw UnsupportedOperationException()

  override val isActive: Boolean
    get() = SwingUtilities.isEventDispatchThread()

  private var _isBeingProcessed = false
  val IsBeingProcessed
    get() = _isBeingProcessed

  override fun queue(action: () -> Unit) {
    application.invokeLater {
      try {
        _isBeingProcessed = true
        action.invoke()
      }
      finally {
        _isBeingProcessed = false
      }
    }
  }
}