// Copyright 2000-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package com.jetbrains.rdserver.startup

import com.intellij.openapi.diagnostic.Logger
import java.io.File

private val LOG = Logger.getInstance("com.jetbrains.rdserver.startup")

fun generateIdeaProject(solutionDirectory: String, virtualSolution: Boolean): String {
  LOG.info("Requested IDEA project in directory $solutionDirectory, virtual solution=$virtualSolution")
  val solutionFile = File(solutionDirectory)
  if (virtualSolution) {
    val dotIdea = File(solutionFile, ".idea")
    if (dotIdea.exists()) {
      LOG.info("Found existing .idea in $solutionFile")
      return solutionFile.absolutePath
    }
  }
  val ideaBackendDir = File(solutionFile, ".idea/.idea-backend")
  val imlFileName = solutionFile.nameWithoutExtension + ".iml"
  val imlPath = File(ideaBackendDir, imlFileName)
  val dotIdea = File(ideaBackendDir, ".idea")
  val modulesXmlPath = File(dotIdea, "modules.xml")
  if (!imlPath.exists() || !modulesXmlPath.exists()) {
    LOG.info("Generating new .idea in $ideaBackendDir")
    ideaBackendDir.mkdirs()
    generateImlFile(imlPath, solutionDirectory)
    dotIdea.mkdirs()
    generateModulesXml(modulesXmlPath, imlFileName)
  }
  else {
    LOG.info("Found existing .idea in $ideaBackendDir")
  }

  return ideaBackendDir.absolutePath
}

private fun generateImlFile(file: File, contentRoot: String) {
  val content = """<?xml version="1.0" encoding="UTF-8"?>
<module type="JAVA_MODULE" version="4">
  <component name="NewModuleRootManager" inherit-compiler-output="true">
    <exclude-output />
    <content url="file:///$contentRoot">
      <sourceFolder url="file:///$contentRoot" isTestSource="false" />
    </content>
    <orderEntry type="inheritedJdk" />
    <orderEntry type="sourceFolder" forTests="false" />
  </component>
</module>"""
  file.writeText(content)
}

private fun generateModulesXml(file: File, imlFile: String) {
  val content = """<?xml version="1.0" encoding="UTF-8"?>
<project version="4">
  <component name="ProjectModuleManager">
    <modules>
      <module fileurl="file://${'$'}PROJECT_DIR${'$'}/$imlFile" filepath="${'$'}PROJECT_DIR${'$'}/$imlFile" />
    </modules>
  </component>
</project>"""
  file.writeText(content)
}
