package com.jetbrains.rdserver.startup

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.util.application
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rd.util.addUnique
import com.jetbrains.rd.util.lifetime.Lifetime

class RdServerApplicationService {

  companion object {
    fun getInstance() = application.getService(RdServerApplicationService::class.java)
  }

  private val clientToProjectMap = hashMapOf<ClientId, Project>()

  fun registerClient(lifetime: Lifetime, clientId: ClientId, project: Project) {
    synchronized(clientToProjectMap) {
      if (clientToProjectMap.containsKey(clientId)) {
        getLogger<RdServerApplicationService>().warn("Duplicate client ID: $clientId")
        return@synchronized
      }

      clientToProjectMap.addUnique(lifetime, clientId, project)
    }
  }

  fun getProjectForClient(clientId: ClientId): Project? {
    return synchronized(clientToProjectMap) { clientToProjectMap[clientId] }
  }
}