package com.jetbrains.rdserver.startup

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.openapi.util.Key
import com.jetbrains.rd.platform.util.getOrCreateUserData
import com.jetbrains.rd.util.addUnique
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.lifetime.onTermination
import com.jetbrains.rdserver.startup.RdServerPerClientSolutions.SOLUTION_KEY
import com.jetbrains.rd.ide.model.Solution
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.ConcurrentMap

val Project.serverSolution: Solution
  get() = serverSolutionFor(ClientId.current)

fun Project.serverSolutionFor(clientId: ClientId): Solution {
  val solutionMap = getUserData(SOLUTION_KEY) ?: error("No solution map in project $this")
  return solutionMap[clientId]
         ?: error("No solution for client $clientId")
}

fun Project.serverSolutionForOrNull(clientId: ClientId): Solution? {
  return getUserData(SOLUTION_KEY)?.get(clientId)
}

fun ClientId.tryGetProject(): Project? {
  return RdServerPerClientSolutions.clientIdToProjectMap[this]
}

object RdServerPerClientSolutions {
  val SOLUTION_KEY = Key.create<ConcurrentMap<ClientId, Solution>>("com.jetbrains.rdserver.solution")
  val clientIdToProjectMap = hashMapOf<ClientId, Project>()

  fun putPerClientIdSolution(
    project: Project, solutionModelLifetime: Lifetime,
    clientId: ClientId,
    solution: Solution
  ) {
    val solutionMap = project.getOrCreateUserData(SOLUTION_KEY) { ConcurrentHashMap() }
    solutionMap.addUnique(solutionModelLifetime, clientId, solution)

    clientIdToProjectMap[clientId] = project
    solutionModelLifetime.onTermination {
      clientIdToProjectMap.remove(clientId)
    }
  }
}