package com.jetbrains.rdserver.telephony

import com.intellij.ide.util.PropertiesComponent
import com.intellij.openapi.project.Project
import com.jetbrains.rd.platform.codeWithMe.telephony.TelephonyManager
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rd.util.reactive.Property
import com.jetbrains.rdserver.joinLinks.RdServerJoinLinkManager

class BackendTelephonyManager(project: Project) : TelephonyManager(project) {


  companion object {
    fun getInstance(project: Project) = TelephonyManager.getInstance(project) as BackendTelephonyManager

    private const val enabledByDefaultKey = "codeWithMe.voiceChat.enabledByDefault"
  }

  val isTelephonyEnabled = Property(
    PropertiesComponent.getInstance(project).getBoolean(enabledByDefaultKey, true)
  ).apply {
    advise(project.lifetime) {
      PropertiesComponent.getInstance(project).setValue(enabledByDefaultKey, it, true)
    }
  }

  override fun getJitsiRoomUrl(): String? {
    val link = RdServerJoinLinkManager.getInstance(project).getExistingJoinLink() ?: return null
    return RdServerJoinLinkManager.getInstance(project).getJoinLinkData(link)?.telephonyUrl
  }
}