package com.jetbrains.rdserver.tests

import com.intellij.codeWithMe.ClientId
import com.intellij.execution.Location
import com.intellij.execution.testframework.TestsUIUtil
import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.actionSystem.DataProvider
import com.intellij.openapi.project.Project
import com.intellij.psi.search.GlobalSearchScope
import com.jetbrains.codeWithMe.model.TestSelectionModel
import com.jetbrains.rd.ide.model.ActionTimestampSetModel
import com.jetbrains.rdserver.actions.*

class BackendTestsViewTimestampProvider(project: Project) : BackendActionTimestampProvider(project) {
  override fun createSet(actionId: String, dataContext: DataContext): ActionTimestampSet {
    return ActionTimestampSet.EMPTY
  }

  override fun createSet(actionId: String, setModel: ActionTimestampSetModel): ActionTimestampSet {
    val timestampModel = setModel.getTimestamp(TestsViewTimestamp.NAME) ?: return ActionTimestampSet.EMPTY
    return ActionTimestampSet(TestsViewTimestamp(timestampModel.executionId, timestampModel.selectedLocation))
  }
}

class TestsViewTimestamp(executionId: Long, locationUrl: String?)
  : ActionTimestamp<TestSelectionModel>(NAME, TestSelectionModel(executionId, locationUrl)) {
  companion object {
    val NAME = ActionTimestampKey<TestsViewTimestamp, TestSelectionModel>(BackendDataKeys.TEST_SELECTION.name)
  }

  override fun getData(): TestSelection {
    return TestSelection(model.executionId, model.selectedLocation)
  }
}

data class TestSelection(val executionId: Long, val selectedLocation: String?)

class BackendTestsDataProvider : BackendActionDataProvider {
  override fun getData(dataId: String, dataProvider: DataProvider): Any? {
    val project = CommonDataKeys.PROJECT.getData(dataProvider) ?: return null
    val tests = BackendDataKeys.TEST_SELECTION.getData(dataProvider) ?: return null
    val selectedLocation = tests.selectedLocation
    if (selectedLocation == null) return null
    fun getProxy(project: Project, tests: TestSelection) = BackendTestsContentHost.getInstance(project, ClientId.current)
                                                                                  .getProxy(tests.executionId, selectedLocation)
    if (CommonDataKeys.NAVIGATABLE_ARRAY.`is`(dataId)) {
      val proxy = getProxy(project, tests) ?: return null
      return arrayOf(TestsUIUtil.getOpenFileDescriptor(proxy, proxy.root.testConsoleProperties))
    }
    if (Location.DATA_KEY.`is`(dataId)) {
      val proxy = getProxy(project, tests) ?: return null
      return proxy.getLocation(project, GlobalSearchScope.allScope(project))
    }
    return null
  }

  
}