package com.jetbrains.rdserver.toolWindow

import com.intellij.openapi.application.invokeLater
import com.intellij.openapi.project.Project
import com.intellij.openapi.wm.ToolWindowManager
import com.intellij.openapi.wm.ex.ToolWindowManagerListener
import com.intellij.ui.viewModel.extraction.ToolWindowExtractorMode
import com.jetbrains.ide.model.uiautomation.BeToolWindowState
import com.jetbrains.rd.platform.codeWithMe.toolWindow.ToolwindowMirrorModeTracker
import com.jetbrains.rd.platform.codeWithMe.users.UserManager
import com.jetbrains.rd.platform.codeWithMe.users.clientId
import com.jetbrains.rd.platform.util.lifetime
import com.jetbrains.rdserver.permissions.PermissionManager
import com.jetbrains.rdserver.permissions.ToolWindowAccessLevel
import com.jetbrains.rdserver.permissions.impl.PermissionManagerImpl

class BackendToolwindowMirrorModeTracker(project: Project) : ToolwindowMirrorModeTracker(project) {
  init {
    project.messageBus.connect().subscribe(ToolWindowManagerListener.TOPIC, object : ToolWindowManagerListener {
      override fun stateChanged(toolWindowManager: ToolWindowManager) {
        invokeLater { updateMirroredWindows() }
      }
    })

    UserManager.getInstance(project).users.change.advise(project.lifetime) {
      invokeLater { updateMirroredWindows() }
    }
    PermissionManagerImpl.getInstanceImpl(project).permissionModelChanged.advise(project.lifetime) {
      invokeLater { updateMirroredWindows() }
    }
    updateMirroredWindows()
  }

  override fun updateMirroredWindows() {
    if (project.isDisposed) return
    val windowManager = ToolWindowManager.getInstance(project)

    for (toolWindowId in windowManager.toolWindowIds) {
      if (getBackendToolWindowMode(toolWindowId) != ToolWindowExtractorMode.MIRROR) continue

      var openCounter = 0

      val hostWindow = windowManager.getToolWindow(toolWindowId)
      if (hostWindow != null && hostWindow.isVisible) openCounter++

      UserManager.getInstance(project).users.forEach {
        val windowHost = BackendToolWindowHost.getInstance(project, it.clientId)
        val clientWindowState = windowHost.getToolWindowState(toolWindowId)
        val access = PermissionManager.getInstance(project).getToolWindowAccess(it.clientId)
        if (clientWindowState != BeToolWindowState.Hidden && access != ToolWindowAccessLevel.Disabled) openCounter++
      }
      val isSharedAccess = openCounter > 1

      UserManager.getInstance(project).users.forEach {
        BackendToolWindowHost.getInstance(project, it.clientId).setSharedAccess(toolWindowId, isSharedAccess)
      }

      if (hostWindow != null) updateWindowNotification(hostWindow, isSharedAccess)
    }
  }
}
