package com.jetbrains.rdserver.ui

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.project.Project
import com.intellij.openapi.rd.createLifetime
import com.intellij.ui.ColorChooserServiceImpl
import com.intellij.ui.ColorPickerListener
import com.intellij.util.concurrency.FutureResult
import com.jetbrains.rdserver.startup.serverSolution
import com.jetbrains.rdserver.toColor
import com.jetbrains.rdserver.toRgbaColor
import com.jetbrains.rd.ide.model.ColorChooserParam
import com.jetbrains.rd.ide.model.ColorChooserSession
import com.jetbrains.rd.ide.model.colorChooserModel
import java.awt.Color
import java.awt.Component

class BackendColorHost : ColorChooserServiceImpl() {
  override fun showDialog(project: Project?,
                          parent: Component?,
                          caption: String?,
                          preselectedColor: Color?,
                          enableOpacity: Boolean,
                          listeners: MutableList<out ColorPickerListener>,
                          opacityInPercent: Boolean): Color? {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.showDialog(project, parent, caption, preselectedColor, enableOpacity, listeners, opacityInPercent)

    if (project == null) {
      logger<BackendColorHost>().error("Trying to choose color for null project")
      return null
    }

    val session = ColorChooserSession(ColorChooserParam(caption, preselectedColor.toRgbaColor(), enableOpacity, opacityInPercent))
    val lifetime = project.createLifetime()

    session.colorChanged.advise(lifetime) { rgbaColor ->
      val color = rgbaColor.toColor()
      listeners?.forEach {
        it.colorChanged(color)
      }
    }
    val result: FutureResult<Color> = FutureResult()
    session.closed.advise(lifetime) { rgbaColor ->
      val color = rgbaColor.toColor()
      listeners?.forEach {
        it.closed(color)
      }
      result.set(color)
    }
    project.serverSolution.colorChooserModel.colorChooserSessions.add(session)
    while (!result.isDone) { // TODO: proper pumping here, or even better async
      project.serverSolution.protocol.scheduler.flush()
      Thread.sleep(0)
    }
    project.serverSolution.colorChooserModel.colorChooserSessions.remove(session)
    return result.get()
  }

  override fun showDialog(parent: Component?,
                          caption: String?,
                          preselectedColor: Color?,
                          enableOpacity: Boolean,
                          listeners: MutableList<out ColorPickerListener>,
                          opacityInPercent: Boolean): Color? {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.showDialog(parent, caption, preselectedColor, enableOpacity, listeners, opacityInPercent)

    return Color.GREEN
  }
}