package com.jetbrains.rdserver.ui

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.openapi.ui.messages.MessagesService
import com.intellij.ui.mac.MacMessages
import com.intellij.ui.messages.getLocalMacMessages
import java.awt.Window

class BackendMacMessageManagerProvider : MacMessages.MacMessageManagerProvider {
  private val messages by lazy { BackendJBMacMessages() }

  override fun getMessageManager(): MacMessages {
    if (ClientId.isCurrentlyUnderLocalId) {
      return getLocalMacMessages()
    }
    return messages
  }
}

class BackendJBMacMessages : MacMessages() {
  override fun showYesNoCancelDialog(title: String,
                                     message: String,
                                     yesText: String,
                                     noText: String,
                                     cancelText: String,
                                     window: Window?,
                                     doNotAskOption: DialogWrapper.DoNotAskOption?): Int {
    val options = arrayOf(yesText, noText, cancelText)
    return MessagesService.getInstance().showMessageDialog(project = null, parentComponent = null,
                                                           message = message, title = title, icon = null,
                                                           options = options,
                                                           doNotAskOption = doNotAskOption)
  }

  override fun showMessageDialog(title: String,
                                 message: String?,
                                 buttons: Array<out String>?,
                                 errorStyle: Boolean,
                                 window: Window?,
                                 defaultOptionIndex: Int,
                                 focusedOptionIndex: Int,
                                 doNotAskDialogOption: DialogWrapper.DoNotAskOption?): Int {
    error("Not supported")
  }

  override fun showOkMessageDialog(title: String, message: String?, okText: String, window: Window?) {
    MessagesService.getInstance().showMessageDialog(project = null, parentComponent = null,
                                                    message = message, title = title, icon = null,
                                                    options = arrayOf(okText),
                                                    doNotAskOption = null)
  }

  override fun showYesNoDialog(title: String,
                               message: String,
                               yesText: String,
                               noText: String,
                               window: Window?,
                               doNotAskDialogOption: DialogWrapper.DoNotAskOption?): Boolean {
    return MessagesService.getInstance().showMessageDialog(project = null, parentComponent = null,
                                                           message = message, title = title, icon = null,
                                                           options = arrayOf(yesText, noText),
                                                           doNotAskOption = doNotAskDialogOption) == 0
  }

  override fun showErrorDialog(title: String, message: String?, okButton: String, window: Window?) {
    MessagesService.getInstance().showMessageDialog(project = null, parentComponent = null,
                                                    message = message, title = title, icon = null,
                                                    options = arrayOf(okButton),
                                                    doNotAskOption = null)
  }
}

