package com.jetbrains.rdserver.ui

import com.intellij.CommonBundle
import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.ui.messages.MessagesServiceImpl
import com.jetbrains.ide.model.notifications.MessageBoxType
import com.jetbrains.rd.platform.util.getLogger
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.startup.RdServerPerClientSolutions
import com.jetbrains.rdserver.ui.feelLuckUi.BackendFeelLuckyUiHelper
import java.awt.Component
import javax.swing.Icon

/**
 * @author yole
 */
internal class BackendMessagesService : MessagesServiceImpl() {
  override fun showMessageDialog(project: Project?,
                                 parentComponent: Component?,
                                 message: String?,
                                 title: String?,
                                 options: Array<String>,
                                 defaultOptionIndex: Int,
                                 focusedOptionIndex: Int,
                                 icon: Icon?,
                                 doNotAskOption: DialogWrapper.DoNotAskOption?,
                                 alwaysUseIdeaUI: Boolean): Int {
    if (ClientId.isCurrentlyUnderLocalId)
      return super.showMessageDialog(project, parentComponent, message, title, options, defaultOptionIndex, focusedOptionIndex, icon, doNotAskOption, alwaysUseIdeaUI)
    val currentProject = project ?: RdServerPerClientSolutions.clientIdToProjectMap[ClientId.current]

    if (currentProject == null) {
      logger.error("Attempting to show message dialog $title : $message to client ${ClientId.current} without project specified")
      return super.showMessageDialog(project, parentComponent, message, title, options, defaultOptionIndex, focusedOptionIndex, icon, doNotAskOption, alwaysUseIdeaUI)
    }

    logger.info("Not showing a modal dialog to a remote client; Title: $title; Message: $message; Options: ${options.joinToString()}")

    val type = when (options.toList()) {
      listOf(CommonBundle.getOkButtonText()) -> MessageBoxType.OK
      listOf(CommonBundle.getOkButtonText(), CommonBundle.getCancelButtonText()) -> MessageBoxType.OKCANCEL
      listOf(CommonBundle.getYesButtonText(), CommonBundle.getNoButtonText()) -> MessageBoxType.YESNO
      listOf(CommonBundle.getYesButtonText(), CommonBundle.getNoButtonText(), CommonBundle.getCancelButtonText()) -> MessageBoxType.YESNOCANCEL
      else -> MessageBoxType.OKCANCEL  // Try to cancel for unknown combination
    }

    if (type == MessageBoxType.OK)
      return 0 // silently ok all ok dialogs


    if(RdServerPerClientComponents.getService<BackendFeelLuckyUiHelper>(currentProject).mayUseDefaultValueInMessageBox(title)){ //currently all dialogs from find usages
      return defaultOptionIndex
    }

    return if(type == MessageBoxType.YESNOCANCEL || type == MessageBoxType.OKCANCEL) -1 else 1 // cancel/no

    /*val model = MessageBoxModel(
      message ?: "",
      title ?: "",
      type
    )
    val notificationsHost = RdServerPerClientComponents.getService(project, BackendNotificationsHost::class.java)
    val response = notificationsHost.notificationsModel.messageBox.sync(model)
    return when (response) {
      MessageBoxResponse.OK -> 0
      MessageBoxResponse.CANCEL -> -1
      MessageBoxResponse.YES -> 0
      MessageBoxResponse.NO -> 1
    }*/
  }

  companion object {
    private val logger = getLogger<BackendMessagesService>()
  }
}