package com.jetbrains.rdserver.ui

import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uicontext.ContextKey
import com.jetbrains.ide.model.uicontext.ContextPair
import com.jetbrains.ide.model.uicontext.UiContextModel
import com.jetbrains.rd.framework.Protocol
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.RdServerPerClientComponents

@Suppress("NonDefaultConstructor")
class BackendUiContextHost(
  lifetime: Lifetime,
  val project: Project,
  protocol: Protocol
) {

  companion object {
    fun getInstance(project: Project): BackendUiContextHost = RdServerPerClientComponents.getService(project)
  }

  private var model = UiContextModel.create(lifetime, protocol)

  fun get(vararg pairs: ContextKey): List<ContextPair> {
    return (pairs zip pairs.map { key ->
      when (key) {
        ContextKey.Caret -> model.caret
        ContextKey.Selection -> model.selection
        ContextKey.Focus -> model.focus
        ContextKey.TextControl -> model.textControl
        ContextKey.Document -> model.document
      }
    }).map { ContextPair(it.first, it.second.valueOrNull ?: 0) }
  }
}
