package com.jetbrains.rdserver.ui

import java.awt.Component
import java.util.*
import javax.swing.JComponent

/**
 * Parse component to it's tree presentation.
 * Leafs are components of JComponent
 *
 * This function is designed to be used in REPL mode
 */
fun explore(component: Component): Tree<Component> {
  val stack = ArrayDeque<Tree<Component>>()
  val result = Tree(component)
  stack.add(result)

  while (stack.isNotEmpty()) {
    val elementTree = stack.removeLast()
    val element = elementTree.head
    if (element !is JComponent) {
      continue
    }

    element.components.forEach {
      val node = Tree(it)
      elementTree.leafs.add(node)
      stack.add(node)
    }
  }
  return result
}

class Tree<T : Any>(val head: T) {
  val leafs: MutableList<Tree<T>> = mutableListOf()

  /**
   * Get element by it's id.
   * Id has format like ":2:4:1"
   *
   * You can get id of an element after evaluating [toString] function
   */
  operator fun get(idString: String): T {
    val ids = idString.split(":").filter { it.isNotEmpty() }
    var result = this
    for (id in ids) {
      result = result.leafs[id.toInt() - 1]
    }
    return result.head
  }

  override fun toString(): String {
    val stack = ArrayDeque<Triple<Tree<T>, Int, String>>()
    stack.add(Triple(this, 0, ""))

    val resultBuilder = StringBuilder()
    while (stack.isNotEmpty()) {
      val (element, depth, id) = stack.removeLast()
      resultBuilder.append(" ".repeat(depth)).append(if (element.leafs.isEmpty()) "-" else "+").append(
        element.head::class.java.simpleName).append(" | $id\n")

      element.leafs.reversed().forEachIndexed { index, value ->
        stack.add(Triple(value, depth + 1, "$id:${element.leafs.size - index}"))
      }
    }
    return resultBuilder.toString()
  }
}
