package com.jetbrains.rdserver.ui

import com.intellij.CommonBundle
import com.intellij.icons.AllIcons
import com.intellij.ide.BrowserUtil
import com.intellij.notification.Notification
import com.intellij.notification.NotificationAction
import com.intellij.notification.NotificationType
import com.intellij.openapi.actionSystem.ActionManager
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.ex.ActionUtil
import com.intellij.openapi.project.Project
import com.intellij.util.concurrency.annotations.RequiresEdt
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.platform.codeWithMe.CwmNotifications
import com.jetbrains.rd.platform.codeWithMe.connection.CwmConnectionStatusService
import com.jetbrains.rd.platform.codeWithMe.ui.CWMDefaultDialog
import com.jetbrains.rdserver.CodeWithMePluginBundle
import java.util.concurrent.TimeUnit

object HostNotifications {
  fun showErrorDialog(project: Project) {
    val action = ActionManager.getInstance().getAction("GetJoinLinkAction")
    CWMDefaultDialog.createBuilder(project,
                                   CodeWithMePluginBundle.message("message.session.ended.title"),
                                   CodeWithMePluginBundle.message("license.expired.during.session.message"))
      .withHeader(CodeWithMePluginBundle.message("message.session.ended.title"))
      .withCancel({
                    action?.let { ActionUtil.performActionDumbAware(action, null) }
                  }, if(action != null) CodeWithMePluginBundle.message("license.start.session") else CommonBundle.message("button.ok"))
      .withOk({ BrowserUtil.browse(CodeWithMePlatformBundle.message("license.buy.link")) },
              CodeWithMePluginBundle.message("license.upgrade"))
      .build()
      .show()
  }

  fun showLicenseExpiredDialog(project: Project) {
    val action = ActionManager.getInstance().getAction("GetJoinLinkAction")
    CWMDefaultDialog.createBuilder(project,
                                   CodeWithMePluginBundle.message("license.expired.title"),
                                   CodeWithMePluginBundle.message("license.expired.during.session.message"))
      .withHeader(CodeWithMePluginBundle.message("license.expired.during.session.header"))
      .withCancel({
                  action?.let { ActionUtil.performActionDumbAware(action, null) }
                  }, if(action != null) CodeWithMePluginBundle.message("license.start.session") else CommonBundle.message("button.ok"))
      .withOk({ BrowserUtil.browse(CodeWithMePlatformBundle.message("license.buy.link")) },
              CodeWithMePluginBundle.message("license.upgrade"))
      .build()
      .show()
  }

  enum class SessionsLimitDialogResult {
    RETRY,
    CLOSE,
  }

  @RequiresEdt
  fun showSessionsLimitDialog(project: Project): SessionsLimitDialogResult {
    val dialog = CWMDefaultDialog.createBuilder(project,
                                   CodeWithMePluginBundle.message("license.session.limit.title"),
                                   CodeWithMePluginBundle.message("license.session.limit.message"))
      .withHeader(CodeWithMePluginBundle.message("license.session.limit.header"))
      .withCancel({}, CodeWithMePlatformBundle.message("license.close"))
      .withOk({ },
              CodeWithMePlatformBundle.message("license.retry"))
      .build()
    dialog.isModal = true
    return if (dialog.showAndGet()) SessionsLimitDialogResult.RETRY else SessionsLimitDialogResult.CLOSE
  }

  fun showTimeIsUpDialog(project: Project) {
    CWMDefaultDialog.createBuilder(project,
                                   CodeWithMePlatformBundle.message("license.ended.title"),
                                   CodeWithMePluginBundle.message("license.time.is.up.message"))
      .withHeader(CodeWithMePlatformBundle.message("license.time.is.up.header"))
      .withCancel({}, CodeWithMePlatformBundle.message("license.end"))
      .withOk({ BrowserUtil.browse(CodeWithMePlatformBundle.message("license.buy.link")) },
              CodeWithMePlatformBundle.message("license.upgrade"))
      .build()
      .show()
  }

  fun showNMinutesLeftNotifications(project: Project) {
    val connectionStatusService = CwmConnectionStatusService.getInstance(project)
    val minutesLeft = connectionStatusService.freeSessionRemainingTimeMs.value?.let {
      TimeUnit.MILLISECONDS.toMinutes(it).toInt()
    } ?: 1

    Notification(
      CwmNotifications.CWM_NOTIFICATION_GROUP_ID,
      AllIcons.CodeWithMe.CwmAccess,
      if(minutesLeft == 1)
        CodeWithMePluginBundle.message("license.five.minutes.left.1.header")
      else
        CodeWithMePluginBundle.message("license.five.minutes.left.header", minutesLeft),
      null,
      CodeWithMePluginBundle.message("license.five.minutes.left.message"),
      NotificationType.INFORMATION, null)
      .addAction(object : NotificationAction(CodeWithMePluginBundle.message("license.upgrade.uncapitalized")){
        override fun actionPerformed(e: AnActionEvent, notification: Notification) {
          BrowserUtil.browse(CodeWithMePlatformBundle.message("license.buy.link"))
        }
      })
      .notify(project)
  }
}

