package com.jetbrains.rdserver.ui.converters

import com.intellij.ide.DataManager
import com.intellij.openapi.actionSystem.*
import com.intellij.openapi.actionSystem.impl.ActionToolbarImpl
import com.intellij.openapi.project.ProjectManager
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.actionButton
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.actions.BeHostRemoteDataContextManager
import javax.swing.SwingConstants

class ActionToolbarConverter : RdConverter<ActionToolbar> {
  override fun convert(component: ActionToolbar, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val toolbar = component as ActionToolbarImpl
    val dock = if (toolbar.orientation == SwingConstants.VERTICAL) BeDock.LEFT else BeDock.TOP
    val beToolbar = BeToolbar(BeSpacer(), dock)
    val actions = toolbar.actions.flatMap { action -> convertAction(action, toolbar, trackOptions, lifetime) }
    beToolbar.leftItems.addAll(actions)

    // wrap to avoid caching
    val delegateContext = DataContext { dataId ->
      DataManager.getInstance().getDataContext(toolbar.targetComponent ?: toolbar).getData(dataId)
    }
    val contextId = BeHostRemoteDataContextManager.getInstance().registerContext(delegateContext, lifetime)
    if (contextId != null) {
      return BeContextIdBehavior(contextId, beToolbar)
    }
    else {
      return beToolbar
    }
  }

  private fun convertAction(action: AnAction,
                            toolbar: ActionToolbar,
                            trackOptions: TrackOptions,
                            lifetime: Lifetime): List<BeControl> {
    if (action is ActionGroup && !action.isPopup) {
      val children = action.getChildren(createEvent(toolbar))
      return children.flatMap { convertAction(it, toolbar, trackOptions, lifetime) }
    }

    val beControl = when (action) {
      is Separator -> BeSeparator(GridOrientation.Horizontal)
      else -> actionButton(action, lifetime)
    }
    return listOf(beControl)
  }

  private fun createEvent(toolbar: ActionToolbar): AnActionEvent {
    val place = if (toolbar is ActionToolbarImpl) toolbar.place else ActionPlaces.UNKNOWN
    return AnActionEvent.createFromDataContext(place, null, createDataContext(toolbar))
  }

  private fun createDataContext(toolbar: ActionToolbar): DataContext {
    return DataContext { dataId ->
      val data = toolbar.toolbarDataContext.getData(dataId)
      when {
        data != null -> data
        CommonDataKeys.PROJECT.`is`(dataId) -> ProjectManager.getInstance().openProjects.firstOrNull()
        else -> null
      }
    }
  }
}
