package com.jetbrains.rdserver.ui.converters

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.util.ClassExtension
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeLabel
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.ui.converters.TrackOptions.Companion.pinClientId
import java.awt.Component

/**
 * @author Alex Plate
 */

interface RdConverter<in T : Any> {
  fun convert(component: T, trackOptions: TrackOptions, lifetime: Lifetime): BeControl

  /*fun convert(component: T, trackOptions: TrackOptions, lifetime: Lifetime, project: Project?): BeControl {
    return convert(component, trackOptions, lifetime)
  }*/
}

/**
 * Use [explore] to explore structure of JComponents
 */
class RdConverterRegistry private constructor() : ClassExtension<RdConverter<Any>>("com.intellij.rdserver.converter") {
  companion object {
    private val instance = RdConverterRegistry()

    fun <T : Any> getConverter(clazz: Class<T>): RdConverter<T> {

      return instance.forClass(clazz) as? RdConverter<T>
             ?: throw RuntimeException("Cannot find converter for class $clazz")
    }
  }
}

fun Component.toBeModel(trackOptions: TrackOptions = TrackOptions.instance, lifetime: Lifetime = Lifetime.Eternal): BeControl {
  val realOptions = trackOptions.pinClientId()
  ClientId.withClientId(realOptions.clientId) {
    return RdConverterRegistry.getConverter(javaClass).convert(this, realOptions, lifetime)
  }
}

//fun String.toBeLabel() = BeRichText(false).also { it.text.set(toRichText()) }
fun String.toBeLabel() = BeLabel(false).also { it.text.set(this) }
