package com.jetbrains.rdserver.ui.converters

import com.intellij.openapi.editor.event.DocumentEvent
import com.intellij.openapi.editor.event.DocumentListener
import com.intellij.ui.EditorTextField
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeTextBox
import com.jetbrains.ide.model.uiautomation.BeTextBoxSettings
import com.jetbrains.rd.util.lifetime.Lifetime

/**
 * @author Alex Plate
 */
class EditorTextFieldConverter : RdConverter<EditorTextField> {
  override fun convert(component: EditorTextField, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val beTextBox = BeTextBox(BeTextBoxSettings())
    val textBoxSettings = beTextBox.settings
    textBoxSettings.readonly.set(component.isViewer)

    // Text
    beTextBox.text.set(component.text)
    if (trackOptions.modelChangeTrack) {
      beTextBox.text.advise(lifetime) {
        trackOptions.beforeModelChange?.invoke()
        val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? EditorTextField ?: component
        componentToChange.text = beTextBox.text.value
        trackOptions.afterModelChange?.invoke()
      }
    }
    if (trackOptions.componentChangeTrack) {
      component.addDocumentListener(object : DocumentListener {
        override fun documentChanged(event: DocumentEvent) {
          trackOptions.beforeComponentChange?.invoke()
          beTextBox.text.set(event.document.text)
          trackOptions.afterComponentChange?.invoke()
        }
      })
    }

    return beTextBox
  }
}
