package com.jetbrains.rdserver.ui.converters

import com.intellij.ui.InplaceButton
import com.jetbrains.ide.model.uiautomation.BeButton
import com.jetbrains.ide.model.uiautomation.BeButtonStyle
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeUserAction
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.toProtocolModel

/**
 * @author Alex Plate
 */
class InplaceButtonConverter : RdConverter<InplaceButton> {
  override fun convert(component: InplaceButton, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {

    val beLabel = component.toolTipText.toBeLabel().apply {
      icon.set(component.icon.toProtocolModel())
    }
    val beButton = BeButton(BeUserAction(), beLabel, BeButtonStyle.ICON)
    beButton.enabled.set(component.isEnabled)
    if (trackOptions.modelChangeTrack) {
      beButton.enabled.advise(lifetime) { value ->
        trackOptions.beforeModelChange?.invoke()
        val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? InplaceButton ?: component
        componentToChange.isEnabled = value
        trackOptions.afterModelChange?.invoke()
      }
    }
    if (trackOptions.componentChangeTrack) {
      component.addPropertyChangeListener {
        trackOptions.beforeComponentChange?.invoke()
        beButton.enabled.set(component.isEnabled)
        trackOptions.afterComponentChange?.invoke()
      }
    }
    beButton.action.click.advise(lifetime) {
      component.doClick()
    }
    return beButton
  }
}