package com.jetbrains.rdserver.ui.converters

import com.intellij.ui.components.editors.JBComboBoxTableCellEditorComponent
import com.jetbrains.ide.model.uiautomation.BeComboBox
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeTextBoxSettings
import com.jetbrains.rd.util.lifetime.Lifetime

/**
 * @author Alex Plate
 */
class JBComboBoxTableCellEditorComponentConverter : RdConverter<JBComboBoxTableCellEditorComponent> {
  override fun convert(component: JBComboBoxTableCellEditorComponent, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val comboBox = BeComboBox(BeTextBoxSettings())
    val toStringFunction = component.toString
    for (option in component.options) {
      comboBox.values.add(toStringFunction.`fun`(option).toBeLabel())
    }

    comboBox.selectedIndex.set(component.options.indexOf(component.editorValue))

    if (trackOptions.modelChangeTrack) {
      comboBox.selectedIndex.advise(lifetime) { newSelectedIndex ->
        trackOptions.beforeModelChange?.invoke()
        val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JBComboBoxTableCellEditorComponent ?: component
        if (newSelectedIndex in componentToChange.options.indices) {
          val newValue = componentToChange.options[newSelectedIndex]
          componentToChange.setDefaultValue(newValue)
        }
        trackOptions.afterModelChange?.invoke()
      }
    }

    return comboBox
  }
}