package com.jetbrains.rdserver.ui.converters

import com.jetbrains.ide.model.uiautomation.BeButton
import com.jetbrains.ide.model.uiautomation.BeButtonStyle
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeUserAction
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JButton

/**
 * @author Alex Plate
 */
class JButtonConverter : RdConverter<JButton> {
  override fun convert(component: JButton, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val beButton = BeButton(BeUserAction(), component.text.toBeLabel(), BeButtonStyle.DEFAULT)
    beButton.enabled.set(component.isEnabled)
    if (trackOptions.modelChangeTrack) {
      beButton.enabled.advise(lifetime) { value ->
        trackOptions.beforeModelChange?.invoke()
        val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JButton ?: component
        componentToChange.isEnabled = value
        trackOptions.afterModelChange?.invoke()
      }
    }
    if (trackOptions.componentChangeTrack) {
      component.addPropertyChangeListener {
        trackOptions.beforeComponentChange?.invoke()
        beButton.enabled.set(component.isEnabled)
        trackOptions.afterComponentChange?.invoke()
      }
    }
    beButton.action.click.advise(lifetime) {
      component.doClick()
    }
    return beButton
  }
}