package com.jetbrains.rdserver.ui.converters

import com.jetbrains.ide.model.uiautomation.BeCheckbox
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeRichText
import com.jetbrains.ide.model.uiautomation.ControlVisibility
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.adviseNotNull
import com.jetbrains.rdserver.toRichText
import javax.swing.JCheckBox

/**
 * @author Alex Plate
 */
class JCheckBoxConverter : RdConverter<JCheckBox> {
  override fun convert(component: JCheckBox, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val label = BeRichText(false).also { it.text.set(component.text.toRichText()) }
    return BeCheckbox(label).also { checkbox ->
      checkbox.property.set(component.isSelected)
      checkbox.visible.set(if (component.isVisible) ControlVisibility.Visible else ControlVisibility.Hidden)

      if (trackOptions.modelChangeTrack) {
        checkbox.property.adviseNotNull(lifetime) { newValue ->
          trackOptions.beforeModelChange?.invoke()
          val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JCheckBox ?: component

          if (componentToChange.isSelected != newValue) {
            componentToChange.doClick()
          }

          trackOptions.afterModelChange?.invoke()
        }
      }
    }
  }
}