package com.jetbrains.rdserver.ui.converters

import com.jetbrains.ide.model.uiautomation.BeAbstractText
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.rd.ui.bedsl.dsl.combobox
import com.jetbrains.rd.ui.bedsl.dsl.editableCombobox
import com.jetbrains.rd.ui.bedsl.dsl.getText
import com.jetbrains.rd.ui.bedsl.dsl.label
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JComboBox
import javax.swing.JList

/**
 * @author Alex Plate
 */
class JComboBoxConverter<T : Any> : RdConverter<JComboBox<T>> {

  override fun convert(component: JComboBox<T>, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val count = component.model.size

    if (!component.isEditable) {
      val items = mutableListOf<BeControl>()
      for (i in 0 until count) {
        val item = component.model.getElementAt(i)
        val renderComponent = component.renderer.getListCellRendererComponent(JList(), item, i, false, false)
        val label = renderComponent.toBeModel(TrackOptions(componentChangeTrack = false), lifetime)
        items.add(label)
      }
      val selectedValue = items.getOrNull(component.selectedIndex) ?: items.firstOrNull()

      return combobox(lifetime, items, selectedValue, "", {
        if (trackOptions.modelChangeTrack) {
          trackOptions.beforeModelChange?.invoke()
          val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JComboBox<*> ?: component
          componentToChange.selectedIndex = items.indexOf(it)
          trackOptions.afterModelChange?.invoke()
        }
      }) { it }
    }
    else {
      val items = mutableListOf<BeAbstractText>()
      for (i in 0 until count) {
        val item = component.model.getElementAt(i)
        val renderComponent = component.renderer.getListCellRendererComponent(JList(), item, i, false, false)
        // BeModel for label should not be subscribed to label changes because `renderer.getListCellRendererComponent` changes label state
        // and does not return new value
        val label = renderComponent.toBeModel(TrackOptions(componentChangeTrack = false), lifetime) as? BeAbstractText
                    ?: item.toString().toBeLabel()
        items.add(label)
      }
      val selectedValue = items.getOrNull(component.selectedIndex) ?: items.firstOrNull()

      val strings = items.map { it.getText() }
      return editableCombobox(lifetime, strings, selectedValue?.getText() ?: component.editor.item.toString(), "") {
        if (trackOptions.modelChangeTrack) {
          trackOptions.beforeModelChange?.invoke()
          val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JComboBox<*> ?: component
          componentToChange.selectedIndex = strings.indexOf(it)
          trackOptions.afterModelChange?.invoke()
        }
      }

    }

    // TODO result.selectedValue.set(component.editor.item.toString())

    /*if (trackOptions.modelChangeTrack) {
      result.selectedIndex.advise(lifetime) {
        trackOptions.beforeModelChange?.invoke()
        val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JComboBox<*> ?: component
        componentToChange.selectedIndex = it
        trackOptions.afterModelChange?.invoke()
      }*/

      /* TODO
    result.selectedValue.advise(lifetime) {
      trackOptions.beforeModelChange?.invoke()
      val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JComboBox<*> ?: component
      componentToChange.editor.item = it
      trackOptions.afterModelChange?.invoke()
    }
       */
    //}
  }
}