package com.jetbrains.rdserver.ui.converters

import com.jetbrains.ide.model.uiautomation.BeAbstractText
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeLabel
import com.jetbrains.ide.model.uiautomation.BeRichText
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.htmlToRichText
import com.jetbrains.rdserver.toProtocolModel
import javax.swing.JLabel

/**
 * @author Alex Plate
 */
class JLabelConverter : RdConverter<JLabel> {
  override fun convert(component: JLabel, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val beLabel: BeAbstractText =
      if (component.text != null) {
        val isRichText = component.text.startsWith("<html>", ignoreCase = true)
        if(isRichText)
        BeRichText(wrapText = isRichText).also {
          val text = it.text
          text.set(component.text.htmlToRichText())
        }
        else BeLabel(false).apply{
          this.text.set(component.text)
        }
      }
      else {
        BeLabel(false)
      }

    component.icon?.let { beLabel.icon.set(it.toProtocolModel()) }

    if (trackOptions.componentChangeTrack) {
      component.addPropertyChangeListener {
        trackOptions.beforeComponentChange?.invoke()
        beLabel.setText(component.text)
        trackOptions.afterComponentChange?.invoke()
      }
    }
    return beLabel
  }
}

private fun BeAbstractText.setText(text: String?) {
  when(this){
    is BeRichText -> this.text.set(text?.htmlToRichText() ?: "".htmlToRichText())
    is BeLabel -> this.text.set(text ?: "")
  }
}
