package com.jetbrains.rdserver.ui.converters

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JScrollPane
import javax.swing.ScrollPaneConstants

/**
 * @author Alex Plate
 */
class JScrollPaneConverter : RdConverter<JScrollPane> {
  override fun convert(component: JScrollPane, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val viewportModel = component.viewport.view.toBeModel(trackOptions, lifetime)
    if (viewportModel.hasOwnScrollPane) return viewportModel

    val hasHorizontal = component.horizontalScrollBarPolicy != ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER
    val hasVertical = component.verticalScrollBarPolicy != ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER
    val scrollBarPolicy = if (hasHorizontal) {
      if (hasVertical) BeScrollbarPolicy.BOTH else BeScrollbarPolicy.HORIZONTAL
    }
    else if (hasVertical) BeScrollbarPolicy.VERTICAL else BeScrollbarPolicy.NONE


    val size = BeControlSizePredefined(BeSize(BeControlSizeType.FIT_TO_CONTENT, 1), BeSize(BeControlSizeType.FIT_TO_CONTENT, 1))
    return BeScrollablePanel(scrollBarPolicy, size, viewportModel)
  }

  private val BeControl.hasOwnScrollPane
    get() = this is BeTreeGrid ||
            this is BeEditorImpl
}