package com.jetbrains.rdserver.ui.converters

import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.dsl.textBox
import com.jetbrains.rd.util.lifetime.Lifetime
import javax.swing.JTextField

/**
 * @author Alex Plate
 */

class JTextFieldConverter : RdConverter<JTextField> {
  override fun convert(component: JTextField, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val beTextBox = textBox(lifetime, !component.isEditable)

    // Text
    beTextBox.text.set(component.text)
    beTextBox.settings.caret.set(component.text.length)
    if (trackOptions.modelChangeTrack) {
      beTextBox.text.advise(lifetime) {
        trackOptions.beforeModelChange?.invoke()
        val componentToChange = trackOptions.modelChangeComponentProvider?.invoke() as? JTextField ?: component
        componentToChange.text = beTextBox.text.value
        trackOptions.afterModelChange?.invoke()
      }
    }
    if (trackOptions.componentChangeTrack) {
      component.addPropertyChangeListener("JComponent.outline"){
        if(it.newValue == "error") beTextBox.validationResult.set(ValidationResult(ValidationStates.validationError))
        if(it.newValue == "warning") beTextBox.validationResult.set(ValidationResult(ValidationStates.validationWarning))
        if(it.newValue == null) beTextBox.validationResult.set(ValidationResult(ValidationStates.validationPassed))
      }

      component.addActionListener {
        val source = it.source
        if (source is JTextField) {
          trackOptions.beforeComponentChange?.invoke()
          beTextBox.text.set(source.text)
          trackOptions.afterComponentChange?.invoke()
        }
      }
    }

    return beTextBox
  }
}