package com.jetbrains.rdserver.ui.converters

import com.intellij.ui.components.JBLayeredPane
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeSpacer
import com.jetbrains.rd.util.lifetime.Lifetime
import java.awt.FlowLayout
import javax.swing.JLayeredPane

class LayeredPaneConverter : RdConverter<JBLayeredPane> {
  override fun convert(component: JBLayeredPane, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    return createDynamicContainer(component, trackOptions, lifetime, ::convertDefaultLayer)
  }

  private fun convertDefaultLayer(component: JBLayeredPane, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    val components = component.getComponentsInLayer(JLayeredPane.DEFAULT_LAYER).asList()
    val layout = component.layout
    return when {
      layout != null -> return BeJPanel(trackOptions, lifetime).buildControl(components, layout)
      components.size > 1 -> return BeJPanel(trackOptions, lifetime).buildControl(components, FlowLayout())
      components.size == 1 -> return components.single().toBeModel(trackOptions, lifetime)
      else -> BeSpacer()
    }
  }
}