package com.jetbrains.rdserver.ui.converters

import com.intellij.openapi.ui.Splitter
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeSpacer
import com.jetbrains.ide.model.uiautomation.BeSplitControl
import com.jetbrains.rd.util.lifetime.Lifetime

/**
 * @author Alex Plate
 */
class SplitterConverter : RdConverter<Splitter> {
  override fun convert(component: Splitter, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    return createDynamicContainer(component, trackOptions, lifetime, ::buildSplitter)
  }

  private fun buildSplitter(component: Splitter,
                            trackOptions: TrackOptions,
                            lifetime: Lifetime): BeControl {
    val firstComponent = component.firstComponent?.takeIf { it.isVisible }?.toBeModel(trackOptions, lifetime)
    val secondComponent = component.secondComponent?.takeIf { it.isVisible }?.toBeModel(trackOptions, lifetime)

    return when {
      firstComponent != null && secondComponent != null -> {
        BeSplitControl(
          firstComponent, secondComponent, component.isVertical, component.proportion
        )
      }
      firstComponent != null -> firstComponent
      secondComponent != null -> secondComponent
      else -> BeSpacer()
    }
  }
}