package com.jetbrains.rdserver.ui.converters.custom

import com.intellij.codeInsight.hint.ImplementationViewComponent
import com.jetbrains.ide.model.uiautomation.BeControl
import com.jetbrains.ide.model.uiautomation.BeDynamicControlContainer
import com.jetbrains.ide.model.uiautomation.BeMarginType
import com.jetbrains.ide.model.uiautomation.BeSizingType
import com.jetbrains.rd.ui.bedsl.dsl.horizontalGrid
import com.jetbrains.rd.ui.bedsl.dsl.verticalGrid
import com.jetbrains.rd.ui.bedsl.dsl.withMargin
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.ui.converters.RdConverter
import com.jetbrains.rdserver.ui.converters.TrackOptions
import com.jetbrains.rdserver.ui.converters.toBeModel
import java.beans.PropertyChangeListener

class ImplementationViewComponentConverter : RdConverter<ImplementationViewComponent> {

  override fun convert(component: ImplementationViewComponent, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {
    return verticalGrid {
      row {
        val dynamicContent = BeDynamicControlContainer() //todo[A.Afonina, V.Petrenko]: convert to dsl method
        dynamicContent.content.set(component.singleEntryPanel.toBeModel(trackOptions, lifetime)
                                     .withMargin { //remove when toolbar is properly added
                                       margin(BeMarginType.DefaultIntercellSpacing,
                                              BeMarginType.DefaultIntercellSpacing,
                                              BeMarginType.DefaultIntercellSpacing)
                                     })
        val fileChooserComboBox = component.fileChooserComboBox
        fileChooserComboBox.addPropertyChangeListener("renderer", PropertyChangeListener {
          if (fileChooserComboBox.isVisible) {
            dynamicContent.content.set(
              horizontalGrid {
                column(BeSizingType.Fill) { fileChooserComboBox.toBeModel(trackOptions, lifetime) }
                // todo: toolbar
                //val actionToolbar = component.toolbar.component
                //column { actionToolbar.toBeModel() }
              })
          }
        })
        dynamicContent
      }
      row(BeSizingType.Fill) {
        component.viewingPanel.toBeModel(trackOptions, lifetime)
      }
    }
  }

}
