package com.jetbrains.rdserver.ui.converters.custom

import com.intellij.ide.ui.newItemPopup.NewItemSimplePopupPanel
import com.intellij.openapi.actionSystem.CommonShortcuts
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.platform.util.TreeTraverse
import com.jetbrains.rd.ui.bedsl.dsl.label
import com.jetbrains.rd.ui.bedsl.dsl.verticalGrid
import com.jetbrains.rd.ui.bedsl.extensions.descendantsOfType
import com.jetbrains.rd.ui.util.getStringPresentation
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.ui.converters.RdConverter
import com.jetbrains.rdserver.ui.converters.TrackOptions
import com.jetbrains.rdserver.ui.converters.toBeModel
import javax.swing.JComponent


class NewItemSimplePopupPanelConverter : RdConverter<NewItemSimplePopupPanel> {
  override fun convert(component: NewItemSimplePopupPanel, trackOptions: TrackOptions, lifetime: Lifetime): BeControl {

    val components = TreeTraverse.descendantsDfs(component as JComponent)
    { it.components.mapNotNull { jc -> jc as? JComponent }.asSequence() }.mapNotNull {
      it.toBeModel(trackOptions, lifetime)
    }
    val beTextBox = components.firstOrNull { it is BeTextBox } as? BeTextBox
    beTextBox?.validationStyle?.set(ValidationStyle.InnerBorder)

    val list = components.firstOrNull { it is BeTreeGrid } as? BeTreeGrid

    val textBoxBehaviorWrapper = beTextBox?.let { BeBehaviorControl(it) }
    textBoxBehaviorWrapper?.behavior?.add(BeShortcutListener(CommonShortcuts.ENTER.getStringPresentation(), true).apply {
      this.reaction.advise(lifetime) {
        component.applyAction.consume(null)
      }
    })

    if (list == null || list.fakeRoot.children.isEmpty()) return textBoxBehaviorWrapper ?: label("Something went wrong on load")
    if (textBoxBehaviorWrapper == null) return list ?: label("Something went wrong on load")

    val descendantsOfType = list.descendantsOfType(BeTreeGridLine::class.java).toList()


    textBoxBehaviorWrapper.behavior.add(BeShortcutListener(CommonShortcuts.getMoveUp().getStringPresentation()).apply {
      this.reaction.advise(lifetime) {
        val currentSelected = descendantsOfType.indexOfFirst { it.isSelected.value }
        if (currentSelected <= 0) return@advise
        descendantsOfType[currentSelected].isSelected.value = false
        descendantsOfType[currentSelected - 1].isSelected.value = true
      }
    })
    textBoxBehaviorWrapper.behavior.add(BeShortcutListener(CommonShortcuts.getMoveDown().getStringPresentation()).apply {
      this.reaction.advise(lifetime) {

        val currentSelected = descendantsOfType.indexOfFirst { it.isSelected.value }
        if (currentSelected < 0 || currentSelected == descendantsOfType.count() - 1) return@advise

        descendantsOfType[currentSelected].isSelected.value = false
        descendantsOfType[currentSelected + 1].isSelected.value = true
      }
    })

    descendantsOfType.forEach {
      it.signal.advise(lifetime) {
        component.applyAction.consume(null)
      }
    }
    return verticalGrid {
      row { textBoxBehaviorWrapper }
      row { list }
    }
  }
}