package com.jetbrains.rdserver.ui.feelLuckUi

import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.diagnostic.trace
import com.intellij.openapi.ui.DialogWrapper
import com.jetbrains.rdserver.actions.ModalUIShowPolicy

enum class ModalDialogSilentOption{
  Accept,
  Cancel,
  SilentForbidden
}
class BackendFeelLuckyUiHelper {
  companion object {
    private val logger = logger<BackendFeelLuckyUiHelper>()
  }

  private var currentMarker: FeelLuckyUiMarker? = null

  fun <T> executeWithUiInteractionMarker(marker: FeelLuckyUiMarker, block: () -> T): T {
    try {
      require(currentMarker == null) { "Expected marker is empty, but actual is $currentMarker." }
      currentMarker = marker
      return block()
    }
    finally {
      currentMarker = null
    }
  }
  fun mayShowPopup(): Boolean {
    val currentMarker = currentMarker ?: return false
    val mayShowPopup = currentMarker.mayShowPopup()
    logger.trace { "Current marker is $currentMarker, MayShowPopup = $mayShowPopup." }
    return mayShowPopup
  }

  fun maySkipModalDialog(dialogWrapper: DialogWrapper): ModalDialogSilentOption {
    val currentMarker = currentMarker ?: return ModalUIShowPolicy.maySkipModalDialog(dialogWrapper.title)
    val maySkipModalDialog = currentMarker.maySkipModalDialog()
    logger.trace { "Current marker is $currentMarker, MaySkipModalDialog = $maySkipModalDialog." }
    return maySkipModalDialog
  }

  fun mayUseDefaultValueInMessageBox(title: String?): Boolean {
    val currentMarker = currentMarker ?: return ModalUIShowPolicy.mayUseDefaultValueInMessageBox(title)
    val mayUseDefaultValueInMessageBox = currentMarker.mayUseDefaultValueInMessageBox()
    logger.trace { "Current marker is $currentMarker, mayUseDefaultValueInMessageBox = $mayUseDefaultValueInMessageBox." }
    return mayUseDefaultValueInMessageBox
  }
}