package com.jetbrains.rdserver.ui.intentions

import com.intellij.codeInsight.intention.IntentionAction
import com.intellij.openapi.extensions.ExtensionPointName

interface BackendIntentionActionFilter {
  companion object {
    var EP = ExtensionPointName<BackendIntentionActionFilter>("com.intellij.rdserver.backendIntentionActionFilter")

    fun acceptIntention(it: IntentionAction): Boolean = accept(it) { acceptIntention(it) }
    fun acceptErrorFix(it: IntentionAction): Boolean = accept(it) { acceptErrorFix(it) }
    fun acceptInspection(it: IntentionAction): Boolean = accept(it) { acceptInspection(it) }
    fun acceptGutterAction(it: IntentionAction): Boolean = accept(it) { acceptGutterAction(it) }
    fun acceptNotificationAction(it: IntentionAction): Boolean = accept(it) { acceptNotificationAction(it) }

    private inline fun accept(it: IntentionAction, crossinline accept: BackendIntentionActionFilter.(IntentionAction) -> Boolean): Boolean {
      for (filter in EP.extensions) {
        if (!filter.accept(it)) return false
      }
      return true
    }
  }

  fun acceptIntention(it: IntentionAction): Boolean = true
  fun acceptErrorFix(it: IntentionAction): Boolean = true
  fun acceptInspection(it: IntentionAction): Boolean = true
  fun acceptGutterAction(it: IntentionAction): Boolean = true
  fun acceptNotificationAction(it: IntentionAction): Boolean = true
}