package com.jetbrains.rdserver.ui.intentions

import com.intellij.codeInsight.intention.IntentionAction
import com.intellij.codeInsight.intention.impl.config.IntentionActionWrapper
import com.intellij.codeInspection.ex.QuickFixWrapper
import com.jetbrains.rdserver.ui.feelLuckUi.FeelLuckyUiMarker
import com.jetbrains.rdserver.ui.feelLuckUi.ModalDialogSilentOption

data class IntentionActionFeelLuckyUiMarker(private var familyName: String) : FeelLuckyUiMarker {
  companion object {
    private val intentionsWithSkippedModalUi = setOf(
      "Kotlin/Make private",
      "Kotlin/Make internal",
      "Kotlin/Create Kotlin subclass",
      "Safe delete",
      "Create Kotlin subclass",
      "Java/Strings/Copy string concatenation text to the clipboard",
      "Create Class from Usage",
      "Create new function",
      "Create field",
      "Create parameter from Usage",
      "Implement abstract methods"
    )

    fun maySkipModalDialog(intention: IntentionAction): Boolean {
      var id = getPersistentStringId(intention)
      if (id.contains(" '"))  {id = id.split(" '")[0]}
      return intentionsWithSkippedModalUi.contains(id)
    }

    fun createToken(action: IntentionAction): IntentionActionFeelLuckyUiMarker {
      val familyName = getPersistentStringId(action)
      return IntentionActionFeelLuckyUiMarker(familyName)
    }
  }

  override fun maySkipModalDialog(): ModalDialogSilentOption {
    if (familyName.contains(" '")) familyName = familyName.split(" '")[0]
    if(intentionsWithSkippedModalUi.contains(familyName))
      return ModalDialogSilentOption.Accept
    return ModalDialogSilentOption.SilentForbidden
  }
}

fun getPersistentStringId(action: IntentionAction) =
  when (action) {
    is IntentionActionWrapper -> action.fullFamilyName
    is QuickFixWrapper -> action.fix.familyName
    else -> action.familyName
  }
