package com.jetbrains.rdserver.ui.modalDialogs

import com.intellij.codeWithMe.ClientId
import com.intellij.ide.util.MemberChooser
import com.intellij.openapi.project.Project
import com.jetbrains.ide.model.uiautomation.BeTreeGrid
import com.jetbrains.ide.model.uiautomation.BeTreeGridLine
import com.jetbrains.rd.ui.bedsl.extensions.descendantsOfType
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.ui.converters.verticalGridOf
import com.jetbrains.rdserver.ui.converters.toBeModel
import java.awt.BorderLayout
import javax.swing.JComponent

class MemberChooserModalAction : RdModalDialogAction<MemberChooser<*>> {
  private fun JComponent.extractCenterPane(): JComponent {
    val borderLayout = layout as? BorderLayout ?: return this
    return borderLayout.getLayoutComponent(this, BorderLayout.CENTER) as? JComponent ?: this
  }

  private fun JComponent.extractSouthPane(): JComponent? {
    val borderLayout = layout as? BorderLayout ?: return null
    return borderLayout.getLayoutComponent(this, BorderLayout.SOUTH) as? JComponent
  }

  override fun execute(dialogWrapper: MemberChooser<*>, lifetime: Lifetime, project: Project, clientId: ClientId) {
    val content = dialogWrapper.contentPanel
    val centerPane = content.extractCenterPane().toBeModel(lifetime = lifetime)
    val southSection = content.extractSouthPane()
    val errorText = southSection?.extractCenterPane()
    val beErrorText = errorText?.toBeModel(lifetime = lifetime)

    val dialogContent = if (beErrorText != null) verticalGridOf(centerPane to true, beErrorText to false) else centerPane
    val listOfMethods = dialogContent.descendantsOfType(BeTreeGrid::class.java).firstOrNull()
    listOfMethods?.descendantsOfType(BeTreeGridLine::class.java)?.forEach {
      it.isSelected.value = true
    }
  }

}

