package com.jetbrains.rdserver.ui.modalDialogs

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.diagnostic.Logger
import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.DialogWrapper
import com.intellij.openapi.util.ActionCallback
import com.intellij.openapi.util.ClassExtension
import com.jetbrains.ide.model.notifications.NotificationModel
import com.jetbrains.ide.model.notifications.RdNotificationEntryType
import com.jetbrains.rd.platform.codeWithMe.CodeWithMePlatformBundle
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rdserver.CodeWithMePluginBundle
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.ui.BackendNotificationsHost
import com.jetbrains.rdserver.ui.feelLuckUi.BackendFeelLuckyUiHelper
import com.jetbrains.rdserver.ui.feelLuckUi.ModalDialogSilentOption

class RdModalDialogActionRegistry private constructor() : ClassExtension<RdModalDialogAction<*>>(
  "com.intellij.rdserver.modalDialogAction") {
  companion object {
    private val instance = RdModalDialogActionRegistry()
    fun <T : DialogWrapper> getModalDialogConverter(clazz: Class<T>): RdModalDialogAction<T> {
      return instance.forClass(clazz) as? RdModalDialogAction<T>
             ?: throw RuntimeException("Cannot find action for class $clazz")
    }
  }
}

fun DialogWrapper.tryProcessModalOnBackend(project: Project,
                                           clientId: ClientId,
                                           lifetime: Lifetime = Lifetime.Eternal,
                                           logger: Logger? = null): ActionCallback? {
  val modalDialogConverter = RdModalDialogActionRegistry.getModalDialogConverter(javaClass)
  // val process = modalDialogConverter.process(this, lifetime, project, clientId)
  // if (process != null) return process
  val canClickOk = RdServerPerClientComponents.getService<BackendFeelLuckyUiHelper>(project).maySkipModalDialog(this)
  when (canClickOk) {
    ModalDialogSilentOption.Accept -> {
      if (this.isOKActionEnabled) {
        logger?.debug("Trying to show a modal dialog to a remote client, pressing ok")
        modalDialogConverter.execute(this, lifetime, project, clientId)
        this.clickDefaultButton()
        if (!this.isOK)
          this.close(DialogWrapper.CANCEL_EXIT_CODE)
      }
      else showNotImplementedNotification(logger, project, clientId)
    }
    ModalDialogSilentOption.Cancel -> {
      logger?.debug("Trying to show a modal dialog to a remote client, pressing cancel")
      this.doCancelAction()
    }
    ModalDialogSilentOption.SilentForbidden -> {
      showNotImplementedNotification(logger, project, clientId)
    }
  }
  return ActionCallback.DONE
}

private fun showNotImplementedNotification(logger: Logger?,
                                           project: Project,
                                           clientId: ClientId) {
  logger?.warn("Trying to show a modal dialog to a remote client, showing notification instead", Throwable())
  RdServerPerClientComponents.getService(project, clientId,
                                         BackendNotificationsHost::class.java).notificationsModel.notification.fire(
    NotificationModel(
      CodeWithMePlatformBundle.message("product.name"),
      CodeWithMePluginBundle.message("notification.not.implemented"),
      true,
      RdNotificationEntryType.ERROR,
      emptyList()))
}