package com.jetbrains.rdserver.ui.popup

import com.intellij.openapi.actionSystem.*
import com.intellij.openapi.actionSystem.ex.ActionManagerEx
import com.intellij.openapi.actionSystem.impl.PresentationFactory
import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.popup.ListPopup
import com.intellij.openapi.ui.popup.ListPopupStep
import com.jetbrains.ide.model.popups.PopupInteractionModel
import com.jetbrains.ide.model.popups.RdPopupMenuModelItem
import com.jetbrains.rdserver.RdServerPerClientComponents
import com.jetbrains.rdserver.actions.AnActionFeelLuckyUiMarker
import com.jetbrains.rdserver.toProtocolModel
import com.jetbrains.rdserver.toRichText
import com.jetbrains.rdserver.ui.feelLuckUi.BackendFeelLuckyUiHelper
import java.awt.event.InputEvent
import javax.swing.event.ListSelectionListener

class RdActionGroupPopup(
  popupInteractionModel: PopupInteractionModel,
  title: String?,
  private val actionGroup: ActionGroup,
  private val dataContext: DataContext,
  private val showDisabledActions: Boolean
) : RdJBPopupBase(popupInteractionModel, title, null), ListPopup {
  private val expandedActions = mutableListOf<AnAction>()

  override fun createPopupModelItems(): List<RdPopupMenuModelItem> {
    val items = mutableListOf<RdPopupMenuModelItem>()
    convertActionGroup(actionGroup, items)
    return items
  }

  private fun convertActionGroup(actionGroup: ActionGroup, items: MutableList<RdPopupMenuModelItem>) {
    val actionEvent = AnActionEvent.createFromDataContext(ActionPlaces.UNKNOWN, null, dataContext)
    val children = actionGroup.getChildren(actionEvent)
    for (child in children) {
      if (child is ActionGroup && !child.isPopup) {
        convertActionGroup(child, items)
      }
      else {
        val element = convertAction(child)
        if (element != null) {
          items.add(element)
        }
      }
    }
  }

  override fun handleExecuteItem(index: Int) {
    val action = expandedActions[index]
    val actionEvent = AnActionEvent.createFromDataContext(ActionPlaces.UNKNOWN, null, dataContext)
    val project = CommonDataKeys.PROJECT.getData(dataContext)
    val feelLuckyUiHelper = project?.let { RdServerPerClientComponents.getService<BackendFeelLuckyUiHelper>(it) }
    val actionManager = ActionManagerEx.getInstanceEx()
    var actionId = actionManager.getId(action)

    if(actionId == null){
      actionId = PresentationFactory().getPresentation(action).text
      if(actionId == null) {
        action.update(actionEvent)
        actionId = actionEvent.presentation.text
      }
    }

    if(feelLuckyUiHelper != null && actionId != null)
      feelLuckyUiHelper.executeWithUiInteractionMarker(AnActionFeelLuckyUiMarker(actionId, action)) {action.actionPerformed(actionEvent)}
    else
      action.actionPerformed(actionEvent)
  }

  override fun handleSelectedItem(index: Int) {

  }

  private fun convertAction(action: AnAction): RdPopupMenuModelItem? {
    val presentation = Presentation()
    presentation.copyFrom(action.templatePresentation)
    val actionEvent = AnActionEvent.createFromDataContext(ActionPlaces.UNKNOWN, presentation, dataContext)
    action.update(actionEvent)
    if (!presentation.isVisible || (!showDisabledActions && !presentation.isEnabled)) return null

    expandedActions.add(action)

    return RdPopupMenuModelItem(
      expandedActions.size - 1,
      presentation.icon?.toProtocolModel(),
      null,
      (presentation.text ?: "").toRichText(),
      presentation.description?.toRichText(),
      null,
      null,
      action is Separator,
      presentation.isEnabled,
      false,
      false,
      null
    )
  }

  override fun addListSelectionListener(listSelectionListener: ListSelectionListener?) {
    TODO("not implemented")
  }

  override fun getListStep(): ListPopupStep<*> {
    TODO("not implemented")
  }

  override fun handleSelect(handleFinalChoices: Boolean) {
    TODO("not implemented")
  }

  override fun handleSelect(handleFinalChoices: Boolean, e: InputEvent?) {
    TODO("not implemented")
  }

  override fun setHandleAutoSelectionBeforeShow(autoHandle: Boolean) {
    TODO("not implemented")
  }

  override fun setShowSubmenuOnHover(showSubmenuOnHover: Boolean) {
    TODO("not implemented")
  }
}