package com.jetbrains.rdserver.ui.popup.bePopups.impl

import com.intellij.openapi.actionSystem.DataContext
import com.intellij.openapi.actionSystem.DataProvider
import com.intellij.openapi.diagnostic.logger
import com.intellij.openapi.editor.Editor
import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.popup.JBPopupListener
import com.intellij.openapi.ui.popup.LightweightWindowEvent
import com.intellij.openapi.util.Disposer
import com.intellij.ui.awt.RelativePoint
import com.intellij.ui.popup.AbstractPopup
import com.jetbrains.codeWithMe.model.PopupsModel
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.AddRemove
import com.jetbrains.rd.util.reactive.fire
import com.jetbrains.rdserver.followMe.popup.BackendPopupsHost
import org.jetbrains.annotations.NotNull
import java.awt.Component
import java.awt.Dimension
import java.awt.Point
import java.awt.event.InputEvent
import java.awt.event.KeyEvent
import java.util.concurrent.atomic.AtomicInteger
import javax.swing.JComponent

class BackendBeAbstractPopup(
  private val popupsModel: PopupsModel?,
  private val popupContent: JComponent,
  private val settings: BePopupSettings,
  private val isOriginalWithReflection: Boolean,
  private val realProject: Project?
) : AbstractPopup() {

  constructor(popupsModel: PopupsModel?,
              popupContent: JComponent,
              settings: BePopupSettings) : this(popupsModel, popupContent, settings, false, null)

  private var isShown: Boolean = false
  private var isVisible: Boolean = false
  private var isDisposed = false

  val lifetime = Lifetime.Eternal

  companion object {
    private val idCounter = AtomicInteger()
    private val logger = logger<BackendBeAbstractPopup>()
  }

  private val popupId: Int = idCounter.incrementAndGet()
  private var popupModel: BeAbstractPopup? = null

  override fun addListener(listener: JBPopupListener) {
    if (isOriginalWithReflection) super.addListener(listener)
    else settings.popupListeners.add(listener)
  }

  override fun setAdText(s: String, alignment: Int) {
    super.setAdText(s, alignment)
    settings.adText = s
  }


  override fun setCaption(title: String) {
    if (isOriginalWithReflection) super.setCaption(title)
    settings.setTitle(title)
  }

  override fun isModalContext(): Boolean {
    if (isOriginalWithReflection) return super.isModalContext()
    return settings.isModal
  }

  override fun setFinalRunnable(finalRunnable: Runnable?) {
    if (isOriginalWithReflection) super.setFinalRunnable(finalRunnable)
    else settings.finalRunnable = finalRunnable
  }

  override fun setRequestFocus(requestFocus: Boolean) {
    if (isOriginalWithReflection) super.setRequestFocus(requestFocus)
    settings.requestFocus = requestFocus
  }

  override fun setMinimumSize(size: Dimension?) {
    if (isOriginalWithReflection) super.setMinimumSize(size)
    settings.minimumSize = size
  }

  override fun isVisible(): Boolean {
    if (isOriginalWithReflection) return super.isVisible()
    return /*popupModel?.visible?.valueOrDefault(ControlVisibility.Hidden) == ControlVisibility.Visible &&*/ isVisible
  }

  override fun isPersistent(): Boolean {
    if (isOriginalWithReflection) return super.isPersistent()
    return !settings.cancelOnClickOutside && !settings.cancelOnOtherWindowOpen
  }


  override fun showUnderneathOf(componentUnder: Component) {
    if (isOriginalWithReflection) super.showUnderneathOf(componentUnder)
    else show()
    // super.showUnderneathOf(componentUnder)//show relative point
  }

  override fun show(point: RelativePoint) {
    if (isOriginalWithReflection) super.show(point)
    else show()
    //super.show(point)//show : Component owner, int aScreenX, int aScreenY, final boolean considerForcedXY
  }

  override fun showInCenterOf(component: Component) {
    if (isOriginalWithReflection) super.showInCenterOf(component)
    else show()
    //super.showInCenterOf(component)//show : Component owner, int aScreenX, int aScreenY, final boolean considerForcedXY
  }

  override fun showInBestPositionFor(dataContext: DataContext) {
    if (isOriginalWithReflection) super.showInBestPositionFor(dataContext)
    else show()
    //super.showInBestPositionFor(dataContext)//show relative point
  }

  override fun showInBestPositionFor(editor: Editor) {
    if (isOriginalWithReflection) super.showInBestPositionFor(editor)
    else show()
    //super.showInBestPositionFor(editor)//show relative point
  }

  override fun show(owner: Component) {
    if (isOriginalWithReflection) super.show(owner)
    else show()
    //super.show(owner)//show : Component owner, int aScreenX, int aScreenY, final boolean considerForcedXY
  }

  override fun showInScreenCoordinates(owner: Component, point: Point) {
    if (isOriginalWithReflection) super.showInScreenCoordinates(owner, point)
    else show()
    //super.showInScreenCoordinates(owner, point)//show : Component owner, int aScreenX, int aScreenY, final boolean considerForcedXY
  }

  override fun showInFocusCenter() {
    if (isOriginalWithReflection) super.showInFocusCenter()
    else {
      trySetContext(BeFocusCenterPopupContext())
      show()
    }
    //super.showInFocusCenter()//show : Component owner, int aScreenX, int aScreenY, final boolean considerForcedXY
  }

  override fun show(@NotNull owner: Component, aScreenX: Int, aScreenY: Int, considerForcedXY: Boolean) {
    if (isOriginalWithReflection) super.show(owner, aScreenX, aScreenY, considerForcedXY)

    // the only "branching"-point among all "show..." methods
    trySetContext(BeScreenPointPopupContext(aScreenX, aScreenY))
    show()
  }

  private fun show() {
    // require(!isShown && !isVisible)
    settings.popupListeners.forEach { listener -> listener?.beforeShown(LightweightWindowEvent(this)) }

    popupModel = settings.getBeModel(lifetime)

    if (popupsModel != null) {
      popupsModel.bePopups[popupId] = popupModel!!
      popupsModel.bePopups.adviseAddRemove(lifetime){type, i, popup ->
        if(i == popupId && type == AddRemove.Remove){
          popupModel?.result?.set(DialogResult.CLOSE)
          popupModel = null
        }
      }
      val popupNum = popupsModel.bePopups.size
      if (popupNum > 1) {
        logger.warn("Popup models is more then one: ${popupNum}.")
      }
    }

    if (isOriginalWithReflection) {
      BackendPopupsHost.fireLeadsPopupWasShown(popupModel!!, Lifetime.Eternal, realProject!!)
    }

    popupModel!!.loaded.fire()
    isShown = true
    isVisible = true
  }

  override fun canShow(): Boolean {
    return true
  }

  override fun showCenteredInCurrentWindow(project: Project) {
    if (isOriginalWithReflection) super.showCenteredInCurrentWindow(project)
    trySetContext(BeWindowCenterPopupContext())
    show() //show : Component owner, int aScreenX, int aScreenY, final boolean considerForcedXY
  }

  private fun trySetContext(popupContext: BePopupContext) {
    if (settings.context != null) return
    settings.context = popupContext
  }

  override fun setSize(size: Dimension) {
    if (isOriginalWithReflection) super.setSize(size)
    settings.size = size
  }

  /*override fun closeOk(e: InputEvent?) {
    //mostly used with e == null
    popupModel?.accept?.fire()
  }*/

  override fun cancel(e: InputEvent?) {
    popupModel?.result?.set(DialogResult.CANCEL)
    super.cancel(e)
  }


  override fun pack(width: Boolean, height: Boolean) {
    if (isOriginalWithReflection) super.pack(width, height)
    popupModel?.resize?.fire()
  }

  override fun <T : Any?> getUserData(userDataClass: Class<T>): T? {
    return super.getUserData(userDataClass)
    //TODO("Not yet implemented")//mostly for accessing update processor
  }

  override fun dispose() {
    if (isDisposed) return

    isDisposed = true
    isShown = false
    isVisible = false
    popupModel?.result?.set(DialogResult.CLOSE)
    popupsModel?.bePopups?.remove(popupId)
    super.dispose()
  }

  override fun isDisposed(): Boolean {
    if (isOriginalWithReflection) return super.isDisposed()
    return isDisposed
  }

  override fun getContent(): JComponent {
    if (isOriginalWithReflection) return super.getContent()
    return settings.component!!
  }


  //rarely used api
  override fun dispatchKeyEvent(e: KeyEvent): Boolean {
    //used only in Popup dispatcher, may be this action used only in frontend
    //convert to super
    //TODO("Not yet implemented")
    return super.dispatchKeyEvent(e)
  }

  override fun getBestPositionFor(dataContext: DataContext): Point {
    return Point()
    //TODO("Not yet implemented")
  }

  override fun setUiVisible(visible: Boolean) {
    //why? may be collapsed visibility can be used here
    //TODO("Not yet implemented")s
    super.setUiVisible(visible)
  }

  override fun setDataProvider(dataProvider: DataProvider) {
    if (isOriginalWithReflection) super.setDataProvider(dataProvider)
    //one usage in show message history action
    TODO("Not yet implemented")
  }

  override fun removeListener(listener: JBPopupListener) {
    if (isOriginalWithReflection) super.removeListener(listener)
    //not used
    TODO("Not yet implemented")
  }

  override fun isFocused(): Boolean {
    if (isOriginalWithReflection) return super.isFocused()
    TODO("Not yet implemented")
  }

  override fun moveToFitScreen() {
    if (isOriginalWithReflection) moveToFitScreen()
    //3 usages: combo, window, frontend popup
  }

  override fun isNativePopup(): Boolean {
    if (isOriginalWithReflection) return super.isNativePopup()
    // used only for restoring, and almost always is true(there are some popups like dialogs, see Dialog popupFactory)
    TODO("Not yet implemented")
  }

  override fun getOwner(): Component {
    if (isOriginalWithReflection) return super.getOwner()
    TODO("Not yet implemented")
  }
}
