package com.jetbrains.rdserver.ui.popup.bePopups.impl

import com.intellij.openapi.ui.ListComponentUpdater
import com.intellij.openapi.ui.popup.*
import com.intellij.openapi.util.Computable
import com.intellij.ui.ActiveComponent
import com.intellij.ui.components.JBScrollPane
import com.intellij.util.Consumer
import com.intellij.util.Function
import com.intellij.util.Processor
import com.jetbrains.codeWithMe.model.PopupsModel
import java.awt.Component
import java.awt.Dimension
import java.awt.Font
import java.awt.event.ActionListener
import javax.swing.*

public class BackendBePopupChooserBuilder<T> : PopupChooserBuilder<T> {
  constructor(popupsModel: PopupsModel?, table: JTable) : super(table){
    settings = BePopupWithListSettings(table)
    this.popupsModel = popupsModel
  }
  constructor(popupsModel: PopupsModel?, jList: JList<T>) : super(jList){
    settings = BePopupWithListSettings(jList)
    this.popupsModel = popupsModel
  } //jb list always used
  constructor(popupsModel: PopupsModel?, tree: JTree) : super(tree){
    settings = BePopupWithListSettings(tree)
    this.popupsModel = popupsModel
  } //is not relevant
  private var settings : BePopupWithListSettings<T>
  private val popupsModel: PopupsModel?

  override fun setTitle(title: String): PopupChooserBuilder<T> {
    settings.setTitle(title)
    return super.setTitle(title)
  }

  override fun setItemChosenCallback(callback: Consumer<in T>): IPopupChooserBuilder<T> {
    // is set in PopupComponentAdapter (from list)
    settings.itemChosenCallBack = callback
    return super.setItemChosenCallback(callback)
  }

  override fun setItemsChosenCallback(callback: Consumer<in MutableSet<out T>>): IPopupChooserBuilder<T> {
    // is set in PopupComponentAdapter (from list, from table, from tree)
    settings.itemsChosenCallBack = callback
    return super.setItemsChosenCallback(callback)
  }

  override fun setItemChoosenCallback(runnable: Runnable): PopupChooserBuilder<T> {
    // is set in PopupComponentAdapter (from list, from table, from tree)
    settings.itemChosenRunnable = runnable
    return super.setItemChoosenCallback(runnable)
  }

  override fun setSouthComponent(cmp: JComponent): PopupChooserBuilder<T> {
    settings.footer = cmp
    return super.setSouthComponent(cmp)
  }

  override fun setCouldPin(callback: Processor<in JBPopup>?): PopupChooserBuilder<T> {
    settings.pinCallback = callback
    return super.setCouldPin(callback)
  }

  override fun setRequestFocus(requestFocus: Boolean): PopupChooserBuilder<T> {
    settings.requestFocus = requestFocus
    return super.setRequestFocus(requestFocus)
  }

  override fun setResizable(forceResizable: Boolean): PopupChooserBuilder<T> {
    settings.resizable = forceResizable
    return super.setResizable(forceResizable)
  }

  override fun setMovable(forceMovable: Boolean): PopupChooserBuilder<T> {
    settings.movable = forceMovable
    return super.setMovable(forceMovable)
  }

  override fun setDimensionServiceKey(key: String?): PopupChooserBuilder<T> {
    settings.dimensionServiceKey = key
    return super.setDimensionServiceKey(key)
  }

  override fun setUseDimensionServiceForXYLocation(use: Boolean): PopupChooserBuilder<T> {
    settings.useKeyForXYLocation = use
    return super.setUseDimensionServiceForXYLocation(use)
  }

  override fun setCommandButton(commandButton: ActiveComponent): PopupChooserBuilder<T> {
    settings.rightToolbarComponent = commandButton
    return super.setCommandButton(commandButton)
  }

  override fun setModalContext(modalContext: Boolean): PopupChooserBuilder<T> {
    settings.isModal = modalContext
    return super.setModalContext(modalContext)
  }

  override fun setMinSize(dimension: Dimension?): PopupChooserBuilder<T> {
    settings.minimumSize = dimension
    return super.setMinSize(dimension)
  }

  override fun registerKeyboardAction(keyStroke: KeyStroke?, actionListener: ActionListener?): PopupChooserBuilder<T> {
    //used in uml and affected test structure + in super for enter [todo]
    return super.registerKeyboardAction(keyStroke, actionListener)
  }

  override fun setCancelKeyEnabled(enabled: Boolean): PopupChooserBuilder<T> {
    settings.cancelKeyEnabled = enabled
    return super.setCancelKeyEnabled(enabled)
  }
  override fun setCancelCallback(callback: Computable<Boolean>?): PopupChooserBuilder<T> {
    settings.cancelCallback = callback
    return super.setCancelCallback(callback)
  }
  override fun setCancelOnClickOutside(cancelOnClickOutside: Boolean): PopupChooserBuilder<T> {
    settings.cancelOnClickOutside = cancelOnClickOutside
    return super.setCancelOnClickOutside(cancelOnClickOutside)
  }

  override fun setSettingButton(button: Component?): PopupChooserBuilder<T> {
    settings.leftToolbarComponent = button
    return super.setSettingButton(button)
  }

  override fun addListener(listener: JBPopupListener?): PopupChooserBuilder<T> {
    settings.popupListeners.add(listener)
    return super.addListener(listener)
  }

  override fun setFilteringEnabled(namer: Function<Any, String>?): PopupChooserBuilder<T> {
    settings.namerForFiltering = if(namer != null) Function { param -> namer.`fun`(param) } else null
    return super.setFilteringEnabled(namer)
  }

  override fun setNamerForFiltering(namer: Function<in T, String>?): PopupChooserBuilder<T> {
    settings.namerForFiltering = namer
    return super.setNamerForFiltering(namer)
  }

  override fun setAutoSelectIfEmpty(autoSelect: Boolean): PopupChooserBuilder<T> {
    settings.autoSelectIfEmpty = autoSelect
    return super.setAutoSelectIfEmpty(autoSelect)
  }

  override fun setSelectionMode(selection: Int): IPopupChooserBuilder<T> {
    //listSettings.selectionMode = selection
    return super.setSelectionMode(selection)
  }

  override fun setSelectedValue(preselection: T, shouldScroll: Boolean): IPopupChooserBuilder<T> {
    //listSettings.selectedValue = preselection
    settings.shouldScrollToPreselectedValue = shouldScroll
    return super.setSelectedValue(preselection, shouldScroll)
  }

  override fun setItemSelectedCallback(c: Consumer<in T>?): IPopupChooserBuilder<T> {
    settings.itemSelectedCallback = c
    return super.setItemSelectedCallback(c)
  }

  override fun setFont(f: Font?): IPopupChooserBuilder<T> {
    settings.font = f
    return super.setFont(f)
  }

  override fun setRenderer(renderer: ListCellRenderer<in T>?): IPopupChooserBuilder<T> {
    return super.setRenderer(renderer)
  }

  override fun createPopup(): JBPopup {
    //todo [alisa]
    return BackendBeAbstractPopup(popupsModel, super.getChooserComponent(), settings)
  }

  //questions
  override fun getBackgroundUpdater(): ListComponentUpdater {
    return super.getBackgroundUpdater()
  }
  override fun withHintUpdateSupply(): IPopupChooserBuilder<T> {
    //??
    return super.withHintUpdateSupply()
  }

  //rarely used api
  override fun setAccessibleName(title: String?): IPopupChooserBuilder<T> {
    //?
    return super.setAccessibleName(title)
  }

  override fun getVisibleRowCount(): Int {
    //used for popup adapter
    return super.getVisibleRowCount()
  }
  override fun setVisibleRowCount(visibleRowCount: Int): IPopupChooserBuilder<T> {
    //one usage
    return super.setVisibleRowCount(visibleRowCount)
  }

  override fun setCloseOnEnter(closeOnEnter: Boolean): PopupChooserBuilder<T> {
    settings.closeOnEnter = closeOnEnter
    return super.setCloseOnEnter(closeOnEnter)
  }
  override fun setFocusOwners(focusOwners: Array<out Component>?): PopupChooserBuilder<T> {
    //where focus moves after closing (one usage, not sure if it's necessaru)
    return super.setFocusOwners(focusOwners)
  }

  override fun setAdText(ad: String?, alignment: Int): PopupChooserBuilder<T> {
    //overload without alignment is used
    settings.adText = ad
    return super.setAdText(ad, alignment)
  }
  override fun getItemsNamer(): Function<in T, String> {
    //used for list adapter only
    return super.getItemsNamer()
  }
  override fun getScrollPane(): JScrollPane {
    //used mostly for removing a border
    return JBScrollPane()
  }
  override fun getChooserComponent(): JComponent {
    //u
    return super.getChooserComponent()
  }
  override fun setEastComponent(cmp: JComponent): PopupChooserBuilder<T> {
    //never used
    return super.setEastComponent(cmp)
  }
  override fun setAlpha(alpha: Float): PopupChooserBuilder<T> {
    //never used
    return super.setAlpha(alpha)
  }

  override fun setMayBeParent(mayBeParent: Boolean): PopupChooserBuilder<T> {
    //vague meaning
    return super.setMayBeParent(mayBeParent)
  }

  override fun getPreferableFocusComponent(): JComponent {
    //used to dirty hack in master detail popup builder(used only in bookmarks) for accessing speed search
    return super.getPreferableFocusComponent()
  }

  override fun addCenterComponentToContentPane(contentPane: JPanel?, component: JComponent?) {
    //used in master detail and in super for wrapping content in scroll pane
    super.addCenterComponentToContentPane(contentPane, component)
  }
  override fun setAutoselectOnMouseMove(doAutoSelect: Boolean): PopupChooserBuilder<T> {
    //3 usages: one in master detail
    return super.setAutoselectOnMouseMove(doAutoSelect)
  }

  override fun isAutoselectOnMouseMove(): Boolean {
    //usages only in adapter
    return super.isAutoselectOnMouseMove()
  }
  override fun setCancelOnWindowDeactivation(cancelOnWindowDeactivation: Boolean): PopupChooserBuilder<T> {
    //one usage in remote vm connetion
    settings.cancelOnWindowDeactivation = cancelOnWindowDeactivation
    return super.setCancelOnWindowDeactivation(cancelOnWindowDeactivation)
  }

  override fun setAutoPackHeightOnFiltering(autoPackHeightOnFiltering: Boolean): IPopupChooserBuilder<T> {
    //todo [alisa]
    return super.setAutoPackHeightOnFiltering(autoPackHeightOnFiltering)
  }

  override fun isAutoPackHeightOnFiltering(): Boolean {
    //todo [alisa]
    return super.isAutoPackHeightOnFiltering()
  }
}
