package com.jetbrains.rdserver.ui.popup.bePopups.impl

import com.intellij.openapi.project.Project
import com.intellij.openapi.ui.popup.*
import com.intellij.openapi.util.Computable
import com.intellij.openapi.util.Condition
import com.intellij.openapi.util.Pair
import com.intellij.ui.ActiveComponent
import com.intellij.ui.popup.ComponentPopupBuilderImpl
import com.intellij.util.BooleanFunction
import com.intellij.util.Processor
import com.jetbrains.codeWithMe.model.PopupsModel
import com.jetbrains.rd.util.lifetime.Lifetime
import java.awt.Component
import java.awt.Dimension
import java.awt.event.ActionListener
import java.awt.event.KeyEvent
import java.util.function.Supplier
import javax.swing.JComponent
import javax.swing.KeyStroke
import javax.swing.SwingConstants

// FIXME 2 such secondary constructors look bad
class BackendComponentBePopupBuilder private constructor(
  private val component: JComponent,
  private val popupsModel: PopupsModel?,
  private val originalComponentPopupBuilder: ComponentPopupBuilder?,
  private val project: Project?
) : ComponentPopupBuilder {

  constructor(component: JComponent, popupsModel: PopupsModel?) :
    this(component, popupsModel, null, null)

  constructor(component: JComponent, originalComponentPopupBuilder: ComponentPopupBuilder?, project: Project) :
    this(component, null, originalComponentPopupBuilder, project)

  val lifetime = Lifetime.Eternal
  private val settings = BePopupSettings(component)

  override fun setCancelOnOtherWindowOpen(cancelOnWindow: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelOnOtherWindowOpen(cancelOnWindow)
    settings.cancelOnOtherWindowOpen = cancelOnWindow
    return this
  }
  override fun setCancelOnClickOutside(cancel: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelOnClickOutside(cancel)
    settings.cancelOnClickOutside = cancel
    return this
  }
  override fun setCancelOnWindowDeactivation(cancelOnWindowDeactivation: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelOnWindowDeactivation(cancelOnWindowDeactivation)
    settings.cancelOnWindowDeactivation = cancelOnWindowDeactivation
    return this
  }
  override fun setCancelCallback(shouldProceed: Computable<Boolean>): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelCallback(shouldProceed)
    settings.cancelCallback = shouldProceed
    return this
  }

  override fun setCancelKeyEnabled(enabled: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelKeyEnabled(enabled)
    settings.cancelKeyEnabled = enabled
    return this
  }

  override fun setResizable(forceResizable: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setResizable(forceResizable)
    settings.resizable = forceResizable
    return this
  }

  override fun setTitle(title: String?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setTitle(title)
    settings.setTitle(title)
    return this
  }
  override fun setAdText(text: String?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setAdText(text)
    return setAdText(text, SwingConstants.LEFT)
  }
  override fun setSettingButtons(button: Component): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setSettingButtons(button)
    settings.leftToolbarComponent = button
    return this
  }

  override fun setModalContext(modal: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setModalContext(modal)
    settings.isModal = modal
    return this
  }

  override fun setMovable(forceMovable: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setMovable(forceMovable)
    settings.movable = forceMovable
    return this
  }

  override fun setFocusable(focusable: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setFocusable(focusable)
    settings.focusable = focusable
    return this
  }

  override fun setRequestFocus(requestFocus: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setRequestFocus(requestFocus)
    settings.requestFocus = requestFocus
    return this
  }

  override fun setCouldPin(callback: Processor<in JBPopup>?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCouldPin(callback)
    settings.pinCallback = callback
    return this
  }

  override fun setMinSize(minSize: Dimension?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setMinSize(minSize)
    settings.minimumSize = minSize
    return this
  }
  override fun setDimensionServiceKey(project: Project?, key: String?, useForXYLocation: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setDimensionServiceKey(project, key, useForXYLocation)
    settings.dimensionServiceKey = key
    settings.useKeyForXYLocation = useForXYLocation
    return this
  }

  override fun setKeyEventHandler(handler: BooleanFunction<in KeyEvent>): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setKeyEventHandler(handler)
    settings.keyEventHandler = handler
    return this
  }

  override fun setCommandButton(commandButton: ActiveComponent): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCommandButton(commandButton)
    settings.rightToolbarComponent = commandButton
    return this
  }

  override fun setKeyboardActions(keyboardActions: MutableList<out Pair<ActionListener, KeyStroke>>): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setKeyboardActions(keyboardActions)
    //[alisa]todo
    return this
  }

  override fun addListener(listener: JBPopupListener): ComponentPopupBuilder {
    originalComponentPopupBuilder?.addListener(listener)
    settings.popupListeners.add(listener)
    return this
  }
  override fun setOkHandler(okHandler: Runnable?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setOkHandler(okHandler)
    settings.okHandler = okHandler
    return this
  }

  override fun createPopup(): JBPopup {
    if (originalComponentPopupBuilder != null) {
      return (originalComponentPopupBuilder as ComponentPopupBuilderImpl).createPopup(
        Supplier { BackendBeAbstractPopup(null, component, settings, true, project) })
      //return JBPopupDelegatorWithReflection(originalPopup, reflectedPopup)  // not possible because ShowUsagesAction casts JBPopup to AbstractPopup
    }
    return BackendBeAbstractPopup(popupsModel, component, settings)
  }

  //rarely used api
  override fun setLocateByContent(byContent: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setLocateByContent(byContent)
    return this
  }

  override fun setAdText(text: String?, textAlignment: Int): ComponentPopupBuilder {
    settings.adText = text
    //alignment used only in one place
    return this
  }
  override fun setShowBorder(show: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setShowBorder(show)
    return this//may be needed?
  }
  override fun setFocusOwners(focusOwners: Array<out Component>?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setFocusOwners(focusOwners)
    return this
  }
  override fun addUserData(`object`: Any?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.addUserData(`object`)
    return this //mostly used for adding simple_window, idk
  }
  override fun setMayBeParent(mayBeParent: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setMayBeParent(mayBeParent)
    return this
  }
  override fun setBelongsToGlobalPopupStack(isInStack: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setBelongsToGlobalPopupStack(isInStack)
    return this // for combo/textbox/other should be false
  }
  override fun setAlpha(alpha: Float): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setAlpha(alpha)
    return this //
  }

  override fun setProject(project: Project?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setProject(project)
    return this
  }

  override fun setNormalWindowLevel(b: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setNormalWindowLevel(b)
    return this
  }
  override fun setRequestFocusCondition(project: Project, condition: Condition<in Project>): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setRequestFocusCondition(project, condition)
    return this
  }
  override fun setShowShadow(show: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setShowShadow(show)
    return this
  }
  override fun setMaskProvider(maskProvider: MaskProvider?): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setMaskProvider(maskProvider)
    return this
  }

  override fun setCancelOnMouseOutCallback(shouldCancel: MouseChecker): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelOnMouseOutCallback(shouldCancel)
    return this //vague meaning, mostly for validation
  }

  override fun setTitleIcon(icon: ActiveIcon): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setTitleIcon(icon)
    return this
  }

  override fun setCancelButton(cancelButton: IconButton): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setCancelButton(cancelButton)
    return this
  }

  override fun setLocateWithinScreenBounds(within: Boolean): ComponentPopupBuilder {
    originalComponentPopupBuilder?.setLocateWithinScreenBounds(within)
    //this.placeWithinScreen = within
    return this
  }
}
