package com.jetbrains.rdserver.ui.popup.bePopups.impl

import com.intellij.openapi.rd.createNestedDisposable
import com.intellij.openapi.ui.popup.IconButton
import com.intellij.openapi.ui.popup.JBPopup
import com.intellij.openapi.ui.popup.JBPopupListener
import com.intellij.openapi.util.Computable
import com.intellij.ui.ActiveComponent
import com.intellij.util.BooleanFunction
import com.intellij.util.Consumer
import com.intellij.util.Function
import com.intellij.util.Processor
import com.intellij.util.ui.update.MergingUpdateQueue
import com.jetbrains.ide.model.uiautomation.*
import com.jetbrains.rd.ui.bedsl.dsl.label
import com.jetbrains.rd.ui.bedsl.dsl.setText
import com.jetbrains.rd.ui.bedsl.dsl.tryGetSettings
import com.jetbrains.rd.ui.bedsl.dsl.withCursor
import com.jetbrains.rd.ui.bedsl.extensions.applyToLiveDescendantsOfType
import com.jetbrains.rd.ui.bedsl.extensions.descendantsOfType
import com.jetbrains.rd.ui.bedsl.extensions.grouppedFire
import com.jetbrains.rd.util.lifetime.Lifetime
import com.jetbrains.rd.util.reactive.fire
import com.jetbrains.rdserver.ui.converters.toBeModel
import java.awt.Component
import java.awt.Dimension
import java.awt.Font
import java.awt.event.KeyEvent
import javax.swing.JComponent
import javax.swing.JList
import javax.swing.SwingUtilities

class BePopupWithListSettings<T>(jComponent: JComponent?) : BePopupSettings(jComponent) {

  init {
    requestFocus = true
    closeOnEnter = true
  }

  var namerForFiltering: Function<in T, String>? = null
  var itemSelectedCallback: Consumer<in T>? = null
  var shouldScrollToPreselectedValue: Boolean = true

  // var selectedValue: T?  = null
  // var selectionMode: Int = 0//change to multiselection
  var autoSelectIfEmpty: Boolean = false
  var itemsChosenCallBack: Consumer<in MutableSet<out T>>? = null
  var itemChosenCallBack: Consumer<in T>? = null
  var itemChosenRunnable: Runnable? = null

  override val content get() = component!!.toBeModel().withCursor(BeCursor.HAND)

  override fun getBeModel(lifetime: Lifetime): BeAbstractPopup {
    val popup = super.getBeModel(lifetime)
    fixModel(popup, lifetime)
    return popup
  }

  private fun fixModel(popup: BeAbstractPopup, lifetime: Lifetime) {
    val model = popup.content.descendantsOfType(BeTreeGrid::class.java).firstOrNull()
    val component = component
    val beList = model ?: return

    val list = component as? JList<*>
    val mergingQueue = MergingUpdateQueue("BePopupResize", 200, true, component, lifetime.createNestedDisposable())
    beList.applyToLiveDescendantsOfType(BeTreeGridLine::class.java, lifetime) { line ->
      popup.resize.grouppedFire(mergingQueue)
      if (list != null) {
        line.isSelected.advise(lifetime) {
          val t = list.selectedValue as? T
          if (t == null) return@advise
          if (it) {
            itemSelectedCallback?.consume(t)
          }
        }

      }
    }
    if(list != null)
      beList.configuration.speedSearch.set(BeSpeedSearch.ENABLED_FILTER_AUTO)
    else
      beList.configuration.speedSearch.set(BeSpeedSearch.ENABLED_HIGHLIGHT)

    beList.mouseClicked.advise(lifetime) {
      popup.accept.fire()
      if (closeOnEnter) { //todo: rethink results sequence set, states may have to be redefined
        popup.result.set(DialogResult.OK)
        popup.result.set(DialogResult.CANCEL)
      }
    }

    if (shouldScrollToPreselectedValue)
      beList.configuration.autoScroll.set(BeScrollSettings.AUTOSCROLL_TO_SELECTION)
    beList.configuration.selectOnMouseOver.set(true)

    //todo: may be this signal should be in treegrid, and be splitted on two - single_click && enter
    popup.accept.advise(lifetime) {
      itemChosenRunnable?.run()
    }
    return
  }
}

open class BePopupSettings(val component: JComponent?) {
  init {
    component?.putClientProperty("THIN_CLIENT", true)
  }

  var context: BePopupContext? = null
  lateinit var size: Dimension
  var font: Font? = null
  var closeOnEnter: Boolean = false
  var footer: JComponent? = null
  var leftToolbarComponent: Component? = null
  var useKeyForXYLocation: Boolean = false
  var dimensionServiceKey: String? = null
  var minimumSize: Dimension? = null
  var requestFocus: Boolean = false
  var focusable: Boolean = true
  var movable: Boolean = false
  var isModal: Boolean = true
  var rightToolbarComponent: ActiveComponent? = null
  var cancelIconButton: IconButton? = null
  private var title: BeLabel? = null
  var adText: String? = ""
  var resizable: Boolean = false
  var cancelKeyEnabled: Boolean = true

  var cancelOnWindowDeactivation: Boolean = true
  var cancelOnClickOutside: Boolean = true
  var cancelOnOtherWindowOpen: Boolean = true

  var cancelCallback: Computable<Boolean>? = null
  var pinCallback: Processor<in JBPopup>? = null
  lateinit var keyEventHandler: BooleanFunction<in KeyEvent>

  //order of calls: listeners -> okHandler -> finalRunnable
  var okHandler: Runnable? = Runnable {}
  var finalRunnable: Runnable? = Runnable {}
  val popupListeners: MutableList<JBPopupListener?> = java.util.concurrent.CopyOnWriteArrayList<JBPopupListener?>()
  /*private fun addCancelCallback(cbb: Computable<Boolean>) {
     val callback: Computable<Boolean> = myCancelCallback
     myCancelCallback = Computable { cbb.compute() && (callback == null || callback.compute()) }
   }
 */

  internal open fun getBeModel(lifetime: Lifetime): BeAbstractPopup {
    val content = content
    if (font != null)
      content.applyToLiveDescendantsOfType(BeAbstractText::class.java, lifetime) {
        it.tryGetSettings()?.fontStyle?.set(BeFontStyle.MONOSPACE)
      }
    return BeAbstractPopup(
      content, //.inScrollablePanel(),//todo: fix sizes
      title,
      leftToolbarComponent?.toBeModel() ?: BeSpacer(),
      rightToolbarComponent?.component?.toBeModel() ?: BeSpacer(),
      footer?.toBeModel() ?: BeSpacer(),
      adText,
      requestFocus,
      focusable,
      resizable,
      movable,
      closeOnEnter,
      cancelKeyEnabled,
      cancelOnWindowDeactivation,
      cancelOnClickOutside,
      cancelOnOtherWindowOpen,
      if (isModal) DialogModality.MODAL else DialogModality.NON_MODAL,
      if (dimensionServiceKey != null) if (useKeyForXYLocation) DialogLayoutPersistenceMode.SIZE_AND_POSITION else DialogLayoutPersistenceMode.SIZE else DialogLayoutPersistenceMode.NONE
    ).apply {
      title
      controlId.value = dimensionServiceKey ?: ""
      this@BePopupSettings.context?.let { context.set(it) }
      result.advise(lifetime) {
        //todo: listeners
        if (it == DialogResult.CANCEL) {
          SwingUtilities.invokeLater { result.set(if (cancelCallback?.compute() != false) DialogResult.CLOSE else DialogResult.NONE) }
          return@advise
        }
        if (it == DialogResult.OK)
          okHandler?.run()
        finalRunnable?.run()
      }
    }
  }

  fun setTitle(titleText: String?) {
    if(titleText == null) return
    if(title == null)
      this.title = label(titleText)
    else
      this.title!!.setText(titleText)
  }

  protected open val content get() = component!!.toBeModel()
}

private fun BeControl.inScrollablePanel(): BeScrollablePanel {
  return BeScrollablePanel(BeScrollbarPolicy.BOTH, BeControlSizePredefined(
    BeSize(BeControlSizeType.FIT_TO_CONTENT),
    BeSize(BeControlSizeType.FIT_TO_CONTENT)
  ),this@inScrollablePanel)
}
