package com.jetbrains.rdserver.unattendedHost

import com.intellij.codeWithMe.ClientId
import com.intellij.openapi.components.Service
import com.intellij.openapi.components.service
import com.intellij.openapi.project.Project
import com.jetbrains.rdserver.startup.SecureClientData
import java.util.*

@Service
class UnattendedHostProjectService(val project: Project) {
  companion object {
    fun getInstance(project: Project) = project.service<UnattendedHostProjectService>()
  }

  private var currentToken: String? = null
  private var lastClientFingerprintToUseValidToken: String? = null
  private var lastClientIdToUseValidToken: ClientId? = null

  fun createSingleUseConnectionToken(): String {
    require(UnattendedHostManager.getInstance().isUnattendedMode)

    return UUID.randomUUID().toString().also { currentToken = it }
  }

  fun createOrGetSingleUseConnectionToken(): String {
    require(UnattendedHostManager.getInstance().isUnattendedMode)

    return currentToken ?: createSingleUseConnectionToken()
  }

  fun verifySingleUseConnectionToken(data: SecureClientData): Boolean {
    val token = data.singleUseToken
    if (token == currentToken) {
      currentToken = null
      lastClientFingerprintToUseValidToken = data.clientFingerprint
      lastClientIdToUseValidToken = data.clientId
      return true
    }

    return false
  }

  fun hasClientUsedValidTokenPreviously(data: SecureClientData) = data.clientFingerprint == lastClientFingerprintToUseValidToken

  fun wasClientJoinedViaToken(client: ClientId) = client == lastClientIdToUseValidToken
}
